#define _CFFI_

/* We try to define Py_LIMITED_API before including Python.h.

   Mess: we can only define it if Py_DEBUG, Py_TRACE_REFS and
   Py_REF_DEBUG are not defined.  This is a best-effort approximation:
   we can learn about Py_DEBUG from pyconfig.h, but it is unclear if
   the same works for the other two macros.  Py_DEBUG implies them,
   but not the other way around.

   Issue #350 is still open: on Windows, the code here causes it to link
   with PYTHON36.DLL (for example) instead of PYTHON3.DLL.  A fix was
   attempted in 164e526a5515 and 14ce6985e1c3, but reverted: virtualenv
   does not make PYTHON3.DLL available, and so the "correctly" compiled
   version would not run inside a virtualenv.  We will re-apply the fix
   after virtualenv has been fixed for some time.  For explanation, see
   issue #355.  For a workaround if you want PYTHON3.DLL and don't worry
   about virtualenv, see issue #350.  See also 'py_limited_api' in
   setuptools_ext.py.
*/
#if !defined(_CFFI_USE_EMBEDDING) && !defined(Py_LIMITED_API)
#  include <pyconfig.h>
#  if !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG)
#    define Py_LIMITED_API
#  endif
#endif

#include <Python.h>
#ifdef __cplusplus
extern "C" {
#endif
#include <stddef.h>

/* This part is from file 'cffi/parse_c_type.h'.  It is copied at the
   beginning of C sources generated by CFFI's ffi.set_source(). */

typedef void *_cffi_opcode_t;

#define _CFFI_OP(opcode, arg)   (_cffi_opcode_t)(opcode | (((uintptr_t)(arg)) << 8))
#define _CFFI_GETOP(cffi_opcode)    ((unsigned char)(uintptr_t)cffi_opcode)
#define _CFFI_GETARG(cffi_opcode)   (((intptr_t)cffi_opcode) >> 8)

#define _CFFI_OP_PRIMITIVE       1
#define _CFFI_OP_POINTER         3
#define _CFFI_OP_ARRAY           5
#define _CFFI_OP_OPEN_ARRAY      7
#define _CFFI_OP_STRUCT_UNION    9
#define _CFFI_OP_ENUM           11
#define _CFFI_OP_FUNCTION       13
#define _CFFI_OP_FUNCTION_END   15
#define _CFFI_OP_NOOP           17
#define _CFFI_OP_BITFIELD       19
#define _CFFI_OP_TYPENAME       21
#define _CFFI_OP_CPYTHON_BLTN_V 23   // varargs
#define _CFFI_OP_CPYTHON_BLTN_N 25   // noargs
#define _CFFI_OP_CPYTHON_BLTN_O 27   // O  (i.e. a single arg)
#define _CFFI_OP_CONSTANT       29
#define _CFFI_OP_CONSTANT_INT   31
#define _CFFI_OP_GLOBAL_VAR     33
#define _CFFI_OP_DLOPEN_FUNC    35
#define _CFFI_OP_DLOPEN_CONST   37
#define _CFFI_OP_GLOBAL_VAR_F   39
#define _CFFI_OP_EXTERN_PYTHON  41

#define _CFFI_PRIM_VOID          0
#define _CFFI_PRIM_BOOL          1
#define _CFFI_PRIM_CHAR          2
#define _CFFI_PRIM_SCHAR         3
#define _CFFI_PRIM_UCHAR         4
#define _CFFI_PRIM_SHORT         5
#define _CFFI_PRIM_USHORT        6
#define _CFFI_PRIM_INT           7
#define _CFFI_PRIM_UINT          8
#define _CFFI_PRIM_LONG          9
#define _CFFI_PRIM_ULONG        10
#define _CFFI_PRIM_LONGLONG     11
#define _CFFI_PRIM_ULONGLONG    12
#define _CFFI_PRIM_FLOAT        13
#define _CFFI_PRIM_DOUBLE       14
#define _CFFI_PRIM_LONGDOUBLE   15

#define _CFFI_PRIM_WCHAR        16
#define _CFFI_PRIM_INT8         17
#define _CFFI_PRIM_UINT8        18
#define _CFFI_PRIM_INT16        19
#define _CFFI_PRIM_UINT16       20
#define _CFFI_PRIM_INT32        21
#define _CFFI_PRIM_UINT32       22
#define _CFFI_PRIM_INT64        23
#define _CFFI_PRIM_UINT64       24
#define _CFFI_PRIM_INTPTR       25
#define _CFFI_PRIM_UINTPTR      26
#define _CFFI_PRIM_PTRDIFF      27
#define _CFFI_PRIM_SIZE         28
#define _CFFI_PRIM_SSIZE        29
#define _CFFI_PRIM_INT_LEAST8   30
#define _CFFI_PRIM_UINT_LEAST8  31
#define _CFFI_PRIM_INT_LEAST16  32
#define _CFFI_PRIM_UINT_LEAST16 33
#define _CFFI_PRIM_INT_LEAST32  34
#define _CFFI_PRIM_UINT_LEAST32 35
#define _CFFI_PRIM_INT_LEAST64  36
#define _CFFI_PRIM_UINT_LEAST64 37
#define _CFFI_PRIM_INT_FAST8    38
#define _CFFI_PRIM_UINT_FAST8   39
#define _CFFI_PRIM_INT_FAST16   40
#define _CFFI_PRIM_UINT_FAST16  41
#define _CFFI_PRIM_INT_FAST32   42
#define _CFFI_PRIM_UINT_FAST32  43
#define _CFFI_PRIM_INT_FAST64   44
#define _CFFI_PRIM_UINT_FAST64  45
#define _CFFI_PRIM_INTMAX       46
#define _CFFI_PRIM_UINTMAX      47
#define _CFFI_PRIM_FLOATCOMPLEX 48
#define _CFFI_PRIM_DOUBLECOMPLEX 49
#define _CFFI_PRIM_CHAR16       50
#define _CFFI_PRIM_CHAR32       51

#define _CFFI__NUM_PRIM         52
#define _CFFI__UNKNOWN_PRIM           (-1)
#define _CFFI__UNKNOWN_FLOAT_PRIM     (-2)
#define _CFFI__UNKNOWN_LONG_DOUBLE    (-3)

#define _CFFI__IO_FILE_STRUCT         (-1)


struct _cffi_global_s {
    const char *name;
    void *address;
    _cffi_opcode_t type_op;
    void *size_or_direct_fn;  // OP_GLOBAL_VAR: size, or 0 if unknown
                              // OP_CPYTHON_BLTN_*: addr of direct function
};

struct _cffi_getconst_s {
    unsigned long long value;
    const struct _cffi_type_context_s *ctx;
    int gindex;
};

struct _cffi_struct_union_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_STRUCT_UNION
    int flags;               // _CFFI_F_* flags below
    size_t size;
    int alignment;
    int first_field_index;   // -> _cffi_fields array
    int num_fields;
};
#define _CFFI_F_UNION         0x01   // is a union, not a struct
#define _CFFI_F_CHECK_FIELDS  0x02   // complain if fields are not in the
                                     // "standard layout" or if some are missing
#define _CFFI_F_PACKED        0x04   // for CHECK_FIELDS, assume a packed struct
#define _CFFI_F_EXTERNAL      0x08   // in some other ffi.include()
#define _CFFI_F_OPAQUE        0x10   // opaque

struct _cffi_field_s {
    const char *name;
    size_t field_offset;
    size_t field_size;
    _cffi_opcode_t field_type_op;
};

struct _cffi_enum_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_ENUM
    int type_prim;           // _CFFI_PRIM_xxx
    const char *enumerators; // comma-delimited string
};

struct _cffi_typename_s {
    const char *name;
    int type_index;   /* if opaque, points to a possibly artificial
                         OP_STRUCT which is itself opaque */
};

struct _cffi_type_context_s {
    _cffi_opcode_t *types;
    const struct _cffi_global_s *globals;
    const struct _cffi_field_s *fields;
    const struct _cffi_struct_union_s *struct_unions;
    const struct _cffi_enum_s *enums;
    const struct _cffi_typename_s *typenames;
    int num_globals;
    int num_struct_unions;
    int num_enums;
    int num_typenames;
    const char *const *includes;
    int num_types;
    int flags;      /* future extension */
};

struct _cffi_parse_info_s {
    const struct _cffi_type_context_s *ctx;
    _cffi_opcode_t *output;
    unsigned int output_size;
    size_t error_location;
    const char *error_message;
};

struct _cffi_externpy_s {
    const char *name;
    size_t size_of_result;
    void *reserved1, *reserved2;
};

#ifdef _CFFI_INTERNAL
static int parse_c_type(struct _cffi_parse_info_s *info, const char *input);
static int search_in_globals(const struct _cffi_type_context_s *ctx,
                             const char *search, size_t search_len);
static int search_in_struct_unions(const struct _cffi_type_context_s *ctx,
                                   const char *search, size_t search_len);
#endif

/* this block of #ifs should be kept exactly identical between
   c/_cffi_backend.c, cffi/vengine_cpy.py, cffi/vengine_gen.py
   and cffi/_cffi_include.h */
#if defined(_MSC_VER)
# include <malloc.h>   /* for alloca() */
# if _MSC_VER < 1600   /* MSVC < 2010 */
   typedef __int8 int8_t;
   typedef __int16 int16_t;
   typedef __int32 int32_t;
   typedef __int64 int64_t;
   typedef unsigned __int8 uint8_t;
   typedef unsigned __int16 uint16_t;
   typedef unsigned __int32 uint32_t;
   typedef unsigned __int64 uint64_t;
   typedef __int8 int_least8_t;
   typedef __int16 int_least16_t;
   typedef __int32 int_least32_t;
   typedef __int64 int_least64_t;
   typedef unsigned __int8 uint_least8_t;
   typedef unsigned __int16 uint_least16_t;
   typedef unsigned __int32 uint_least32_t;
   typedef unsigned __int64 uint_least64_t;
   typedef __int8 int_fast8_t;
   typedef __int16 int_fast16_t;
   typedef __int32 int_fast32_t;
   typedef __int64 int_fast64_t;
   typedef unsigned __int8 uint_fast8_t;
   typedef unsigned __int16 uint_fast16_t;
   typedef unsigned __int32 uint_fast32_t;
   typedef unsigned __int64 uint_fast64_t;
   typedef __int64 intmax_t;
   typedef unsigned __int64 uintmax_t;
# else
#  include <stdint.h>
# endif
# if _MSC_VER < 1800   /* MSVC < 2013 */
#  ifndef __cplusplus
    typedef unsigned char _Bool;
#  endif
# endif
#else
# include <stdint.h>
# if (defined (__SVR4) && defined (__sun)) || defined(_AIX) || defined(__hpux)
#  include <alloca.h>
# endif
#endif

#ifdef __GNUC__
# define _CFFI_UNUSED_FN  __attribute__((unused))
#else
# define _CFFI_UNUSED_FN  /* nothing */
#endif

#ifdef __cplusplus
# ifndef _Bool
   typedef bool _Bool;   /* semi-hackish: C++ has no _Bool; bool is builtin */
# endif
#endif

/**********  CPython-specific section  **********/
#ifndef PYPY_VERSION


#if PY_MAJOR_VERSION >= 3
# define PyInt_FromLong PyLong_FromLong
#endif

#define _cffi_from_c_double PyFloat_FromDouble
#define _cffi_from_c_float PyFloat_FromDouble
#define _cffi_from_c_long PyInt_FromLong
#define _cffi_from_c_ulong PyLong_FromUnsignedLong
#define _cffi_from_c_longlong PyLong_FromLongLong
#define _cffi_from_c_ulonglong PyLong_FromUnsignedLongLong
#define _cffi_from_c__Bool PyBool_FromLong

#define _cffi_to_c_double PyFloat_AsDouble
#define _cffi_to_c_float PyFloat_AsDouble

#define _cffi_from_c_int(x, type)                                        \
    (((type)-1) > 0 ? /* unsigned */                                     \
        (sizeof(type) < sizeof(long) ?                                   \
            PyInt_FromLong((long)x) :                                    \
         sizeof(type) == sizeof(long) ?                                  \
            PyLong_FromUnsignedLong((unsigned long)x) :                  \
            PyLong_FromUnsignedLongLong((unsigned long long)x)) :        \
        (sizeof(type) <= sizeof(long) ?                                  \
            PyInt_FromLong((long)x) :                                    \
            PyLong_FromLongLong((long long)x)))

#define _cffi_to_c_int(o, type)                                          \
    ((type)(                                                             \
     sizeof(type) == 1 ? (((type)-1) > 0 ? (type)_cffi_to_c_u8(o)        \
                                         : (type)_cffi_to_c_i8(o)) :     \
     sizeof(type) == 2 ? (((type)-1) > 0 ? (type)_cffi_to_c_u16(o)       \
                                         : (type)_cffi_to_c_i16(o)) :    \
     sizeof(type) == 4 ? (((type)-1) > 0 ? (type)_cffi_to_c_u32(o)       \
                                         : (type)_cffi_to_c_i32(o)) :    \
     sizeof(type) == 8 ? (((type)-1) > 0 ? (type)_cffi_to_c_u64(o)       \
                                         : (type)_cffi_to_c_i64(o)) :    \
     (Py_FatalError("unsupported size for type " #type), (type)0)))

#define _cffi_to_c_i8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[1])
#define _cffi_to_c_u8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[2])
#define _cffi_to_c_i16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[3])
#define _cffi_to_c_u16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[4])
#define _cffi_to_c_i32                                                   \
                 ((int(*)(PyObject *))_cffi_exports[5])
#define _cffi_to_c_u32                                                   \
                 ((unsigned int(*)(PyObject *))_cffi_exports[6])
#define _cffi_to_c_i64                                                   \
                 ((long long(*)(PyObject *))_cffi_exports[7])
#define _cffi_to_c_u64                                                   \
                 ((unsigned long long(*)(PyObject *))_cffi_exports[8])
#define _cffi_to_c_char                                                  \
                 ((int(*)(PyObject *))_cffi_exports[9])
#define _cffi_from_c_pointer                                             \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[10])
#define _cffi_to_c_pointer                                               \
    ((char *(*)(PyObject *, struct _cffi_ctypedescr *))_cffi_exports[11])
#define _cffi_get_struct_layout                                          \
    not used any more
#define _cffi_restore_errno                                              \
    ((void(*)(void))_cffi_exports[13])
#define _cffi_save_errno                                                 \
    ((void(*)(void))_cffi_exports[14])
#define _cffi_from_c_char                                                \
    ((PyObject *(*)(char))_cffi_exports[15])
#define _cffi_from_c_deref                                               \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[16])
#define _cffi_to_c                                                       \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[17])
#define _cffi_from_c_struct                                              \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[18])
#define _cffi_to_c_wchar_t                                               \
    ((_cffi_wchar_t(*)(PyObject *))_cffi_exports[19])
#define _cffi_from_c_wchar_t                                             \
    ((PyObject *(*)(_cffi_wchar_t))_cffi_exports[20])
#define _cffi_to_c_long_double                                           \
    ((long double(*)(PyObject *))_cffi_exports[21])
#define _cffi_to_c__Bool                                                 \
    ((_Bool(*)(PyObject *))_cffi_exports[22])
#define _cffi_prepare_pointer_call_argument                              \
    ((Py_ssize_t(*)(struct _cffi_ctypedescr *,                           \
                    PyObject *, char **))_cffi_exports[23])
#define _cffi_convert_array_from_object                                  \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[24])
#define _CFFI_CPIDX  25
#define _cffi_call_python                                                \
    ((void(*)(struct _cffi_externpy_s *, char *))_cffi_exports[_CFFI_CPIDX])
#define _cffi_to_c_wchar3216_t                                           \
    ((int(*)(PyObject *))_cffi_exports[26])
#define _cffi_from_c_wchar3216_t                                         \
    ((PyObject *(*)(int))_cffi_exports[27])
#define _CFFI_NUM_EXPORTS 28

struct _cffi_ctypedescr;

static void *_cffi_exports[_CFFI_NUM_EXPORTS];

#define _cffi_type(index)   (                           \
    assert((((uintptr_t)_cffi_types[index]) & 1) == 0), \
    (struct _cffi_ctypedescr *)_cffi_types[index])

static PyObject *_cffi_init(const char *module_name, Py_ssize_t version,
                            const struct _cffi_type_context_s *ctx)
{
    PyObject *module, *o_arg, *new_module;
    void *raw[] = {
        (void *)module_name,
        (void *)version,
        (void *)_cffi_exports,
        (void *)ctx,
    };

    module = PyImport_ImportModule("_cffi_backend");
    if (module == NULL)
        goto failure;

    o_arg = PyLong_FromVoidPtr((void *)raw);
    if (o_arg == NULL)
        goto failure;

    new_module = PyObject_CallMethod(
        module, (char *)"_init_cffi_1_0_external_module", (char *)"O", o_arg);

    Py_DECREF(o_arg);
    Py_DECREF(module);
    return new_module;

  failure:
    Py_XDECREF(module);
    return NULL;
}


#ifdef HAVE_WCHAR_H
typedef wchar_t _cffi_wchar_t;
#else
typedef uint16_t _cffi_wchar_t;   /* same random pick as _cffi_backend.c */
#endif

_CFFI_UNUSED_FN static uint16_t _cffi_to_c_char16_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return (uint16_t)_cffi_to_c_wchar_t(o);
    else
        return (uint16_t)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char16_t(uint16_t x)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

_CFFI_UNUSED_FN static int _cffi_to_c_char32_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return (int)_cffi_to_c_wchar_t(o);
    else
        return (int)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char32_t(unsigned int x)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

union _cffi_union_alignment_u {
    unsigned char m_char;
    unsigned short m_short;
    unsigned int m_int;
    unsigned long m_long;
    unsigned long long m_longlong;
    float m_float;
    double m_double;
    long double m_longdouble;
};

struct _cffi_freeme_s {
    struct _cffi_freeme_s *next;
    union _cffi_union_alignment_u alignment;
};

_CFFI_UNUSED_FN static int
_cffi_convert_array_argument(struct _cffi_ctypedescr *ctptr, PyObject *arg,
                             char **output_data, Py_ssize_t datasize,
                             struct _cffi_freeme_s **freeme)
{
    char *p;
    if (datasize < 0)
        return -1;

    p = *output_data;
    if (p == NULL) {
        struct _cffi_freeme_s *fp = (struct _cffi_freeme_s *)PyObject_Malloc(
            offsetof(struct _cffi_freeme_s, alignment) + (size_t)datasize);
        if (fp == NULL)
            return -1;
        fp->next = *freeme;
        *freeme = fp;
        p = *output_data = (char *)&fp->alignment;
    }
    memset((void *)p, 0, (size_t)datasize);
    return _cffi_convert_array_from_object(p, ctptr, arg);
}

_CFFI_UNUSED_FN static void
_cffi_free_array_arguments(struct _cffi_freeme_s *freeme)
{
    do {
        void *p = (void *)freeme;
        freeme = freeme->next;
        PyObject_Free(p);
    } while (freeme != NULL);
}

/**********  end CPython-specific section  **********/
#else
_CFFI_UNUSED_FN
static void (*_cffi_call_python_org)(struct _cffi_externpy_s *, char *);
# define _cffi_call_python  _cffi_call_python_org
#endif


#define _cffi_array_len(array)   (sizeof(array) / sizeof((array)[0]))

#define _cffi_prim_int(size, sign)                                      \
    ((size) == 1 ? ((sign) ? _CFFI_PRIM_INT8  : _CFFI_PRIM_UINT8)  :    \
     (size) == 2 ? ((sign) ? _CFFI_PRIM_INT16 : _CFFI_PRIM_UINT16) :    \
     (size) == 4 ? ((sign) ? _CFFI_PRIM_INT32 : _CFFI_PRIM_UINT32) :    \
     (size) == 8 ? ((sign) ? _CFFI_PRIM_INT64 : _CFFI_PRIM_UINT64) :    \
     _CFFI__UNKNOWN_PRIM)

#define _cffi_prim_float(size)                                          \
    ((size) == sizeof(float) ? _CFFI_PRIM_FLOAT :                       \
     (size) == sizeof(double) ? _CFFI_PRIM_DOUBLE :                     \
     (size) == sizeof(long double) ? _CFFI__UNKNOWN_LONG_DOUBLE :       \
     _CFFI__UNKNOWN_FLOAT_PRIM)

#define _cffi_check_int(got, got_nonpos, expected)      \
    ((got_nonpos) == (expected <= 0) &&                 \
     (got) == (unsigned long long)expected)

#ifdef MS_WIN32
# define _cffi_stdcall  __stdcall
#else
# define _cffi_stdcall  /* nothing */
#endif

#ifdef __cplusplus
}
#endif

/************************************************************/


	#include "../raylib.h"   
	

/************************************************************/

static void *_cffi_types[] = {
/*  0 */ _CFFI_OP(_CFFI_OP_FUNCTION, 384), // AudioStream()(unsigned int, unsigned int, unsigned int)
/*  1 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 8), // unsigned int
/*  2 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 8),
/*  3 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 8),
/*  4 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/*  5 */ _CFFI_OP(_CFFI_OP_FUNCTION, 387), // BoundingBox()(Mesh)
/*  6 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 12), // Mesh
/*  7 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/*  8 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1148), // CharInfo *()(char const *, int, int *, int, int)
/*  9 */ _CFFI_OP(_CFFI_OP_POINTER, 337), // char const *
/* 10 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7), // int
/* 11 */ _CFFI_OP(_CFFI_OP_POINTER, 10), // int *
/* 12 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 13 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 14 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 15 */ _CFFI_OP(_CFFI_OP_FUNCTION, 61), // Color *()(Image)
/* 16 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 8), // Image
/* 17 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 18 */ _CFFI_OP(_CFFI_OP_FUNCTION, 61), // Color *()(Image, int, int *)
/* 19 */ _CFFI_OP(_CFFI_OP_NOOP, 16),
/* 20 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 21 */ _CFFI_OP(_CFFI_OP_NOOP, 11),
/* 22 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 23 */ _CFFI_OP(_CFFI_OP_FUNCTION, 24), // Color()(Color, float)
/* 24 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 6), // Color
/* 25 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13), // float
/* 26 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 27 */ _CFFI_OP(_CFFI_OP_FUNCTION, 24), // Color()(Vector3)
/* 28 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 26), // Vector3
/* 29 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 30 */ _CFFI_OP(_CFFI_OP_FUNCTION, 24), // Color()(Vector4)
/* 31 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 27), // Vector4
/* 32 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 33 */ _CFFI_OP(_CFFI_OP_FUNCTION, 24), // Color()(int)
/* 34 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 35 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 36 */ _CFFI_OP(_CFFI_OP_FUNCTION, 66), // Font()(Image, Color, int)
/* 37 */ _CFFI_OP(_CFFI_OP_NOOP, 16),
/* 38 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 39 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 40 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 41 */ _CFFI_OP(_CFFI_OP_FUNCTION, 66), // Font()(char const *)
/* 42 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 43 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 44 */ _CFFI_OP(_CFFI_OP_FUNCTION, 66), // Font()(char const *, int, int *, int)
/* 45 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 46 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 47 */ _CFFI_OP(_CFFI_OP_NOOP, 11),
/* 48 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 49 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 50 */ _CFFI_OP(_CFFI_OP_FUNCTION, 66), // Font()(void)
/* 51 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 52 */ _CFFI_OP(_CFFI_OP_FUNCTION, 16), // Image()(CharInfo const *, Rectangle * *, int, int, int, int)
/* 53 */ _CFFI_OP(_CFFI_OP_POINTER, 1149), // CharInfo const *
/* 54 */ _CFFI_OP(_CFFI_OP_POINTER, 1166), // Rectangle * *
/* 55 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 56 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 57 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 58 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 59 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 60 */ _CFFI_OP(_CFFI_OP_FUNCTION, 16), // Image()(Color *, int, int)
/* 61 */ _CFFI_OP(_CFFI_OP_POINTER, 24), // Color *
/* 62 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 63 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 64 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 65 */ _CFFI_OP(_CFFI_OP_FUNCTION, 16), // Image()(Font, char const *, float, float, Color)
/* 66 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 7), // Font
/* 67 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 68 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 69 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 70 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 71 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 72 */ _CFFI_OP(_CFFI_OP_FUNCTION, 16), // Image()(Image)
/* 73 */ _CFFI_OP(_CFFI_OP_NOOP, 16),
/* 74 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 75 */ _CFFI_OP(_CFFI_OP_FUNCTION, 16), // Image()(Image, Rectangle)
/* 76 */ _CFFI_OP(_CFFI_OP_NOOP, 16),
/* 77 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 19), // Rectangle
/* 78 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 79 */ _CFFI_OP(_CFFI_OP_FUNCTION, 16), // Image()(Texture2D)
/* 80 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 23), // Texture2D
/* 81 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 82 */ _CFFI_OP(_CFFI_OP_FUNCTION, 16), // Image()(char const *)
/* 83 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 84 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 85 */ _CFFI_OP(_CFFI_OP_FUNCTION, 16), // Image()(char const *, int, Color)
/* 86 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 87 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 88 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 89 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 90 */ _CFFI_OP(_CFFI_OP_FUNCTION, 16), // Image()(char const *, int, int, int, int)
/* 91 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 92 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 93 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 94 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 95 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 96 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 97 */ _CFFI_OP(_CFFI_OP_FUNCTION, 16), // Image()(int, int, Color)
/* 98 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 99 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 100 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 101 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 102 */ _CFFI_OP(_CFFI_OP_FUNCTION, 16), // Image()(int, int, Color, Color)
/* 103 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 104 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 105 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 106 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 107 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 108 */ _CFFI_OP(_CFFI_OP_FUNCTION, 16), // Image()(int, int, float)
/* 109 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 110 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 111 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 112 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 113 */ _CFFI_OP(_CFFI_OP_FUNCTION, 16), // Image()(int, int, float, Color, Color)
/* 114 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 115 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 116 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 117 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 118 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 119 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 120 */ _CFFI_OP(_CFFI_OP_FUNCTION, 16), // Image()(int, int, int)
/* 121 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 122 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 123 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 124 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 125 */ _CFFI_OP(_CFFI_OP_FUNCTION, 16), // Image()(int, int, int, int, Color, Color)
/* 126 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 127 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 128 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 129 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 130 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 131 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 132 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 133 */ _CFFI_OP(_CFFI_OP_FUNCTION, 16), // Image()(int, int, int, int, float)
/* 134 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 135 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 136 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 137 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 138 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 139 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 140 */ _CFFI_OP(_CFFI_OP_FUNCTION, 16), // Image()(void *, int, int, int)
/* 141 */ _CFFI_OP(_CFFI_OP_POINTER, 1192), // void *
/* 142 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 143 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 144 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 145 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 146 */ _CFFI_OP(_CFFI_OP_FUNCTION, 16), // Image()(void)
/* 147 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 148 */ _CFFI_OP(_CFFI_OP_FUNCTION, 699), // Material *()(char const *, int *)
/* 149 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 150 */ _CFFI_OP(_CFFI_OP_NOOP, 11),
/* 151 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 152 */ _CFFI_OP(_CFFI_OP_FUNCTION, 704), // Material()(void)
/* 153 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 154 */ _CFFI_OP(_CFFI_OP_FUNCTION, 707), // Matrix()(Camera2D)
/* 155 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 3), // Camera2D
/* 156 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 157 */ _CFFI_OP(_CFFI_OP_FUNCTION, 707), // Matrix()(Camera3D)
/* 158 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 4), // Camera3D
/* 159 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 160 */ _CFFI_OP(_CFFI_OP_FUNCTION, 707), // Matrix()(void)
/* 161 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 162 */ _CFFI_OP(_CFFI_OP_FUNCTION, 710), // Mesh *()(char const *, int *)
/* 163 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 164 */ _CFFI_OP(_CFFI_OP_NOOP, 11),
/* 165 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 166 */ _CFFI_OP(_CFFI_OP_FUNCTION, 6), // Mesh()(Image, Vector3)
/* 167 */ _CFFI_OP(_CFFI_OP_NOOP, 16),
/* 168 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 169 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 170 */ _CFFI_OP(_CFFI_OP_FUNCTION, 6), // Mesh()(float, float, float)
/* 171 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 172 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 173 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 174 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 175 */ _CFFI_OP(_CFFI_OP_FUNCTION, 6), // Mesh()(float, float, int)
/* 176 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 177 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 178 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 179 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 180 */ _CFFI_OP(_CFFI_OP_FUNCTION, 6), // Mesh()(float, float, int, int)
/* 181 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 182 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 183 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 184 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 185 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 186 */ _CFFI_OP(_CFFI_OP_FUNCTION, 6), // Mesh()(float, int, int)
/* 187 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 188 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 189 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 190 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 191 */ _CFFI_OP(_CFFI_OP_FUNCTION, 6), // Mesh()(int, float)
/* 192 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 193 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 194 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 195 */ _CFFI_OP(_CFFI_OP_FUNCTION, 214), // Model()(Mesh)
/* 196 */ _CFFI_OP(_CFFI_OP_NOOP, 6),
/* 197 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 198 */ _CFFI_OP(_CFFI_OP_FUNCTION, 214), // Model()(char const *)
/* 199 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 200 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 201 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1161), // ModelAnimation *()(char const *, int *)
/* 202 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 203 */ _CFFI_OP(_CFFI_OP_NOOP, 11),
/* 204 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 205 */ _CFFI_OP(_CFFI_OP_FUNCTION, 371), // Music()(char const *)
/* 206 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 207 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 208 */ _CFFI_OP(_CFFI_OP_FUNCTION, 213), // Ray()(Vector2, Camera3D)
/* 209 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 25), // Vector2
/* 210 */ _CFFI_OP(_CFFI_OP_NOOP, 158),
/* 211 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 212 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1165), // RayHitInfo()(Ray, Model)
/* 213 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 17), // Ray
/* 214 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 13), // Model
/* 215 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 216 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1165), // RayHitInfo()(Ray, Vector3, Vector3, Vector3)
/* 217 */ _CFFI_OP(_CFFI_OP_NOOP, 213),
/* 218 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 219 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 220 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 221 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 222 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1165), // RayHitInfo()(Ray, float)
/* 223 */ _CFFI_OP(_CFFI_OP_NOOP, 213),
/* 224 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 225 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 226 */ _CFFI_OP(_CFFI_OP_FUNCTION, 77), // Rectangle()(Image, float)
/* 227 */ _CFFI_OP(_CFFI_OP_NOOP, 16),
/* 228 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 229 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 230 */ _CFFI_OP(_CFFI_OP_FUNCTION, 77), // Rectangle()(Rectangle, Rectangle)
/* 231 */ _CFFI_OP(_CFFI_OP_NOOP, 77),
/* 232 */ _CFFI_OP(_CFFI_OP_NOOP, 77),
/* 233 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 234 */ _CFFI_OP(_CFFI_OP_FUNCTION, 800), // RenderTexture2D()(int, int)
/* 235 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 236 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 237 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 238 */ _CFFI_OP(_CFFI_OP_FUNCTION, 258), // Shader()(char const *, char const *)
/* 239 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 240 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 241 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 242 */ _CFFI_OP(_CFFI_OP_FUNCTION, 258), // Shader()(void)
/* 243 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 244 */ _CFFI_OP(_CFFI_OP_FUNCTION, 433), // Sound()(Wave)
/* 245 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 29), // Wave
/* 246 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 247 */ _CFFI_OP(_CFFI_OP_FUNCTION, 433), // Sound()(char const *)
/* 248 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 249 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 250 */ _CFFI_OP(_CFFI_OP_FUNCTION, 80), // Texture2D()(Image)
/* 251 */ _CFFI_OP(_CFFI_OP_NOOP, 16),
/* 252 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 253 */ _CFFI_OP(_CFFI_OP_FUNCTION, 80), // Texture2D()(Image, int)
/* 254 */ _CFFI_OP(_CFFI_OP_NOOP, 16),
/* 255 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 256 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 257 */ _CFFI_OP(_CFFI_OP_FUNCTION, 80), // Texture2D()(Shader, Texture2D, int)
/* 258 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 21), // Shader
/* 259 */ _CFFI_OP(_CFFI_OP_NOOP, 80),
/* 260 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 261 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 262 */ _CFFI_OP(_CFFI_OP_FUNCTION, 80), // Texture2D()(Shader, int)
/* 263 */ _CFFI_OP(_CFFI_OP_NOOP, 258),
/* 264 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 265 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 266 */ _CFFI_OP(_CFFI_OP_FUNCTION, 80), // Texture2D()(char const *)
/* 267 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 268 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 269 */ _CFFI_OP(_CFFI_OP_FUNCTION, 80), // Texture2D()(void)
/* 270 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 271 */ _CFFI_OP(_CFFI_OP_FUNCTION, 209), // Vector2()(Font, char const *, float, float)
/* 272 */ _CFFI_OP(_CFFI_OP_NOOP, 66),
/* 273 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 274 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 275 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 276 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 277 */ _CFFI_OP(_CFFI_OP_FUNCTION, 209), // Vector2()(Vector2, Camera2D)
/* 278 */ _CFFI_OP(_CFFI_OP_NOOP, 209),
/* 279 */ _CFFI_OP(_CFFI_OP_NOOP, 155),
/* 280 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 281 */ _CFFI_OP(_CFFI_OP_FUNCTION, 209), // Vector2()(Vector3, Camera3D)
/* 282 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 283 */ _CFFI_OP(_CFFI_OP_NOOP, 158),
/* 284 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 285 */ _CFFI_OP(_CFFI_OP_FUNCTION, 209), // Vector2()(Vector3, Camera3D, int, int)
/* 286 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 287 */ _CFFI_OP(_CFFI_OP_NOOP, 158),
/* 288 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 289 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 290 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 291 */ _CFFI_OP(_CFFI_OP_FUNCTION, 209), // Vector2()(int)
/* 292 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 293 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 294 */ _CFFI_OP(_CFFI_OP_FUNCTION, 209), // Vector2()(void)
/* 295 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 296 */ _CFFI_OP(_CFFI_OP_FUNCTION, 28), // Vector3()(Color)
/* 297 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 298 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 299 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1175), // Vector4 *()(Image)
/* 300 */ _CFFI_OP(_CFFI_OP_NOOP, 16),
/* 301 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 302 */ _CFFI_OP(_CFFI_OP_FUNCTION, 31), // Vector4()(Color)
/* 303 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 304 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 305 */ _CFFI_OP(_CFFI_OP_FUNCTION, 245), // Wave()(Wave)
/* 306 */ _CFFI_OP(_CFFI_OP_NOOP, 245),
/* 307 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 308 */ _CFFI_OP(_CFFI_OP_FUNCTION, 245), // Wave()(char const *)
/* 309 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 310 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 311 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1176), // char * *()(char const *, int *)
/* 312 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 313 */ _CFFI_OP(_CFFI_OP_NOOP, 11),
/* 314 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 315 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1176), // char * *()(int *)
/* 316 */ _CFFI_OP(_CFFI_OP_NOOP, 11),
/* 317 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 318 */ _CFFI_OP(_CFFI_OP_FUNCTION, 319), // char *()(char *, char const *, char const *)
/* 319 */ _CFFI_OP(_CFFI_OP_POINTER, 337), // char *
/* 320 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 321 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 322 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 323 */ _CFFI_OP(_CFFI_OP_FUNCTION, 319), // char *()(char const *)
/* 324 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 325 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 326 */ _CFFI_OP(_CFFI_OP_FUNCTION, 319), // char *()(char const *, char const *, int)
/* 327 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 328 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 329 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 330 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 331 */ _CFFI_OP(_CFFI_OP_FUNCTION, 319), // char *()(int *, int)
/* 332 */ _CFFI_OP(_CFFI_OP_NOOP, 11),
/* 333 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 334 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 335 */ _CFFI_OP(_CFFI_OP_FUNCTION, 341), // char const * *()(char const *, char, int *)
/* 336 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 337 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 2), // char
/* 338 */ _CFFI_OP(_CFFI_OP_NOOP, 11),
/* 339 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 340 */ _CFFI_OP(_CFFI_OP_FUNCTION, 9), // char const *()(char const * *, int, char const *)
/* 341 */ _CFFI_OP(_CFFI_OP_POINTER, 9), // char const * *
/* 342 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 343 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 344 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 345 */ _CFFI_OP(_CFFI_OP_FUNCTION, 9), // char const *()(char const *)
/* 346 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 347 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 348 */ _CFFI_OP(_CFFI_OP_FUNCTION, 9), // char const *()(char const *, ...)
/* 349 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 350 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 1),
/* 351 */ _CFFI_OP(_CFFI_OP_FUNCTION, 9), // char const *()(char const *, int, int)
/* 352 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 353 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 354 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 355 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 356 */ _CFFI_OP(_CFFI_OP_FUNCTION, 9), // char const *()(int)
/* 357 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 358 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 359 */ _CFFI_OP(_CFFI_OP_FUNCTION, 9), // char const *()(int, int *)
/* 360 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 361 */ _CFFI_OP(_CFFI_OP_NOOP, 11),
/* 362 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 363 */ _CFFI_OP(_CFFI_OP_FUNCTION, 9), // char const *()(void)
/* 364 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 365 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1180), // double()(void)
/* 366 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 367 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1181), // float *()(Wave)
/* 368 */ _CFFI_OP(_CFFI_OP_NOOP, 245),
/* 369 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 370 */ _CFFI_OP(_CFFI_OP_FUNCTION, 25), // float()(Music)
/* 371 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 15), // Music
/* 372 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 373 */ _CFFI_OP(_CFFI_OP_FUNCTION, 25), // float()(int, int)
/* 374 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 375 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 376 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 377 */ _CFFI_OP(_CFFI_OP_FUNCTION, 25), // float()(void)
/* 378 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 379 */ _CFFI_OP(_CFFI_OP_FUNCTION, 11), // int *()(char const *, int *)
/* 380 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 381 */ _CFFI_OP(_CFFI_OP_NOOP, 11),
/* 382 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 383 */ _CFFI_OP(_CFFI_OP_FUNCTION, 10), // int()(AudioStream)
/* 384 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 0), // AudioStream
/* 385 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 386 */ _CFFI_OP(_CFFI_OP_FUNCTION, 10), // int()(BoundingBox, BoundingBox)
/* 387 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 2), // BoundingBox
/* 388 */ _CFFI_OP(_CFFI_OP_NOOP, 387),
/* 389 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 390 */ _CFFI_OP(_CFFI_OP_FUNCTION, 10), // int()(BoundingBox, Vector3, float)
/* 391 */ _CFFI_OP(_CFFI_OP_NOOP, 387),
/* 392 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 393 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 394 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 395 */ _CFFI_OP(_CFFI_OP_FUNCTION, 10), // int()(Color)
/* 396 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 397 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 398 */ _CFFI_OP(_CFFI_OP_FUNCTION, 10), // int()(Font, int)
/* 399 */ _CFFI_OP(_CFFI_OP_NOOP, 66),
/* 400 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 401 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 402 */ _CFFI_OP(_CFFI_OP_FUNCTION, 10), // int()(Model, ModelAnimation)
/* 403 */ _CFFI_OP(_CFFI_OP_NOOP, 214),
/* 404 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 14), // ModelAnimation
/* 405 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 406 */ _CFFI_OP(_CFFI_OP_FUNCTION, 10), // int()(Music)
/* 407 */ _CFFI_OP(_CFFI_OP_NOOP, 371),
/* 408 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 409 */ _CFFI_OP(_CFFI_OP_FUNCTION, 10), // int()(Ray, BoundingBox)
/* 410 */ _CFFI_OP(_CFFI_OP_NOOP, 213),
/* 411 */ _CFFI_OP(_CFFI_OP_NOOP, 387),
/* 412 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 413 */ _CFFI_OP(_CFFI_OP_FUNCTION, 10), // int()(Ray, Vector3, float)
/* 414 */ _CFFI_OP(_CFFI_OP_NOOP, 213),
/* 415 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 416 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 417 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 418 */ _CFFI_OP(_CFFI_OP_FUNCTION, 10), // int()(Ray, Vector3, float, Vector3 *)
/* 419 */ _CFFI_OP(_CFFI_OP_NOOP, 213),
/* 420 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 421 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 422 */ _CFFI_OP(_CFFI_OP_POINTER, 28), // Vector3 *
/* 423 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 424 */ _CFFI_OP(_CFFI_OP_FUNCTION, 10), // int()(Rectangle, Rectangle)
/* 425 */ _CFFI_OP(_CFFI_OP_NOOP, 77),
/* 426 */ _CFFI_OP(_CFFI_OP_NOOP, 77),
/* 427 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 428 */ _CFFI_OP(_CFFI_OP_FUNCTION, 10), // int()(Shader, char const *)
/* 429 */ _CFFI_OP(_CFFI_OP_NOOP, 258),
/* 430 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 431 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 432 */ _CFFI_OP(_CFFI_OP_FUNCTION, 10), // int()(Sound)
/* 433 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 22), // Sound
/* 434 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 435 */ _CFFI_OP(_CFFI_OP_FUNCTION, 10), // int()(Vector2, Rectangle)
/* 436 */ _CFFI_OP(_CFFI_OP_NOOP, 209),
/* 437 */ _CFFI_OP(_CFFI_OP_NOOP, 77),
/* 438 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 439 */ _CFFI_OP(_CFFI_OP_FUNCTION, 10), // int()(Vector2, Vector2, Vector2, Vector2)
/* 440 */ _CFFI_OP(_CFFI_OP_NOOP, 209),
/* 441 */ _CFFI_OP(_CFFI_OP_NOOP, 209),
/* 442 */ _CFFI_OP(_CFFI_OP_NOOP, 209),
/* 443 */ _CFFI_OP(_CFFI_OP_NOOP, 209),
/* 444 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 445 */ _CFFI_OP(_CFFI_OP_FUNCTION, 10), // int()(Vector2, Vector2, float)
/* 446 */ _CFFI_OP(_CFFI_OP_NOOP, 209),
/* 447 */ _CFFI_OP(_CFFI_OP_NOOP, 209),
/* 448 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 449 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 450 */ _CFFI_OP(_CFFI_OP_FUNCTION, 10), // int()(Vector2, float, Rectangle)
/* 451 */ _CFFI_OP(_CFFI_OP_NOOP, 209),
/* 452 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 453 */ _CFFI_OP(_CFFI_OP_NOOP, 77),
/* 454 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 455 */ _CFFI_OP(_CFFI_OP_FUNCTION, 10), // int()(Vector2, float, Vector2, float)
/* 456 */ _CFFI_OP(_CFFI_OP_NOOP, 209),
/* 457 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 458 */ _CFFI_OP(_CFFI_OP_NOOP, 209),
/* 459 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 460 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 461 */ _CFFI_OP(_CFFI_OP_FUNCTION, 10), // int()(Vector3, float, Vector3, float)
/* 462 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 463 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 464 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 465 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 466 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 467 */ _CFFI_OP(_CFFI_OP_FUNCTION, 10), // int()(char *, char const *)
/* 468 */ _CFFI_OP(_CFFI_OP_NOOP, 319),
/* 469 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 470 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 471 */ _CFFI_OP(_CFFI_OP_FUNCTION, 10), // int()(char const *)
/* 472 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 473 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 474 */ _CFFI_OP(_CFFI_OP_FUNCTION, 10), // int()(char const *, char const *)
/* 475 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 476 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 477 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 478 */ _CFFI_OP(_CFFI_OP_FUNCTION, 10), // int()(char const *, int *)
/* 479 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 480 */ _CFFI_OP(_CFFI_OP_NOOP, 11),
/* 481 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 482 */ _CFFI_OP(_CFFI_OP_FUNCTION, 10), // int()(char const *, int)
/* 483 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 484 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 485 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 486 */ _CFFI_OP(_CFFI_OP_FUNCTION, 10), // int()(int)
/* 487 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 488 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 489 */ _CFFI_OP(_CFFI_OP_FUNCTION, 10), // int()(int, char const *)
/* 490 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 491 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 492 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 493 */ _CFFI_OP(_CFFI_OP_FUNCTION, 10), // int()(int, int)
/* 494 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 495 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 496 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 497 */ _CFFI_OP(_CFFI_OP_FUNCTION, 10), // int()(int, int, int)
/* 498 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 499 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 500 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 501 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 502 */ _CFFI_OP(_CFFI_OP_FUNCTION, 10), // int()(void)
/* 503 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 504 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1184), // long()(char const *)
/* 505 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 506 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 507 */ _CFFI_OP(_CFFI_OP_FUNCTION, 508), // unsigned char *()(unsigned char *, int, int *)
/* 508 */ _CFFI_OP(_CFFI_OP_POINTER, 1187), // unsigned char *
/* 509 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 510 */ _CFFI_OP(_CFFI_OP_NOOP, 11),
/* 511 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 512 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1), // unsigned int()(char const *)
/* 513 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 514 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 515 */ _CFFI_OP(_CFFI_OP_FUNCTION, 141), // void *()(void)
/* 516 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 517 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(AudioStream)
/* 518 */ _CFFI_OP(_CFFI_OP_NOOP, 384),
/* 519 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 520 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(AudioStream, float)
/* 521 */ _CFFI_OP(_CFFI_OP_NOOP, 384),
/* 522 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 523 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 524 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(AudioStream, void const *, int)
/* 525 */ _CFFI_OP(_CFFI_OP_NOOP, 384),
/* 526 */ _CFFI_OP(_CFFI_OP_POINTER, 1192), // void const *
/* 527 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 528 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 529 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(BoundingBox, Color)
/* 530 */ _CFFI_OP(_CFFI_OP_NOOP, 387),
/* 531 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 532 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 533 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Camera2D)
/* 534 */ _CFFI_OP(_CFFI_OP_NOOP, 155),
/* 535 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 536 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Camera3D *)
/* 537 */ _CFFI_OP(_CFFI_OP_POINTER, 158), // Camera3D *
/* 538 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 539 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Camera3D)
/* 540 */ _CFFI_OP(_CFFI_OP_NOOP, 158),
/* 541 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 542 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Camera3D, Texture2D, Rectangle, Vector3, float, Color)
/* 543 */ _CFFI_OP(_CFFI_OP_NOOP, 158),
/* 544 */ _CFFI_OP(_CFFI_OP_NOOP, 80),
/* 545 */ _CFFI_OP(_CFFI_OP_NOOP, 77),
/* 546 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 547 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 548 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 549 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 550 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Camera3D, Texture2D, Vector3, float, Color)
/* 551 */ _CFFI_OP(_CFFI_OP_NOOP, 158),
/* 552 */ _CFFI_OP(_CFFI_OP_NOOP, 80),
/* 553 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 554 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 555 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 556 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 557 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Camera3D, int)
/* 558 */ _CFFI_OP(_CFFI_OP_NOOP, 158),
/* 559 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 560 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 561 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Color)
/* 562 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 563 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 564 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Font)
/* 565 */ _CFFI_OP(_CFFI_OP_NOOP, 66),
/* 566 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 567 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Font, char const *, Rectangle, float, float, int, Color)
/* 568 */ _CFFI_OP(_CFFI_OP_NOOP, 66),
/* 569 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 570 */ _CFFI_OP(_CFFI_OP_NOOP, 77),
/* 571 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 572 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 573 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 574 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 575 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 576 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Font, char const *, Rectangle, float, float, int, Color, int, int, Color, Color)
/* 577 */ _CFFI_OP(_CFFI_OP_NOOP, 66),
/* 578 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 579 */ _CFFI_OP(_CFFI_OP_NOOP, 77),
/* 580 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 581 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 582 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 583 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 584 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 585 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 586 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 587 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 588 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 589 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Font, char const *, Vector2, float, float, Color)
/* 590 */ _CFFI_OP(_CFFI_OP_NOOP, 66),
/* 591 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 592 */ _CFFI_OP(_CFFI_OP_NOOP, 209),
/* 593 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 594 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 595 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 596 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 597 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Font, int, Vector2, float, Color)
/* 598 */ _CFFI_OP(_CFFI_OP_NOOP, 66),
/* 599 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 600 */ _CFFI_OP(_CFFI_OP_NOOP, 209),
/* 601 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 602 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 603 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 604 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Image *)
/* 605 */ _CFFI_OP(_CFFI_OP_POINTER, 16), // Image *
/* 606 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 607 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Image *, Color)
/* 608 */ _CFFI_OP(_CFFI_OP_NOOP, 605),
/* 609 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 610 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 611 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Image *, Color, Color)
/* 612 */ _CFFI_OP(_CFFI_OP_NOOP, 605),
/* 613 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 614 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 615 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 616 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Image *, Color, float)
/* 617 */ _CFFI_OP(_CFFI_OP_NOOP, 605),
/* 618 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 619 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 620 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 621 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Image *, Image)
/* 622 */ _CFFI_OP(_CFFI_OP_NOOP, 605),
/* 623 */ _CFFI_OP(_CFFI_OP_NOOP, 16),
/* 624 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 625 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Image *, Image, Rectangle, Rectangle, Color)
/* 626 */ _CFFI_OP(_CFFI_OP_NOOP, 605),
/* 627 */ _CFFI_OP(_CFFI_OP_NOOP, 16),
/* 628 */ _CFFI_OP(_CFFI_OP_NOOP, 77),
/* 629 */ _CFFI_OP(_CFFI_OP_NOOP, 77),
/* 630 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 631 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 632 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Image *, Rectangle)
/* 633 */ _CFFI_OP(_CFFI_OP_NOOP, 605),
/* 634 */ _CFFI_OP(_CFFI_OP_NOOP, 77),
/* 635 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 636 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Image *, Rectangle, Color)
/* 637 */ _CFFI_OP(_CFFI_OP_NOOP, 605),
/* 638 */ _CFFI_OP(_CFFI_OP_NOOP, 77),
/* 639 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 640 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 641 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Image *, Rectangle, int, Color)
/* 642 */ _CFFI_OP(_CFFI_OP_NOOP, 605),
/* 643 */ _CFFI_OP(_CFFI_OP_NOOP, 77),
/* 644 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 645 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 646 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 647 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Image *, Vector2, Font, char const *, float, float, Color)
/* 648 */ _CFFI_OP(_CFFI_OP_NOOP, 605),
/* 649 */ _CFFI_OP(_CFFI_OP_NOOP, 209),
/* 650 */ _CFFI_OP(_CFFI_OP_NOOP, 66),
/* 651 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 652 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 653 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 654 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 655 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 656 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Image *, Vector2, char const *, int, Color)
/* 657 */ _CFFI_OP(_CFFI_OP_NOOP, 605),
/* 658 */ _CFFI_OP(_CFFI_OP_NOOP, 209),
/* 659 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 660 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 661 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 662 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 663 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Image *, float)
/* 664 */ _CFFI_OP(_CFFI_OP_NOOP, 605),
/* 665 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 666 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 667 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Image *, int)
/* 668 */ _CFFI_OP(_CFFI_OP_NOOP, 605),
/* 669 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 670 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 671 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Image *, int, int)
/* 672 */ _CFFI_OP(_CFFI_OP_NOOP, 605),
/* 673 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 674 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 675 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 676 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Image *, int, int, int, int)
/* 677 */ _CFFI_OP(_CFFI_OP_NOOP, 605),
/* 678 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 679 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 680 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 681 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 682 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 683 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Image *, int, int, int, int, Color)
/* 684 */ _CFFI_OP(_CFFI_OP_NOOP, 605),
/* 685 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 686 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 687 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 688 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 689 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 690 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 691 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Image)
/* 692 */ _CFFI_OP(_CFFI_OP_NOOP, 16),
/* 693 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 694 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Image, char const *)
/* 695 */ _CFFI_OP(_CFFI_OP_NOOP, 16),
/* 696 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 697 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 698 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Material *, int, Texture2D)
/* 699 */ _CFFI_OP(_CFFI_OP_POINTER, 704), // Material *
/* 700 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 701 */ _CFFI_OP(_CFFI_OP_NOOP, 80),
/* 702 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 703 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Material)
/* 704 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 9), // Material
/* 705 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 706 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Matrix)
/* 707 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 11), // Matrix
/* 708 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 709 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Mesh *)
/* 710 */ _CFFI_OP(_CFFI_OP_POINTER, 6), // Mesh *
/* 711 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 712 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Mesh)
/* 713 */ _CFFI_OP(_CFFI_OP_NOOP, 6),
/* 714 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 715 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Mesh, char const *)
/* 716 */ _CFFI_OP(_CFFI_OP_NOOP, 6),
/* 717 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 718 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 719 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Model *, int, int)
/* 720 */ _CFFI_OP(_CFFI_OP_POINTER, 214), // Model *
/* 721 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 722 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 723 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 724 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Model)
/* 725 */ _CFFI_OP(_CFFI_OP_NOOP, 214),
/* 726 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 727 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Model, ModelAnimation, int)
/* 728 */ _CFFI_OP(_CFFI_OP_NOOP, 214),
/* 729 */ _CFFI_OP(_CFFI_OP_NOOP, 404),
/* 730 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 731 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 732 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Model, Vector3, Vector3, float, Vector3, Color)
/* 733 */ _CFFI_OP(_CFFI_OP_NOOP, 214),
/* 734 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 735 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 736 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 737 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 738 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 739 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 740 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Model, Vector3, float, Color)
/* 741 */ _CFFI_OP(_CFFI_OP_NOOP, 214),
/* 742 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 743 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 744 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 745 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 746 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(ModelAnimation)
/* 747 */ _CFFI_OP(_CFFI_OP_NOOP, 404),
/* 748 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 749 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Music)
/* 750 */ _CFFI_OP(_CFFI_OP_NOOP, 371),
/* 751 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 752 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Music, float)
/* 753 */ _CFFI_OP(_CFFI_OP_NOOP, 371),
/* 754 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 755 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 756 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Music, int)
/* 757 */ _CFFI_OP(_CFFI_OP_NOOP, 371),
/* 758 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 759 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 760 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Ray, Color)
/* 761 */ _CFFI_OP(_CFFI_OP_NOOP, 213),
/* 762 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 763 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 764 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Rectangle, Color)
/* 765 */ _CFFI_OP(_CFFI_OP_NOOP, 77),
/* 766 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 767 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 768 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Rectangle, Color, Color, Color, Color)
/* 769 */ _CFFI_OP(_CFFI_OP_NOOP, 77),
/* 770 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 771 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 772 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 773 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 774 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 775 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Rectangle, Vector2, float, Color)
/* 776 */ _CFFI_OP(_CFFI_OP_NOOP, 77),
/* 777 */ _CFFI_OP(_CFFI_OP_NOOP, 209),
/* 778 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 779 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 780 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 781 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Rectangle, float, int, Color)
/* 782 */ _CFFI_OP(_CFFI_OP_NOOP, 77),
/* 783 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 784 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 785 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 786 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 787 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Rectangle, float, int, int, Color)
/* 788 */ _CFFI_OP(_CFFI_OP_NOOP, 77),
/* 789 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 790 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 791 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 792 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 793 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 794 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Rectangle, int, Color)
/* 795 */ _CFFI_OP(_CFFI_OP_NOOP, 77),
/* 796 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 797 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 798 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 799 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(RenderTexture2D)
/* 800 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 20), // RenderTexture2D
/* 801 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 802 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Shader)
/* 803 */ _CFFI_OP(_CFFI_OP_NOOP, 258),
/* 804 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 805 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Shader, int, Matrix)
/* 806 */ _CFFI_OP(_CFFI_OP_NOOP, 258),
/* 807 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 808 */ _CFFI_OP(_CFFI_OP_NOOP, 707),
/* 809 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 810 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Shader, int, Texture2D)
/* 811 */ _CFFI_OP(_CFFI_OP_NOOP, 258),
/* 812 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 813 */ _CFFI_OP(_CFFI_OP_NOOP, 80),
/* 814 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 815 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Shader, int, void const *, int)
/* 816 */ _CFFI_OP(_CFFI_OP_NOOP, 258),
/* 817 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 818 */ _CFFI_OP(_CFFI_OP_NOOP, 526),
/* 819 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 820 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 821 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Shader, int, void const *, int, int)
/* 822 */ _CFFI_OP(_CFFI_OP_NOOP, 258),
/* 823 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 824 */ _CFFI_OP(_CFFI_OP_NOOP, 526),
/* 825 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 826 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 827 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 828 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Sound)
/* 829 */ _CFFI_OP(_CFFI_OP_NOOP, 433),
/* 830 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 831 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Sound, float)
/* 832 */ _CFFI_OP(_CFFI_OP_NOOP, 433),
/* 833 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 834 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 835 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Sound, void const *, int)
/* 836 */ _CFFI_OP(_CFFI_OP_NOOP, 433),
/* 837 */ _CFFI_OP(_CFFI_OP_NOOP, 526),
/* 838 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 839 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 840 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Texture2D *)
/* 841 */ _CFFI_OP(_CFFI_OP_POINTER, 80), // Texture2D *
/* 842 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 843 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Texture2D)
/* 844 */ _CFFI_OP(_CFFI_OP_NOOP, 80),
/* 845 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 846 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Texture2D, NPatchInfo, Rectangle, Vector2, float, Color)
/* 847 */ _CFFI_OP(_CFFI_OP_NOOP, 80),
/* 848 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 16), // NPatchInfo
/* 849 */ _CFFI_OP(_CFFI_OP_NOOP, 77),
/* 850 */ _CFFI_OP(_CFFI_OP_NOOP, 209),
/* 851 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 852 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 853 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 854 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Texture2D, Rectangle)
/* 855 */ _CFFI_OP(_CFFI_OP_NOOP, 80),
/* 856 */ _CFFI_OP(_CFFI_OP_NOOP, 77),
/* 857 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 858 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Texture2D, Rectangle, Rectangle, Vector2, float, Color)
/* 859 */ _CFFI_OP(_CFFI_OP_NOOP, 80),
/* 860 */ _CFFI_OP(_CFFI_OP_NOOP, 77),
/* 861 */ _CFFI_OP(_CFFI_OP_NOOP, 77),
/* 862 */ _CFFI_OP(_CFFI_OP_NOOP, 209),
/* 863 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 864 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 865 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 866 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Texture2D, Rectangle, Vector2, Color)
/* 867 */ _CFFI_OP(_CFFI_OP_NOOP, 80),
/* 868 */ _CFFI_OP(_CFFI_OP_NOOP, 77),
/* 869 */ _CFFI_OP(_CFFI_OP_NOOP, 209),
/* 870 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 871 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 872 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Texture2D, Vector2, Color)
/* 873 */ _CFFI_OP(_CFFI_OP_NOOP, 80),
/* 874 */ _CFFI_OP(_CFFI_OP_NOOP, 209),
/* 875 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 876 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 877 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Texture2D, Vector2, Vector2, Rectangle, Color)
/* 878 */ _CFFI_OP(_CFFI_OP_NOOP, 80),
/* 879 */ _CFFI_OP(_CFFI_OP_NOOP, 209),
/* 880 */ _CFFI_OP(_CFFI_OP_NOOP, 209),
/* 881 */ _CFFI_OP(_CFFI_OP_NOOP, 77),
/* 882 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 883 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 884 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Texture2D, Vector2, float, float, Color)
/* 885 */ _CFFI_OP(_CFFI_OP_NOOP, 80),
/* 886 */ _CFFI_OP(_CFFI_OP_NOOP, 209),
/* 887 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 888 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 889 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 890 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 891 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Texture2D, Vector3, float, float, float, Color)
/* 892 */ _CFFI_OP(_CFFI_OP_NOOP, 80),
/* 893 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 894 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 895 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 896 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 897 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 898 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 899 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Texture2D, int)
/* 900 */ _CFFI_OP(_CFFI_OP_NOOP, 80),
/* 901 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 902 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 903 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Texture2D, int, int, Color)
/* 904 */ _CFFI_OP(_CFFI_OP_NOOP, 80),
/* 905 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 906 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 907 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 908 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 909 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Texture2D, void const *)
/* 910 */ _CFFI_OP(_CFFI_OP_NOOP, 80),
/* 911 */ _CFFI_OP(_CFFI_OP_NOOP, 526),
/* 912 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 913 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Vector2 *, int, Color)
/* 914 */ _CFFI_OP(_CFFI_OP_POINTER, 209), // Vector2 *
/* 915 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 916 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 917 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 918 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Vector2, Color)
/* 919 */ _CFFI_OP(_CFFI_OP_NOOP, 209),
/* 920 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 921 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 922 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Vector2, Vector2, Color)
/* 923 */ _CFFI_OP(_CFFI_OP_NOOP, 209),
/* 924 */ _CFFI_OP(_CFFI_OP_NOOP, 209),
/* 925 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 926 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 927 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Vector2, Vector2, Vector2, Color)
/* 928 */ _CFFI_OP(_CFFI_OP_NOOP, 209),
/* 929 */ _CFFI_OP(_CFFI_OP_NOOP, 209),
/* 930 */ _CFFI_OP(_CFFI_OP_NOOP, 209),
/* 931 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 932 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 933 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Vector2, Vector2, float, Color)
/* 934 */ _CFFI_OP(_CFFI_OP_NOOP, 209),
/* 935 */ _CFFI_OP(_CFFI_OP_NOOP, 209),
/* 936 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 937 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 938 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 939 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Vector2, float, Color)
/* 940 */ _CFFI_OP(_CFFI_OP_NOOP, 209),
/* 941 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 942 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 943 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 944 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Vector2, float, float, int, int, int, Color)
/* 945 */ _CFFI_OP(_CFFI_OP_NOOP, 209),
/* 946 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 947 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 948 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 949 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 950 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 951 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 952 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 953 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Vector2, float, int, int, int, Color)
/* 954 */ _CFFI_OP(_CFFI_OP_NOOP, 209),
/* 955 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 956 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 957 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 958 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 959 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 960 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 961 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Vector2, int, float, float, Color)
/* 962 */ _CFFI_OP(_CFFI_OP_NOOP, 209),
/* 963 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 964 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 965 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 966 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 967 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 968 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Vector3)
/* 969 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 970 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 971 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Vector3, Color)
/* 972 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 973 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 974 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 975 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Vector3, Vector2, Color)
/* 976 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 977 */ _CFFI_OP(_CFFI_OP_NOOP, 209),
/* 978 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 979 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 980 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Vector3, Vector3, Color)
/* 981 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 982 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 983 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 984 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 985 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Vector3, float, Color)
/* 986 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 987 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 988 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 989 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 990 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Vector3, float, Vector3, float, Color)
/* 991 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 992 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 993 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 994 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 995 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 996 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 997 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Vector3, float, float, float, Color)
/* 998 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 999 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 1000 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 1001 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 1002 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 1003 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 1004 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Vector3, float, float, float, int, Color)
/* 1005 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 1006 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 1007 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 1008 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 1009 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1010 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 1011 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 1012 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Vector3, float, int, int, Color)
/* 1013 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 1014 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 1015 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1016 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1017 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 1018 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 1019 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(VrDeviceInfo, Shader)
/* 1020 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 28), // VrDeviceInfo
/* 1021 */ _CFFI_OP(_CFFI_OP_NOOP, 258),
/* 1022 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 1023 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Wave *, int, int)
/* 1024 */ _CFFI_OP(_CFFI_OP_POINTER, 245), // Wave *
/* 1025 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1026 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1027 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 1028 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Wave *, int, int, int)
/* 1029 */ _CFFI_OP(_CFFI_OP_NOOP, 1024),
/* 1030 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1031 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1032 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1033 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 1034 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Wave)
/* 1035 */ _CFFI_OP(_CFFI_OP_NOOP, 245),
/* 1036 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 1037 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(Wave, char const *)
/* 1038 */ _CFFI_OP(_CFFI_OP_NOOP, 245),
/* 1039 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 1040 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 1041 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(char *, char const *, int *)
/* 1042 */ _CFFI_OP(_CFFI_OP_NOOP, 319),
/* 1043 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 1044 */ _CFFI_OP(_CFFI_OP_NOOP, 11),
/* 1045 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 1046 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(char const *)
/* 1047 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 1048 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 1049 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(char const *, int, int, int, Color)
/* 1050 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 1051 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1052 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1053 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1054 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 1055 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 1056 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(float)
/* 1057 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 1058 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 1059 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(float, float)
/* 1060 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 1061 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 1062 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 1063 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(int)
/* 1064 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1065 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 1066 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(int, char const *, ...)
/* 1067 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1068 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 1069 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 1),
/* 1070 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(int, float)
/* 1071 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1072 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 1073 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 1074 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(int, int)
/* 1075 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1076 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1077 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 1078 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(int, int, Color)
/* 1079 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1080 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1081 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 1082 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 1083 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(int, int, char const *)
/* 1084 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1085 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1086 */ _CFFI_OP(_CFFI_OP_NOOP, 9),
/* 1087 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 1088 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(int, int, float, Color)
/* 1089 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1090 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1091 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 1092 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 1093 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 1094 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(int, int, float, Color, Color)
/* 1095 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1096 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1097 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 1098 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 1099 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 1100 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 1101 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(int, int, float, float, Color)
/* 1102 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1103 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1104 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 1105 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 1106 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 1107 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 1108 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(int, int, int, int)
/* 1109 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1110 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1111 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1112 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1113 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 1114 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(int, int, int, int, Color)
/* 1115 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1116 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1117 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1118 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1119 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 1120 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 1121 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(int, int, int, int, Color, Color)
/* 1122 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1123 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1124 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1125 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1126 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 1127 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 1128 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 1129 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(int, int, int, int, int, int)
/* 1130 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1131 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1132 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1133 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1134 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1135 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 1136 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 1137 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(unsigned int)
/* 1138 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 8),
/* 1139 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 1140 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1192), // void()(void)
/* 1141 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 1142 */ _CFFI_OP(_CFFI_OP_ENUM, 0), // AndroidButton
/* 1143 */ _CFFI_OP(_CFFI_OP_ENUM, 1), // BlendMode
/* 1144 */ _CFFI_OP(_CFFI_OP_POINTER, 1145), // BoneInfo *
/* 1145 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 1), // BoneInfo
/* 1146 */ _CFFI_OP(_CFFI_OP_ENUM, 2), // CameraMode
/* 1147 */ _CFFI_OP(_CFFI_OP_ENUM, 3), // CameraType
/* 1148 */ _CFFI_OP(_CFFI_OP_POINTER, 1149), // CharInfo *
/* 1149 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 5), // CharInfo
/* 1150 */ _CFFI_OP(_CFFI_OP_ENUM, 4), // ConfigFlag
/* 1151 */ _CFFI_OP(_CFFI_OP_ENUM, 5), // CubemapLayoutType
/* 1152 */ _CFFI_OP(_CFFI_OP_ENUM, 6), // FontType
/* 1153 */ _CFFI_OP(_CFFI_OP_ENUM, 7), // GamepadAxis
/* 1154 */ _CFFI_OP(_CFFI_OP_ENUM, 8), // GamepadButton
/* 1155 */ _CFFI_OP(_CFFI_OP_ENUM, 9), // GamepadNumber
/* 1156 */ _CFFI_OP(_CFFI_OP_ENUM, 10), // GestureType
/* 1157 */ _CFFI_OP(_CFFI_OP_ENUM, 11), // KeyboardKey
/* 1158 */ _CFFI_OP(_CFFI_OP_POINTER, 1159), // MaterialMap *
/* 1159 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 10), // MaterialMap
/* 1160 */ _CFFI_OP(_CFFI_OP_ENUM, 12), // MaterialMapType
/* 1161 */ _CFFI_OP(_CFFI_OP_POINTER, 404), // ModelAnimation *
/* 1162 */ _CFFI_OP(_CFFI_OP_ENUM, 13), // MouseButton
/* 1163 */ _CFFI_OP(_CFFI_OP_ENUM, 14), // NPatchType
/* 1164 */ _CFFI_OP(_CFFI_OP_ENUM, 15), // PixelFormat
/* 1165 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 18), // RayHitInfo
/* 1166 */ _CFFI_OP(_CFFI_OP_POINTER, 77), // Rectangle *
/* 1167 */ _CFFI_OP(_CFFI_OP_ENUM, 16), // ShaderLocationIndex
/* 1168 */ _CFFI_OP(_CFFI_OP_ENUM, 17), // ShaderUniformDataType
/* 1169 */ _CFFI_OP(_CFFI_OP_ENUM, 18), // TextureFilterMode
/* 1170 */ _CFFI_OP(_CFFI_OP_ENUM, 19), // TextureWrapMode
/* 1171 */ _CFFI_OP(_CFFI_OP_ENUM, 20), // TraceLogType
/* 1172 */ _CFFI_OP(_CFFI_OP_POINTER, 1173), // Transform * *
/* 1173 */ _CFFI_OP(_CFFI_OP_POINTER, 1174), // Transform *
/* 1174 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 24), // Transform
/* 1175 */ _CFFI_OP(_CFFI_OP_POINTER, 31), // Vector4 *
/* 1176 */ _CFFI_OP(_CFFI_OP_POINTER, 319), // char * *
/* 1177 */ _CFFI_OP(_CFFI_OP_POINTER, 348), // char const *(*)(char const *, ...)
/* 1178 */ _CFFI_OP(_CFFI_OP_ARRAY, 337), // char[32]
/* 1179 */ (_cffi_opcode_t)(32),
/* 1180 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 14), // double
/* 1181 */ _CFFI_OP(_CFFI_OP_POINTER, 25), // float *
/* 1182 */ _CFFI_OP(_CFFI_OP_ARRAY, 25), // float[4]
/* 1183 */ (_cffi_opcode_t)(4),
/* 1184 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 9), // long
/* 1185 */ _CFFI_OP(_CFFI_OP_POINTER, 1186), // rAudioBuffer *
/* 1186 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 30), // rAudioBuffer
/* 1187 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 4), // unsigned char
/* 1188 */ _CFFI_OP(_CFFI_OP_POINTER, 1), // unsigned int *
/* 1189 */ _CFFI_OP(_CFFI_OP_POINTER, 1190), // unsigned short *
/* 1190 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 6), // unsigned short
/* 1191 */ _CFFI_OP(_CFFI_OP_POINTER, 1066), // void(*)(int, char const *, ...)
/* 1192 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 0), // void
};

static int _cffi_const_KEY_BACK(unsigned long long *o)
{
  int n = (KEY_BACK) <= 0;
  *o = (unsigned long long)((KEY_BACK) | 0);  /* check that KEY_BACK is an integer */
  return n;
}

static int _cffi_const_KEY_MENU(unsigned long long *o)
{
  int n = (KEY_MENU) <= 0;
  *o = (unsigned long long)((KEY_MENU) | 0);  /* check that KEY_MENU is an integer */
  return n;
}

static int _cffi_const_KEY_VOLUME_UP(unsigned long long *o)
{
  int n = (KEY_VOLUME_UP) <= 0;
  *o = (unsigned long long)((KEY_VOLUME_UP) | 0);  /* check that KEY_VOLUME_UP is an integer */
  return n;
}

static int _cffi_const_KEY_VOLUME_DOWN(unsigned long long *o)
{
  int n = (KEY_VOLUME_DOWN) <= 0;
  *o = (unsigned long long)((KEY_VOLUME_DOWN) | 0);  /* check that KEY_VOLUME_DOWN is an integer */
  return n;
}

static int _cffi_const_BLEND_ALPHA(unsigned long long *o)
{
  int n = (BLEND_ALPHA) <= 0;
  *o = (unsigned long long)((BLEND_ALPHA) | 0);  /* check that BLEND_ALPHA is an integer */
  return n;
}

static int _cffi_const_BLEND_ADDITIVE(unsigned long long *o)
{
  int n = (BLEND_ADDITIVE) <= 0;
  *o = (unsigned long long)((BLEND_ADDITIVE) | 0);  /* check that BLEND_ADDITIVE is an integer */
  return n;
}

static int _cffi_const_BLEND_MULTIPLIED(unsigned long long *o)
{
  int n = (BLEND_MULTIPLIED) <= 0;
  *o = (unsigned long long)((BLEND_MULTIPLIED) | 0);  /* check that BLEND_MULTIPLIED is an integer */
  return n;
}

static int _cffi_const_CAMERA_CUSTOM(unsigned long long *o)
{
  int n = (CAMERA_CUSTOM) <= 0;
  *o = (unsigned long long)((CAMERA_CUSTOM) | 0);  /* check that CAMERA_CUSTOM is an integer */
  return n;
}

static int _cffi_const_CAMERA_FREE(unsigned long long *o)
{
  int n = (CAMERA_FREE) <= 0;
  *o = (unsigned long long)((CAMERA_FREE) | 0);  /* check that CAMERA_FREE is an integer */
  return n;
}

static int _cffi_const_CAMERA_ORBITAL(unsigned long long *o)
{
  int n = (CAMERA_ORBITAL) <= 0;
  *o = (unsigned long long)((CAMERA_ORBITAL) | 0);  /* check that CAMERA_ORBITAL is an integer */
  return n;
}

static int _cffi_const_CAMERA_FIRST_PERSON(unsigned long long *o)
{
  int n = (CAMERA_FIRST_PERSON) <= 0;
  *o = (unsigned long long)((CAMERA_FIRST_PERSON) | 0);  /* check that CAMERA_FIRST_PERSON is an integer */
  return n;
}

static int _cffi_const_CAMERA_THIRD_PERSON(unsigned long long *o)
{
  int n = (CAMERA_THIRD_PERSON) <= 0;
  *o = (unsigned long long)((CAMERA_THIRD_PERSON) | 0);  /* check that CAMERA_THIRD_PERSON is an integer */
  return n;
}

static int _cffi_const_CAMERA_PERSPECTIVE(unsigned long long *o)
{
  int n = (CAMERA_PERSPECTIVE) <= 0;
  *o = (unsigned long long)((CAMERA_PERSPECTIVE) | 0);  /* check that CAMERA_PERSPECTIVE is an integer */
  return n;
}

static int _cffi_const_CAMERA_ORTHOGRAPHIC(unsigned long long *o)
{
  int n = (CAMERA_ORTHOGRAPHIC) <= 0;
  *o = (unsigned long long)((CAMERA_ORTHOGRAPHIC) | 0);  /* check that CAMERA_ORTHOGRAPHIC is an integer */
  return n;
}

static int _cffi_const_FLAG_RESERVED(unsigned long long *o)
{
  int n = (FLAG_RESERVED) <= 0;
  *o = (unsigned long long)((FLAG_RESERVED) | 0);  /* check that FLAG_RESERVED is an integer */
  return n;
}

static int _cffi_const_FLAG_FULLSCREEN_MODE(unsigned long long *o)
{
  int n = (FLAG_FULLSCREEN_MODE) <= 0;
  *o = (unsigned long long)((FLAG_FULLSCREEN_MODE) | 0);  /* check that FLAG_FULLSCREEN_MODE is an integer */
  return n;
}

static int _cffi_const_FLAG_WINDOW_RESIZABLE(unsigned long long *o)
{
  int n = (FLAG_WINDOW_RESIZABLE) <= 0;
  *o = (unsigned long long)((FLAG_WINDOW_RESIZABLE) | 0);  /* check that FLAG_WINDOW_RESIZABLE is an integer */
  return n;
}

static int _cffi_const_FLAG_WINDOW_UNDECORATED(unsigned long long *o)
{
  int n = (FLAG_WINDOW_UNDECORATED) <= 0;
  *o = (unsigned long long)((FLAG_WINDOW_UNDECORATED) | 0);  /* check that FLAG_WINDOW_UNDECORATED is an integer */
  return n;
}

static int _cffi_const_FLAG_WINDOW_TRANSPARENT(unsigned long long *o)
{
  int n = (FLAG_WINDOW_TRANSPARENT) <= 0;
  *o = (unsigned long long)((FLAG_WINDOW_TRANSPARENT) | 0);  /* check that FLAG_WINDOW_TRANSPARENT is an integer */
  return n;
}

static int _cffi_const_FLAG_WINDOW_HIDDEN(unsigned long long *o)
{
  int n = (FLAG_WINDOW_HIDDEN) <= 0;
  *o = (unsigned long long)((FLAG_WINDOW_HIDDEN) | 0);  /* check that FLAG_WINDOW_HIDDEN is an integer */
  return n;
}

static int _cffi_const_FLAG_WINDOW_ALWAYS_RUN(unsigned long long *o)
{
  int n = (FLAG_WINDOW_ALWAYS_RUN) <= 0;
  *o = (unsigned long long)((FLAG_WINDOW_ALWAYS_RUN) | 0);  /* check that FLAG_WINDOW_ALWAYS_RUN is an integer */
  return n;
}

static int _cffi_const_FLAG_MSAA_4X_HINT(unsigned long long *o)
{
  int n = (FLAG_MSAA_4X_HINT) <= 0;
  *o = (unsigned long long)((FLAG_MSAA_4X_HINT) | 0);  /* check that FLAG_MSAA_4X_HINT is an integer */
  return n;
}

static int _cffi_const_FLAG_VSYNC_HINT(unsigned long long *o)
{
  int n = (FLAG_VSYNC_HINT) <= 0;
  *o = (unsigned long long)((FLAG_VSYNC_HINT) | 0);  /* check that FLAG_VSYNC_HINT is an integer */
  return n;
}

static int _cffi_const_CUBEMAP_AUTO_DETECT(unsigned long long *o)
{
  int n = (CUBEMAP_AUTO_DETECT) <= 0;
  *o = (unsigned long long)((CUBEMAP_AUTO_DETECT) | 0);  /* check that CUBEMAP_AUTO_DETECT is an integer */
  return n;
}

static int _cffi_const_CUBEMAP_LINE_VERTICAL(unsigned long long *o)
{
  int n = (CUBEMAP_LINE_VERTICAL) <= 0;
  *o = (unsigned long long)((CUBEMAP_LINE_VERTICAL) | 0);  /* check that CUBEMAP_LINE_VERTICAL is an integer */
  return n;
}

static int _cffi_const_CUBEMAP_LINE_HORIZONTAL(unsigned long long *o)
{
  int n = (CUBEMAP_LINE_HORIZONTAL) <= 0;
  *o = (unsigned long long)((CUBEMAP_LINE_HORIZONTAL) | 0);  /* check that CUBEMAP_LINE_HORIZONTAL is an integer */
  return n;
}

static int _cffi_const_CUBEMAP_CROSS_THREE_BY_FOUR(unsigned long long *o)
{
  int n = (CUBEMAP_CROSS_THREE_BY_FOUR) <= 0;
  *o = (unsigned long long)((CUBEMAP_CROSS_THREE_BY_FOUR) | 0);  /* check that CUBEMAP_CROSS_THREE_BY_FOUR is an integer */
  return n;
}

static int _cffi_const_CUBEMAP_CROSS_FOUR_BY_THREE(unsigned long long *o)
{
  int n = (CUBEMAP_CROSS_FOUR_BY_THREE) <= 0;
  *o = (unsigned long long)((CUBEMAP_CROSS_FOUR_BY_THREE) | 0);  /* check that CUBEMAP_CROSS_FOUR_BY_THREE is an integer */
  return n;
}

static int _cffi_const_CUBEMAP_PANORAMA(unsigned long long *o)
{
  int n = (CUBEMAP_PANORAMA) <= 0;
  *o = (unsigned long long)((CUBEMAP_PANORAMA) | 0);  /* check that CUBEMAP_PANORAMA is an integer */
  return n;
}

static int _cffi_const_FONT_DEFAULT(unsigned long long *o)
{
  int n = (FONT_DEFAULT) <= 0;
  *o = (unsigned long long)((FONT_DEFAULT) | 0);  /* check that FONT_DEFAULT is an integer */
  return n;
}

static int _cffi_const_FONT_BITMAP(unsigned long long *o)
{
  int n = (FONT_BITMAP) <= 0;
  *o = (unsigned long long)((FONT_BITMAP) | 0);  /* check that FONT_BITMAP is an integer */
  return n;
}

static int _cffi_const_FONT_SDF(unsigned long long *o)
{
  int n = (FONT_SDF) <= 0;
  *o = (unsigned long long)((FONT_SDF) | 0);  /* check that FONT_SDF is an integer */
  return n;
}

static int _cffi_const_GAMEPAD_AXIS_UNKNOWN(unsigned long long *o)
{
  int n = (GAMEPAD_AXIS_UNKNOWN) <= 0;
  *o = (unsigned long long)((GAMEPAD_AXIS_UNKNOWN) | 0);  /* check that GAMEPAD_AXIS_UNKNOWN is an integer */
  return n;
}

static int _cffi_const_GAMEPAD_AXIS_LEFT_X(unsigned long long *o)
{
  int n = (GAMEPAD_AXIS_LEFT_X) <= 0;
  *o = (unsigned long long)((GAMEPAD_AXIS_LEFT_X) | 0);  /* check that GAMEPAD_AXIS_LEFT_X is an integer */
  return n;
}

static int _cffi_const_GAMEPAD_AXIS_LEFT_Y(unsigned long long *o)
{
  int n = (GAMEPAD_AXIS_LEFT_Y) <= 0;
  *o = (unsigned long long)((GAMEPAD_AXIS_LEFT_Y) | 0);  /* check that GAMEPAD_AXIS_LEFT_Y is an integer */
  return n;
}

static int _cffi_const_GAMEPAD_AXIS_RIGHT_X(unsigned long long *o)
{
  int n = (GAMEPAD_AXIS_RIGHT_X) <= 0;
  *o = (unsigned long long)((GAMEPAD_AXIS_RIGHT_X) | 0);  /* check that GAMEPAD_AXIS_RIGHT_X is an integer */
  return n;
}

static int _cffi_const_GAMEPAD_AXIS_RIGHT_Y(unsigned long long *o)
{
  int n = (GAMEPAD_AXIS_RIGHT_Y) <= 0;
  *o = (unsigned long long)((GAMEPAD_AXIS_RIGHT_Y) | 0);  /* check that GAMEPAD_AXIS_RIGHT_Y is an integer */
  return n;
}

static int _cffi_const_GAMEPAD_AXIS_LEFT_TRIGGER(unsigned long long *o)
{
  int n = (GAMEPAD_AXIS_LEFT_TRIGGER) <= 0;
  *o = (unsigned long long)((GAMEPAD_AXIS_LEFT_TRIGGER) | 0);  /* check that GAMEPAD_AXIS_LEFT_TRIGGER is an integer */
  return n;
}

static int _cffi_const_GAMEPAD_AXIS_RIGHT_TRIGGER(unsigned long long *o)
{
  int n = (GAMEPAD_AXIS_RIGHT_TRIGGER) <= 0;
  *o = (unsigned long long)((GAMEPAD_AXIS_RIGHT_TRIGGER) | 0);  /* check that GAMEPAD_AXIS_RIGHT_TRIGGER is an integer */
  return n;
}

static int _cffi_const_GAMEPAD_BUTTON_UNKNOWN(unsigned long long *o)
{
  int n = (GAMEPAD_BUTTON_UNKNOWN) <= 0;
  *o = (unsigned long long)((GAMEPAD_BUTTON_UNKNOWN) | 0);  /* check that GAMEPAD_BUTTON_UNKNOWN is an integer */
  return n;
}

static int _cffi_const_GAMEPAD_BUTTON_LEFT_FACE_UP(unsigned long long *o)
{
  int n = (GAMEPAD_BUTTON_LEFT_FACE_UP) <= 0;
  *o = (unsigned long long)((GAMEPAD_BUTTON_LEFT_FACE_UP) | 0);  /* check that GAMEPAD_BUTTON_LEFT_FACE_UP is an integer */
  return n;
}

static int _cffi_const_GAMEPAD_BUTTON_LEFT_FACE_RIGHT(unsigned long long *o)
{
  int n = (GAMEPAD_BUTTON_LEFT_FACE_RIGHT) <= 0;
  *o = (unsigned long long)((GAMEPAD_BUTTON_LEFT_FACE_RIGHT) | 0);  /* check that GAMEPAD_BUTTON_LEFT_FACE_RIGHT is an integer */
  return n;
}

static int _cffi_const_GAMEPAD_BUTTON_LEFT_FACE_DOWN(unsigned long long *o)
{
  int n = (GAMEPAD_BUTTON_LEFT_FACE_DOWN) <= 0;
  *o = (unsigned long long)((GAMEPAD_BUTTON_LEFT_FACE_DOWN) | 0);  /* check that GAMEPAD_BUTTON_LEFT_FACE_DOWN is an integer */
  return n;
}

static int _cffi_const_GAMEPAD_BUTTON_LEFT_FACE_LEFT(unsigned long long *o)
{
  int n = (GAMEPAD_BUTTON_LEFT_FACE_LEFT) <= 0;
  *o = (unsigned long long)((GAMEPAD_BUTTON_LEFT_FACE_LEFT) | 0);  /* check that GAMEPAD_BUTTON_LEFT_FACE_LEFT is an integer */
  return n;
}

static int _cffi_const_GAMEPAD_BUTTON_RIGHT_FACE_UP(unsigned long long *o)
{
  int n = (GAMEPAD_BUTTON_RIGHT_FACE_UP) <= 0;
  *o = (unsigned long long)((GAMEPAD_BUTTON_RIGHT_FACE_UP) | 0);  /* check that GAMEPAD_BUTTON_RIGHT_FACE_UP is an integer */
  return n;
}

static int _cffi_const_GAMEPAD_BUTTON_RIGHT_FACE_RIGHT(unsigned long long *o)
{
  int n = (GAMEPAD_BUTTON_RIGHT_FACE_RIGHT) <= 0;
  *o = (unsigned long long)((GAMEPAD_BUTTON_RIGHT_FACE_RIGHT) | 0);  /* check that GAMEPAD_BUTTON_RIGHT_FACE_RIGHT is an integer */
  return n;
}

static int _cffi_const_GAMEPAD_BUTTON_RIGHT_FACE_DOWN(unsigned long long *o)
{
  int n = (GAMEPAD_BUTTON_RIGHT_FACE_DOWN) <= 0;
  *o = (unsigned long long)((GAMEPAD_BUTTON_RIGHT_FACE_DOWN) | 0);  /* check that GAMEPAD_BUTTON_RIGHT_FACE_DOWN is an integer */
  return n;
}

static int _cffi_const_GAMEPAD_BUTTON_RIGHT_FACE_LEFT(unsigned long long *o)
{
  int n = (GAMEPAD_BUTTON_RIGHT_FACE_LEFT) <= 0;
  *o = (unsigned long long)((GAMEPAD_BUTTON_RIGHT_FACE_LEFT) | 0);  /* check that GAMEPAD_BUTTON_RIGHT_FACE_LEFT is an integer */
  return n;
}

static int _cffi_const_GAMEPAD_BUTTON_LEFT_TRIGGER_1(unsigned long long *o)
{
  int n = (GAMEPAD_BUTTON_LEFT_TRIGGER_1) <= 0;
  *o = (unsigned long long)((GAMEPAD_BUTTON_LEFT_TRIGGER_1) | 0);  /* check that GAMEPAD_BUTTON_LEFT_TRIGGER_1 is an integer */
  return n;
}

static int _cffi_const_GAMEPAD_BUTTON_LEFT_TRIGGER_2(unsigned long long *o)
{
  int n = (GAMEPAD_BUTTON_LEFT_TRIGGER_2) <= 0;
  *o = (unsigned long long)((GAMEPAD_BUTTON_LEFT_TRIGGER_2) | 0);  /* check that GAMEPAD_BUTTON_LEFT_TRIGGER_2 is an integer */
  return n;
}

static int _cffi_const_GAMEPAD_BUTTON_RIGHT_TRIGGER_1(unsigned long long *o)
{
  int n = (GAMEPAD_BUTTON_RIGHT_TRIGGER_1) <= 0;
  *o = (unsigned long long)((GAMEPAD_BUTTON_RIGHT_TRIGGER_1) | 0);  /* check that GAMEPAD_BUTTON_RIGHT_TRIGGER_1 is an integer */
  return n;
}

static int _cffi_const_GAMEPAD_BUTTON_RIGHT_TRIGGER_2(unsigned long long *o)
{
  int n = (GAMEPAD_BUTTON_RIGHT_TRIGGER_2) <= 0;
  *o = (unsigned long long)((GAMEPAD_BUTTON_RIGHT_TRIGGER_2) | 0);  /* check that GAMEPAD_BUTTON_RIGHT_TRIGGER_2 is an integer */
  return n;
}

static int _cffi_const_GAMEPAD_BUTTON_MIDDLE_LEFT(unsigned long long *o)
{
  int n = (GAMEPAD_BUTTON_MIDDLE_LEFT) <= 0;
  *o = (unsigned long long)((GAMEPAD_BUTTON_MIDDLE_LEFT) | 0);  /* check that GAMEPAD_BUTTON_MIDDLE_LEFT is an integer */
  return n;
}

static int _cffi_const_GAMEPAD_BUTTON_MIDDLE(unsigned long long *o)
{
  int n = (GAMEPAD_BUTTON_MIDDLE) <= 0;
  *o = (unsigned long long)((GAMEPAD_BUTTON_MIDDLE) | 0);  /* check that GAMEPAD_BUTTON_MIDDLE is an integer */
  return n;
}

static int _cffi_const_GAMEPAD_BUTTON_MIDDLE_RIGHT(unsigned long long *o)
{
  int n = (GAMEPAD_BUTTON_MIDDLE_RIGHT) <= 0;
  *o = (unsigned long long)((GAMEPAD_BUTTON_MIDDLE_RIGHT) | 0);  /* check that GAMEPAD_BUTTON_MIDDLE_RIGHT is an integer */
  return n;
}

static int _cffi_const_GAMEPAD_BUTTON_LEFT_THUMB(unsigned long long *o)
{
  int n = (GAMEPAD_BUTTON_LEFT_THUMB) <= 0;
  *o = (unsigned long long)((GAMEPAD_BUTTON_LEFT_THUMB) | 0);  /* check that GAMEPAD_BUTTON_LEFT_THUMB is an integer */
  return n;
}

static int _cffi_const_GAMEPAD_BUTTON_RIGHT_THUMB(unsigned long long *o)
{
  int n = (GAMEPAD_BUTTON_RIGHT_THUMB) <= 0;
  *o = (unsigned long long)((GAMEPAD_BUTTON_RIGHT_THUMB) | 0);  /* check that GAMEPAD_BUTTON_RIGHT_THUMB is an integer */
  return n;
}

static int _cffi_const_GAMEPAD_PLAYER1(unsigned long long *o)
{
  int n = (GAMEPAD_PLAYER1) <= 0;
  *o = (unsigned long long)((GAMEPAD_PLAYER1) | 0);  /* check that GAMEPAD_PLAYER1 is an integer */
  return n;
}

static int _cffi_const_GAMEPAD_PLAYER2(unsigned long long *o)
{
  int n = (GAMEPAD_PLAYER2) <= 0;
  *o = (unsigned long long)((GAMEPAD_PLAYER2) | 0);  /* check that GAMEPAD_PLAYER2 is an integer */
  return n;
}

static int _cffi_const_GAMEPAD_PLAYER3(unsigned long long *o)
{
  int n = (GAMEPAD_PLAYER3) <= 0;
  *o = (unsigned long long)((GAMEPAD_PLAYER3) | 0);  /* check that GAMEPAD_PLAYER3 is an integer */
  return n;
}

static int _cffi_const_GAMEPAD_PLAYER4(unsigned long long *o)
{
  int n = (GAMEPAD_PLAYER4) <= 0;
  *o = (unsigned long long)((GAMEPAD_PLAYER4) | 0);  /* check that GAMEPAD_PLAYER4 is an integer */
  return n;
}

static int _cffi_const_GESTURE_NONE(unsigned long long *o)
{
  int n = (GESTURE_NONE) <= 0;
  *o = (unsigned long long)((GESTURE_NONE) | 0);  /* check that GESTURE_NONE is an integer */
  return n;
}

static int _cffi_const_GESTURE_TAP(unsigned long long *o)
{
  int n = (GESTURE_TAP) <= 0;
  *o = (unsigned long long)((GESTURE_TAP) | 0);  /* check that GESTURE_TAP is an integer */
  return n;
}

static int _cffi_const_GESTURE_DOUBLETAP(unsigned long long *o)
{
  int n = (GESTURE_DOUBLETAP) <= 0;
  *o = (unsigned long long)((GESTURE_DOUBLETAP) | 0);  /* check that GESTURE_DOUBLETAP is an integer */
  return n;
}

static int _cffi_const_GESTURE_HOLD(unsigned long long *o)
{
  int n = (GESTURE_HOLD) <= 0;
  *o = (unsigned long long)((GESTURE_HOLD) | 0);  /* check that GESTURE_HOLD is an integer */
  return n;
}

static int _cffi_const_GESTURE_DRAG(unsigned long long *o)
{
  int n = (GESTURE_DRAG) <= 0;
  *o = (unsigned long long)((GESTURE_DRAG) | 0);  /* check that GESTURE_DRAG is an integer */
  return n;
}

static int _cffi_const_GESTURE_SWIPE_RIGHT(unsigned long long *o)
{
  int n = (GESTURE_SWIPE_RIGHT) <= 0;
  *o = (unsigned long long)((GESTURE_SWIPE_RIGHT) | 0);  /* check that GESTURE_SWIPE_RIGHT is an integer */
  return n;
}

static int _cffi_const_GESTURE_SWIPE_LEFT(unsigned long long *o)
{
  int n = (GESTURE_SWIPE_LEFT) <= 0;
  *o = (unsigned long long)((GESTURE_SWIPE_LEFT) | 0);  /* check that GESTURE_SWIPE_LEFT is an integer */
  return n;
}

static int _cffi_const_GESTURE_SWIPE_UP(unsigned long long *o)
{
  int n = (GESTURE_SWIPE_UP) <= 0;
  *o = (unsigned long long)((GESTURE_SWIPE_UP) | 0);  /* check that GESTURE_SWIPE_UP is an integer */
  return n;
}

static int _cffi_const_GESTURE_SWIPE_DOWN(unsigned long long *o)
{
  int n = (GESTURE_SWIPE_DOWN) <= 0;
  *o = (unsigned long long)((GESTURE_SWIPE_DOWN) | 0);  /* check that GESTURE_SWIPE_DOWN is an integer */
  return n;
}

static int _cffi_const_GESTURE_PINCH_IN(unsigned long long *o)
{
  int n = (GESTURE_PINCH_IN) <= 0;
  *o = (unsigned long long)((GESTURE_PINCH_IN) | 0);  /* check that GESTURE_PINCH_IN is an integer */
  return n;
}

static int _cffi_const_GESTURE_PINCH_OUT(unsigned long long *o)
{
  int n = (GESTURE_PINCH_OUT) <= 0;
  *o = (unsigned long long)((GESTURE_PINCH_OUT) | 0);  /* check that GESTURE_PINCH_OUT is an integer */
  return n;
}

static int _cffi_const_KEY_APOSTROPHE(unsigned long long *o)
{
  int n = (KEY_APOSTROPHE) <= 0;
  *o = (unsigned long long)((KEY_APOSTROPHE) | 0);  /* check that KEY_APOSTROPHE is an integer */
  return n;
}

static int _cffi_const_KEY_COMMA(unsigned long long *o)
{
  int n = (KEY_COMMA) <= 0;
  *o = (unsigned long long)((KEY_COMMA) | 0);  /* check that KEY_COMMA is an integer */
  return n;
}

static int _cffi_const_KEY_MINUS(unsigned long long *o)
{
  int n = (KEY_MINUS) <= 0;
  *o = (unsigned long long)((KEY_MINUS) | 0);  /* check that KEY_MINUS is an integer */
  return n;
}

static int _cffi_const_KEY_PERIOD(unsigned long long *o)
{
  int n = (KEY_PERIOD) <= 0;
  *o = (unsigned long long)((KEY_PERIOD) | 0);  /* check that KEY_PERIOD is an integer */
  return n;
}

static int _cffi_const_KEY_SLASH(unsigned long long *o)
{
  int n = (KEY_SLASH) <= 0;
  *o = (unsigned long long)((KEY_SLASH) | 0);  /* check that KEY_SLASH is an integer */
  return n;
}

static int _cffi_const_KEY_ZERO(unsigned long long *o)
{
  int n = (KEY_ZERO) <= 0;
  *o = (unsigned long long)((KEY_ZERO) | 0);  /* check that KEY_ZERO is an integer */
  return n;
}

static int _cffi_const_KEY_ONE(unsigned long long *o)
{
  int n = (KEY_ONE) <= 0;
  *o = (unsigned long long)((KEY_ONE) | 0);  /* check that KEY_ONE is an integer */
  return n;
}

static int _cffi_const_KEY_TWO(unsigned long long *o)
{
  int n = (KEY_TWO) <= 0;
  *o = (unsigned long long)((KEY_TWO) | 0);  /* check that KEY_TWO is an integer */
  return n;
}

static int _cffi_const_KEY_THREE(unsigned long long *o)
{
  int n = (KEY_THREE) <= 0;
  *o = (unsigned long long)((KEY_THREE) | 0);  /* check that KEY_THREE is an integer */
  return n;
}

static int _cffi_const_KEY_FOUR(unsigned long long *o)
{
  int n = (KEY_FOUR) <= 0;
  *o = (unsigned long long)((KEY_FOUR) | 0);  /* check that KEY_FOUR is an integer */
  return n;
}

static int _cffi_const_KEY_FIVE(unsigned long long *o)
{
  int n = (KEY_FIVE) <= 0;
  *o = (unsigned long long)((KEY_FIVE) | 0);  /* check that KEY_FIVE is an integer */
  return n;
}

static int _cffi_const_KEY_SIX(unsigned long long *o)
{
  int n = (KEY_SIX) <= 0;
  *o = (unsigned long long)((KEY_SIX) | 0);  /* check that KEY_SIX is an integer */
  return n;
}

static int _cffi_const_KEY_SEVEN(unsigned long long *o)
{
  int n = (KEY_SEVEN) <= 0;
  *o = (unsigned long long)((KEY_SEVEN) | 0);  /* check that KEY_SEVEN is an integer */
  return n;
}

static int _cffi_const_KEY_EIGHT(unsigned long long *o)
{
  int n = (KEY_EIGHT) <= 0;
  *o = (unsigned long long)((KEY_EIGHT) | 0);  /* check that KEY_EIGHT is an integer */
  return n;
}

static int _cffi_const_KEY_NINE(unsigned long long *o)
{
  int n = (KEY_NINE) <= 0;
  *o = (unsigned long long)((KEY_NINE) | 0);  /* check that KEY_NINE is an integer */
  return n;
}

static int _cffi_const_KEY_SEMICOLON(unsigned long long *o)
{
  int n = (KEY_SEMICOLON) <= 0;
  *o = (unsigned long long)((KEY_SEMICOLON) | 0);  /* check that KEY_SEMICOLON is an integer */
  return n;
}

static int _cffi_const_KEY_EQUAL(unsigned long long *o)
{
  int n = (KEY_EQUAL) <= 0;
  *o = (unsigned long long)((KEY_EQUAL) | 0);  /* check that KEY_EQUAL is an integer */
  return n;
}

static int _cffi_const_KEY_A(unsigned long long *o)
{
  int n = (KEY_A) <= 0;
  *o = (unsigned long long)((KEY_A) | 0);  /* check that KEY_A is an integer */
  return n;
}

static int _cffi_const_KEY_B(unsigned long long *o)
{
  int n = (KEY_B) <= 0;
  *o = (unsigned long long)((KEY_B) | 0);  /* check that KEY_B is an integer */
  return n;
}

static int _cffi_const_KEY_C(unsigned long long *o)
{
  int n = (KEY_C) <= 0;
  *o = (unsigned long long)((KEY_C) | 0);  /* check that KEY_C is an integer */
  return n;
}

static int _cffi_const_KEY_D(unsigned long long *o)
{
  int n = (KEY_D) <= 0;
  *o = (unsigned long long)((KEY_D) | 0);  /* check that KEY_D is an integer */
  return n;
}

static int _cffi_const_KEY_E(unsigned long long *o)
{
  int n = (KEY_E) <= 0;
  *o = (unsigned long long)((KEY_E) | 0);  /* check that KEY_E is an integer */
  return n;
}

static int _cffi_const_KEY_F(unsigned long long *o)
{
  int n = (KEY_F) <= 0;
  *o = (unsigned long long)((KEY_F) | 0);  /* check that KEY_F is an integer */
  return n;
}

static int _cffi_const_KEY_G(unsigned long long *o)
{
  int n = (KEY_G) <= 0;
  *o = (unsigned long long)((KEY_G) | 0);  /* check that KEY_G is an integer */
  return n;
}

static int _cffi_const_KEY_H(unsigned long long *o)
{
  int n = (KEY_H) <= 0;
  *o = (unsigned long long)((KEY_H) | 0);  /* check that KEY_H is an integer */
  return n;
}

static int _cffi_const_KEY_I(unsigned long long *o)
{
  int n = (KEY_I) <= 0;
  *o = (unsigned long long)((KEY_I) | 0);  /* check that KEY_I is an integer */
  return n;
}

static int _cffi_const_KEY_J(unsigned long long *o)
{
  int n = (KEY_J) <= 0;
  *o = (unsigned long long)((KEY_J) | 0);  /* check that KEY_J is an integer */
  return n;
}

static int _cffi_const_KEY_K(unsigned long long *o)
{
  int n = (KEY_K) <= 0;
  *o = (unsigned long long)((KEY_K) | 0);  /* check that KEY_K is an integer */
  return n;
}

static int _cffi_const_KEY_L(unsigned long long *o)
{
  int n = (KEY_L) <= 0;
  *o = (unsigned long long)((KEY_L) | 0);  /* check that KEY_L is an integer */
  return n;
}

static int _cffi_const_KEY_M(unsigned long long *o)
{
  int n = (KEY_M) <= 0;
  *o = (unsigned long long)((KEY_M) | 0);  /* check that KEY_M is an integer */
  return n;
}

static int _cffi_const_KEY_N(unsigned long long *o)
{
  int n = (KEY_N) <= 0;
  *o = (unsigned long long)((KEY_N) | 0);  /* check that KEY_N is an integer */
  return n;
}

static int _cffi_const_KEY_O(unsigned long long *o)
{
  int n = (KEY_O) <= 0;
  *o = (unsigned long long)((KEY_O) | 0);  /* check that KEY_O is an integer */
  return n;
}

static int _cffi_const_KEY_P(unsigned long long *o)
{
  int n = (KEY_P) <= 0;
  *o = (unsigned long long)((KEY_P) | 0);  /* check that KEY_P is an integer */
  return n;
}

static int _cffi_const_KEY_Q(unsigned long long *o)
{
  int n = (KEY_Q) <= 0;
  *o = (unsigned long long)((KEY_Q) | 0);  /* check that KEY_Q is an integer */
  return n;
}

static int _cffi_const_KEY_R(unsigned long long *o)
{
  int n = (KEY_R) <= 0;
  *o = (unsigned long long)((KEY_R) | 0);  /* check that KEY_R is an integer */
  return n;
}

static int _cffi_const_KEY_S(unsigned long long *o)
{
  int n = (KEY_S) <= 0;
  *o = (unsigned long long)((KEY_S) | 0);  /* check that KEY_S is an integer */
  return n;
}

static int _cffi_const_KEY_T(unsigned long long *o)
{
  int n = (KEY_T) <= 0;
  *o = (unsigned long long)((KEY_T) | 0);  /* check that KEY_T is an integer */
  return n;
}

static int _cffi_const_KEY_U(unsigned long long *o)
{
  int n = (KEY_U) <= 0;
  *o = (unsigned long long)((KEY_U) | 0);  /* check that KEY_U is an integer */
  return n;
}

static int _cffi_const_KEY_V(unsigned long long *o)
{
  int n = (KEY_V) <= 0;
  *o = (unsigned long long)((KEY_V) | 0);  /* check that KEY_V is an integer */
  return n;
}

static int _cffi_const_KEY_W(unsigned long long *o)
{
  int n = (KEY_W) <= 0;
  *o = (unsigned long long)((KEY_W) | 0);  /* check that KEY_W is an integer */
  return n;
}

static int _cffi_const_KEY_X(unsigned long long *o)
{
  int n = (KEY_X) <= 0;
  *o = (unsigned long long)((KEY_X) | 0);  /* check that KEY_X is an integer */
  return n;
}

static int _cffi_const_KEY_Y(unsigned long long *o)
{
  int n = (KEY_Y) <= 0;
  *o = (unsigned long long)((KEY_Y) | 0);  /* check that KEY_Y is an integer */
  return n;
}

static int _cffi_const_KEY_Z(unsigned long long *o)
{
  int n = (KEY_Z) <= 0;
  *o = (unsigned long long)((KEY_Z) | 0);  /* check that KEY_Z is an integer */
  return n;
}

static int _cffi_const_KEY_SPACE(unsigned long long *o)
{
  int n = (KEY_SPACE) <= 0;
  *o = (unsigned long long)((KEY_SPACE) | 0);  /* check that KEY_SPACE is an integer */
  return n;
}

static int _cffi_const_KEY_ESCAPE(unsigned long long *o)
{
  int n = (KEY_ESCAPE) <= 0;
  *o = (unsigned long long)((KEY_ESCAPE) | 0);  /* check that KEY_ESCAPE is an integer */
  return n;
}

static int _cffi_const_KEY_ENTER(unsigned long long *o)
{
  int n = (KEY_ENTER) <= 0;
  *o = (unsigned long long)((KEY_ENTER) | 0);  /* check that KEY_ENTER is an integer */
  return n;
}

static int _cffi_const_KEY_TAB(unsigned long long *o)
{
  int n = (KEY_TAB) <= 0;
  *o = (unsigned long long)((KEY_TAB) | 0);  /* check that KEY_TAB is an integer */
  return n;
}

static int _cffi_const_KEY_BACKSPACE(unsigned long long *o)
{
  int n = (KEY_BACKSPACE) <= 0;
  *o = (unsigned long long)((KEY_BACKSPACE) | 0);  /* check that KEY_BACKSPACE is an integer */
  return n;
}

static int _cffi_const_KEY_INSERT(unsigned long long *o)
{
  int n = (KEY_INSERT) <= 0;
  *o = (unsigned long long)((KEY_INSERT) | 0);  /* check that KEY_INSERT is an integer */
  return n;
}

static int _cffi_const_KEY_DELETE(unsigned long long *o)
{
  int n = (KEY_DELETE) <= 0;
  *o = (unsigned long long)((KEY_DELETE) | 0);  /* check that KEY_DELETE is an integer */
  return n;
}

static int _cffi_const_KEY_RIGHT(unsigned long long *o)
{
  int n = (KEY_RIGHT) <= 0;
  *o = (unsigned long long)((KEY_RIGHT) | 0);  /* check that KEY_RIGHT is an integer */
  return n;
}

static int _cffi_const_KEY_LEFT(unsigned long long *o)
{
  int n = (KEY_LEFT) <= 0;
  *o = (unsigned long long)((KEY_LEFT) | 0);  /* check that KEY_LEFT is an integer */
  return n;
}

static int _cffi_const_KEY_DOWN(unsigned long long *o)
{
  int n = (KEY_DOWN) <= 0;
  *o = (unsigned long long)((KEY_DOWN) | 0);  /* check that KEY_DOWN is an integer */
  return n;
}

static int _cffi_const_KEY_UP(unsigned long long *o)
{
  int n = (KEY_UP) <= 0;
  *o = (unsigned long long)((KEY_UP) | 0);  /* check that KEY_UP is an integer */
  return n;
}

static int _cffi_const_KEY_PAGE_UP(unsigned long long *o)
{
  int n = (KEY_PAGE_UP) <= 0;
  *o = (unsigned long long)((KEY_PAGE_UP) | 0);  /* check that KEY_PAGE_UP is an integer */
  return n;
}

static int _cffi_const_KEY_PAGE_DOWN(unsigned long long *o)
{
  int n = (KEY_PAGE_DOWN) <= 0;
  *o = (unsigned long long)((KEY_PAGE_DOWN) | 0);  /* check that KEY_PAGE_DOWN is an integer */
  return n;
}

static int _cffi_const_KEY_HOME(unsigned long long *o)
{
  int n = (KEY_HOME) <= 0;
  *o = (unsigned long long)((KEY_HOME) | 0);  /* check that KEY_HOME is an integer */
  return n;
}

static int _cffi_const_KEY_END(unsigned long long *o)
{
  int n = (KEY_END) <= 0;
  *o = (unsigned long long)((KEY_END) | 0);  /* check that KEY_END is an integer */
  return n;
}

static int _cffi_const_KEY_CAPS_LOCK(unsigned long long *o)
{
  int n = (KEY_CAPS_LOCK) <= 0;
  *o = (unsigned long long)((KEY_CAPS_LOCK) | 0);  /* check that KEY_CAPS_LOCK is an integer */
  return n;
}

static int _cffi_const_KEY_SCROLL_LOCK(unsigned long long *o)
{
  int n = (KEY_SCROLL_LOCK) <= 0;
  *o = (unsigned long long)((KEY_SCROLL_LOCK) | 0);  /* check that KEY_SCROLL_LOCK is an integer */
  return n;
}

static int _cffi_const_KEY_NUM_LOCK(unsigned long long *o)
{
  int n = (KEY_NUM_LOCK) <= 0;
  *o = (unsigned long long)((KEY_NUM_LOCK) | 0);  /* check that KEY_NUM_LOCK is an integer */
  return n;
}

static int _cffi_const_KEY_PRINT_SCREEN(unsigned long long *o)
{
  int n = (KEY_PRINT_SCREEN) <= 0;
  *o = (unsigned long long)((KEY_PRINT_SCREEN) | 0);  /* check that KEY_PRINT_SCREEN is an integer */
  return n;
}

static int _cffi_const_KEY_PAUSE(unsigned long long *o)
{
  int n = (KEY_PAUSE) <= 0;
  *o = (unsigned long long)((KEY_PAUSE) | 0);  /* check that KEY_PAUSE is an integer */
  return n;
}

static int _cffi_const_KEY_F1(unsigned long long *o)
{
  int n = (KEY_F1) <= 0;
  *o = (unsigned long long)((KEY_F1) | 0);  /* check that KEY_F1 is an integer */
  return n;
}

static int _cffi_const_KEY_F2(unsigned long long *o)
{
  int n = (KEY_F2) <= 0;
  *o = (unsigned long long)((KEY_F2) | 0);  /* check that KEY_F2 is an integer */
  return n;
}

static int _cffi_const_KEY_F3(unsigned long long *o)
{
  int n = (KEY_F3) <= 0;
  *o = (unsigned long long)((KEY_F3) | 0);  /* check that KEY_F3 is an integer */
  return n;
}

static int _cffi_const_KEY_F4(unsigned long long *o)
{
  int n = (KEY_F4) <= 0;
  *o = (unsigned long long)((KEY_F4) | 0);  /* check that KEY_F4 is an integer */
  return n;
}

static int _cffi_const_KEY_F5(unsigned long long *o)
{
  int n = (KEY_F5) <= 0;
  *o = (unsigned long long)((KEY_F5) | 0);  /* check that KEY_F5 is an integer */
  return n;
}

static int _cffi_const_KEY_F6(unsigned long long *o)
{
  int n = (KEY_F6) <= 0;
  *o = (unsigned long long)((KEY_F6) | 0);  /* check that KEY_F6 is an integer */
  return n;
}

static int _cffi_const_KEY_F7(unsigned long long *o)
{
  int n = (KEY_F7) <= 0;
  *o = (unsigned long long)((KEY_F7) | 0);  /* check that KEY_F7 is an integer */
  return n;
}

static int _cffi_const_KEY_F8(unsigned long long *o)
{
  int n = (KEY_F8) <= 0;
  *o = (unsigned long long)((KEY_F8) | 0);  /* check that KEY_F8 is an integer */
  return n;
}

static int _cffi_const_KEY_F9(unsigned long long *o)
{
  int n = (KEY_F9) <= 0;
  *o = (unsigned long long)((KEY_F9) | 0);  /* check that KEY_F9 is an integer */
  return n;
}

static int _cffi_const_KEY_F10(unsigned long long *o)
{
  int n = (KEY_F10) <= 0;
  *o = (unsigned long long)((KEY_F10) | 0);  /* check that KEY_F10 is an integer */
  return n;
}

static int _cffi_const_KEY_F11(unsigned long long *o)
{
  int n = (KEY_F11) <= 0;
  *o = (unsigned long long)((KEY_F11) | 0);  /* check that KEY_F11 is an integer */
  return n;
}

static int _cffi_const_KEY_F12(unsigned long long *o)
{
  int n = (KEY_F12) <= 0;
  *o = (unsigned long long)((KEY_F12) | 0);  /* check that KEY_F12 is an integer */
  return n;
}

static int _cffi_const_KEY_LEFT_SHIFT(unsigned long long *o)
{
  int n = (KEY_LEFT_SHIFT) <= 0;
  *o = (unsigned long long)((KEY_LEFT_SHIFT) | 0);  /* check that KEY_LEFT_SHIFT is an integer */
  return n;
}

static int _cffi_const_KEY_LEFT_CONTROL(unsigned long long *o)
{
  int n = (KEY_LEFT_CONTROL) <= 0;
  *o = (unsigned long long)((KEY_LEFT_CONTROL) | 0);  /* check that KEY_LEFT_CONTROL is an integer */
  return n;
}

static int _cffi_const_KEY_LEFT_ALT(unsigned long long *o)
{
  int n = (KEY_LEFT_ALT) <= 0;
  *o = (unsigned long long)((KEY_LEFT_ALT) | 0);  /* check that KEY_LEFT_ALT is an integer */
  return n;
}

static int _cffi_const_KEY_LEFT_SUPER(unsigned long long *o)
{
  int n = (KEY_LEFT_SUPER) <= 0;
  *o = (unsigned long long)((KEY_LEFT_SUPER) | 0);  /* check that KEY_LEFT_SUPER is an integer */
  return n;
}

static int _cffi_const_KEY_RIGHT_SHIFT(unsigned long long *o)
{
  int n = (KEY_RIGHT_SHIFT) <= 0;
  *o = (unsigned long long)((KEY_RIGHT_SHIFT) | 0);  /* check that KEY_RIGHT_SHIFT is an integer */
  return n;
}

static int _cffi_const_KEY_RIGHT_CONTROL(unsigned long long *o)
{
  int n = (KEY_RIGHT_CONTROL) <= 0;
  *o = (unsigned long long)((KEY_RIGHT_CONTROL) | 0);  /* check that KEY_RIGHT_CONTROL is an integer */
  return n;
}

static int _cffi_const_KEY_RIGHT_ALT(unsigned long long *o)
{
  int n = (KEY_RIGHT_ALT) <= 0;
  *o = (unsigned long long)((KEY_RIGHT_ALT) | 0);  /* check that KEY_RIGHT_ALT is an integer */
  return n;
}

static int _cffi_const_KEY_RIGHT_SUPER(unsigned long long *o)
{
  int n = (KEY_RIGHT_SUPER) <= 0;
  *o = (unsigned long long)((KEY_RIGHT_SUPER) | 0);  /* check that KEY_RIGHT_SUPER is an integer */
  return n;
}

static int _cffi_const_KEY_KB_MENU(unsigned long long *o)
{
  int n = (KEY_KB_MENU) <= 0;
  *o = (unsigned long long)((KEY_KB_MENU) | 0);  /* check that KEY_KB_MENU is an integer */
  return n;
}

static int _cffi_const_KEY_LEFT_BRACKET(unsigned long long *o)
{
  int n = (KEY_LEFT_BRACKET) <= 0;
  *o = (unsigned long long)((KEY_LEFT_BRACKET) | 0);  /* check that KEY_LEFT_BRACKET is an integer */
  return n;
}

static int _cffi_const_KEY_BACKSLASH(unsigned long long *o)
{
  int n = (KEY_BACKSLASH) <= 0;
  *o = (unsigned long long)((KEY_BACKSLASH) | 0);  /* check that KEY_BACKSLASH is an integer */
  return n;
}

static int _cffi_const_KEY_RIGHT_BRACKET(unsigned long long *o)
{
  int n = (KEY_RIGHT_BRACKET) <= 0;
  *o = (unsigned long long)((KEY_RIGHT_BRACKET) | 0);  /* check that KEY_RIGHT_BRACKET is an integer */
  return n;
}

static int _cffi_const_KEY_GRAVE(unsigned long long *o)
{
  int n = (KEY_GRAVE) <= 0;
  *o = (unsigned long long)((KEY_GRAVE) | 0);  /* check that KEY_GRAVE is an integer */
  return n;
}

static int _cffi_const_KEY_KP_0(unsigned long long *o)
{
  int n = (KEY_KP_0) <= 0;
  *o = (unsigned long long)((KEY_KP_0) | 0);  /* check that KEY_KP_0 is an integer */
  return n;
}

static int _cffi_const_KEY_KP_1(unsigned long long *o)
{
  int n = (KEY_KP_1) <= 0;
  *o = (unsigned long long)((KEY_KP_1) | 0);  /* check that KEY_KP_1 is an integer */
  return n;
}

static int _cffi_const_KEY_KP_2(unsigned long long *o)
{
  int n = (KEY_KP_2) <= 0;
  *o = (unsigned long long)((KEY_KP_2) | 0);  /* check that KEY_KP_2 is an integer */
  return n;
}

static int _cffi_const_KEY_KP_3(unsigned long long *o)
{
  int n = (KEY_KP_3) <= 0;
  *o = (unsigned long long)((KEY_KP_3) | 0);  /* check that KEY_KP_3 is an integer */
  return n;
}

static int _cffi_const_KEY_KP_4(unsigned long long *o)
{
  int n = (KEY_KP_4) <= 0;
  *o = (unsigned long long)((KEY_KP_4) | 0);  /* check that KEY_KP_4 is an integer */
  return n;
}

static int _cffi_const_KEY_KP_5(unsigned long long *o)
{
  int n = (KEY_KP_5) <= 0;
  *o = (unsigned long long)((KEY_KP_5) | 0);  /* check that KEY_KP_5 is an integer */
  return n;
}

static int _cffi_const_KEY_KP_6(unsigned long long *o)
{
  int n = (KEY_KP_6) <= 0;
  *o = (unsigned long long)((KEY_KP_6) | 0);  /* check that KEY_KP_6 is an integer */
  return n;
}

static int _cffi_const_KEY_KP_7(unsigned long long *o)
{
  int n = (KEY_KP_7) <= 0;
  *o = (unsigned long long)((KEY_KP_7) | 0);  /* check that KEY_KP_7 is an integer */
  return n;
}

static int _cffi_const_KEY_KP_8(unsigned long long *o)
{
  int n = (KEY_KP_8) <= 0;
  *o = (unsigned long long)((KEY_KP_8) | 0);  /* check that KEY_KP_8 is an integer */
  return n;
}

static int _cffi_const_KEY_KP_9(unsigned long long *o)
{
  int n = (KEY_KP_9) <= 0;
  *o = (unsigned long long)((KEY_KP_9) | 0);  /* check that KEY_KP_9 is an integer */
  return n;
}

static int _cffi_const_KEY_KP_DECIMAL(unsigned long long *o)
{
  int n = (KEY_KP_DECIMAL) <= 0;
  *o = (unsigned long long)((KEY_KP_DECIMAL) | 0);  /* check that KEY_KP_DECIMAL is an integer */
  return n;
}

static int _cffi_const_KEY_KP_DIVIDE(unsigned long long *o)
{
  int n = (KEY_KP_DIVIDE) <= 0;
  *o = (unsigned long long)((KEY_KP_DIVIDE) | 0);  /* check that KEY_KP_DIVIDE is an integer */
  return n;
}

static int _cffi_const_KEY_KP_MULTIPLY(unsigned long long *o)
{
  int n = (KEY_KP_MULTIPLY) <= 0;
  *o = (unsigned long long)((KEY_KP_MULTIPLY) | 0);  /* check that KEY_KP_MULTIPLY is an integer */
  return n;
}

static int _cffi_const_KEY_KP_SUBTRACT(unsigned long long *o)
{
  int n = (KEY_KP_SUBTRACT) <= 0;
  *o = (unsigned long long)((KEY_KP_SUBTRACT) | 0);  /* check that KEY_KP_SUBTRACT is an integer */
  return n;
}

static int _cffi_const_KEY_KP_ADD(unsigned long long *o)
{
  int n = (KEY_KP_ADD) <= 0;
  *o = (unsigned long long)((KEY_KP_ADD) | 0);  /* check that KEY_KP_ADD is an integer */
  return n;
}

static int _cffi_const_KEY_KP_ENTER(unsigned long long *o)
{
  int n = (KEY_KP_ENTER) <= 0;
  *o = (unsigned long long)((KEY_KP_ENTER) | 0);  /* check that KEY_KP_ENTER is an integer */
  return n;
}

static int _cffi_const_KEY_KP_EQUAL(unsigned long long *o)
{
  int n = (KEY_KP_EQUAL) <= 0;
  *o = (unsigned long long)((KEY_KP_EQUAL) | 0);  /* check that KEY_KP_EQUAL is an integer */
  return n;
}

static int _cffi_const_MAP_ALBEDO(unsigned long long *o)
{
  int n = (MAP_ALBEDO) <= 0;
  *o = (unsigned long long)((MAP_ALBEDO) | 0);  /* check that MAP_ALBEDO is an integer */
  return n;
}

static int _cffi_const_MAP_METALNESS(unsigned long long *o)
{
  int n = (MAP_METALNESS) <= 0;
  *o = (unsigned long long)((MAP_METALNESS) | 0);  /* check that MAP_METALNESS is an integer */
  return n;
}

static int _cffi_const_MAP_NORMAL(unsigned long long *o)
{
  int n = (MAP_NORMAL) <= 0;
  *o = (unsigned long long)((MAP_NORMAL) | 0);  /* check that MAP_NORMAL is an integer */
  return n;
}

static int _cffi_const_MAP_ROUGHNESS(unsigned long long *o)
{
  int n = (MAP_ROUGHNESS) <= 0;
  *o = (unsigned long long)((MAP_ROUGHNESS) | 0);  /* check that MAP_ROUGHNESS is an integer */
  return n;
}

static int _cffi_const_MAP_OCCLUSION(unsigned long long *o)
{
  int n = (MAP_OCCLUSION) <= 0;
  *o = (unsigned long long)((MAP_OCCLUSION) | 0);  /* check that MAP_OCCLUSION is an integer */
  return n;
}

static int _cffi_const_MAP_EMISSION(unsigned long long *o)
{
  int n = (MAP_EMISSION) <= 0;
  *o = (unsigned long long)((MAP_EMISSION) | 0);  /* check that MAP_EMISSION is an integer */
  return n;
}

static int _cffi_const_MAP_HEIGHT(unsigned long long *o)
{
  int n = (MAP_HEIGHT) <= 0;
  *o = (unsigned long long)((MAP_HEIGHT) | 0);  /* check that MAP_HEIGHT is an integer */
  return n;
}

static int _cffi_const_MAP_CUBEMAP(unsigned long long *o)
{
  int n = (MAP_CUBEMAP) <= 0;
  *o = (unsigned long long)((MAP_CUBEMAP) | 0);  /* check that MAP_CUBEMAP is an integer */
  return n;
}

static int _cffi_const_MAP_IRRADIANCE(unsigned long long *o)
{
  int n = (MAP_IRRADIANCE) <= 0;
  *o = (unsigned long long)((MAP_IRRADIANCE) | 0);  /* check that MAP_IRRADIANCE is an integer */
  return n;
}

static int _cffi_const_MAP_PREFILTER(unsigned long long *o)
{
  int n = (MAP_PREFILTER) <= 0;
  *o = (unsigned long long)((MAP_PREFILTER) | 0);  /* check that MAP_PREFILTER is an integer */
  return n;
}

static int _cffi_const_MAP_BRDF(unsigned long long *o)
{
  int n = (MAP_BRDF) <= 0;
  *o = (unsigned long long)((MAP_BRDF) | 0);  /* check that MAP_BRDF is an integer */
  return n;
}

static int _cffi_const_MOUSE_LEFT_BUTTON(unsigned long long *o)
{
  int n = (MOUSE_LEFT_BUTTON) <= 0;
  *o = (unsigned long long)((MOUSE_LEFT_BUTTON) | 0);  /* check that MOUSE_LEFT_BUTTON is an integer */
  return n;
}

static int _cffi_const_MOUSE_RIGHT_BUTTON(unsigned long long *o)
{
  int n = (MOUSE_RIGHT_BUTTON) <= 0;
  *o = (unsigned long long)((MOUSE_RIGHT_BUTTON) | 0);  /* check that MOUSE_RIGHT_BUTTON is an integer */
  return n;
}

static int _cffi_const_MOUSE_MIDDLE_BUTTON(unsigned long long *o)
{
  int n = (MOUSE_MIDDLE_BUTTON) <= 0;
  *o = (unsigned long long)((MOUSE_MIDDLE_BUTTON) | 0);  /* check that MOUSE_MIDDLE_BUTTON is an integer */
  return n;
}

static int _cffi_const_NPT_9PATCH(unsigned long long *o)
{
  int n = (NPT_9PATCH) <= 0;
  *o = (unsigned long long)((NPT_9PATCH) | 0);  /* check that NPT_9PATCH is an integer */
  return n;
}

static int _cffi_const_NPT_3PATCH_VERTICAL(unsigned long long *o)
{
  int n = (NPT_3PATCH_VERTICAL) <= 0;
  *o = (unsigned long long)((NPT_3PATCH_VERTICAL) | 0);  /* check that NPT_3PATCH_VERTICAL is an integer */
  return n;
}

static int _cffi_const_NPT_3PATCH_HORIZONTAL(unsigned long long *o)
{
  int n = (NPT_3PATCH_HORIZONTAL) <= 0;
  *o = (unsigned long long)((NPT_3PATCH_HORIZONTAL) | 0);  /* check that NPT_3PATCH_HORIZONTAL is an integer */
  return n;
}

static int _cffi_const_UNCOMPRESSED_GRAYSCALE(unsigned long long *o)
{
  int n = (UNCOMPRESSED_GRAYSCALE) <= 0;
  *o = (unsigned long long)((UNCOMPRESSED_GRAYSCALE) | 0);  /* check that UNCOMPRESSED_GRAYSCALE is an integer */
  return n;
}

static int _cffi_const_UNCOMPRESSED_GRAY_ALPHA(unsigned long long *o)
{
  int n = (UNCOMPRESSED_GRAY_ALPHA) <= 0;
  *o = (unsigned long long)((UNCOMPRESSED_GRAY_ALPHA) | 0);  /* check that UNCOMPRESSED_GRAY_ALPHA is an integer */
  return n;
}

static int _cffi_const_UNCOMPRESSED_R5G6B5(unsigned long long *o)
{
  int n = (UNCOMPRESSED_R5G6B5) <= 0;
  *o = (unsigned long long)((UNCOMPRESSED_R5G6B5) | 0);  /* check that UNCOMPRESSED_R5G6B5 is an integer */
  return n;
}

static int _cffi_const_UNCOMPRESSED_R8G8B8(unsigned long long *o)
{
  int n = (UNCOMPRESSED_R8G8B8) <= 0;
  *o = (unsigned long long)((UNCOMPRESSED_R8G8B8) | 0);  /* check that UNCOMPRESSED_R8G8B8 is an integer */
  return n;
}

static int _cffi_const_UNCOMPRESSED_R5G5B5A1(unsigned long long *o)
{
  int n = (UNCOMPRESSED_R5G5B5A1) <= 0;
  *o = (unsigned long long)((UNCOMPRESSED_R5G5B5A1) | 0);  /* check that UNCOMPRESSED_R5G5B5A1 is an integer */
  return n;
}

static int _cffi_const_UNCOMPRESSED_R4G4B4A4(unsigned long long *o)
{
  int n = (UNCOMPRESSED_R4G4B4A4) <= 0;
  *o = (unsigned long long)((UNCOMPRESSED_R4G4B4A4) | 0);  /* check that UNCOMPRESSED_R4G4B4A4 is an integer */
  return n;
}

static int _cffi_const_UNCOMPRESSED_R8G8B8A8(unsigned long long *o)
{
  int n = (UNCOMPRESSED_R8G8B8A8) <= 0;
  *o = (unsigned long long)((UNCOMPRESSED_R8G8B8A8) | 0);  /* check that UNCOMPRESSED_R8G8B8A8 is an integer */
  return n;
}

static int _cffi_const_UNCOMPRESSED_R32(unsigned long long *o)
{
  int n = (UNCOMPRESSED_R32) <= 0;
  *o = (unsigned long long)((UNCOMPRESSED_R32) | 0);  /* check that UNCOMPRESSED_R32 is an integer */
  return n;
}

static int _cffi_const_UNCOMPRESSED_R32G32B32(unsigned long long *o)
{
  int n = (UNCOMPRESSED_R32G32B32) <= 0;
  *o = (unsigned long long)((UNCOMPRESSED_R32G32B32) | 0);  /* check that UNCOMPRESSED_R32G32B32 is an integer */
  return n;
}

static int _cffi_const_UNCOMPRESSED_R32G32B32A32(unsigned long long *o)
{
  int n = (UNCOMPRESSED_R32G32B32A32) <= 0;
  *o = (unsigned long long)((UNCOMPRESSED_R32G32B32A32) | 0);  /* check that UNCOMPRESSED_R32G32B32A32 is an integer */
  return n;
}

static int _cffi_const_COMPRESSED_DXT1_RGB(unsigned long long *o)
{
  int n = (COMPRESSED_DXT1_RGB) <= 0;
  *o = (unsigned long long)((COMPRESSED_DXT1_RGB) | 0);  /* check that COMPRESSED_DXT1_RGB is an integer */
  return n;
}

static int _cffi_const_COMPRESSED_DXT1_RGBA(unsigned long long *o)
{
  int n = (COMPRESSED_DXT1_RGBA) <= 0;
  *o = (unsigned long long)((COMPRESSED_DXT1_RGBA) | 0);  /* check that COMPRESSED_DXT1_RGBA is an integer */
  return n;
}

static int _cffi_const_COMPRESSED_DXT3_RGBA(unsigned long long *o)
{
  int n = (COMPRESSED_DXT3_RGBA) <= 0;
  *o = (unsigned long long)((COMPRESSED_DXT3_RGBA) | 0);  /* check that COMPRESSED_DXT3_RGBA is an integer */
  return n;
}

static int _cffi_const_COMPRESSED_DXT5_RGBA(unsigned long long *o)
{
  int n = (COMPRESSED_DXT5_RGBA) <= 0;
  *o = (unsigned long long)((COMPRESSED_DXT5_RGBA) | 0);  /* check that COMPRESSED_DXT5_RGBA is an integer */
  return n;
}

static int _cffi_const_COMPRESSED_ETC1_RGB(unsigned long long *o)
{
  int n = (COMPRESSED_ETC1_RGB) <= 0;
  *o = (unsigned long long)((COMPRESSED_ETC1_RGB) | 0);  /* check that COMPRESSED_ETC1_RGB is an integer */
  return n;
}

static int _cffi_const_COMPRESSED_ETC2_RGB(unsigned long long *o)
{
  int n = (COMPRESSED_ETC2_RGB) <= 0;
  *o = (unsigned long long)((COMPRESSED_ETC2_RGB) | 0);  /* check that COMPRESSED_ETC2_RGB is an integer */
  return n;
}

static int _cffi_const_COMPRESSED_ETC2_EAC_RGBA(unsigned long long *o)
{
  int n = (COMPRESSED_ETC2_EAC_RGBA) <= 0;
  *o = (unsigned long long)((COMPRESSED_ETC2_EAC_RGBA) | 0);  /* check that COMPRESSED_ETC2_EAC_RGBA is an integer */
  return n;
}

static int _cffi_const_COMPRESSED_PVRT_RGB(unsigned long long *o)
{
  int n = (COMPRESSED_PVRT_RGB) <= 0;
  *o = (unsigned long long)((COMPRESSED_PVRT_RGB) | 0);  /* check that COMPRESSED_PVRT_RGB is an integer */
  return n;
}

static int _cffi_const_COMPRESSED_PVRT_RGBA(unsigned long long *o)
{
  int n = (COMPRESSED_PVRT_RGBA) <= 0;
  *o = (unsigned long long)((COMPRESSED_PVRT_RGBA) | 0);  /* check that COMPRESSED_PVRT_RGBA is an integer */
  return n;
}

static int _cffi_const_COMPRESSED_ASTC_4x4_RGBA(unsigned long long *o)
{
  int n = (COMPRESSED_ASTC_4x4_RGBA) <= 0;
  *o = (unsigned long long)((COMPRESSED_ASTC_4x4_RGBA) | 0);  /* check that COMPRESSED_ASTC_4x4_RGBA is an integer */
  return n;
}

static int _cffi_const_COMPRESSED_ASTC_8x8_RGBA(unsigned long long *o)
{
  int n = (COMPRESSED_ASTC_8x8_RGBA) <= 0;
  *o = (unsigned long long)((COMPRESSED_ASTC_8x8_RGBA) | 0);  /* check that COMPRESSED_ASTC_8x8_RGBA is an integer */
  return n;
}

static int _cffi_const_LOC_VERTEX_POSITION(unsigned long long *o)
{
  int n = (LOC_VERTEX_POSITION) <= 0;
  *o = (unsigned long long)((LOC_VERTEX_POSITION) | 0);  /* check that LOC_VERTEX_POSITION is an integer */
  return n;
}

static int _cffi_const_LOC_VERTEX_TEXCOORD01(unsigned long long *o)
{
  int n = (LOC_VERTEX_TEXCOORD01) <= 0;
  *o = (unsigned long long)((LOC_VERTEX_TEXCOORD01) | 0);  /* check that LOC_VERTEX_TEXCOORD01 is an integer */
  return n;
}

static int _cffi_const_LOC_VERTEX_TEXCOORD02(unsigned long long *o)
{
  int n = (LOC_VERTEX_TEXCOORD02) <= 0;
  *o = (unsigned long long)((LOC_VERTEX_TEXCOORD02) | 0);  /* check that LOC_VERTEX_TEXCOORD02 is an integer */
  return n;
}

static int _cffi_const_LOC_VERTEX_NORMAL(unsigned long long *o)
{
  int n = (LOC_VERTEX_NORMAL) <= 0;
  *o = (unsigned long long)((LOC_VERTEX_NORMAL) | 0);  /* check that LOC_VERTEX_NORMAL is an integer */
  return n;
}

static int _cffi_const_LOC_VERTEX_TANGENT(unsigned long long *o)
{
  int n = (LOC_VERTEX_TANGENT) <= 0;
  *o = (unsigned long long)((LOC_VERTEX_TANGENT) | 0);  /* check that LOC_VERTEX_TANGENT is an integer */
  return n;
}

static int _cffi_const_LOC_VERTEX_COLOR(unsigned long long *o)
{
  int n = (LOC_VERTEX_COLOR) <= 0;
  *o = (unsigned long long)((LOC_VERTEX_COLOR) | 0);  /* check that LOC_VERTEX_COLOR is an integer */
  return n;
}

static int _cffi_const_LOC_MATRIX_MVP(unsigned long long *o)
{
  int n = (LOC_MATRIX_MVP) <= 0;
  *o = (unsigned long long)((LOC_MATRIX_MVP) | 0);  /* check that LOC_MATRIX_MVP is an integer */
  return n;
}

static int _cffi_const_LOC_MATRIX_MODEL(unsigned long long *o)
{
  int n = (LOC_MATRIX_MODEL) <= 0;
  *o = (unsigned long long)((LOC_MATRIX_MODEL) | 0);  /* check that LOC_MATRIX_MODEL is an integer */
  return n;
}

static int _cffi_const_LOC_MATRIX_VIEW(unsigned long long *o)
{
  int n = (LOC_MATRIX_VIEW) <= 0;
  *o = (unsigned long long)((LOC_MATRIX_VIEW) | 0);  /* check that LOC_MATRIX_VIEW is an integer */
  return n;
}

static int _cffi_const_LOC_MATRIX_PROJECTION(unsigned long long *o)
{
  int n = (LOC_MATRIX_PROJECTION) <= 0;
  *o = (unsigned long long)((LOC_MATRIX_PROJECTION) | 0);  /* check that LOC_MATRIX_PROJECTION is an integer */
  return n;
}

static int _cffi_const_LOC_VECTOR_VIEW(unsigned long long *o)
{
  int n = (LOC_VECTOR_VIEW) <= 0;
  *o = (unsigned long long)((LOC_VECTOR_VIEW) | 0);  /* check that LOC_VECTOR_VIEW is an integer */
  return n;
}

static int _cffi_const_LOC_COLOR_DIFFUSE(unsigned long long *o)
{
  int n = (LOC_COLOR_DIFFUSE) <= 0;
  *o = (unsigned long long)((LOC_COLOR_DIFFUSE) | 0);  /* check that LOC_COLOR_DIFFUSE is an integer */
  return n;
}

static int _cffi_const_LOC_COLOR_SPECULAR(unsigned long long *o)
{
  int n = (LOC_COLOR_SPECULAR) <= 0;
  *o = (unsigned long long)((LOC_COLOR_SPECULAR) | 0);  /* check that LOC_COLOR_SPECULAR is an integer */
  return n;
}

static int _cffi_const_LOC_COLOR_AMBIENT(unsigned long long *o)
{
  int n = (LOC_COLOR_AMBIENT) <= 0;
  *o = (unsigned long long)((LOC_COLOR_AMBIENT) | 0);  /* check that LOC_COLOR_AMBIENT is an integer */
  return n;
}

static int _cffi_const_LOC_MAP_ALBEDO(unsigned long long *o)
{
  int n = (LOC_MAP_ALBEDO) <= 0;
  *o = (unsigned long long)((LOC_MAP_ALBEDO) | 0);  /* check that LOC_MAP_ALBEDO is an integer */
  return n;
}

static int _cffi_const_LOC_MAP_METALNESS(unsigned long long *o)
{
  int n = (LOC_MAP_METALNESS) <= 0;
  *o = (unsigned long long)((LOC_MAP_METALNESS) | 0);  /* check that LOC_MAP_METALNESS is an integer */
  return n;
}

static int _cffi_const_LOC_MAP_NORMAL(unsigned long long *o)
{
  int n = (LOC_MAP_NORMAL) <= 0;
  *o = (unsigned long long)((LOC_MAP_NORMAL) | 0);  /* check that LOC_MAP_NORMAL is an integer */
  return n;
}

static int _cffi_const_LOC_MAP_ROUGHNESS(unsigned long long *o)
{
  int n = (LOC_MAP_ROUGHNESS) <= 0;
  *o = (unsigned long long)((LOC_MAP_ROUGHNESS) | 0);  /* check that LOC_MAP_ROUGHNESS is an integer */
  return n;
}

static int _cffi_const_LOC_MAP_OCCLUSION(unsigned long long *o)
{
  int n = (LOC_MAP_OCCLUSION) <= 0;
  *o = (unsigned long long)((LOC_MAP_OCCLUSION) | 0);  /* check that LOC_MAP_OCCLUSION is an integer */
  return n;
}

static int _cffi_const_LOC_MAP_EMISSION(unsigned long long *o)
{
  int n = (LOC_MAP_EMISSION) <= 0;
  *o = (unsigned long long)((LOC_MAP_EMISSION) | 0);  /* check that LOC_MAP_EMISSION is an integer */
  return n;
}

static int _cffi_const_LOC_MAP_HEIGHT(unsigned long long *o)
{
  int n = (LOC_MAP_HEIGHT) <= 0;
  *o = (unsigned long long)((LOC_MAP_HEIGHT) | 0);  /* check that LOC_MAP_HEIGHT is an integer */
  return n;
}

static int _cffi_const_LOC_MAP_CUBEMAP(unsigned long long *o)
{
  int n = (LOC_MAP_CUBEMAP) <= 0;
  *o = (unsigned long long)((LOC_MAP_CUBEMAP) | 0);  /* check that LOC_MAP_CUBEMAP is an integer */
  return n;
}

static int _cffi_const_LOC_MAP_IRRADIANCE(unsigned long long *o)
{
  int n = (LOC_MAP_IRRADIANCE) <= 0;
  *o = (unsigned long long)((LOC_MAP_IRRADIANCE) | 0);  /* check that LOC_MAP_IRRADIANCE is an integer */
  return n;
}

static int _cffi_const_LOC_MAP_PREFILTER(unsigned long long *o)
{
  int n = (LOC_MAP_PREFILTER) <= 0;
  *o = (unsigned long long)((LOC_MAP_PREFILTER) | 0);  /* check that LOC_MAP_PREFILTER is an integer */
  return n;
}

static int _cffi_const_LOC_MAP_BRDF(unsigned long long *o)
{
  int n = (LOC_MAP_BRDF) <= 0;
  *o = (unsigned long long)((LOC_MAP_BRDF) | 0);  /* check that LOC_MAP_BRDF is an integer */
  return n;
}

static int _cffi_const_UNIFORM_FLOAT(unsigned long long *o)
{
  int n = (UNIFORM_FLOAT) <= 0;
  *o = (unsigned long long)((UNIFORM_FLOAT) | 0);  /* check that UNIFORM_FLOAT is an integer */
  return n;
}

static int _cffi_const_UNIFORM_VEC2(unsigned long long *o)
{
  int n = (UNIFORM_VEC2) <= 0;
  *o = (unsigned long long)((UNIFORM_VEC2) | 0);  /* check that UNIFORM_VEC2 is an integer */
  return n;
}

static int _cffi_const_UNIFORM_VEC3(unsigned long long *o)
{
  int n = (UNIFORM_VEC3) <= 0;
  *o = (unsigned long long)((UNIFORM_VEC3) | 0);  /* check that UNIFORM_VEC3 is an integer */
  return n;
}

static int _cffi_const_UNIFORM_VEC4(unsigned long long *o)
{
  int n = (UNIFORM_VEC4) <= 0;
  *o = (unsigned long long)((UNIFORM_VEC4) | 0);  /* check that UNIFORM_VEC4 is an integer */
  return n;
}

static int _cffi_const_UNIFORM_INT(unsigned long long *o)
{
  int n = (UNIFORM_INT) <= 0;
  *o = (unsigned long long)((UNIFORM_INT) | 0);  /* check that UNIFORM_INT is an integer */
  return n;
}

static int _cffi_const_UNIFORM_IVEC2(unsigned long long *o)
{
  int n = (UNIFORM_IVEC2) <= 0;
  *o = (unsigned long long)((UNIFORM_IVEC2) | 0);  /* check that UNIFORM_IVEC2 is an integer */
  return n;
}

static int _cffi_const_UNIFORM_IVEC3(unsigned long long *o)
{
  int n = (UNIFORM_IVEC3) <= 0;
  *o = (unsigned long long)((UNIFORM_IVEC3) | 0);  /* check that UNIFORM_IVEC3 is an integer */
  return n;
}

static int _cffi_const_UNIFORM_IVEC4(unsigned long long *o)
{
  int n = (UNIFORM_IVEC4) <= 0;
  *o = (unsigned long long)((UNIFORM_IVEC4) | 0);  /* check that UNIFORM_IVEC4 is an integer */
  return n;
}

static int _cffi_const_UNIFORM_SAMPLER2D(unsigned long long *o)
{
  int n = (UNIFORM_SAMPLER2D) <= 0;
  *o = (unsigned long long)((UNIFORM_SAMPLER2D) | 0);  /* check that UNIFORM_SAMPLER2D is an integer */
  return n;
}

static int _cffi_const_FILTER_POINT(unsigned long long *o)
{
  int n = (FILTER_POINT) <= 0;
  *o = (unsigned long long)((FILTER_POINT) | 0);  /* check that FILTER_POINT is an integer */
  return n;
}

static int _cffi_const_FILTER_BILINEAR(unsigned long long *o)
{
  int n = (FILTER_BILINEAR) <= 0;
  *o = (unsigned long long)((FILTER_BILINEAR) | 0);  /* check that FILTER_BILINEAR is an integer */
  return n;
}

static int _cffi_const_FILTER_TRILINEAR(unsigned long long *o)
{
  int n = (FILTER_TRILINEAR) <= 0;
  *o = (unsigned long long)((FILTER_TRILINEAR) | 0);  /* check that FILTER_TRILINEAR is an integer */
  return n;
}

static int _cffi_const_FILTER_ANISOTROPIC_4X(unsigned long long *o)
{
  int n = (FILTER_ANISOTROPIC_4X) <= 0;
  *o = (unsigned long long)((FILTER_ANISOTROPIC_4X) | 0);  /* check that FILTER_ANISOTROPIC_4X is an integer */
  return n;
}

static int _cffi_const_FILTER_ANISOTROPIC_8X(unsigned long long *o)
{
  int n = (FILTER_ANISOTROPIC_8X) <= 0;
  *o = (unsigned long long)((FILTER_ANISOTROPIC_8X) | 0);  /* check that FILTER_ANISOTROPIC_8X is an integer */
  return n;
}

static int _cffi_const_FILTER_ANISOTROPIC_16X(unsigned long long *o)
{
  int n = (FILTER_ANISOTROPIC_16X) <= 0;
  *o = (unsigned long long)((FILTER_ANISOTROPIC_16X) | 0);  /* check that FILTER_ANISOTROPIC_16X is an integer */
  return n;
}

static int _cffi_const_WRAP_REPEAT(unsigned long long *o)
{
  int n = (WRAP_REPEAT) <= 0;
  *o = (unsigned long long)((WRAP_REPEAT) | 0);  /* check that WRAP_REPEAT is an integer */
  return n;
}

static int _cffi_const_WRAP_CLAMP(unsigned long long *o)
{
  int n = (WRAP_CLAMP) <= 0;
  *o = (unsigned long long)((WRAP_CLAMP) | 0);  /* check that WRAP_CLAMP is an integer */
  return n;
}

static int _cffi_const_WRAP_MIRROR_REPEAT(unsigned long long *o)
{
  int n = (WRAP_MIRROR_REPEAT) <= 0;
  *o = (unsigned long long)((WRAP_MIRROR_REPEAT) | 0);  /* check that WRAP_MIRROR_REPEAT is an integer */
  return n;
}

static int _cffi_const_WRAP_MIRROR_CLAMP(unsigned long long *o)
{
  int n = (WRAP_MIRROR_CLAMP) <= 0;
  *o = (unsigned long long)((WRAP_MIRROR_CLAMP) | 0);  /* check that WRAP_MIRROR_CLAMP is an integer */
  return n;
}

static int _cffi_const_LOG_ALL(unsigned long long *o)
{
  int n = (LOG_ALL) <= 0;
  *o = (unsigned long long)((LOG_ALL) | 0);  /* check that LOG_ALL is an integer */
  return n;
}

static int _cffi_const_LOG_TRACE(unsigned long long *o)
{
  int n = (LOG_TRACE) <= 0;
  *o = (unsigned long long)((LOG_TRACE) | 0);  /* check that LOG_TRACE is an integer */
  return n;
}

static int _cffi_const_LOG_DEBUG(unsigned long long *o)
{
  int n = (LOG_DEBUG) <= 0;
  *o = (unsigned long long)((LOG_DEBUG) | 0);  /* check that LOG_DEBUG is an integer */
  return n;
}

static int _cffi_const_LOG_INFO(unsigned long long *o)
{
  int n = (LOG_INFO) <= 0;
  *o = (unsigned long long)((LOG_INFO) | 0);  /* check that LOG_INFO is an integer */
  return n;
}

static int _cffi_const_LOG_WARNING(unsigned long long *o)
{
  int n = (LOG_WARNING) <= 0;
  *o = (unsigned long long)((LOG_WARNING) | 0);  /* check that LOG_WARNING is an integer */
  return n;
}

static int _cffi_const_LOG_ERROR(unsigned long long *o)
{
  int n = (LOG_ERROR) <= 0;
  *o = (unsigned long long)((LOG_ERROR) | 0);  /* check that LOG_ERROR is an integer */
  return n;
}

static int _cffi_const_LOG_FATAL(unsigned long long *o)
{
  int n = (LOG_FATAL) <= 0;
  *o = (unsigned long long)((LOG_FATAL) | 0);  /* check that LOG_FATAL is an integer */
  return n;
}

static int _cffi_const_LOG_NONE(unsigned long long *o)
{
  int n = (LOG_NONE) <= 0;
  *o = (unsigned long long)((LOG_NONE) | 0);  /* check that LOG_NONE is an integer */
  return n;
}

static void _cffi_d_BeginBlendMode(int x0)
{
  BeginBlendMode(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_BeginBlendMode(PyObject *self, PyObject *arg0)
{
  int x0;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { BeginBlendMode(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_BeginBlendMode _cffi_d_BeginBlendMode
#endif

static void _cffi_d_BeginDrawing(void)
{
  BeginDrawing();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_BeginDrawing(PyObject *self, PyObject *noarg)
{

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { BeginDrawing(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_BeginDrawing _cffi_d_BeginDrawing
#endif

static void _cffi_d_BeginMode2D(Camera2D x0)
{
  BeginMode2D(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_BeginMode2D(PyObject *self, PyObject *arg0)
{
  Camera2D x0;

  if (_cffi_to_c((char *)&x0, _cffi_type(155), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { BeginMode2D(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_BeginMode2D(Camera2D *x0)
{
  { BeginMode2D(*x0); }
}
#endif

static void _cffi_d_BeginMode3D(Camera3D x0)
{
  BeginMode3D(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_BeginMode3D(PyObject *self, PyObject *arg0)
{
  Camera3D x0;

  if (_cffi_to_c((char *)&x0, _cffi_type(158), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { BeginMode3D(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_BeginMode3D(Camera3D *x0)
{
  { BeginMode3D(*x0); }
}
#endif

static void _cffi_d_BeginScissorMode(int x0, int x1, int x2, int x3)
{
  BeginScissorMode(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_BeginScissorMode(PyObject *self, PyObject *args)
{
  int x0;
  int x1;
  int x2;
  int x3;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "BeginScissorMode", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { BeginScissorMode(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_BeginScissorMode _cffi_d_BeginScissorMode
#endif

static void _cffi_d_BeginShaderMode(Shader x0)
{
  BeginShaderMode(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_BeginShaderMode(PyObject *self, PyObject *arg0)
{
  Shader x0;

  if (_cffi_to_c((char *)&x0, _cffi_type(258), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { BeginShaderMode(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_BeginShaderMode(Shader *x0)
{
  { BeginShaderMode(*x0); }
}
#endif

static void _cffi_d_BeginTextureMode(RenderTexture2D x0)
{
  BeginTextureMode(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_BeginTextureMode(PyObject *self, PyObject *arg0)
{
  RenderTexture2D x0;

  if (_cffi_to_c((char *)&x0, _cffi_type(800), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { BeginTextureMode(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_BeginTextureMode(RenderTexture2D *x0)
{
  { BeginTextureMode(*x0); }
}
#endif

static void _cffi_d_BeginVrDrawing(void)
{
  BeginVrDrawing();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_BeginVrDrawing(PyObject *self, PyObject *noarg)
{

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { BeginVrDrawing(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_BeginVrDrawing _cffi_d_BeginVrDrawing
#endif

static int _cffi_d_ChangeDirectory(char const * x0)
{
  return ChangeDirectory(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ChangeDirectory(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ChangeDirectory(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ChangeDirectory _cffi_d_ChangeDirectory
#endif

static int _cffi_d_CheckCollisionBoxSphere(BoundingBox x0, Vector3 x1, float x2)
{
  return CheckCollisionBoxSphere(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_CheckCollisionBoxSphere(PyObject *self, PyObject *args)
{
  BoundingBox x0;
  Vector3 x1;
  float x2;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "CheckCollisionBoxSphere", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(387), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(28), arg1) < 0)
    return NULL;

  x2 = (float)_cffi_to_c_float(arg2);
  if (x2 == (float)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = CheckCollisionBoxSphere(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
static int _cffi_f_CheckCollisionBoxSphere(BoundingBox *x0, Vector3 *x1, float x2)
{
  int result;
  { result = CheckCollisionBoxSphere(*x0, *x1, x2); }
  return result;
}
#endif

static int _cffi_d_CheckCollisionBoxes(BoundingBox x0, BoundingBox x1)
{
  return CheckCollisionBoxes(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_CheckCollisionBoxes(PyObject *self, PyObject *args)
{
  BoundingBox x0;
  BoundingBox x1;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "CheckCollisionBoxes", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(387), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(387), arg1) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = CheckCollisionBoxes(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
static int _cffi_f_CheckCollisionBoxes(BoundingBox *x0, BoundingBox *x1)
{
  int result;
  { result = CheckCollisionBoxes(*x0, *x1); }
  return result;
}
#endif

static int _cffi_d_CheckCollisionCircleRec(Vector2 x0, float x1, Rectangle x2)
{
  return CheckCollisionCircleRec(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_CheckCollisionCircleRec(PyObject *self, PyObject *args)
{
  Vector2 x0;
  float x1;
  Rectangle x2;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "CheckCollisionCircleRec", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(209), arg0) < 0)
    return NULL;

  x1 = (float)_cffi_to_c_float(arg1);
  if (x1 == (float)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(77), arg2) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = CheckCollisionCircleRec(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
static int _cffi_f_CheckCollisionCircleRec(Vector2 *x0, float x1, Rectangle *x2)
{
  int result;
  { result = CheckCollisionCircleRec(*x0, x1, *x2); }
  return result;
}
#endif

static int _cffi_d_CheckCollisionCircles(Vector2 x0, float x1, Vector2 x2, float x3)
{
  return CheckCollisionCircles(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_CheckCollisionCircles(PyObject *self, PyObject *args)
{
  Vector2 x0;
  float x1;
  Vector2 x2;
  float x3;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "CheckCollisionCircles", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(209), arg0) < 0)
    return NULL;

  x1 = (float)_cffi_to_c_float(arg1);
  if (x1 == (float)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(209), arg2) < 0)
    return NULL;

  x3 = (float)_cffi_to_c_float(arg3);
  if (x3 == (float)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = CheckCollisionCircles(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
static int _cffi_f_CheckCollisionCircles(Vector2 *x0, float x1, Vector2 *x2, float x3)
{
  int result;
  { result = CheckCollisionCircles(*x0, x1, *x2, x3); }
  return result;
}
#endif

static int _cffi_d_CheckCollisionPointCircle(Vector2 x0, Vector2 x1, float x2)
{
  return CheckCollisionPointCircle(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_CheckCollisionPointCircle(PyObject *self, PyObject *args)
{
  Vector2 x0;
  Vector2 x1;
  float x2;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "CheckCollisionPointCircle", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(209), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(209), arg1) < 0)
    return NULL;

  x2 = (float)_cffi_to_c_float(arg2);
  if (x2 == (float)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = CheckCollisionPointCircle(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
static int _cffi_f_CheckCollisionPointCircle(Vector2 *x0, Vector2 *x1, float x2)
{
  int result;
  { result = CheckCollisionPointCircle(*x0, *x1, x2); }
  return result;
}
#endif

static int _cffi_d_CheckCollisionPointRec(Vector2 x0, Rectangle x1)
{
  return CheckCollisionPointRec(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_CheckCollisionPointRec(PyObject *self, PyObject *args)
{
  Vector2 x0;
  Rectangle x1;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "CheckCollisionPointRec", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(209), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(77), arg1) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = CheckCollisionPointRec(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
static int _cffi_f_CheckCollisionPointRec(Vector2 *x0, Rectangle *x1)
{
  int result;
  { result = CheckCollisionPointRec(*x0, *x1); }
  return result;
}
#endif

static int _cffi_d_CheckCollisionPointTriangle(Vector2 x0, Vector2 x1, Vector2 x2, Vector2 x3)
{
  return CheckCollisionPointTriangle(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_CheckCollisionPointTriangle(PyObject *self, PyObject *args)
{
  Vector2 x0;
  Vector2 x1;
  Vector2 x2;
  Vector2 x3;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "CheckCollisionPointTriangle", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(209), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(209), arg1) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(209), arg2) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x3, _cffi_type(209), arg3) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = CheckCollisionPointTriangle(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
static int _cffi_f_CheckCollisionPointTriangle(Vector2 *x0, Vector2 *x1, Vector2 *x2, Vector2 *x3)
{
  int result;
  { result = CheckCollisionPointTriangle(*x0, *x1, *x2, *x3); }
  return result;
}
#endif

static int _cffi_d_CheckCollisionRayBox(Ray x0, BoundingBox x1)
{
  return CheckCollisionRayBox(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_CheckCollisionRayBox(PyObject *self, PyObject *args)
{
  Ray x0;
  BoundingBox x1;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "CheckCollisionRayBox", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(213), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(387), arg1) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = CheckCollisionRayBox(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
static int _cffi_f_CheckCollisionRayBox(Ray *x0, BoundingBox *x1)
{
  int result;
  { result = CheckCollisionRayBox(*x0, *x1); }
  return result;
}
#endif

static int _cffi_d_CheckCollisionRaySphere(Ray x0, Vector3 x1, float x2)
{
  return CheckCollisionRaySphere(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_CheckCollisionRaySphere(PyObject *self, PyObject *args)
{
  Ray x0;
  Vector3 x1;
  float x2;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "CheckCollisionRaySphere", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(213), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(28), arg1) < 0)
    return NULL;

  x2 = (float)_cffi_to_c_float(arg2);
  if (x2 == (float)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = CheckCollisionRaySphere(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
static int _cffi_f_CheckCollisionRaySphere(Ray *x0, Vector3 *x1, float x2)
{
  int result;
  { result = CheckCollisionRaySphere(*x0, *x1, x2); }
  return result;
}
#endif

static int _cffi_d_CheckCollisionRaySphereEx(Ray x0, Vector3 x1, float x2, Vector3 * x3)
{
  return CheckCollisionRaySphereEx(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_CheckCollisionRaySphereEx(PyObject *self, PyObject *args)
{
  Ray x0;
  Vector3 x1;
  float x2;
  Vector3 * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "CheckCollisionRaySphereEx", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(213), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(28), arg1) < 0)
    return NULL;

  x2 = (float)_cffi_to_c_float(arg2);
  if (x2 == (float)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(422), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (Vector3 *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(422), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = CheckCollisionRaySphereEx(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
static int _cffi_f_CheckCollisionRaySphereEx(Ray *x0, Vector3 *x1, float x2, Vector3 * x3)
{
  int result;
  { result = CheckCollisionRaySphereEx(*x0, *x1, x2, x3); }
  return result;
}
#endif

static int _cffi_d_CheckCollisionRecs(Rectangle x0, Rectangle x1)
{
  return CheckCollisionRecs(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_CheckCollisionRecs(PyObject *self, PyObject *args)
{
  Rectangle x0;
  Rectangle x1;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "CheckCollisionRecs", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(77), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(77), arg1) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = CheckCollisionRecs(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
static int _cffi_f_CheckCollisionRecs(Rectangle *x0, Rectangle *x1)
{
  int result;
  { result = CheckCollisionRecs(*x0, *x1); }
  return result;
}
#endif

static int _cffi_d_CheckCollisionSpheres(Vector3 x0, float x1, Vector3 x2, float x3)
{
  return CheckCollisionSpheres(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_CheckCollisionSpheres(PyObject *self, PyObject *args)
{
  Vector3 x0;
  float x1;
  Vector3 x2;
  float x3;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "CheckCollisionSpheres", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(28), arg0) < 0)
    return NULL;

  x1 = (float)_cffi_to_c_float(arg1);
  if (x1 == (float)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(28), arg2) < 0)
    return NULL;

  x3 = (float)_cffi_to_c_float(arg3);
  if (x3 == (float)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = CheckCollisionSpheres(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
static int _cffi_f_CheckCollisionSpheres(Vector3 *x0, float x1, Vector3 *x2, float x3)
{
  int result;
  { result = CheckCollisionSpheres(*x0, x1, *x2, x3); }
  return result;
}
#endif

static void _cffi_d_ClearBackground(Color x0)
{
  ClearBackground(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ClearBackground(PyObject *self, PyObject *arg0)
{
  Color x0;

  if (_cffi_to_c((char *)&x0, _cffi_type(24), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ClearBackground(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_ClearBackground(Color *x0)
{
  { ClearBackground(*x0); }
}
#endif

static void _cffi_d_ClearDirectoryFiles(void)
{
  ClearDirectoryFiles();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ClearDirectoryFiles(PyObject *self, PyObject *noarg)
{

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ClearDirectoryFiles(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ClearDirectoryFiles _cffi_d_ClearDirectoryFiles
#endif

static void _cffi_d_ClearDroppedFiles(void)
{
  ClearDroppedFiles();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ClearDroppedFiles(PyObject *self, PyObject *noarg)
{

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ClearDroppedFiles(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ClearDroppedFiles _cffi_d_ClearDroppedFiles
#endif

static void _cffi_d_CloseAudioDevice(void)
{
  CloseAudioDevice();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_CloseAudioDevice(PyObject *self, PyObject *noarg)
{

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { CloseAudioDevice(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_CloseAudioDevice _cffi_d_CloseAudioDevice
#endif

static void _cffi_d_CloseAudioStream(AudioStream x0)
{
  CloseAudioStream(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_CloseAudioStream(PyObject *self, PyObject *arg0)
{
  AudioStream x0;

  if (_cffi_to_c((char *)&x0, _cffi_type(384), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { CloseAudioStream(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_CloseAudioStream(AudioStream *x0)
{
  { CloseAudioStream(*x0); }
}
#endif

static void _cffi_d_CloseVrSimulator(void)
{
  CloseVrSimulator();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_CloseVrSimulator(PyObject *self, PyObject *noarg)
{

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { CloseVrSimulator(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_CloseVrSimulator _cffi_d_CloseVrSimulator
#endif

static void _cffi_d_CloseWindow(void)
{
  CloseWindow();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_CloseWindow(PyObject *self, PyObject *noarg)
{

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { CloseWindow(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_CloseWindow _cffi_d_CloseWindow
#endif

static char const * _cffi_d_CodepointToUtf8(int x0, int * x1)
{
  return CodepointToUtf8(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_CodepointToUtf8(PyObject *self, PyObject *args)
{
  int x0;
  int * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char const * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "CodepointToUtf8", 2, 2, &arg0, &arg1))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(11), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(11), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = CodepointToUtf8(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(9));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_CodepointToUtf8 _cffi_d_CodepointToUtf8
#endif

static Color _cffi_d_ColorFromHSV(Vector3 x0)
{
  return ColorFromHSV(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ColorFromHSV(PyObject *self, PyObject *arg0)
{
  Vector3 x0;
  Color result;
  PyObject *pyresult;

  if (_cffi_to_c((char *)&x0, _cffi_type(28), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ColorFromHSV(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(24));
  return pyresult;
}
#else
static void _cffi_f_ColorFromHSV(Color *result, Vector3 *x0)
{
  { *result = ColorFromHSV(*x0); }
}
#endif

static Color _cffi_d_ColorFromNormalized(Vector4 x0)
{
  return ColorFromNormalized(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ColorFromNormalized(PyObject *self, PyObject *arg0)
{
  Vector4 x0;
  Color result;
  PyObject *pyresult;

  if (_cffi_to_c((char *)&x0, _cffi_type(31), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ColorFromNormalized(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(24));
  return pyresult;
}
#else
static void _cffi_f_ColorFromNormalized(Color *result, Vector4 *x0)
{
  { *result = ColorFromNormalized(*x0); }
}
#endif

static Vector4 _cffi_d_ColorNormalize(Color x0)
{
  return ColorNormalize(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ColorNormalize(PyObject *self, PyObject *arg0)
{
  Color x0;
  Vector4 result;
  PyObject *pyresult;

  if (_cffi_to_c((char *)&x0, _cffi_type(24), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ColorNormalize(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(31));
  return pyresult;
}
#else
static void _cffi_f_ColorNormalize(Vector4 *result, Color *x0)
{
  { *result = ColorNormalize(*x0); }
}
#endif

static Vector3 _cffi_d_ColorToHSV(Color x0)
{
  return ColorToHSV(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ColorToHSV(PyObject *self, PyObject *arg0)
{
  Color x0;
  Vector3 result;
  PyObject *pyresult;

  if (_cffi_to_c((char *)&x0, _cffi_type(24), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ColorToHSV(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(28));
  return pyresult;
}
#else
static void _cffi_f_ColorToHSV(Vector3 *result, Color *x0)
{
  { *result = ColorToHSV(*x0); }
}
#endif

static int _cffi_d_ColorToInt(Color x0)
{
  return ColorToInt(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ColorToInt(PyObject *self, PyObject *arg0)
{
  Color x0;
  int result;
  PyObject *pyresult;

  if (_cffi_to_c((char *)&x0, _cffi_type(24), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ColorToInt(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
static int _cffi_f_ColorToInt(Color *x0)
{
  int result;
  { result = ColorToInt(*x0); }
  return result;
}
#endif

static unsigned char * _cffi_d_CompressData(unsigned char * x0, int x1, int * x2)
{
  return CompressData(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_CompressData(PyObject *self, PyObject *args)
{
  unsigned char * x0;
  int x1;
  int * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  unsigned char * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "CompressData", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(508), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (unsigned char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(508), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(11), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(11), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = CompressData(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(508));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_CompressData _cffi_d_CompressData
#endif

static unsigned char * _cffi_d_DecompressData(unsigned char * x0, int x1, int * x2)
{
  return DecompressData(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DecompressData(PyObject *self, PyObject *args)
{
  unsigned char * x0;
  int x1;
  int * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  unsigned char * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "DecompressData", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(508), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (unsigned char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(508), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(11), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(11), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = DecompressData(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(508));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_DecompressData _cffi_d_DecompressData
#endif

static int _cffi_d_DirectoryExists(char const * x0)
{
  return DirectoryExists(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DirectoryExists(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = DirectoryExists(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_DirectoryExists _cffi_d_DirectoryExists
#endif

static void _cffi_d_DisableCursor(void)
{
  DisableCursor();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DisableCursor(PyObject *self, PyObject *noarg)
{

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DisableCursor(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_DisableCursor _cffi_d_DisableCursor
#endif

static void _cffi_d_DrawBillboard(Camera3D x0, Texture2D x1, Vector3 x2, float x3, Color x4)
{
  DrawBillboard(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawBillboard(PyObject *self, PyObject *args)
{
  Camera3D x0;
  Texture2D x1;
  Vector3 x2;
  float x3;
  Color x4;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "DrawBillboard", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(158), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(80), arg1) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(28), arg2) < 0)
    return NULL;

  x3 = (float)_cffi_to_c_float(arg3);
  if (x3 == (float)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x4, _cffi_type(24), arg4) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawBillboard(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawBillboard(Camera3D *x0, Texture2D *x1, Vector3 *x2, float x3, Color *x4)
{
  { DrawBillboard(*x0, *x1, *x2, x3, *x4); }
}
#endif

static void _cffi_d_DrawBillboardRec(Camera3D x0, Texture2D x1, Rectangle x2, Vector3 x3, float x4, Color x5)
{
  DrawBillboardRec(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawBillboardRec(PyObject *self, PyObject *args)
{
  Camera3D x0;
  Texture2D x1;
  Rectangle x2;
  Vector3 x3;
  float x4;
  Color x5;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "DrawBillboardRec", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(158), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(80), arg1) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(77), arg2) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x3, _cffi_type(28), arg3) < 0)
    return NULL;

  x4 = (float)_cffi_to_c_float(arg4);
  if (x4 == (float)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x5, _cffi_type(24), arg5) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawBillboardRec(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawBillboardRec(Camera3D *x0, Texture2D *x1, Rectangle *x2, Vector3 *x3, float x4, Color *x5)
{
  { DrawBillboardRec(*x0, *x1, *x2, *x3, x4, *x5); }
}
#endif

static void _cffi_d_DrawBoundingBox(BoundingBox x0, Color x1)
{
  DrawBoundingBox(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawBoundingBox(PyObject *self, PyObject *args)
{
  BoundingBox x0;
  Color x1;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "DrawBoundingBox", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(387), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(24), arg1) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawBoundingBox(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawBoundingBox(BoundingBox *x0, Color *x1)
{
  { DrawBoundingBox(*x0, *x1); }
}
#endif

static void _cffi_d_DrawCircle(int x0, int x1, float x2, Color x3)
{
  DrawCircle(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawCircle(PyObject *self, PyObject *args)
{
  int x0;
  int x1;
  float x2;
  Color x3;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "DrawCircle", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = (float)_cffi_to_c_float(arg2);
  if (x2 == (float)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x3, _cffi_type(24), arg3) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawCircle(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawCircle(int x0, int x1, float x2, Color *x3)
{
  { DrawCircle(x0, x1, x2, *x3); }
}
#endif

static void _cffi_d_DrawCircle3D(Vector3 x0, float x1, Vector3 x2, float x3, Color x4)
{
  DrawCircle3D(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawCircle3D(PyObject *self, PyObject *args)
{
  Vector3 x0;
  float x1;
  Vector3 x2;
  float x3;
  Color x4;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "DrawCircle3D", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(28), arg0) < 0)
    return NULL;

  x1 = (float)_cffi_to_c_float(arg1);
  if (x1 == (float)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(28), arg2) < 0)
    return NULL;

  x3 = (float)_cffi_to_c_float(arg3);
  if (x3 == (float)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x4, _cffi_type(24), arg4) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawCircle3D(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawCircle3D(Vector3 *x0, float x1, Vector3 *x2, float x3, Color *x4)
{
  { DrawCircle3D(*x0, x1, *x2, x3, *x4); }
}
#endif

static void _cffi_d_DrawCircleGradient(int x0, int x1, float x2, Color x3, Color x4)
{
  DrawCircleGradient(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawCircleGradient(PyObject *self, PyObject *args)
{
  int x0;
  int x1;
  float x2;
  Color x3;
  Color x4;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "DrawCircleGradient", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = (float)_cffi_to_c_float(arg2);
  if (x2 == (float)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x3, _cffi_type(24), arg3) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x4, _cffi_type(24), arg4) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawCircleGradient(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawCircleGradient(int x0, int x1, float x2, Color *x3, Color *x4)
{
  { DrawCircleGradient(x0, x1, x2, *x3, *x4); }
}
#endif

static void _cffi_d_DrawCircleLines(int x0, int x1, float x2, Color x3)
{
  DrawCircleLines(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawCircleLines(PyObject *self, PyObject *args)
{
  int x0;
  int x1;
  float x2;
  Color x3;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "DrawCircleLines", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = (float)_cffi_to_c_float(arg2);
  if (x2 == (float)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x3, _cffi_type(24), arg3) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawCircleLines(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawCircleLines(int x0, int x1, float x2, Color *x3)
{
  { DrawCircleLines(x0, x1, x2, *x3); }
}
#endif

static void _cffi_d_DrawCircleSector(Vector2 x0, float x1, int x2, int x3, int x4, Color x5)
{
  DrawCircleSector(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawCircleSector(PyObject *self, PyObject *args)
{
  Vector2 x0;
  float x1;
  int x2;
  int x3;
  int x4;
  Color x5;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "DrawCircleSector", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(209), arg0) < 0)
    return NULL;

  x1 = (float)_cffi_to_c_float(arg1);
  if (x1 == (float)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  x4 = _cffi_to_c_int(arg4, int);
  if (x4 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x5, _cffi_type(24), arg5) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawCircleSector(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawCircleSector(Vector2 *x0, float x1, int x2, int x3, int x4, Color *x5)
{
  { DrawCircleSector(*x0, x1, x2, x3, x4, *x5); }
}
#endif

static void _cffi_d_DrawCircleSectorLines(Vector2 x0, float x1, int x2, int x3, int x4, Color x5)
{
  DrawCircleSectorLines(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawCircleSectorLines(PyObject *self, PyObject *args)
{
  Vector2 x0;
  float x1;
  int x2;
  int x3;
  int x4;
  Color x5;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "DrawCircleSectorLines", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(209), arg0) < 0)
    return NULL;

  x1 = (float)_cffi_to_c_float(arg1);
  if (x1 == (float)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  x4 = _cffi_to_c_int(arg4, int);
  if (x4 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x5, _cffi_type(24), arg5) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawCircleSectorLines(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawCircleSectorLines(Vector2 *x0, float x1, int x2, int x3, int x4, Color *x5)
{
  { DrawCircleSectorLines(*x0, x1, x2, x3, x4, *x5); }
}
#endif

static void _cffi_d_DrawCircleV(Vector2 x0, float x1, Color x2)
{
  DrawCircleV(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawCircleV(PyObject *self, PyObject *args)
{
  Vector2 x0;
  float x1;
  Color x2;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "DrawCircleV", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(209), arg0) < 0)
    return NULL;

  x1 = (float)_cffi_to_c_float(arg1);
  if (x1 == (float)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(24), arg2) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawCircleV(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawCircleV(Vector2 *x0, float x1, Color *x2)
{
  { DrawCircleV(*x0, x1, *x2); }
}
#endif

static void _cffi_d_DrawCube(Vector3 x0, float x1, float x2, float x3, Color x4)
{
  DrawCube(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawCube(PyObject *self, PyObject *args)
{
  Vector3 x0;
  float x1;
  float x2;
  float x3;
  Color x4;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "DrawCube", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(28), arg0) < 0)
    return NULL;

  x1 = (float)_cffi_to_c_float(arg1);
  if (x1 == (float)-1 && PyErr_Occurred())
    return NULL;

  x2 = (float)_cffi_to_c_float(arg2);
  if (x2 == (float)-1 && PyErr_Occurred())
    return NULL;

  x3 = (float)_cffi_to_c_float(arg3);
  if (x3 == (float)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x4, _cffi_type(24), arg4) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawCube(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawCube(Vector3 *x0, float x1, float x2, float x3, Color *x4)
{
  { DrawCube(*x0, x1, x2, x3, *x4); }
}
#endif

static void _cffi_d_DrawCubeTexture(Texture2D x0, Vector3 x1, float x2, float x3, float x4, Color x5)
{
  DrawCubeTexture(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawCubeTexture(PyObject *self, PyObject *args)
{
  Texture2D x0;
  Vector3 x1;
  float x2;
  float x3;
  float x4;
  Color x5;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "DrawCubeTexture", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(80), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(28), arg1) < 0)
    return NULL;

  x2 = (float)_cffi_to_c_float(arg2);
  if (x2 == (float)-1 && PyErr_Occurred())
    return NULL;

  x3 = (float)_cffi_to_c_float(arg3);
  if (x3 == (float)-1 && PyErr_Occurred())
    return NULL;

  x4 = (float)_cffi_to_c_float(arg4);
  if (x4 == (float)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x5, _cffi_type(24), arg5) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawCubeTexture(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawCubeTexture(Texture2D *x0, Vector3 *x1, float x2, float x3, float x4, Color *x5)
{
  { DrawCubeTexture(*x0, *x1, x2, x3, x4, *x5); }
}
#endif

static void _cffi_d_DrawCubeV(Vector3 x0, Vector3 x1, Color x2)
{
  DrawCubeV(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawCubeV(PyObject *self, PyObject *args)
{
  Vector3 x0;
  Vector3 x1;
  Color x2;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "DrawCubeV", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(28), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(28), arg1) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(24), arg2) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawCubeV(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawCubeV(Vector3 *x0, Vector3 *x1, Color *x2)
{
  { DrawCubeV(*x0, *x1, *x2); }
}
#endif

static void _cffi_d_DrawCubeWires(Vector3 x0, float x1, float x2, float x3, Color x4)
{
  DrawCubeWires(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawCubeWires(PyObject *self, PyObject *args)
{
  Vector3 x0;
  float x1;
  float x2;
  float x3;
  Color x4;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "DrawCubeWires", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(28), arg0) < 0)
    return NULL;

  x1 = (float)_cffi_to_c_float(arg1);
  if (x1 == (float)-1 && PyErr_Occurred())
    return NULL;

  x2 = (float)_cffi_to_c_float(arg2);
  if (x2 == (float)-1 && PyErr_Occurred())
    return NULL;

  x3 = (float)_cffi_to_c_float(arg3);
  if (x3 == (float)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x4, _cffi_type(24), arg4) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawCubeWires(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawCubeWires(Vector3 *x0, float x1, float x2, float x3, Color *x4)
{
  { DrawCubeWires(*x0, x1, x2, x3, *x4); }
}
#endif

static void _cffi_d_DrawCubeWiresV(Vector3 x0, Vector3 x1, Color x2)
{
  DrawCubeWiresV(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawCubeWiresV(PyObject *self, PyObject *args)
{
  Vector3 x0;
  Vector3 x1;
  Color x2;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "DrawCubeWiresV", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(28), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(28), arg1) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(24), arg2) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawCubeWiresV(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawCubeWiresV(Vector3 *x0, Vector3 *x1, Color *x2)
{
  { DrawCubeWiresV(*x0, *x1, *x2); }
}
#endif

static void _cffi_d_DrawCylinder(Vector3 x0, float x1, float x2, float x3, int x4, Color x5)
{
  DrawCylinder(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawCylinder(PyObject *self, PyObject *args)
{
  Vector3 x0;
  float x1;
  float x2;
  float x3;
  int x4;
  Color x5;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "DrawCylinder", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(28), arg0) < 0)
    return NULL;

  x1 = (float)_cffi_to_c_float(arg1);
  if (x1 == (float)-1 && PyErr_Occurred())
    return NULL;

  x2 = (float)_cffi_to_c_float(arg2);
  if (x2 == (float)-1 && PyErr_Occurred())
    return NULL;

  x3 = (float)_cffi_to_c_float(arg3);
  if (x3 == (float)-1 && PyErr_Occurred())
    return NULL;

  x4 = _cffi_to_c_int(arg4, int);
  if (x4 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x5, _cffi_type(24), arg5) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawCylinder(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawCylinder(Vector3 *x0, float x1, float x2, float x3, int x4, Color *x5)
{
  { DrawCylinder(*x0, x1, x2, x3, x4, *x5); }
}
#endif

static void _cffi_d_DrawCylinderWires(Vector3 x0, float x1, float x2, float x3, int x4, Color x5)
{
  DrawCylinderWires(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawCylinderWires(PyObject *self, PyObject *args)
{
  Vector3 x0;
  float x1;
  float x2;
  float x3;
  int x4;
  Color x5;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "DrawCylinderWires", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(28), arg0) < 0)
    return NULL;

  x1 = (float)_cffi_to_c_float(arg1);
  if (x1 == (float)-1 && PyErr_Occurred())
    return NULL;

  x2 = (float)_cffi_to_c_float(arg2);
  if (x2 == (float)-1 && PyErr_Occurred())
    return NULL;

  x3 = (float)_cffi_to_c_float(arg3);
  if (x3 == (float)-1 && PyErr_Occurred())
    return NULL;

  x4 = _cffi_to_c_int(arg4, int);
  if (x4 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x5, _cffi_type(24), arg5) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawCylinderWires(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawCylinderWires(Vector3 *x0, float x1, float x2, float x3, int x4, Color *x5)
{
  { DrawCylinderWires(*x0, x1, x2, x3, x4, *x5); }
}
#endif

static void _cffi_d_DrawEllipse(int x0, int x1, float x2, float x3, Color x4)
{
  DrawEllipse(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawEllipse(PyObject *self, PyObject *args)
{
  int x0;
  int x1;
  float x2;
  float x3;
  Color x4;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "DrawEllipse", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = (float)_cffi_to_c_float(arg2);
  if (x2 == (float)-1 && PyErr_Occurred())
    return NULL;

  x3 = (float)_cffi_to_c_float(arg3);
  if (x3 == (float)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x4, _cffi_type(24), arg4) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawEllipse(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawEllipse(int x0, int x1, float x2, float x3, Color *x4)
{
  { DrawEllipse(x0, x1, x2, x3, *x4); }
}
#endif

static void _cffi_d_DrawEllipseLines(int x0, int x1, float x2, float x3, Color x4)
{
  DrawEllipseLines(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawEllipseLines(PyObject *self, PyObject *args)
{
  int x0;
  int x1;
  float x2;
  float x3;
  Color x4;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "DrawEllipseLines", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = (float)_cffi_to_c_float(arg2);
  if (x2 == (float)-1 && PyErr_Occurred())
    return NULL;

  x3 = (float)_cffi_to_c_float(arg3);
  if (x3 == (float)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x4, _cffi_type(24), arg4) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawEllipseLines(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawEllipseLines(int x0, int x1, float x2, float x3, Color *x4)
{
  { DrawEllipseLines(x0, x1, x2, x3, *x4); }
}
#endif

static void _cffi_d_DrawFPS(int x0, int x1)
{
  DrawFPS(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawFPS(PyObject *self, PyObject *args)
{
  int x0;
  int x1;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "DrawFPS", 2, 2, &arg0, &arg1))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawFPS(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_DrawFPS _cffi_d_DrawFPS
#endif

static void _cffi_d_DrawGizmo(Vector3 x0)
{
  DrawGizmo(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawGizmo(PyObject *self, PyObject *arg0)
{
  Vector3 x0;

  if (_cffi_to_c((char *)&x0, _cffi_type(28), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawGizmo(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawGizmo(Vector3 *x0)
{
  { DrawGizmo(*x0); }
}
#endif

static void _cffi_d_DrawGrid(int x0, float x1)
{
  DrawGrid(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawGrid(PyObject *self, PyObject *args)
{
  int x0;
  float x1;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "DrawGrid", 2, 2, &arg0, &arg1))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = (float)_cffi_to_c_float(arg1);
  if (x1 == (float)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawGrid(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_DrawGrid _cffi_d_DrawGrid
#endif

static void _cffi_d_DrawLine(int x0, int x1, int x2, int x3, Color x4)
{
  DrawLine(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawLine(PyObject *self, PyObject *args)
{
  int x0;
  int x1;
  int x2;
  int x3;
  Color x4;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "DrawLine", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x4, _cffi_type(24), arg4) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawLine(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawLine(int x0, int x1, int x2, int x3, Color *x4)
{
  { DrawLine(x0, x1, x2, x3, *x4); }
}
#endif

static void _cffi_d_DrawLine3D(Vector3 x0, Vector3 x1, Color x2)
{
  DrawLine3D(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawLine3D(PyObject *self, PyObject *args)
{
  Vector3 x0;
  Vector3 x1;
  Color x2;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "DrawLine3D", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(28), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(28), arg1) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(24), arg2) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawLine3D(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawLine3D(Vector3 *x0, Vector3 *x1, Color *x2)
{
  { DrawLine3D(*x0, *x1, *x2); }
}
#endif

static void _cffi_d_DrawLineBezier(Vector2 x0, Vector2 x1, float x2, Color x3)
{
  DrawLineBezier(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawLineBezier(PyObject *self, PyObject *args)
{
  Vector2 x0;
  Vector2 x1;
  float x2;
  Color x3;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "DrawLineBezier", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(209), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(209), arg1) < 0)
    return NULL;

  x2 = (float)_cffi_to_c_float(arg2);
  if (x2 == (float)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x3, _cffi_type(24), arg3) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawLineBezier(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawLineBezier(Vector2 *x0, Vector2 *x1, float x2, Color *x3)
{
  { DrawLineBezier(*x0, *x1, x2, *x3); }
}
#endif

static void _cffi_d_DrawLineEx(Vector2 x0, Vector2 x1, float x2, Color x3)
{
  DrawLineEx(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawLineEx(PyObject *self, PyObject *args)
{
  Vector2 x0;
  Vector2 x1;
  float x2;
  Color x3;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "DrawLineEx", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(209), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(209), arg1) < 0)
    return NULL;

  x2 = (float)_cffi_to_c_float(arg2);
  if (x2 == (float)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x3, _cffi_type(24), arg3) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawLineEx(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawLineEx(Vector2 *x0, Vector2 *x1, float x2, Color *x3)
{
  { DrawLineEx(*x0, *x1, x2, *x3); }
}
#endif

static void _cffi_d_DrawLineStrip(Vector2 * x0, int x1, Color x2)
{
  DrawLineStrip(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawLineStrip(PyObject *self, PyObject *args)
{
  Vector2 * x0;
  int x1;
  Color x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "DrawLineStrip", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(914), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Vector2 *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(914), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(24), arg2) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawLineStrip(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawLineStrip(Vector2 * x0, int x1, Color *x2)
{
  { DrawLineStrip(x0, x1, *x2); }
}
#endif

static void _cffi_d_DrawLineV(Vector2 x0, Vector2 x1, Color x2)
{
  DrawLineV(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawLineV(PyObject *self, PyObject *args)
{
  Vector2 x0;
  Vector2 x1;
  Color x2;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "DrawLineV", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(209), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(209), arg1) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(24), arg2) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawLineV(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawLineV(Vector2 *x0, Vector2 *x1, Color *x2)
{
  { DrawLineV(*x0, *x1, *x2); }
}
#endif

static void _cffi_d_DrawModel(Model x0, Vector3 x1, float x2, Color x3)
{
  DrawModel(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawModel(PyObject *self, PyObject *args)
{
  Model x0;
  Vector3 x1;
  float x2;
  Color x3;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "DrawModel", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(214), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(28), arg1) < 0)
    return NULL;

  x2 = (float)_cffi_to_c_float(arg2);
  if (x2 == (float)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x3, _cffi_type(24), arg3) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawModel(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawModel(Model *x0, Vector3 *x1, float x2, Color *x3)
{
  { DrawModel(*x0, *x1, x2, *x3); }
}
#endif

static void _cffi_d_DrawModelEx(Model x0, Vector3 x1, Vector3 x2, float x3, Vector3 x4, Color x5)
{
  DrawModelEx(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawModelEx(PyObject *self, PyObject *args)
{
  Model x0;
  Vector3 x1;
  Vector3 x2;
  float x3;
  Vector3 x4;
  Color x5;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "DrawModelEx", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(214), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(28), arg1) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(28), arg2) < 0)
    return NULL;

  x3 = (float)_cffi_to_c_float(arg3);
  if (x3 == (float)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x4, _cffi_type(28), arg4) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x5, _cffi_type(24), arg5) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawModelEx(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawModelEx(Model *x0, Vector3 *x1, Vector3 *x2, float x3, Vector3 *x4, Color *x5)
{
  { DrawModelEx(*x0, *x1, *x2, x3, *x4, *x5); }
}
#endif

static void _cffi_d_DrawModelWires(Model x0, Vector3 x1, float x2, Color x3)
{
  DrawModelWires(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawModelWires(PyObject *self, PyObject *args)
{
  Model x0;
  Vector3 x1;
  float x2;
  Color x3;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "DrawModelWires", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(214), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(28), arg1) < 0)
    return NULL;

  x2 = (float)_cffi_to_c_float(arg2);
  if (x2 == (float)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x3, _cffi_type(24), arg3) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawModelWires(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawModelWires(Model *x0, Vector3 *x1, float x2, Color *x3)
{
  { DrawModelWires(*x0, *x1, x2, *x3); }
}
#endif

static void _cffi_d_DrawModelWiresEx(Model x0, Vector3 x1, Vector3 x2, float x3, Vector3 x4, Color x5)
{
  DrawModelWiresEx(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawModelWiresEx(PyObject *self, PyObject *args)
{
  Model x0;
  Vector3 x1;
  Vector3 x2;
  float x3;
  Vector3 x4;
  Color x5;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "DrawModelWiresEx", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(214), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(28), arg1) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(28), arg2) < 0)
    return NULL;

  x3 = (float)_cffi_to_c_float(arg3);
  if (x3 == (float)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x4, _cffi_type(28), arg4) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x5, _cffi_type(24), arg5) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawModelWiresEx(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawModelWiresEx(Model *x0, Vector3 *x1, Vector3 *x2, float x3, Vector3 *x4, Color *x5)
{
  { DrawModelWiresEx(*x0, *x1, *x2, x3, *x4, *x5); }
}
#endif

static void _cffi_d_DrawPixel(int x0, int x1, Color x2)
{
  DrawPixel(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawPixel(PyObject *self, PyObject *args)
{
  int x0;
  int x1;
  Color x2;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "DrawPixel", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(24), arg2) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawPixel(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawPixel(int x0, int x1, Color *x2)
{
  { DrawPixel(x0, x1, *x2); }
}
#endif

static void _cffi_d_DrawPixelV(Vector2 x0, Color x1)
{
  DrawPixelV(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawPixelV(PyObject *self, PyObject *args)
{
  Vector2 x0;
  Color x1;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "DrawPixelV", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(209), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(24), arg1) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawPixelV(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawPixelV(Vector2 *x0, Color *x1)
{
  { DrawPixelV(*x0, *x1); }
}
#endif

static void _cffi_d_DrawPlane(Vector3 x0, Vector2 x1, Color x2)
{
  DrawPlane(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawPlane(PyObject *self, PyObject *args)
{
  Vector3 x0;
  Vector2 x1;
  Color x2;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "DrawPlane", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(28), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(209), arg1) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(24), arg2) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawPlane(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawPlane(Vector3 *x0, Vector2 *x1, Color *x2)
{
  { DrawPlane(*x0, *x1, *x2); }
}
#endif

static void _cffi_d_DrawPoint3D(Vector3 x0, Color x1)
{
  DrawPoint3D(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawPoint3D(PyObject *self, PyObject *args)
{
  Vector3 x0;
  Color x1;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "DrawPoint3D", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(28), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(24), arg1) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawPoint3D(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawPoint3D(Vector3 *x0, Color *x1)
{
  { DrawPoint3D(*x0, *x1); }
}
#endif

static void _cffi_d_DrawPoly(Vector2 x0, int x1, float x2, float x3, Color x4)
{
  DrawPoly(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawPoly(PyObject *self, PyObject *args)
{
  Vector2 x0;
  int x1;
  float x2;
  float x3;
  Color x4;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "DrawPoly", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(209), arg0) < 0)
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = (float)_cffi_to_c_float(arg2);
  if (x2 == (float)-1 && PyErr_Occurred())
    return NULL;

  x3 = (float)_cffi_to_c_float(arg3);
  if (x3 == (float)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x4, _cffi_type(24), arg4) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawPoly(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawPoly(Vector2 *x0, int x1, float x2, float x3, Color *x4)
{
  { DrawPoly(*x0, x1, x2, x3, *x4); }
}
#endif

static void _cffi_d_DrawPolyLines(Vector2 x0, int x1, float x2, float x3, Color x4)
{
  DrawPolyLines(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawPolyLines(PyObject *self, PyObject *args)
{
  Vector2 x0;
  int x1;
  float x2;
  float x3;
  Color x4;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "DrawPolyLines", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(209), arg0) < 0)
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = (float)_cffi_to_c_float(arg2);
  if (x2 == (float)-1 && PyErr_Occurred())
    return NULL;

  x3 = (float)_cffi_to_c_float(arg3);
  if (x3 == (float)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x4, _cffi_type(24), arg4) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawPolyLines(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawPolyLines(Vector2 *x0, int x1, float x2, float x3, Color *x4)
{
  { DrawPolyLines(*x0, x1, x2, x3, *x4); }
}
#endif

static void _cffi_d_DrawRay(Ray x0, Color x1)
{
  DrawRay(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawRay(PyObject *self, PyObject *args)
{
  Ray x0;
  Color x1;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "DrawRay", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(213), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(24), arg1) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawRay(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawRay(Ray *x0, Color *x1)
{
  { DrawRay(*x0, *x1); }
}
#endif

static void _cffi_d_DrawRectangle(int x0, int x1, int x2, int x3, Color x4)
{
  DrawRectangle(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawRectangle(PyObject *self, PyObject *args)
{
  int x0;
  int x1;
  int x2;
  int x3;
  Color x4;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "DrawRectangle", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x4, _cffi_type(24), arg4) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawRectangle(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawRectangle(int x0, int x1, int x2, int x3, Color *x4)
{
  { DrawRectangle(x0, x1, x2, x3, *x4); }
}
#endif

static void _cffi_d_DrawRectangleGradientEx(Rectangle x0, Color x1, Color x2, Color x3, Color x4)
{
  DrawRectangleGradientEx(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawRectangleGradientEx(PyObject *self, PyObject *args)
{
  Rectangle x0;
  Color x1;
  Color x2;
  Color x3;
  Color x4;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "DrawRectangleGradientEx", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(77), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(24), arg1) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(24), arg2) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x3, _cffi_type(24), arg3) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x4, _cffi_type(24), arg4) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawRectangleGradientEx(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawRectangleGradientEx(Rectangle *x0, Color *x1, Color *x2, Color *x3, Color *x4)
{
  { DrawRectangleGradientEx(*x0, *x1, *x2, *x3, *x4); }
}
#endif

static void _cffi_d_DrawRectangleGradientH(int x0, int x1, int x2, int x3, Color x4, Color x5)
{
  DrawRectangleGradientH(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawRectangleGradientH(PyObject *self, PyObject *args)
{
  int x0;
  int x1;
  int x2;
  int x3;
  Color x4;
  Color x5;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "DrawRectangleGradientH", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x4, _cffi_type(24), arg4) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x5, _cffi_type(24), arg5) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawRectangleGradientH(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawRectangleGradientH(int x0, int x1, int x2, int x3, Color *x4, Color *x5)
{
  { DrawRectangleGradientH(x0, x1, x2, x3, *x4, *x5); }
}
#endif

static void _cffi_d_DrawRectangleGradientV(int x0, int x1, int x2, int x3, Color x4, Color x5)
{
  DrawRectangleGradientV(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawRectangleGradientV(PyObject *self, PyObject *args)
{
  int x0;
  int x1;
  int x2;
  int x3;
  Color x4;
  Color x5;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "DrawRectangleGradientV", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x4, _cffi_type(24), arg4) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x5, _cffi_type(24), arg5) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawRectangleGradientV(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawRectangleGradientV(int x0, int x1, int x2, int x3, Color *x4, Color *x5)
{
  { DrawRectangleGradientV(x0, x1, x2, x3, *x4, *x5); }
}
#endif

static void _cffi_d_DrawRectangleLines(int x0, int x1, int x2, int x3, Color x4)
{
  DrawRectangleLines(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawRectangleLines(PyObject *self, PyObject *args)
{
  int x0;
  int x1;
  int x2;
  int x3;
  Color x4;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "DrawRectangleLines", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x4, _cffi_type(24), arg4) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawRectangleLines(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawRectangleLines(int x0, int x1, int x2, int x3, Color *x4)
{
  { DrawRectangleLines(x0, x1, x2, x3, *x4); }
}
#endif

static void _cffi_d_DrawRectangleLinesEx(Rectangle x0, int x1, Color x2)
{
  DrawRectangleLinesEx(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawRectangleLinesEx(PyObject *self, PyObject *args)
{
  Rectangle x0;
  int x1;
  Color x2;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "DrawRectangleLinesEx", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(77), arg0) < 0)
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(24), arg2) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawRectangleLinesEx(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawRectangleLinesEx(Rectangle *x0, int x1, Color *x2)
{
  { DrawRectangleLinesEx(*x0, x1, *x2); }
}
#endif

static void _cffi_d_DrawRectanglePro(Rectangle x0, Vector2 x1, float x2, Color x3)
{
  DrawRectanglePro(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawRectanglePro(PyObject *self, PyObject *args)
{
  Rectangle x0;
  Vector2 x1;
  float x2;
  Color x3;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "DrawRectanglePro", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(77), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(209), arg1) < 0)
    return NULL;

  x2 = (float)_cffi_to_c_float(arg2);
  if (x2 == (float)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x3, _cffi_type(24), arg3) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawRectanglePro(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawRectanglePro(Rectangle *x0, Vector2 *x1, float x2, Color *x3)
{
  { DrawRectanglePro(*x0, *x1, x2, *x3); }
}
#endif

static void _cffi_d_DrawRectangleRec(Rectangle x0, Color x1)
{
  DrawRectangleRec(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawRectangleRec(PyObject *self, PyObject *args)
{
  Rectangle x0;
  Color x1;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "DrawRectangleRec", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(77), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(24), arg1) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawRectangleRec(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawRectangleRec(Rectangle *x0, Color *x1)
{
  { DrawRectangleRec(*x0, *x1); }
}
#endif

static void _cffi_d_DrawRectangleRounded(Rectangle x0, float x1, int x2, Color x3)
{
  DrawRectangleRounded(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawRectangleRounded(PyObject *self, PyObject *args)
{
  Rectangle x0;
  float x1;
  int x2;
  Color x3;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "DrawRectangleRounded", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(77), arg0) < 0)
    return NULL;

  x1 = (float)_cffi_to_c_float(arg1);
  if (x1 == (float)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x3, _cffi_type(24), arg3) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawRectangleRounded(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawRectangleRounded(Rectangle *x0, float x1, int x2, Color *x3)
{
  { DrawRectangleRounded(*x0, x1, x2, *x3); }
}
#endif

static void _cffi_d_DrawRectangleRoundedLines(Rectangle x0, float x1, int x2, int x3, Color x4)
{
  DrawRectangleRoundedLines(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawRectangleRoundedLines(PyObject *self, PyObject *args)
{
  Rectangle x0;
  float x1;
  int x2;
  int x3;
  Color x4;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "DrawRectangleRoundedLines", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(77), arg0) < 0)
    return NULL;

  x1 = (float)_cffi_to_c_float(arg1);
  if (x1 == (float)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x4, _cffi_type(24), arg4) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawRectangleRoundedLines(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawRectangleRoundedLines(Rectangle *x0, float x1, int x2, int x3, Color *x4)
{
  { DrawRectangleRoundedLines(*x0, x1, x2, x3, *x4); }
}
#endif

static void _cffi_d_DrawRectangleV(Vector2 x0, Vector2 x1, Color x2)
{
  DrawRectangleV(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawRectangleV(PyObject *self, PyObject *args)
{
  Vector2 x0;
  Vector2 x1;
  Color x2;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "DrawRectangleV", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(209), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(209), arg1) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(24), arg2) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawRectangleV(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawRectangleV(Vector2 *x0, Vector2 *x1, Color *x2)
{
  { DrawRectangleV(*x0, *x1, *x2); }
}
#endif

static void _cffi_d_DrawRing(Vector2 x0, float x1, float x2, int x3, int x4, int x5, Color x6)
{
  DrawRing(x0, x1, x2, x3, x4, x5, x6);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawRing(PyObject *self, PyObject *args)
{
  Vector2 x0;
  float x1;
  float x2;
  int x3;
  int x4;
  int x5;
  Color x6;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;
  PyObject *arg6;

  if (!PyArg_UnpackTuple(args, "DrawRing", 7, 7, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5, &arg6))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(209), arg0) < 0)
    return NULL;

  x1 = (float)_cffi_to_c_float(arg1);
  if (x1 == (float)-1 && PyErr_Occurred())
    return NULL;

  x2 = (float)_cffi_to_c_float(arg2);
  if (x2 == (float)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  x4 = _cffi_to_c_int(arg4, int);
  if (x4 == (int)-1 && PyErr_Occurred())
    return NULL;

  x5 = _cffi_to_c_int(arg5, int);
  if (x5 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x6, _cffi_type(24), arg6) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawRing(x0, x1, x2, x3, x4, x5, x6); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawRing(Vector2 *x0, float x1, float x2, int x3, int x4, int x5, Color *x6)
{
  { DrawRing(*x0, x1, x2, x3, x4, x5, *x6); }
}
#endif

static void _cffi_d_DrawRingLines(Vector2 x0, float x1, float x2, int x3, int x4, int x5, Color x6)
{
  DrawRingLines(x0, x1, x2, x3, x4, x5, x6);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawRingLines(PyObject *self, PyObject *args)
{
  Vector2 x0;
  float x1;
  float x2;
  int x3;
  int x4;
  int x5;
  Color x6;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;
  PyObject *arg6;

  if (!PyArg_UnpackTuple(args, "DrawRingLines", 7, 7, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5, &arg6))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(209), arg0) < 0)
    return NULL;

  x1 = (float)_cffi_to_c_float(arg1);
  if (x1 == (float)-1 && PyErr_Occurred())
    return NULL;

  x2 = (float)_cffi_to_c_float(arg2);
  if (x2 == (float)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  x4 = _cffi_to_c_int(arg4, int);
  if (x4 == (int)-1 && PyErr_Occurred())
    return NULL;

  x5 = _cffi_to_c_int(arg5, int);
  if (x5 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x6, _cffi_type(24), arg6) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawRingLines(x0, x1, x2, x3, x4, x5, x6); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawRingLines(Vector2 *x0, float x1, float x2, int x3, int x4, int x5, Color *x6)
{
  { DrawRingLines(*x0, x1, x2, x3, x4, x5, *x6); }
}
#endif

static void _cffi_d_DrawSphere(Vector3 x0, float x1, Color x2)
{
  DrawSphere(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawSphere(PyObject *self, PyObject *args)
{
  Vector3 x0;
  float x1;
  Color x2;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "DrawSphere", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(28), arg0) < 0)
    return NULL;

  x1 = (float)_cffi_to_c_float(arg1);
  if (x1 == (float)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(24), arg2) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawSphere(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawSphere(Vector3 *x0, float x1, Color *x2)
{
  { DrawSphere(*x0, x1, *x2); }
}
#endif

static void _cffi_d_DrawSphereEx(Vector3 x0, float x1, int x2, int x3, Color x4)
{
  DrawSphereEx(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawSphereEx(PyObject *self, PyObject *args)
{
  Vector3 x0;
  float x1;
  int x2;
  int x3;
  Color x4;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "DrawSphereEx", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(28), arg0) < 0)
    return NULL;

  x1 = (float)_cffi_to_c_float(arg1);
  if (x1 == (float)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x4, _cffi_type(24), arg4) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawSphereEx(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawSphereEx(Vector3 *x0, float x1, int x2, int x3, Color *x4)
{
  { DrawSphereEx(*x0, x1, x2, x3, *x4); }
}
#endif

static void _cffi_d_DrawSphereWires(Vector3 x0, float x1, int x2, int x3, Color x4)
{
  DrawSphereWires(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawSphereWires(PyObject *self, PyObject *args)
{
  Vector3 x0;
  float x1;
  int x2;
  int x3;
  Color x4;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "DrawSphereWires", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(28), arg0) < 0)
    return NULL;

  x1 = (float)_cffi_to_c_float(arg1);
  if (x1 == (float)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x4, _cffi_type(24), arg4) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawSphereWires(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawSphereWires(Vector3 *x0, float x1, int x2, int x3, Color *x4)
{
  { DrawSphereWires(*x0, x1, x2, x3, *x4); }
}
#endif

static void _cffi_d_DrawText(char const * x0, int x1, int x2, int x3, Color x4)
{
  DrawText(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawText(PyObject *self, PyObject *args)
{
  char const * x0;
  int x1;
  int x2;
  int x3;
  Color x4;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "DrawText", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x4, _cffi_type(24), arg4) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawText(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawText(char const * x0, int x1, int x2, int x3, Color *x4)
{
  { DrawText(x0, x1, x2, x3, *x4); }
}
#endif

static void _cffi_d_DrawTextCodepoint(Font x0, int x1, Vector2 x2, float x3, Color x4)
{
  DrawTextCodepoint(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawTextCodepoint(PyObject *self, PyObject *args)
{
  Font x0;
  int x1;
  Vector2 x2;
  float x3;
  Color x4;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "DrawTextCodepoint", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(66), arg0) < 0)
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(209), arg2) < 0)
    return NULL;

  x3 = (float)_cffi_to_c_float(arg3);
  if (x3 == (float)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x4, _cffi_type(24), arg4) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawTextCodepoint(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawTextCodepoint(Font *x0, int x1, Vector2 *x2, float x3, Color *x4)
{
  { DrawTextCodepoint(*x0, x1, *x2, x3, *x4); }
}
#endif

static void _cffi_d_DrawTextEx(Font x0, char const * x1, Vector2 x2, float x3, float x4, Color x5)
{
  DrawTextEx(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawTextEx(PyObject *self, PyObject *args)
{
  Font x0;
  char const * x1;
  Vector2 x2;
  float x3;
  float x4;
  Color x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "DrawTextEx", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(66), arg0) < 0)
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  if (_cffi_to_c((char *)&x2, _cffi_type(209), arg2) < 0)
    return NULL;

  x3 = (float)_cffi_to_c_float(arg3);
  if (x3 == (float)-1 && PyErr_Occurred())
    return NULL;

  x4 = (float)_cffi_to_c_float(arg4);
  if (x4 == (float)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x5, _cffi_type(24), arg5) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawTextEx(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawTextEx(Font *x0, char const * x1, Vector2 *x2, float x3, float x4, Color *x5)
{
  { DrawTextEx(*x0, x1, *x2, x3, x4, *x5); }
}
#endif

static void _cffi_d_DrawTextRec(Font x0, char const * x1, Rectangle x2, float x3, float x4, int x5, Color x6)
{
  DrawTextRec(x0, x1, x2, x3, x4, x5, x6);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawTextRec(PyObject *self, PyObject *args)
{
  Font x0;
  char const * x1;
  Rectangle x2;
  float x3;
  float x4;
  int x5;
  Color x6;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;
  PyObject *arg6;

  if (!PyArg_UnpackTuple(args, "DrawTextRec", 7, 7, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5, &arg6))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(66), arg0) < 0)
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  if (_cffi_to_c((char *)&x2, _cffi_type(77), arg2) < 0)
    return NULL;

  x3 = (float)_cffi_to_c_float(arg3);
  if (x3 == (float)-1 && PyErr_Occurred())
    return NULL;

  x4 = (float)_cffi_to_c_float(arg4);
  if (x4 == (float)-1 && PyErr_Occurred())
    return NULL;

  x5 = _cffi_to_c_int(arg5, int);
  if (x5 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x6, _cffi_type(24), arg6) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawTextRec(x0, x1, x2, x3, x4, x5, x6); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawTextRec(Font *x0, char const * x1, Rectangle *x2, float x3, float x4, int x5, Color *x6)
{
  { DrawTextRec(*x0, x1, *x2, x3, x4, x5, *x6); }
}
#endif

static void _cffi_d_DrawTextRecEx(Font x0, char const * x1, Rectangle x2, float x3, float x4, int x5, Color x6, int x7, int x8, Color x9, Color x10)
{
  DrawTextRecEx(x0, x1, x2, x3, x4, x5, x6, x7, x8, x9, x10);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawTextRecEx(PyObject *self, PyObject *args)
{
  Font x0;
  char const * x1;
  Rectangle x2;
  float x3;
  float x4;
  int x5;
  Color x6;
  int x7;
  int x8;
  Color x9;
  Color x10;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;
  PyObject *arg6;
  PyObject *arg7;
  PyObject *arg8;
  PyObject *arg9;
  PyObject *arg10;

  if (!PyArg_UnpackTuple(args, "DrawTextRecEx", 11, 11, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5, &arg6, &arg7, &arg8, &arg9, &arg10))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(66), arg0) < 0)
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  if (_cffi_to_c((char *)&x2, _cffi_type(77), arg2) < 0)
    return NULL;

  x3 = (float)_cffi_to_c_float(arg3);
  if (x3 == (float)-1 && PyErr_Occurred())
    return NULL;

  x4 = (float)_cffi_to_c_float(arg4);
  if (x4 == (float)-1 && PyErr_Occurred())
    return NULL;

  x5 = _cffi_to_c_int(arg5, int);
  if (x5 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x6, _cffi_type(24), arg6) < 0)
    return NULL;

  x7 = _cffi_to_c_int(arg7, int);
  if (x7 == (int)-1 && PyErr_Occurred())
    return NULL;

  x8 = _cffi_to_c_int(arg8, int);
  if (x8 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x9, _cffi_type(24), arg9) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x10, _cffi_type(24), arg10) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawTextRecEx(x0, x1, x2, x3, x4, x5, x6, x7, x8, x9, x10); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawTextRecEx(Font *x0, char const * x1, Rectangle *x2, float x3, float x4, int x5, Color *x6, int x7, int x8, Color *x9, Color *x10)
{
  { DrawTextRecEx(*x0, x1, *x2, x3, x4, x5, *x6, x7, x8, *x9, *x10); }
}
#endif

static void _cffi_d_DrawTexture(Texture2D x0, int x1, int x2, Color x3)
{
  DrawTexture(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawTexture(PyObject *self, PyObject *args)
{
  Texture2D x0;
  int x1;
  int x2;
  Color x3;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "DrawTexture", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(80), arg0) < 0)
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x3, _cffi_type(24), arg3) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawTexture(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawTexture(Texture2D *x0, int x1, int x2, Color *x3)
{
  { DrawTexture(*x0, x1, x2, *x3); }
}
#endif

static void _cffi_d_DrawTextureEx(Texture2D x0, Vector2 x1, float x2, float x3, Color x4)
{
  DrawTextureEx(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawTextureEx(PyObject *self, PyObject *args)
{
  Texture2D x0;
  Vector2 x1;
  float x2;
  float x3;
  Color x4;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "DrawTextureEx", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(80), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(209), arg1) < 0)
    return NULL;

  x2 = (float)_cffi_to_c_float(arg2);
  if (x2 == (float)-1 && PyErr_Occurred())
    return NULL;

  x3 = (float)_cffi_to_c_float(arg3);
  if (x3 == (float)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x4, _cffi_type(24), arg4) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawTextureEx(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawTextureEx(Texture2D *x0, Vector2 *x1, float x2, float x3, Color *x4)
{
  { DrawTextureEx(*x0, *x1, x2, x3, *x4); }
}
#endif

static void _cffi_d_DrawTextureNPatch(Texture2D x0, NPatchInfo x1, Rectangle x2, Vector2 x3, float x4, Color x5)
{
  DrawTextureNPatch(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawTextureNPatch(PyObject *self, PyObject *args)
{
  Texture2D x0;
  NPatchInfo x1;
  Rectangle x2;
  Vector2 x3;
  float x4;
  Color x5;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "DrawTextureNPatch", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(80), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(848), arg1) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(77), arg2) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x3, _cffi_type(209), arg3) < 0)
    return NULL;

  x4 = (float)_cffi_to_c_float(arg4);
  if (x4 == (float)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x5, _cffi_type(24), arg5) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawTextureNPatch(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawTextureNPatch(Texture2D *x0, NPatchInfo *x1, Rectangle *x2, Vector2 *x3, float x4, Color *x5)
{
  { DrawTextureNPatch(*x0, *x1, *x2, *x3, x4, *x5); }
}
#endif

static void _cffi_d_DrawTexturePro(Texture2D x0, Rectangle x1, Rectangle x2, Vector2 x3, float x4, Color x5)
{
  DrawTexturePro(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawTexturePro(PyObject *self, PyObject *args)
{
  Texture2D x0;
  Rectangle x1;
  Rectangle x2;
  Vector2 x3;
  float x4;
  Color x5;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "DrawTexturePro", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(80), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(77), arg1) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(77), arg2) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x3, _cffi_type(209), arg3) < 0)
    return NULL;

  x4 = (float)_cffi_to_c_float(arg4);
  if (x4 == (float)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x5, _cffi_type(24), arg5) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawTexturePro(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawTexturePro(Texture2D *x0, Rectangle *x1, Rectangle *x2, Vector2 *x3, float x4, Color *x5)
{
  { DrawTexturePro(*x0, *x1, *x2, *x3, x4, *x5); }
}
#endif

static void _cffi_d_DrawTextureQuad(Texture2D x0, Vector2 x1, Vector2 x2, Rectangle x3, Color x4)
{
  DrawTextureQuad(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawTextureQuad(PyObject *self, PyObject *args)
{
  Texture2D x0;
  Vector2 x1;
  Vector2 x2;
  Rectangle x3;
  Color x4;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "DrawTextureQuad", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(80), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(209), arg1) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(209), arg2) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x3, _cffi_type(77), arg3) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x4, _cffi_type(24), arg4) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawTextureQuad(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawTextureQuad(Texture2D *x0, Vector2 *x1, Vector2 *x2, Rectangle *x3, Color *x4)
{
  { DrawTextureQuad(*x0, *x1, *x2, *x3, *x4); }
}
#endif

static void _cffi_d_DrawTextureRec(Texture2D x0, Rectangle x1, Vector2 x2, Color x3)
{
  DrawTextureRec(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawTextureRec(PyObject *self, PyObject *args)
{
  Texture2D x0;
  Rectangle x1;
  Vector2 x2;
  Color x3;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "DrawTextureRec", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(80), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(77), arg1) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(209), arg2) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x3, _cffi_type(24), arg3) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawTextureRec(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawTextureRec(Texture2D *x0, Rectangle *x1, Vector2 *x2, Color *x3)
{
  { DrawTextureRec(*x0, *x1, *x2, *x3); }
}
#endif

static void _cffi_d_DrawTextureV(Texture2D x0, Vector2 x1, Color x2)
{
  DrawTextureV(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawTextureV(PyObject *self, PyObject *args)
{
  Texture2D x0;
  Vector2 x1;
  Color x2;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "DrawTextureV", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(80), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(209), arg1) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(24), arg2) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawTextureV(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawTextureV(Texture2D *x0, Vector2 *x1, Color *x2)
{
  { DrawTextureV(*x0, *x1, *x2); }
}
#endif

static void _cffi_d_DrawTriangle(Vector2 x0, Vector2 x1, Vector2 x2, Color x3)
{
  DrawTriangle(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawTriangle(PyObject *self, PyObject *args)
{
  Vector2 x0;
  Vector2 x1;
  Vector2 x2;
  Color x3;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "DrawTriangle", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(209), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(209), arg1) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(209), arg2) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x3, _cffi_type(24), arg3) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawTriangle(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawTriangle(Vector2 *x0, Vector2 *x1, Vector2 *x2, Color *x3)
{
  { DrawTriangle(*x0, *x1, *x2, *x3); }
}
#endif

static void _cffi_d_DrawTriangleFan(Vector2 * x0, int x1, Color x2)
{
  DrawTriangleFan(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawTriangleFan(PyObject *self, PyObject *args)
{
  Vector2 * x0;
  int x1;
  Color x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "DrawTriangleFan", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(914), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Vector2 *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(914), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(24), arg2) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawTriangleFan(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawTriangleFan(Vector2 * x0, int x1, Color *x2)
{
  { DrawTriangleFan(x0, x1, *x2); }
}
#endif

static void _cffi_d_DrawTriangleLines(Vector2 x0, Vector2 x1, Vector2 x2, Color x3)
{
  DrawTriangleLines(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawTriangleLines(PyObject *self, PyObject *args)
{
  Vector2 x0;
  Vector2 x1;
  Vector2 x2;
  Color x3;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "DrawTriangleLines", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(209), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(209), arg1) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(209), arg2) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x3, _cffi_type(24), arg3) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawTriangleLines(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawTriangleLines(Vector2 *x0, Vector2 *x1, Vector2 *x2, Color *x3)
{
  { DrawTriangleLines(*x0, *x1, *x2, *x3); }
}
#endif

static void _cffi_d_DrawTriangleStrip(Vector2 * x0, int x1, Color x2)
{
  DrawTriangleStrip(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_DrawTriangleStrip(PyObject *self, PyObject *args)
{
  Vector2 * x0;
  int x1;
  Color x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "DrawTriangleStrip", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(914), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Vector2 *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(914), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(24), arg2) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { DrawTriangleStrip(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_DrawTriangleStrip(Vector2 * x0, int x1, Color *x2)
{
  { DrawTriangleStrip(x0, x1, *x2); }
}
#endif

static void _cffi_d_EnableCursor(void)
{
  EnableCursor();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_EnableCursor(PyObject *self, PyObject *noarg)
{

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { EnableCursor(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_EnableCursor _cffi_d_EnableCursor
#endif

static void _cffi_d_EndBlendMode(void)
{
  EndBlendMode();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_EndBlendMode(PyObject *self, PyObject *noarg)
{

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { EndBlendMode(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_EndBlendMode _cffi_d_EndBlendMode
#endif

static void _cffi_d_EndDrawing(void)
{
  EndDrawing();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_EndDrawing(PyObject *self, PyObject *noarg)
{

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { EndDrawing(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_EndDrawing _cffi_d_EndDrawing
#endif

static void _cffi_d_EndMode2D(void)
{
  EndMode2D();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_EndMode2D(PyObject *self, PyObject *noarg)
{

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { EndMode2D(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_EndMode2D _cffi_d_EndMode2D
#endif

static void _cffi_d_EndMode3D(void)
{
  EndMode3D();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_EndMode3D(PyObject *self, PyObject *noarg)
{

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { EndMode3D(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_EndMode3D _cffi_d_EndMode3D
#endif

static void _cffi_d_EndScissorMode(void)
{
  EndScissorMode();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_EndScissorMode(PyObject *self, PyObject *noarg)
{

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { EndScissorMode(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_EndScissorMode _cffi_d_EndScissorMode
#endif

static void _cffi_d_EndShaderMode(void)
{
  EndShaderMode();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_EndShaderMode(PyObject *self, PyObject *noarg)
{

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { EndShaderMode(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_EndShaderMode _cffi_d_EndShaderMode
#endif

static void _cffi_d_EndTextureMode(void)
{
  EndTextureMode();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_EndTextureMode(PyObject *self, PyObject *noarg)
{

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { EndTextureMode(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_EndTextureMode _cffi_d_EndTextureMode
#endif

static void _cffi_d_EndVrDrawing(void)
{
  EndVrDrawing();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_EndVrDrawing(PyObject *self, PyObject *noarg)
{

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { EndVrDrawing(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_EndVrDrawing _cffi_d_EndVrDrawing
#endif

static void _cffi_d_ExportImage(Image x0, char const * x1)
{
  ExportImage(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ExportImage(PyObject *self, PyObject *args)
{
  Image x0;
  char const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ExportImage", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(16), arg0) < 0)
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ExportImage(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_ExportImage(Image *x0, char const * x1)
{
  { ExportImage(*x0, x1); }
}
#endif

static void _cffi_d_ExportImageAsCode(Image x0, char const * x1)
{
  ExportImageAsCode(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ExportImageAsCode(PyObject *self, PyObject *args)
{
  Image x0;
  char const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ExportImageAsCode", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(16), arg0) < 0)
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ExportImageAsCode(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_ExportImageAsCode(Image *x0, char const * x1)
{
  { ExportImageAsCode(*x0, x1); }
}
#endif

static void _cffi_d_ExportMesh(Mesh x0, char const * x1)
{
  ExportMesh(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ExportMesh(PyObject *self, PyObject *args)
{
  Mesh x0;
  char const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ExportMesh", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(6), arg0) < 0)
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ExportMesh(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_ExportMesh(Mesh *x0, char const * x1)
{
  { ExportMesh(*x0, x1); }
}
#endif

static void _cffi_d_ExportWave(Wave x0, char const * x1)
{
  ExportWave(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ExportWave(PyObject *self, PyObject *args)
{
  Wave x0;
  char const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ExportWave", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(245), arg0) < 0)
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ExportWave(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_ExportWave(Wave *x0, char const * x1)
{
  { ExportWave(*x0, x1); }
}
#endif

static void _cffi_d_ExportWaveAsCode(Wave x0, char const * x1)
{
  ExportWaveAsCode(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ExportWaveAsCode(PyObject *self, PyObject *args)
{
  Wave x0;
  char const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ExportWaveAsCode", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(245), arg0) < 0)
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ExportWaveAsCode(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_ExportWaveAsCode(Wave *x0, char const * x1)
{
  { ExportWaveAsCode(*x0, x1); }
}
#endif

static Color _cffi_d_Fade(Color x0, float x1)
{
  return Fade(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_Fade(PyObject *self, PyObject *args)
{
  Color x0;
  float x1;
  Color result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "Fade", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(24), arg0) < 0)
    return NULL;

  x1 = (float)_cffi_to_c_float(arg1);
  if (x1 == (float)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = Fade(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(24));
  return pyresult;
}
#else
static void _cffi_f_Fade(Color *result, Color *x0, float x1)
{
  { *result = Fade(*x0, x1); }
}
#endif

static int _cffi_d_FileExists(char const * x0)
{
  return FileExists(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_FileExists(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = FileExists(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_FileExists _cffi_d_FileExists
#endif

static Image _cffi_d_GenImageCellular(int x0, int x1, int x2)
{
  return GenImageCellular(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GenImageCellular(PyObject *self, PyObject *args)
{
  int x0;
  int x1;
  int x2;
  Image result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "GenImageCellular", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GenImageCellular(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(16));
  return pyresult;
}
#else
static void _cffi_f_GenImageCellular(Image *result, int x0, int x1, int x2)
{
  { *result = GenImageCellular(x0, x1, x2); }
}
#endif

static Image _cffi_d_GenImageChecked(int x0, int x1, int x2, int x3, Color x4, Color x5)
{
  return GenImageChecked(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GenImageChecked(PyObject *self, PyObject *args)
{
  int x0;
  int x1;
  int x2;
  int x3;
  Color x4;
  Color x5;
  Image result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "GenImageChecked", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x4, _cffi_type(24), arg4) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x5, _cffi_type(24), arg5) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GenImageChecked(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(16));
  return pyresult;
}
#else
static void _cffi_f_GenImageChecked(Image *result, int x0, int x1, int x2, int x3, Color *x4, Color *x5)
{
  { *result = GenImageChecked(x0, x1, x2, x3, *x4, *x5); }
}
#endif

static Image _cffi_d_GenImageColor(int x0, int x1, Color x2)
{
  return GenImageColor(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GenImageColor(PyObject *self, PyObject *args)
{
  int x0;
  int x1;
  Color x2;
  Image result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "GenImageColor", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(24), arg2) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GenImageColor(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(16));
  return pyresult;
}
#else
static void _cffi_f_GenImageColor(Image *result, int x0, int x1, Color *x2)
{
  { *result = GenImageColor(x0, x1, *x2); }
}
#endif

static Image _cffi_d_GenImageFontAtlas(CharInfo const * x0, Rectangle * * x1, int x2, int x3, int x4, int x5)
{
  return GenImageFontAtlas(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GenImageFontAtlas(PyObject *self, PyObject *args)
{
  CharInfo const * x0;
  Rectangle * * x1;
  int x2;
  int x3;
  int x4;
  int x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  Image result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "GenImageFontAtlas", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(53), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (CharInfo const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(53), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(54), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (Rectangle * *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(54), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  x4 = _cffi_to_c_int(arg4, int);
  if (x4 == (int)-1 && PyErr_Occurred())
    return NULL;

  x5 = _cffi_to_c_int(arg5, int);
  if (x5 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GenImageFontAtlas(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(16));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
static void _cffi_f_GenImageFontAtlas(Image *result, CharInfo const * x0, Rectangle * * x1, int x2, int x3, int x4, int x5)
{
  { *result = GenImageFontAtlas(x0, x1, x2, x3, x4, x5); }
}
#endif

static Image _cffi_d_GenImageGradientH(int x0, int x1, Color x2, Color x3)
{
  return GenImageGradientH(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GenImageGradientH(PyObject *self, PyObject *args)
{
  int x0;
  int x1;
  Color x2;
  Color x3;
  Image result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "GenImageGradientH", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(24), arg2) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x3, _cffi_type(24), arg3) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GenImageGradientH(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(16));
  return pyresult;
}
#else
static void _cffi_f_GenImageGradientH(Image *result, int x0, int x1, Color *x2, Color *x3)
{
  { *result = GenImageGradientH(x0, x1, *x2, *x3); }
}
#endif

static Image _cffi_d_GenImageGradientRadial(int x0, int x1, float x2, Color x3, Color x4)
{
  return GenImageGradientRadial(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GenImageGradientRadial(PyObject *self, PyObject *args)
{
  int x0;
  int x1;
  float x2;
  Color x3;
  Color x4;
  Image result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "GenImageGradientRadial", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = (float)_cffi_to_c_float(arg2);
  if (x2 == (float)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x3, _cffi_type(24), arg3) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x4, _cffi_type(24), arg4) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GenImageGradientRadial(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(16));
  return pyresult;
}
#else
static void _cffi_f_GenImageGradientRadial(Image *result, int x0, int x1, float x2, Color *x3, Color *x4)
{
  { *result = GenImageGradientRadial(x0, x1, x2, *x3, *x4); }
}
#endif

static Image _cffi_d_GenImageGradientV(int x0, int x1, Color x2, Color x3)
{
  return GenImageGradientV(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GenImageGradientV(PyObject *self, PyObject *args)
{
  int x0;
  int x1;
  Color x2;
  Color x3;
  Image result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "GenImageGradientV", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(24), arg2) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x3, _cffi_type(24), arg3) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GenImageGradientV(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(16));
  return pyresult;
}
#else
static void _cffi_f_GenImageGradientV(Image *result, int x0, int x1, Color *x2, Color *x3)
{
  { *result = GenImageGradientV(x0, x1, *x2, *x3); }
}
#endif

static Image _cffi_d_GenImagePerlinNoise(int x0, int x1, int x2, int x3, float x4)
{
  return GenImagePerlinNoise(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GenImagePerlinNoise(PyObject *self, PyObject *args)
{
  int x0;
  int x1;
  int x2;
  int x3;
  float x4;
  Image result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "GenImagePerlinNoise", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  x4 = (float)_cffi_to_c_float(arg4);
  if (x4 == (float)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GenImagePerlinNoise(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(16));
  return pyresult;
}
#else
static void _cffi_f_GenImagePerlinNoise(Image *result, int x0, int x1, int x2, int x3, float x4)
{
  { *result = GenImagePerlinNoise(x0, x1, x2, x3, x4); }
}
#endif

static Image _cffi_d_GenImageWhiteNoise(int x0, int x1, float x2)
{
  return GenImageWhiteNoise(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GenImageWhiteNoise(PyObject *self, PyObject *args)
{
  int x0;
  int x1;
  float x2;
  Image result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "GenImageWhiteNoise", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = (float)_cffi_to_c_float(arg2);
  if (x2 == (float)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GenImageWhiteNoise(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(16));
  return pyresult;
}
#else
static void _cffi_f_GenImageWhiteNoise(Image *result, int x0, int x1, float x2)
{
  { *result = GenImageWhiteNoise(x0, x1, x2); }
}
#endif

static Mesh _cffi_d_GenMeshCube(float x0, float x1, float x2)
{
  return GenMeshCube(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GenMeshCube(PyObject *self, PyObject *args)
{
  float x0;
  float x1;
  float x2;
  Mesh result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "GenMeshCube", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  x0 = (float)_cffi_to_c_float(arg0);
  if (x0 == (float)-1 && PyErr_Occurred())
    return NULL;

  x1 = (float)_cffi_to_c_float(arg1);
  if (x1 == (float)-1 && PyErr_Occurred())
    return NULL;

  x2 = (float)_cffi_to_c_float(arg2);
  if (x2 == (float)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GenMeshCube(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(6));
  return pyresult;
}
#else
static void _cffi_f_GenMeshCube(Mesh *result, float x0, float x1, float x2)
{
  { *result = GenMeshCube(x0, x1, x2); }
}
#endif

static Mesh _cffi_d_GenMeshCubicmap(Image x0, Vector3 x1)
{
  return GenMeshCubicmap(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GenMeshCubicmap(PyObject *self, PyObject *args)
{
  Image x0;
  Vector3 x1;
  Mesh result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "GenMeshCubicmap", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(16), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(28), arg1) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GenMeshCubicmap(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(6));
  return pyresult;
}
#else
static void _cffi_f_GenMeshCubicmap(Mesh *result, Image *x0, Vector3 *x1)
{
  { *result = GenMeshCubicmap(*x0, *x1); }
}
#endif

static Mesh _cffi_d_GenMeshCylinder(float x0, float x1, int x2)
{
  return GenMeshCylinder(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GenMeshCylinder(PyObject *self, PyObject *args)
{
  float x0;
  float x1;
  int x2;
  Mesh result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "GenMeshCylinder", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  x0 = (float)_cffi_to_c_float(arg0);
  if (x0 == (float)-1 && PyErr_Occurred())
    return NULL;

  x1 = (float)_cffi_to_c_float(arg1);
  if (x1 == (float)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GenMeshCylinder(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(6));
  return pyresult;
}
#else
static void _cffi_f_GenMeshCylinder(Mesh *result, float x0, float x1, int x2)
{
  { *result = GenMeshCylinder(x0, x1, x2); }
}
#endif

static Mesh _cffi_d_GenMeshHeightmap(Image x0, Vector3 x1)
{
  return GenMeshHeightmap(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GenMeshHeightmap(PyObject *self, PyObject *args)
{
  Image x0;
  Vector3 x1;
  Mesh result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "GenMeshHeightmap", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(16), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(28), arg1) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GenMeshHeightmap(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(6));
  return pyresult;
}
#else
static void _cffi_f_GenMeshHeightmap(Mesh *result, Image *x0, Vector3 *x1)
{
  { *result = GenMeshHeightmap(*x0, *x1); }
}
#endif

static Mesh _cffi_d_GenMeshHemiSphere(float x0, int x1, int x2)
{
  return GenMeshHemiSphere(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GenMeshHemiSphere(PyObject *self, PyObject *args)
{
  float x0;
  int x1;
  int x2;
  Mesh result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "GenMeshHemiSphere", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  x0 = (float)_cffi_to_c_float(arg0);
  if (x0 == (float)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GenMeshHemiSphere(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(6));
  return pyresult;
}
#else
static void _cffi_f_GenMeshHemiSphere(Mesh *result, float x0, int x1, int x2)
{
  { *result = GenMeshHemiSphere(x0, x1, x2); }
}
#endif

static Mesh _cffi_d_GenMeshKnot(float x0, float x1, int x2, int x3)
{
  return GenMeshKnot(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GenMeshKnot(PyObject *self, PyObject *args)
{
  float x0;
  float x1;
  int x2;
  int x3;
  Mesh result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "GenMeshKnot", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  x0 = (float)_cffi_to_c_float(arg0);
  if (x0 == (float)-1 && PyErr_Occurred())
    return NULL;

  x1 = (float)_cffi_to_c_float(arg1);
  if (x1 == (float)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GenMeshKnot(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(6));
  return pyresult;
}
#else
static void _cffi_f_GenMeshKnot(Mesh *result, float x0, float x1, int x2, int x3)
{
  { *result = GenMeshKnot(x0, x1, x2, x3); }
}
#endif

static Mesh _cffi_d_GenMeshPlane(float x0, float x1, int x2, int x3)
{
  return GenMeshPlane(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GenMeshPlane(PyObject *self, PyObject *args)
{
  float x0;
  float x1;
  int x2;
  int x3;
  Mesh result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "GenMeshPlane", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  x0 = (float)_cffi_to_c_float(arg0);
  if (x0 == (float)-1 && PyErr_Occurred())
    return NULL;

  x1 = (float)_cffi_to_c_float(arg1);
  if (x1 == (float)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GenMeshPlane(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(6));
  return pyresult;
}
#else
static void _cffi_f_GenMeshPlane(Mesh *result, float x0, float x1, int x2, int x3)
{
  { *result = GenMeshPlane(x0, x1, x2, x3); }
}
#endif

static Mesh _cffi_d_GenMeshPoly(int x0, float x1)
{
  return GenMeshPoly(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GenMeshPoly(PyObject *self, PyObject *args)
{
  int x0;
  float x1;
  Mesh result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "GenMeshPoly", 2, 2, &arg0, &arg1))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = (float)_cffi_to_c_float(arg1);
  if (x1 == (float)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GenMeshPoly(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(6));
  return pyresult;
}
#else
static void _cffi_f_GenMeshPoly(Mesh *result, int x0, float x1)
{
  { *result = GenMeshPoly(x0, x1); }
}
#endif

static Mesh _cffi_d_GenMeshSphere(float x0, int x1, int x2)
{
  return GenMeshSphere(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GenMeshSphere(PyObject *self, PyObject *args)
{
  float x0;
  int x1;
  int x2;
  Mesh result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "GenMeshSphere", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  x0 = (float)_cffi_to_c_float(arg0);
  if (x0 == (float)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GenMeshSphere(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(6));
  return pyresult;
}
#else
static void _cffi_f_GenMeshSphere(Mesh *result, float x0, int x1, int x2)
{
  { *result = GenMeshSphere(x0, x1, x2); }
}
#endif

static Mesh _cffi_d_GenMeshTorus(float x0, float x1, int x2, int x3)
{
  return GenMeshTorus(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GenMeshTorus(PyObject *self, PyObject *args)
{
  float x0;
  float x1;
  int x2;
  int x3;
  Mesh result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "GenMeshTorus", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  x0 = (float)_cffi_to_c_float(arg0);
  if (x0 == (float)-1 && PyErr_Occurred())
    return NULL;

  x1 = (float)_cffi_to_c_float(arg1);
  if (x1 == (float)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GenMeshTorus(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(6));
  return pyresult;
}
#else
static void _cffi_f_GenMeshTorus(Mesh *result, float x0, float x1, int x2, int x3)
{
  { *result = GenMeshTorus(x0, x1, x2, x3); }
}
#endif

static Texture2D _cffi_d_GenTextureBRDF(Shader x0, int x1)
{
  return GenTextureBRDF(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GenTextureBRDF(PyObject *self, PyObject *args)
{
  Shader x0;
  int x1;
  Texture2D result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "GenTextureBRDF", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(258), arg0) < 0)
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GenTextureBRDF(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(80));
  return pyresult;
}
#else
static void _cffi_f_GenTextureBRDF(Texture2D *result, Shader *x0, int x1)
{
  { *result = GenTextureBRDF(*x0, x1); }
}
#endif

static Texture2D _cffi_d_GenTextureCubemap(Shader x0, Texture2D x1, int x2)
{
  return GenTextureCubemap(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GenTextureCubemap(PyObject *self, PyObject *args)
{
  Shader x0;
  Texture2D x1;
  int x2;
  Texture2D result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "GenTextureCubemap", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(258), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(80), arg1) < 0)
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GenTextureCubemap(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(80));
  return pyresult;
}
#else
static void _cffi_f_GenTextureCubemap(Texture2D *result, Shader *x0, Texture2D *x1, int x2)
{
  { *result = GenTextureCubemap(*x0, *x1, x2); }
}
#endif

static Texture2D _cffi_d_GenTextureIrradiance(Shader x0, Texture2D x1, int x2)
{
  return GenTextureIrradiance(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GenTextureIrradiance(PyObject *self, PyObject *args)
{
  Shader x0;
  Texture2D x1;
  int x2;
  Texture2D result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "GenTextureIrradiance", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(258), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(80), arg1) < 0)
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GenTextureIrradiance(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(80));
  return pyresult;
}
#else
static void _cffi_f_GenTextureIrradiance(Texture2D *result, Shader *x0, Texture2D *x1, int x2)
{
  { *result = GenTextureIrradiance(*x0, *x1, x2); }
}
#endif

static void _cffi_d_GenTextureMipmaps(Texture2D * x0)
{
  GenTextureMipmaps(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GenTextureMipmaps(PyObject *self, PyObject *arg0)
{
  Texture2D * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(841), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Texture2D *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(841), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { GenTextureMipmaps(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_GenTextureMipmaps _cffi_d_GenTextureMipmaps
#endif

static Texture2D _cffi_d_GenTexturePrefilter(Shader x0, Texture2D x1, int x2)
{
  return GenTexturePrefilter(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GenTexturePrefilter(PyObject *self, PyObject *args)
{
  Shader x0;
  Texture2D x1;
  int x2;
  Texture2D result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "GenTexturePrefilter", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(258), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(80), arg1) < 0)
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GenTexturePrefilter(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(80));
  return pyresult;
}
#else
static void _cffi_f_GenTexturePrefilter(Texture2D *result, Shader *x0, Texture2D *x1, int x2)
{
  { *result = GenTexturePrefilter(*x0, *x1, x2); }
}
#endif

static Matrix _cffi_d_GetCameraMatrix(Camera3D x0)
{
  return GetCameraMatrix(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetCameraMatrix(PyObject *self, PyObject *arg0)
{
  Camera3D x0;
  Matrix result;
  PyObject *pyresult;

  if (_cffi_to_c((char *)&x0, _cffi_type(158), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetCameraMatrix(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(707));
  return pyresult;
}
#else
static void _cffi_f_GetCameraMatrix(Matrix *result, Camera3D *x0)
{
  { *result = GetCameraMatrix(*x0); }
}
#endif

static Matrix _cffi_d_GetCameraMatrix2D(Camera2D x0)
{
  return GetCameraMatrix2D(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetCameraMatrix2D(PyObject *self, PyObject *arg0)
{
  Camera2D x0;
  Matrix result;
  PyObject *pyresult;

  if (_cffi_to_c((char *)&x0, _cffi_type(155), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetCameraMatrix2D(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(707));
  return pyresult;
}
#else
static void _cffi_f_GetCameraMatrix2D(Matrix *result, Camera2D *x0)
{
  { *result = GetCameraMatrix2D(*x0); }
}
#endif

static char const * _cffi_d_GetClipboardText(void)
{
  return GetClipboardText();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetClipboardText(PyObject *self, PyObject *noarg)
{
  char const * result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetClipboardText(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(9));
  return pyresult;
}
#else
#  define _cffi_f_GetClipboardText _cffi_d_GetClipboardText
#endif

static int * _cffi_d_GetCodepoints(char const * x0, int * x1)
{
  return GetCodepoints(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetCodepoints(PyObject *self, PyObject *args)
{
  char const * x0;
  int * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "GetCodepoints", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(11), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(11), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetCodepoints(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(11));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_GetCodepoints _cffi_d_GetCodepoints
#endif

static int _cffi_d_GetCodepointsCount(char const * x0)
{
  return GetCodepointsCount(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetCodepointsCount(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetCodepointsCount(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_GetCodepointsCount _cffi_d_GetCodepointsCount
#endif

static RayHitInfo _cffi_d_GetCollisionRayGround(Ray x0, float x1)
{
  return GetCollisionRayGround(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetCollisionRayGround(PyObject *self, PyObject *args)
{
  Ray x0;
  float x1;
  RayHitInfo result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "GetCollisionRayGround", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(213), arg0) < 0)
    return NULL;

  x1 = (float)_cffi_to_c_float(arg1);
  if (x1 == (float)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetCollisionRayGround(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(1165));
  return pyresult;
}
#else
static void _cffi_f_GetCollisionRayGround(RayHitInfo *result, Ray *x0, float x1)
{
  { *result = GetCollisionRayGround(*x0, x1); }
}
#endif

static RayHitInfo _cffi_d_GetCollisionRayModel(Ray x0, Model x1)
{
  return GetCollisionRayModel(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetCollisionRayModel(PyObject *self, PyObject *args)
{
  Ray x0;
  Model x1;
  RayHitInfo result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "GetCollisionRayModel", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(213), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(214), arg1) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetCollisionRayModel(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(1165));
  return pyresult;
}
#else
static void _cffi_f_GetCollisionRayModel(RayHitInfo *result, Ray *x0, Model *x1)
{
  { *result = GetCollisionRayModel(*x0, *x1); }
}
#endif

static RayHitInfo _cffi_d_GetCollisionRayTriangle(Ray x0, Vector3 x1, Vector3 x2, Vector3 x3)
{
  return GetCollisionRayTriangle(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetCollisionRayTriangle(PyObject *self, PyObject *args)
{
  Ray x0;
  Vector3 x1;
  Vector3 x2;
  Vector3 x3;
  RayHitInfo result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "GetCollisionRayTriangle", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(213), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(28), arg1) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(28), arg2) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x3, _cffi_type(28), arg3) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetCollisionRayTriangle(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(1165));
  return pyresult;
}
#else
static void _cffi_f_GetCollisionRayTriangle(RayHitInfo *result, Ray *x0, Vector3 *x1, Vector3 *x2, Vector3 *x3)
{
  { *result = GetCollisionRayTriangle(*x0, *x1, *x2, *x3); }
}
#endif

static Rectangle _cffi_d_GetCollisionRec(Rectangle x0, Rectangle x1)
{
  return GetCollisionRec(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetCollisionRec(PyObject *self, PyObject *args)
{
  Rectangle x0;
  Rectangle x1;
  Rectangle result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "GetCollisionRec", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(77), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(77), arg1) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetCollisionRec(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(77));
  return pyresult;
}
#else
static void _cffi_f_GetCollisionRec(Rectangle *result, Rectangle *x0, Rectangle *x1)
{
  { *result = GetCollisionRec(*x0, *x1); }
}
#endif

static Color _cffi_d_GetColor(int x0)
{
  return GetColor(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetColor(PyObject *self, PyObject *arg0)
{
  int x0;
  Color result;
  PyObject *pyresult;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetColor(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(24));
  return pyresult;
}
#else
static void _cffi_f_GetColor(Color *result, int x0)
{
  { *result = GetColor(x0); }
}
#endif

static char * * _cffi_d_GetDirectoryFiles(char const * x0, int * x1)
{
  return GetDirectoryFiles(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetDirectoryFiles(PyObject *self, PyObject *args)
{
  char const * x0;
  int * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char * * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "GetDirectoryFiles", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(11), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(11), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetDirectoryFiles(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(1176));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_GetDirectoryFiles _cffi_d_GetDirectoryFiles
#endif

static char const * _cffi_d_GetDirectoryPath(char const * x0)
{
  return GetDirectoryPath(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetDirectoryPath(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char const * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetDirectoryPath(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(9));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_GetDirectoryPath _cffi_d_GetDirectoryPath
#endif

static char * * _cffi_d_GetDroppedFiles(int * x0)
{
  return GetDroppedFiles(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetDroppedFiles(PyObject *self, PyObject *arg0)
{
  int * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char * * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(11), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(11), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetDroppedFiles(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(1176));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_GetDroppedFiles _cffi_d_GetDroppedFiles
#endif

static char const * _cffi_d_GetExtension(char const * x0)
{
  return GetExtension(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetExtension(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char const * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetExtension(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(9));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_GetExtension _cffi_d_GetExtension
#endif

static int _cffi_d_GetFPS(void)
{
  return GetFPS();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetFPS(PyObject *self, PyObject *noarg)
{
  int result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetFPS(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_GetFPS _cffi_d_GetFPS
#endif

static long _cffi_d_GetFileModTime(char const * x0)
{
  return GetFileModTime(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetFileModTime(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  long result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetFileModTime(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, long);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_GetFileModTime _cffi_d_GetFileModTime
#endif

static char const * _cffi_d_GetFileName(char const * x0)
{
  return GetFileName(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetFileName(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char const * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetFileName(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(9));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_GetFileName _cffi_d_GetFileName
#endif

static char const * _cffi_d_GetFileNameWithoutExt(char const * x0)
{
  return GetFileNameWithoutExt(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetFileNameWithoutExt(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char const * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetFileNameWithoutExt(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(9));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_GetFileNameWithoutExt _cffi_d_GetFileNameWithoutExt
#endif

static Font _cffi_d_GetFontDefault(void)
{
  return GetFontDefault();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetFontDefault(PyObject *self, PyObject *noarg)
{
  Font result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetFontDefault(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(66));
  return pyresult;
}
#else
static void _cffi_f_GetFontDefault(Font *result)
{
  { *result = GetFontDefault(); }
}
#endif

static float _cffi_d_GetFrameTime(void)
{
  return GetFrameTime();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetFrameTime(PyObject *self, PyObject *noarg)
{
  float result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetFrameTime(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_float(result);
  return pyresult;
}
#else
#  define _cffi_f_GetFrameTime _cffi_d_GetFrameTime
#endif

static int _cffi_d_GetGamepadAxisCount(int x0)
{
  return GetGamepadAxisCount(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetGamepadAxisCount(PyObject *self, PyObject *arg0)
{
  int x0;
  int result;
  PyObject *pyresult;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetGamepadAxisCount(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_GetGamepadAxisCount _cffi_d_GetGamepadAxisCount
#endif

static float _cffi_d_GetGamepadAxisMovement(int x0, int x1)
{
  return GetGamepadAxisMovement(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetGamepadAxisMovement(PyObject *self, PyObject *args)
{
  int x0;
  int x1;
  float result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "GetGamepadAxisMovement", 2, 2, &arg0, &arg1))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetGamepadAxisMovement(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_float(result);
  return pyresult;
}
#else
#  define _cffi_f_GetGamepadAxisMovement _cffi_d_GetGamepadAxisMovement
#endif

static int _cffi_d_GetGamepadButtonPressed(void)
{
  return GetGamepadButtonPressed();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetGamepadButtonPressed(PyObject *self, PyObject *noarg)
{
  int result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetGamepadButtonPressed(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_GetGamepadButtonPressed _cffi_d_GetGamepadButtonPressed
#endif

static char const * _cffi_d_GetGamepadName(int x0)
{
  return GetGamepadName(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetGamepadName(PyObject *self, PyObject *arg0)
{
  int x0;
  char const * result;
  PyObject *pyresult;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetGamepadName(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(9));
  return pyresult;
}
#else
#  define _cffi_f_GetGamepadName _cffi_d_GetGamepadName
#endif

static int _cffi_d_GetGestureDetected(void)
{
  return GetGestureDetected();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetGestureDetected(PyObject *self, PyObject *noarg)
{
  int result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetGestureDetected(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_GetGestureDetected _cffi_d_GetGestureDetected
#endif

static float _cffi_d_GetGestureDragAngle(void)
{
  return GetGestureDragAngle();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetGestureDragAngle(PyObject *self, PyObject *noarg)
{
  float result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetGestureDragAngle(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_float(result);
  return pyresult;
}
#else
#  define _cffi_f_GetGestureDragAngle _cffi_d_GetGestureDragAngle
#endif

static Vector2 _cffi_d_GetGestureDragVector(void)
{
  return GetGestureDragVector();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetGestureDragVector(PyObject *self, PyObject *noarg)
{
  Vector2 result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetGestureDragVector(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(209));
  return pyresult;
}
#else
static void _cffi_f_GetGestureDragVector(Vector2 *result)
{
  { *result = GetGestureDragVector(); }
}
#endif

static float _cffi_d_GetGestureHoldDuration(void)
{
  return GetGestureHoldDuration();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetGestureHoldDuration(PyObject *self, PyObject *noarg)
{
  float result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetGestureHoldDuration(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_float(result);
  return pyresult;
}
#else
#  define _cffi_f_GetGestureHoldDuration _cffi_d_GetGestureHoldDuration
#endif

static float _cffi_d_GetGesturePinchAngle(void)
{
  return GetGesturePinchAngle();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetGesturePinchAngle(PyObject *self, PyObject *noarg)
{
  float result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetGesturePinchAngle(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_float(result);
  return pyresult;
}
#else
#  define _cffi_f_GetGesturePinchAngle _cffi_d_GetGesturePinchAngle
#endif

static Vector2 _cffi_d_GetGesturePinchVector(void)
{
  return GetGesturePinchVector();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetGesturePinchVector(PyObject *self, PyObject *noarg)
{
  Vector2 result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetGesturePinchVector(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(209));
  return pyresult;
}
#else
static void _cffi_f_GetGesturePinchVector(Vector2 *result)
{
  { *result = GetGesturePinchVector(); }
}
#endif

static int _cffi_d_GetGlyphIndex(Font x0, int x1)
{
  return GetGlyphIndex(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetGlyphIndex(PyObject *self, PyObject *args)
{
  Font x0;
  int x1;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "GetGlyphIndex", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(66), arg0) < 0)
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetGlyphIndex(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
static int _cffi_f_GetGlyphIndex(Font *x0, int x1)
{
  int result;
  { result = GetGlyphIndex(*x0, x1); }
  return result;
}
#endif

static Rectangle _cffi_d_GetImageAlphaBorder(Image x0, float x1)
{
  return GetImageAlphaBorder(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetImageAlphaBorder(PyObject *self, PyObject *args)
{
  Image x0;
  float x1;
  Rectangle result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "GetImageAlphaBorder", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(16), arg0) < 0)
    return NULL;

  x1 = (float)_cffi_to_c_float(arg1);
  if (x1 == (float)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetImageAlphaBorder(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(77));
  return pyresult;
}
#else
static void _cffi_f_GetImageAlphaBorder(Rectangle *result, Image *x0, float x1)
{
  { *result = GetImageAlphaBorder(*x0, x1); }
}
#endif

static Color * _cffi_d_GetImageData(Image x0)
{
  return GetImageData(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetImageData(PyObject *self, PyObject *arg0)
{
  Image x0;
  Color * result;
  PyObject *pyresult;

  if (_cffi_to_c((char *)&x0, _cffi_type(16), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetImageData(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(61));
  return pyresult;
}
#else
static Color * _cffi_f_GetImageData(Image *x0)
{
  Color * result;
  { result = GetImageData(*x0); }
  return result;
}
#endif

static Vector4 * _cffi_d_GetImageDataNormalized(Image x0)
{
  return GetImageDataNormalized(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetImageDataNormalized(PyObject *self, PyObject *arg0)
{
  Image x0;
  Vector4 * result;
  PyObject *pyresult;

  if (_cffi_to_c((char *)&x0, _cffi_type(16), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetImageDataNormalized(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(1175));
  return pyresult;
}
#else
static Vector4 * _cffi_f_GetImageDataNormalized(Image *x0)
{
  Vector4 * result;
  { result = GetImageDataNormalized(*x0); }
  return result;
}
#endif

static int _cffi_d_GetKeyPressed(void)
{
  return GetKeyPressed();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetKeyPressed(PyObject *self, PyObject *noarg)
{
  int result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetKeyPressed(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_GetKeyPressed _cffi_d_GetKeyPressed
#endif

static Matrix _cffi_d_GetMatrixModelview(void)
{
  return GetMatrixModelview();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetMatrixModelview(PyObject *self, PyObject *noarg)
{
  Matrix result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetMatrixModelview(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(707));
  return pyresult;
}
#else
static void _cffi_f_GetMatrixModelview(Matrix *result)
{
  { *result = GetMatrixModelview(); }
}
#endif

static Matrix _cffi_d_GetMatrixProjection(void)
{
  return GetMatrixProjection();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetMatrixProjection(PyObject *self, PyObject *noarg)
{
  Matrix result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetMatrixProjection(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(707));
  return pyresult;
}
#else
static void _cffi_f_GetMatrixProjection(Matrix *result)
{
  { *result = GetMatrixProjection(); }
}
#endif

static int _cffi_d_GetMonitorCount(void)
{
  return GetMonitorCount();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetMonitorCount(PyObject *self, PyObject *noarg)
{
  int result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetMonitorCount(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_GetMonitorCount _cffi_d_GetMonitorCount
#endif

static int _cffi_d_GetMonitorHeight(int x0)
{
  return GetMonitorHeight(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetMonitorHeight(PyObject *self, PyObject *arg0)
{
  int x0;
  int result;
  PyObject *pyresult;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetMonitorHeight(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_GetMonitorHeight _cffi_d_GetMonitorHeight
#endif

static char const * _cffi_d_GetMonitorName(int x0)
{
  return GetMonitorName(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetMonitorName(PyObject *self, PyObject *arg0)
{
  int x0;
  char const * result;
  PyObject *pyresult;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetMonitorName(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(9));
  return pyresult;
}
#else
#  define _cffi_f_GetMonitorName _cffi_d_GetMonitorName
#endif

static int _cffi_d_GetMonitorPhysicalHeight(int x0)
{
  return GetMonitorPhysicalHeight(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetMonitorPhysicalHeight(PyObject *self, PyObject *arg0)
{
  int x0;
  int result;
  PyObject *pyresult;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetMonitorPhysicalHeight(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_GetMonitorPhysicalHeight _cffi_d_GetMonitorPhysicalHeight
#endif

static int _cffi_d_GetMonitorPhysicalWidth(int x0)
{
  return GetMonitorPhysicalWidth(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetMonitorPhysicalWidth(PyObject *self, PyObject *arg0)
{
  int x0;
  int result;
  PyObject *pyresult;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetMonitorPhysicalWidth(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_GetMonitorPhysicalWidth _cffi_d_GetMonitorPhysicalWidth
#endif

static int _cffi_d_GetMonitorWidth(int x0)
{
  return GetMonitorWidth(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetMonitorWidth(PyObject *self, PyObject *arg0)
{
  int x0;
  int result;
  PyObject *pyresult;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetMonitorWidth(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_GetMonitorWidth _cffi_d_GetMonitorWidth
#endif

static Vector2 _cffi_d_GetMousePosition(void)
{
  return GetMousePosition();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetMousePosition(PyObject *self, PyObject *noarg)
{
  Vector2 result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetMousePosition(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(209));
  return pyresult;
}
#else
static void _cffi_f_GetMousePosition(Vector2 *result)
{
  { *result = GetMousePosition(); }
}
#endif

static Ray _cffi_d_GetMouseRay(Vector2 x0, Camera3D x1)
{
  return GetMouseRay(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetMouseRay(PyObject *self, PyObject *args)
{
  Vector2 x0;
  Camera3D x1;
  Ray result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "GetMouseRay", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(209), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(158), arg1) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetMouseRay(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(213));
  return pyresult;
}
#else
static void _cffi_f_GetMouseRay(Ray *result, Vector2 *x0, Camera3D *x1)
{
  { *result = GetMouseRay(*x0, *x1); }
}
#endif

static int _cffi_d_GetMouseWheelMove(void)
{
  return GetMouseWheelMove();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetMouseWheelMove(PyObject *self, PyObject *noarg)
{
  int result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetMouseWheelMove(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_GetMouseWheelMove _cffi_d_GetMouseWheelMove
#endif

static int _cffi_d_GetMouseX(void)
{
  return GetMouseX();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetMouseX(PyObject *self, PyObject *noarg)
{
  int result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetMouseX(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_GetMouseX _cffi_d_GetMouseX
#endif

static int _cffi_d_GetMouseY(void)
{
  return GetMouseY();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetMouseY(PyObject *self, PyObject *noarg)
{
  int result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetMouseY(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_GetMouseY _cffi_d_GetMouseY
#endif

static float _cffi_d_GetMusicTimeLength(Music x0)
{
  return GetMusicTimeLength(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetMusicTimeLength(PyObject *self, PyObject *arg0)
{
  Music x0;
  float result;
  PyObject *pyresult;

  if (_cffi_to_c((char *)&x0, _cffi_type(371), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetMusicTimeLength(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_float(result);
  return pyresult;
}
#else
static float _cffi_f_GetMusicTimeLength(Music *x0)
{
  float result;
  { result = GetMusicTimeLength(*x0); }
  return result;
}
#endif

static float _cffi_d_GetMusicTimePlayed(Music x0)
{
  return GetMusicTimePlayed(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetMusicTimePlayed(PyObject *self, PyObject *arg0)
{
  Music x0;
  float result;
  PyObject *pyresult;

  if (_cffi_to_c((char *)&x0, _cffi_type(371), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetMusicTimePlayed(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_float(result);
  return pyresult;
}
#else
static float _cffi_f_GetMusicTimePlayed(Music *x0)
{
  float result;
  { result = GetMusicTimePlayed(*x0); }
  return result;
}
#endif

static int _cffi_d_GetNextCodepoint(char const * x0, int * x1)
{
  return GetNextCodepoint(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetNextCodepoint(PyObject *self, PyObject *args)
{
  char const * x0;
  int * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "GetNextCodepoint", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(11), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(11), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetNextCodepoint(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_GetNextCodepoint _cffi_d_GetNextCodepoint
#endif

static int _cffi_d_GetPixelDataSize(int x0, int x1, int x2)
{
  return GetPixelDataSize(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetPixelDataSize(PyObject *self, PyObject *args)
{
  int x0;
  int x1;
  int x2;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "GetPixelDataSize", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetPixelDataSize(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_GetPixelDataSize _cffi_d_GetPixelDataSize
#endif

static char const * _cffi_d_GetPrevDirectoryPath(char const * x0)
{
  return GetPrevDirectoryPath(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetPrevDirectoryPath(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char const * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetPrevDirectoryPath(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(9));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_GetPrevDirectoryPath _cffi_d_GetPrevDirectoryPath
#endif

static int _cffi_d_GetRandomValue(int x0, int x1)
{
  return GetRandomValue(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetRandomValue(PyObject *self, PyObject *args)
{
  int x0;
  int x1;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "GetRandomValue", 2, 2, &arg0, &arg1))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetRandomValue(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_GetRandomValue _cffi_d_GetRandomValue
#endif

static Image _cffi_d_GetScreenData(void)
{
  return GetScreenData();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetScreenData(PyObject *self, PyObject *noarg)
{
  Image result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetScreenData(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(16));
  return pyresult;
}
#else
static void _cffi_f_GetScreenData(Image *result)
{
  { *result = GetScreenData(); }
}
#endif

static int _cffi_d_GetScreenHeight(void)
{
  return GetScreenHeight();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetScreenHeight(PyObject *self, PyObject *noarg)
{
  int result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetScreenHeight(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_GetScreenHeight _cffi_d_GetScreenHeight
#endif

static Vector2 _cffi_d_GetScreenToWorld2D(Vector2 x0, Camera2D x1)
{
  return GetScreenToWorld2D(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetScreenToWorld2D(PyObject *self, PyObject *args)
{
  Vector2 x0;
  Camera2D x1;
  Vector2 result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "GetScreenToWorld2D", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(209), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(155), arg1) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetScreenToWorld2D(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(209));
  return pyresult;
}
#else
static void _cffi_f_GetScreenToWorld2D(Vector2 *result, Vector2 *x0, Camera2D *x1)
{
  { *result = GetScreenToWorld2D(*x0, *x1); }
}
#endif

static int _cffi_d_GetScreenWidth(void)
{
  return GetScreenWidth();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetScreenWidth(PyObject *self, PyObject *noarg)
{
  int result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetScreenWidth(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_GetScreenWidth _cffi_d_GetScreenWidth
#endif

static Shader _cffi_d_GetShaderDefault(void)
{
  return GetShaderDefault();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetShaderDefault(PyObject *self, PyObject *noarg)
{
  Shader result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetShaderDefault(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(258));
  return pyresult;
}
#else
static void _cffi_f_GetShaderDefault(Shader *result)
{
  { *result = GetShaderDefault(); }
}
#endif

static int _cffi_d_GetShaderLocation(Shader x0, char const * x1)
{
  return GetShaderLocation(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetShaderLocation(PyObject *self, PyObject *args)
{
  Shader x0;
  char const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "GetShaderLocation", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(258), arg0) < 0)
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetShaderLocation(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
static int _cffi_f_GetShaderLocation(Shader *x0, char const * x1)
{
  int result;
  { result = GetShaderLocation(*x0, x1); }
  return result;
}
#endif

static int _cffi_d_GetSoundsPlaying(void)
{
  return GetSoundsPlaying();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetSoundsPlaying(PyObject *self, PyObject *noarg)
{
  int result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetSoundsPlaying(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_GetSoundsPlaying _cffi_d_GetSoundsPlaying
#endif

static Image _cffi_d_GetTextureData(Texture2D x0)
{
  return GetTextureData(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetTextureData(PyObject *self, PyObject *arg0)
{
  Texture2D x0;
  Image result;
  PyObject *pyresult;

  if (_cffi_to_c((char *)&x0, _cffi_type(80), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetTextureData(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(16));
  return pyresult;
}
#else
static void _cffi_f_GetTextureData(Image *result, Texture2D *x0)
{
  { *result = GetTextureData(*x0); }
}
#endif

static Texture2D _cffi_d_GetTextureDefault(void)
{
  return GetTextureDefault();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetTextureDefault(PyObject *self, PyObject *noarg)
{
  Texture2D result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetTextureDefault(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(80));
  return pyresult;
}
#else
static void _cffi_f_GetTextureDefault(Texture2D *result)
{
  { *result = GetTextureDefault(); }
}
#endif

static double _cffi_d_GetTime(void)
{
  return GetTime();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetTime(PyObject *self, PyObject *noarg)
{
  double result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetTime(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_double(result);
  return pyresult;
}
#else
#  define _cffi_f_GetTime _cffi_d_GetTime
#endif

static int _cffi_d_GetTouchPointsCount(void)
{
  return GetTouchPointsCount();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetTouchPointsCount(PyObject *self, PyObject *noarg)
{
  int result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetTouchPointsCount(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_GetTouchPointsCount _cffi_d_GetTouchPointsCount
#endif

static Vector2 _cffi_d_GetTouchPosition(int x0)
{
  return GetTouchPosition(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetTouchPosition(PyObject *self, PyObject *arg0)
{
  int x0;
  Vector2 result;
  PyObject *pyresult;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetTouchPosition(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(209));
  return pyresult;
}
#else
static void _cffi_f_GetTouchPosition(Vector2 *result, int x0)
{
  { *result = GetTouchPosition(x0); }
}
#endif

static int _cffi_d_GetTouchX(void)
{
  return GetTouchX();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetTouchX(PyObject *self, PyObject *noarg)
{
  int result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetTouchX(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_GetTouchX _cffi_d_GetTouchX
#endif

static int _cffi_d_GetTouchY(void)
{
  return GetTouchY();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetTouchY(PyObject *self, PyObject *noarg)
{
  int result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetTouchY(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_GetTouchY _cffi_d_GetTouchY
#endif

static float * _cffi_d_GetWaveData(Wave x0)
{
  return GetWaveData(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetWaveData(PyObject *self, PyObject *arg0)
{
  Wave x0;
  float * result;
  PyObject *pyresult;

  if (_cffi_to_c((char *)&x0, _cffi_type(245), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetWaveData(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(1181));
  return pyresult;
}
#else
static float * _cffi_f_GetWaveData(Wave *x0)
{
  float * result;
  { result = GetWaveData(*x0); }
  return result;
}
#endif

static void * _cffi_d_GetWindowHandle(void)
{
  return GetWindowHandle();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetWindowHandle(PyObject *self, PyObject *noarg)
{
  void * result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetWindowHandle(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(141));
  return pyresult;
}
#else
#  define _cffi_f_GetWindowHandle _cffi_d_GetWindowHandle
#endif

static Vector2 _cffi_d_GetWindowPosition(void)
{
  return GetWindowPosition();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetWindowPosition(PyObject *self, PyObject *noarg)
{
  Vector2 result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetWindowPosition(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(209));
  return pyresult;
}
#else
static void _cffi_f_GetWindowPosition(Vector2 *result)
{
  { *result = GetWindowPosition(); }
}
#endif

static char const * _cffi_d_GetWorkingDirectory(void)
{
  return GetWorkingDirectory();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetWorkingDirectory(PyObject *self, PyObject *noarg)
{
  char const * result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetWorkingDirectory(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(9));
  return pyresult;
}
#else
#  define _cffi_f_GetWorkingDirectory _cffi_d_GetWorkingDirectory
#endif

static Vector2 _cffi_d_GetWorldToScreen(Vector3 x0, Camera3D x1)
{
  return GetWorldToScreen(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetWorldToScreen(PyObject *self, PyObject *args)
{
  Vector3 x0;
  Camera3D x1;
  Vector2 result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "GetWorldToScreen", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(28), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(158), arg1) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetWorldToScreen(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(209));
  return pyresult;
}
#else
static void _cffi_f_GetWorldToScreen(Vector2 *result, Vector3 *x0, Camera3D *x1)
{
  { *result = GetWorldToScreen(*x0, *x1); }
}
#endif

static Vector2 _cffi_d_GetWorldToScreen2D(Vector2 x0, Camera2D x1)
{
  return GetWorldToScreen2D(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetWorldToScreen2D(PyObject *self, PyObject *args)
{
  Vector2 x0;
  Camera2D x1;
  Vector2 result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "GetWorldToScreen2D", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(209), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(155), arg1) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetWorldToScreen2D(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(209));
  return pyresult;
}
#else
static void _cffi_f_GetWorldToScreen2D(Vector2 *result, Vector2 *x0, Camera2D *x1)
{
  { *result = GetWorldToScreen2D(*x0, *x1); }
}
#endif

static Vector2 _cffi_d_GetWorldToScreenEx(Vector3 x0, Camera3D x1, int x2, int x3)
{
  return GetWorldToScreenEx(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_GetWorldToScreenEx(PyObject *self, PyObject *args)
{
  Vector3 x0;
  Camera3D x1;
  int x2;
  int x3;
  Vector2 result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "GetWorldToScreenEx", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(28), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(158), arg1) < 0)
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = GetWorldToScreenEx(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(209));
  return pyresult;
}
#else
static void _cffi_f_GetWorldToScreenEx(Vector2 *result, Vector3 *x0, Camera3D *x1, int x2, int x3)
{
  { *result = GetWorldToScreenEx(*x0, *x1, x2, x3); }
}
#endif

static void _cffi_d_HideCursor(void)
{
  HideCursor();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_HideCursor(PyObject *self, PyObject *noarg)
{

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { HideCursor(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_HideCursor _cffi_d_HideCursor
#endif

static void _cffi_d_HideWindow(void)
{
  HideWindow();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_HideWindow(PyObject *self, PyObject *noarg)
{

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { HideWindow(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_HideWindow _cffi_d_HideWindow
#endif

static void _cffi_d_ImageAlphaClear(Image * x0, Color x1, float x2)
{
  ImageAlphaClear(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ImageAlphaClear(PyObject *self, PyObject *args)
{
  Image * x0;
  Color x1;
  float x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "ImageAlphaClear", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(605), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Image *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(605), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  if (_cffi_to_c((char *)&x1, _cffi_type(24), arg1) < 0)
    return NULL;

  x2 = (float)_cffi_to_c_float(arg2);
  if (x2 == (float)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ImageAlphaClear(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_ImageAlphaClear(Image * x0, Color *x1, float x2)
{
  { ImageAlphaClear(x0, *x1, x2); }
}
#endif

static void _cffi_d_ImageAlphaCrop(Image * x0, float x1)
{
  ImageAlphaCrop(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ImageAlphaCrop(PyObject *self, PyObject *args)
{
  Image * x0;
  float x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ImageAlphaCrop", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(605), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Image *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(605), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = (float)_cffi_to_c_float(arg1);
  if (x1 == (float)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ImageAlphaCrop(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ImageAlphaCrop _cffi_d_ImageAlphaCrop
#endif

static void _cffi_d_ImageAlphaMask(Image * x0, Image x1)
{
  ImageAlphaMask(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ImageAlphaMask(PyObject *self, PyObject *args)
{
  Image * x0;
  Image x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ImageAlphaMask", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(605), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Image *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(605), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  if (_cffi_to_c((char *)&x1, _cffi_type(16), arg1) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ImageAlphaMask(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_ImageAlphaMask(Image * x0, Image *x1)
{
  { ImageAlphaMask(x0, *x1); }
}
#endif

static void _cffi_d_ImageAlphaPremultiply(Image * x0)
{
  ImageAlphaPremultiply(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ImageAlphaPremultiply(PyObject *self, PyObject *arg0)
{
  Image * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(605), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Image *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(605), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ImageAlphaPremultiply(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ImageAlphaPremultiply _cffi_d_ImageAlphaPremultiply
#endif

static void _cffi_d_ImageColorBrightness(Image * x0, int x1)
{
  ImageColorBrightness(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ImageColorBrightness(PyObject *self, PyObject *args)
{
  Image * x0;
  int x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ImageColorBrightness", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(605), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Image *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(605), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ImageColorBrightness(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ImageColorBrightness _cffi_d_ImageColorBrightness
#endif

static void _cffi_d_ImageColorContrast(Image * x0, float x1)
{
  ImageColorContrast(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ImageColorContrast(PyObject *self, PyObject *args)
{
  Image * x0;
  float x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ImageColorContrast", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(605), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Image *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(605), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = (float)_cffi_to_c_float(arg1);
  if (x1 == (float)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ImageColorContrast(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ImageColorContrast _cffi_d_ImageColorContrast
#endif

static void _cffi_d_ImageColorGrayscale(Image * x0)
{
  ImageColorGrayscale(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ImageColorGrayscale(PyObject *self, PyObject *arg0)
{
  Image * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(605), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Image *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(605), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ImageColorGrayscale(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ImageColorGrayscale _cffi_d_ImageColorGrayscale
#endif

static void _cffi_d_ImageColorInvert(Image * x0)
{
  ImageColorInvert(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ImageColorInvert(PyObject *self, PyObject *arg0)
{
  Image * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(605), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Image *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(605), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ImageColorInvert(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ImageColorInvert _cffi_d_ImageColorInvert
#endif

static void _cffi_d_ImageColorReplace(Image * x0, Color x1, Color x2)
{
  ImageColorReplace(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ImageColorReplace(PyObject *self, PyObject *args)
{
  Image * x0;
  Color x1;
  Color x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "ImageColorReplace", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(605), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Image *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(605), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  if (_cffi_to_c((char *)&x1, _cffi_type(24), arg1) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(24), arg2) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ImageColorReplace(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_ImageColorReplace(Image * x0, Color *x1, Color *x2)
{
  { ImageColorReplace(x0, *x1, *x2); }
}
#endif

static void _cffi_d_ImageColorTint(Image * x0, Color x1)
{
  ImageColorTint(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ImageColorTint(PyObject *self, PyObject *args)
{
  Image * x0;
  Color x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ImageColorTint", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(605), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Image *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(605), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  if (_cffi_to_c((char *)&x1, _cffi_type(24), arg1) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ImageColorTint(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_ImageColorTint(Image * x0, Color *x1)
{
  { ImageColorTint(x0, *x1); }
}
#endif

static Image _cffi_d_ImageCopy(Image x0)
{
  return ImageCopy(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ImageCopy(PyObject *self, PyObject *arg0)
{
  Image x0;
  Image result;
  PyObject *pyresult;

  if (_cffi_to_c((char *)&x0, _cffi_type(16), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ImageCopy(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(16));
  return pyresult;
}
#else
static void _cffi_f_ImageCopy(Image *result, Image *x0)
{
  { *result = ImageCopy(*x0); }
}
#endif

static void _cffi_d_ImageCrop(Image * x0, Rectangle x1)
{
  ImageCrop(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ImageCrop(PyObject *self, PyObject *args)
{
  Image * x0;
  Rectangle x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ImageCrop", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(605), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Image *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(605), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  if (_cffi_to_c((char *)&x1, _cffi_type(77), arg1) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ImageCrop(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_ImageCrop(Image * x0, Rectangle *x1)
{
  { ImageCrop(x0, *x1); }
}
#endif

static void _cffi_d_ImageDither(Image * x0, int x1, int x2, int x3, int x4)
{
  ImageDither(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ImageDither(PyObject *self, PyObject *args)
{
  Image * x0;
  int x1;
  int x2;
  int x3;
  int x4;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "ImageDither", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(605), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Image *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(605), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  x4 = _cffi_to_c_int(arg4, int);
  if (x4 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ImageDither(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ImageDither _cffi_d_ImageDither
#endif

static void _cffi_d_ImageDraw(Image * x0, Image x1, Rectangle x2, Rectangle x3, Color x4)
{
  ImageDraw(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ImageDraw(PyObject *self, PyObject *args)
{
  Image * x0;
  Image x1;
  Rectangle x2;
  Rectangle x3;
  Color x4;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "ImageDraw", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(605), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Image *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(605), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  if (_cffi_to_c((char *)&x1, _cffi_type(16), arg1) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(77), arg2) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x3, _cffi_type(77), arg3) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x4, _cffi_type(24), arg4) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ImageDraw(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_ImageDraw(Image * x0, Image *x1, Rectangle *x2, Rectangle *x3, Color *x4)
{
  { ImageDraw(x0, *x1, *x2, *x3, *x4); }
}
#endif

static void _cffi_d_ImageDrawRectangle(Image * x0, Rectangle x1, Color x2)
{
  ImageDrawRectangle(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ImageDrawRectangle(PyObject *self, PyObject *args)
{
  Image * x0;
  Rectangle x1;
  Color x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "ImageDrawRectangle", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(605), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Image *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(605), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  if (_cffi_to_c((char *)&x1, _cffi_type(77), arg1) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(24), arg2) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ImageDrawRectangle(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_ImageDrawRectangle(Image * x0, Rectangle *x1, Color *x2)
{
  { ImageDrawRectangle(x0, *x1, *x2); }
}
#endif

static void _cffi_d_ImageDrawRectangleLines(Image * x0, Rectangle x1, int x2, Color x3)
{
  ImageDrawRectangleLines(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ImageDrawRectangleLines(PyObject *self, PyObject *args)
{
  Image * x0;
  Rectangle x1;
  int x2;
  Color x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "ImageDrawRectangleLines", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(605), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Image *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(605), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  if (_cffi_to_c((char *)&x1, _cffi_type(77), arg1) < 0)
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x3, _cffi_type(24), arg3) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ImageDrawRectangleLines(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_ImageDrawRectangleLines(Image * x0, Rectangle *x1, int x2, Color *x3)
{
  { ImageDrawRectangleLines(x0, *x1, x2, *x3); }
}
#endif

static void _cffi_d_ImageDrawText(Image * x0, Vector2 x1, char const * x2, int x3, Color x4)
{
  ImageDrawText(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ImageDrawText(PyObject *self, PyObject *args)
{
  Image * x0;
  Vector2 x1;
  char const * x2;
  int x3;
  Color x4;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "ImageDrawText", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(605), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Image *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(605), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  if (_cffi_to_c((char *)&x1, _cffi_type(209), arg1) < 0)
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x4, _cffi_type(24), arg4) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ImageDrawText(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_ImageDrawText(Image * x0, Vector2 *x1, char const * x2, int x3, Color *x4)
{
  { ImageDrawText(x0, *x1, x2, x3, *x4); }
}
#endif

static void _cffi_d_ImageDrawTextEx(Image * x0, Vector2 x1, Font x2, char const * x3, float x4, float x5, Color x6)
{
  ImageDrawTextEx(x0, x1, x2, x3, x4, x5, x6);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ImageDrawTextEx(PyObject *self, PyObject *args)
{
  Image * x0;
  Vector2 x1;
  Font x2;
  char const * x3;
  float x4;
  float x5;
  Color x6;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;
  PyObject *arg6;

  if (!PyArg_UnpackTuple(args, "ImageDrawTextEx", 7, 7, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5, &arg6))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(605), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Image *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(605), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  if (_cffi_to_c((char *)&x1, _cffi_type(209), arg1) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(66), arg2) < 0)
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x4 = (float)_cffi_to_c_float(arg4);
  if (x4 == (float)-1 && PyErr_Occurred())
    return NULL;

  x5 = (float)_cffi_to_c_float(arg5);
  if (x5 == (float)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x6, _cffi_type(24), arg6) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ImageDrawTextEx(x0, x1, x2, x3, x4, x5, x6); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_ImageDrawTextEx(Image * x0, Vector2 *x1, Font *x2, char const * x3, float x4, float x5, Color *x6)
{
  { ImageDrawTextEx(x0, *x1, *x2, x3, x4, x5, *x6); }
}
#endif

static Color * _cffi_d_ImageExtractPalette(Image x0, int x1, int * x2)
{
  return ImageExtractPalette(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ImageExtractPalette(PyObject *self, PyObject *args)
{
  Image x0;
  int x1;
  int * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  Color * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "ImageExtractPalette", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(16), arg0) < 0)
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(11), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(11), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ImageExtractPalette(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(61));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
static Color * _cffi_f_ImageExtractPalette(Image *x0, int x1, int * x2)
{
  Color * result;
  { result = ImageExtractPalette(*x0, x1, x2); }
  return result;
}
#endif

static void _cffi_d_ImageFlipHorizontal(Image * x0)
{
  ImageFlipHorizontal(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ImageFlipHorizontal(PyObject *self, PyObject *arg0)
{
  Image * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(605), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Image *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(605), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ImageFlipHorizontal(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ImageFlipHorizontal _cffi_d_ImageFlipHorizontal
#endif

static void _cffi_d_ImageFlipVertical(Image * x0)
{
  ImageFlipVertical(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ImageFlipVertical(PyObject *self, PyObject *arg0)
{
  Image * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(605), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Image *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(605), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ImageFlipVertical(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ImageFlipVertical _cffi_d_ImageFlipVertical
#endif

static void _cffi_d_ImageFormat(Image * x0, int x1)
{
  ImageFormat(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ImageFormat(PyObject *self, PyObject *args)
{
  Image * x0;
  int x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ImageFormat", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(605), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Image *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(605), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ImageFormat(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ImageFormat _cffi_d_ImageFormat
#endif

static Image _cffi_d_ImageFromImage(Image x0, Rectangle x1)
{
  return ImageFromImage(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ImageFromImage(PyObject *self, PyObject *args)
{
  Image x0;
  Rectangle x1;
  Image result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ImageFromImage", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(16), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(77), arg1) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ImageFromImage(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(16));
  return pyresult;
}
#else
static void _cffi_f_ImageFromImage(Image *result, Image *x0, Rectangle *x1)
{
  { *result = ImageFromImage(*x0, *x1); }
}
#endif

static void _cffi_d_ImageMipmaps(Image * x0)
{
  ImageMipmaps(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ImageMipmaps(PyObject *self, PyObject *arg0)
{
  Image * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(605), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Image *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(605), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ImageMipmaps(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ImageMipmaps _cffi_d_ImageMipmaps
#endif

static void _cffi_d_ImageResize(Image * x0, int x1, int x2)
{
  ImageResize(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ImageResize(PyObject *self, PyObject *args)
{
  Image * x0;
  int x1;
  int x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "ImageResize", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(605), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Image *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(605), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ImageResize(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ImageResize _cffi_d_ImageResize
#endif

static void _cffi_d_ImageResizeCanvas(Image * x0, int x1, int x2, int x3, int x4, Color x5)
{
  ImageResizeCanvas(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ImageResizeCanvas(PyObject *self, PyObject *args)
{
  Image * x0;
  int x1;
  int x2;
  int x3;
  int x4;
  Color x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "ImageResizeCanvas", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(605), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Image *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(605), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  x4 = _cffi_to_c_int(arg4, int);
  if (x4 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x5, _cffi_type(24), arg5) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ImageResizeCanvas(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_ImageResizeCanvas(Image * x0, int x1, int x2, int x3, int x4, Color *x5)
{
  { ImageResizeCanvas(x0, x1, x2, x3, x4, *x5); }
}
#endif

static void _cffi_d_ImageResizeNN(Image * x0, int x1, int x2)
{
  ImageResizeNN(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ImageResizeNN(PyObject *self, PyObject *args)
{
  Image * x0;
  int x1;
  int x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "ImageResizeNN", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(605), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Image *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(605), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ImageResizeNN(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ImageResizeNN _cffi_d_ImageResizeNN
#endif

static void _cffi_d_ImageRotateCCW(Image * x0)
{
  ImageRotateCCW(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ImageRotateCCW(PyObject *self, PyObject *arg0)
{
  Image * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(605), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Image *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(605), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ImageRotateCCW(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ImageRotateCCW _cffi_d_ImageRotateCCW
#endif

static void _cffi_d_ImageRotateCW(Image * x0)
{
  ImageRotateCW(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ImageRotateCW(PyObject *self, PyObject *arg0)
{
  Image * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(605), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Image *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(605), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ImageRotateCW(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ImageRotateCW _cffi_d_ImageRotateCW
#endif

static Image _cffi_d_ImageText(char const * x0, int x1, Color x2)
{
  return ImageText(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ImageText(PyObject *self, PyObject *args)
{
  char const * x0;
  int x1;
  Color x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  Image result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "ImageText", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(24), arg2) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ImageText(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(16));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
static void _cffi_f_ImageText(Image *result, char const * x0, int x1, Color *x2)
{
  { *result = ImageText(x0, x1, *x2); }
}
#endif

static Image _cffi_d_ImageTextEx(Font x0, char const * x1, float x2, float x3, Color x4)
{
  return ImageTextEx(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ImageTextEx(PyObject *self, PyObject *args)
{
  Font x0;
  char const * x1;
  float x2;
  float x3;
  Color x4;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  Image result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "ImageTextEx", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(66), arg0) < 0)
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = (float)_cffi_to_c_float(arg2);
  if (x2 == (float)-1 && PyErr_Occurred())
    return NULL;

  x3 = (float)_cffi_to_c_float(arg3);
  if (x3 == (float)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x4, _cffi_type(24), arg4) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ImageTextEx(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(16));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
static void _cffi_f_ImageTextEx(Image *result, Font *x0, char const * x1, float x2, float x3, Color *x4)
{
  { *result = ImageTextEx(*x0, x1, x2, x3, *x4); }
}
#endif

static void _cffi_d_ImageToPOT(Image * x0, Color x1)
{
  ImageToPOT(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ImageToPOT(PyObject *self, PyObject *args)
{
  Image * x0;
  Color x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ImageToPOT", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(605), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Image *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(605), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  if (_cffi_to_c((char *)&x1, _cffi_type(24), arg1) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ImageToPOT(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_ImageToPOT(Image * x0, Color *x1)
{
  { ImageToPOT(x0, *x1); }
}
#endif

static void _cffi_d_InitAudioDevice(void)
{
  InitAudioDevice();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_InitAudioDevice(PyObject *self, PyObject *noarg)
{

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { InitAudioDevice(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_InitAudioDevice _cffi_d_InitAudioDevice
#endif

static AudioStream _cffi_d_InitAudioStream(unsigned int x0, unsigned int x1, unsigned int x2)
{
  return InitAudioStream(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_InitAudioStream(PyObject *self, PyObject *args)
{
  unsigned int x0;
  unsigned int x1;
  unsigned int x2;
  AudioStream result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "InitAudioStream", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  x0 = _cffi_to_c_int(arg0, unsigned int);
  if (x0 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, unsigned int);
  if (x1 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, unsigned int);
  if (x2 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = InitAudioStream(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(384));
  return pyresult;
}
#else
static void _cffi_f_InitAudioStream(AudioStream *result, unsigned int x0, unsigned int x1, unsigned int x2)
{
  { *result = InitAudioStream(x0, x1, x2); }
}
#endif

static void _cffi_d_InitVrSimulator(void)
{
  InitVrSimulator();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_InitVrSimulator(PyObject *self, PyObject *noarg)
{

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { InitVrSimulator(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_InitVrSimulator _cffi_d_InitVrSimulator
#endif

static void _cffi_d_InitWindow(int x0, int x1, char const * x2)
{
  InitWindow(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_InitWindow(PyObject *self, PyObject *args)
{
  int x0;
  int x1;
  char const * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "InitWindow", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { InitWindow(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_InitWindow _cffi_d_InitWindow
#endif

static int _cffi_d_IsAudioDeviceReady(void)
{
  return IsAudioDeviceReady();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_IsAudioDeviceReady(PyObject *self, PyObject *noarg)
{
  int result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = IsAudioDeviceReady(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_IsAudioDeviceReady _cffi_d_IsAudioDeviceReady
#endif

static int _cffi_d_IsAudioStreamPlaying(AudioStream x0)
{
  return IsAudioStreamPlaying(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_IsAudioStreamPlaying(PyObject *self, PyObject *arg0)
{
  AudioStream x0;
  int result;
  PyObject *pyresult;

  if (_cffi_to_c((char *)&x0, _cffi_type(384), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = IsAudioStreamPlaying(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
static int _cffi_f_IsAudioStreamPlaying(AudioStream *x0)
{
  int result;
  { result = IsAudioStreamPlaying(*x0); }
  return result;
}
#endif

static int _cffi_d_IsAudioStreamProcessed(AudioStream x0)
{
  return IsAudioStreamProcessed(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_IsAudioStreamProcessed(PyObject *self, PyObject *arg0)
{
  AudioStream x0;
  int result;
  PyObject *pyresult;

  if (_cffi_to_c((char *)&x0, _cffi_type(384), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = IsAudioStreamProcessed(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
static int _cffi_f_IsAudioStreamProcessed(AudioStream *x0)
{
  int result;
  { result = IsAudioStreamProcessed(*x0); }
  return result;
}
#endif

static int _cffi_d_IsCursorHidden(void)
{
  return IsCursorHidden();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_IsCursorHidden(PyObject *self, PyObject *noarg)
{
  int result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = IsCursorHidden(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_IsCursorHidden _cffi_d_IsCursorHidden
#endif

static int _cffi_d_IsFileDropped(void)
{
  return IsFileDropped();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_IsFileDropped(PyObject *self, PyObject *noarg)
{
  int result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = IsFileDropped(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_IsFileDropped _cffi_d_IsFileDropped
#endif

static int _cffi_d_IsFileExtension(char const * x0, char const * x1)
{
  return IsFileExtension(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_IsFileExtension(PyObject *self, PyObject *args)
{
  char const * x0;
  char const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "IsFileExtension", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = IsFileExtension(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_IsFileExtension _cffi_d_IsFileExtension
#endif

static int _cffi_d_IsGamepadAvailable(int x0)
{
  return IsGamepadAvailable(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_IsGamepadAvailable(PyObject *self, PyObject *arg0)
{
  int x0;
  int result;
  PyObject *pyresult;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = IsGamepadAvailable(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_IsGamepadAvailable _cffi_d_IsGamepadAvailable
#endif

static int _cffi_d_IsGamepadButtonDown(int x0, int x1)
{
  return IsGamepadButtonDown(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_IsGamepadButtonDown(PyObject *self, PyObject *args)
{
  int x0;
  int x1;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "IsGamepadButtonDown", 2, 2, &arg0, &arg1))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = IsGamepadButtonDown(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_IsGamepadButtonDown _cffi_d_IsGamepadButtonDown
#endif

static int _cffi_d_IsGamepadButtonPressed(int x0, int x1)
{
  return IsGamepadButtonPressed(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_IsGamepadButtonPressed(PyObject *self, PyObject *args)
{
  int x0;
  int x1;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "IsGamepadButtonPressed", 2, 2, &arg0, &arg1))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = IsGamepadButtonPressed(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_IsGamepadButtonPressed _cffi_d_IsGamepadButtonPressed
#endif

static int _cffi_d_IsGamepadButtonReleased(int x0, int x1)
{
  return IsGamepadButtonReleased(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_IsGamepadButtonReleased(PyObject *self, PyObject *args)
{
  int x0;
  int x1;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "IsGamepadButtonReleased", 2, 2, &arg0, &arg1))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = IsGamepadButtonReleased(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_IsGamepadButtonReleased _cffi_d_IsGamepadButtonReleased
#endif

static int _cffi_d_IsGamepadButtonUp(int x0, int x1)
{
  return IsGamepadButtonUp(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_IsGamepadButtonUp(PyObject *self, PyObject *args)
{
  int x0;
  int x1;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "IsGamepadButtonUp", 2, 2, &arg0, &arg1))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = IsGamepadButtonUp(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_IsGamepadButtonUp _cffi_d_IsGamepadButtonUp
#endif

static int _cffi_d_IsGamepadName(int x0, char const * x1)
{
  return IsGamepadName(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_IsGamepadName(PyObject *self, PyObject *args)
{
  int x0;
  char const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "IsGamepadName", 2, 2, &arg0, &arg1))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = IsGamepadName(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_IsGamepadName _cffi_d_IsGamepadName
#endif

static int _cffi_d_IsGestureDetected(int x0)
{
  return IsGestureDetected(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_IsGestureDetected(PyObject *self, PyObject *arg0)
{
  int x0;
  int result;
  PyObject *pyresult;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = IsGestureDetected(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_IsGestureDetected _cffi_d_IsGestureDetected
#endif

static int _cffi_d_IsKeyDown(int x0)
{
  return IsKeyDown(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_IsKeyDown(PyObject *self, PyObject *arg0)
{
  int x0;
  int result;
  PyObject *pyresult;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = IsKeyDown(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_IsKeyDown _cffi_d_IsKeyDown
#endif

static int _cffi_d_IsKeyPressed(int x0)
{
  return IsKeyPressed(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_IsKeyPressed(PyObject *self, PyObject *arg0)
{
  int x0;
  int result;
  PyObject *pyresult;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = IsKeyPressed(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_IsKeyPressed _cffi_d_IsKeyPressed
#endif

static int _cffi_d_IsKeyReleased(int x0)
{
  return IsKeyReleased(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_IsKeyReleased(PyObject *self, PyObject *arg0)
{
  int x0;
  int result;
  PyObject *pyresult;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = IsKeyReleased(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_IsKeyReleased _cffi_d_IsKeyReleased
#endif

static int _cffi_d_IsKeyUp(int x0)
{
  return IsKeyUp(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_IsKeyUp(PyObject *self, PyObject *arg0)
{
  int x0;
  int result;
  PyObject *pyresult;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = IsKeyUp(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_IsKeyUp _cffi_d_IsKeyUp
#endif

static int _cffi_d_IsModelAnimationValid(Model x0, ModelAnimation x1)
{
  return IsModelAnimationValid(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_IsModelAnimationValid(PyObject *self, PyObject *args)
{
  Model x0;
  ModelAnimation x1;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "IsModelAnimationValid", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(214), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(404), arg1) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = IsModelAnimationValid(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
static int _cffi_f_IsModelAnimationValid(Model *x0, ModelAnimation *x1)
{
  int result;
  { result = IsModelAnimationValid(*x0, *x1); }
  return result;
}
#endif

static int _cffi_d_IsMouseButtonDown(int x0)
{
  return IsMouseButtonDown(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_IsMouseButtonDown(PyObject *self, PyObject *arg0)
{
  int x0;
  int result;
  PyObject *pyresult;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = IsMouseButtonDown(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_IsMouseButtonDown _cffi_d_IsMouseButtonDown
#endif

static int _cffi_d_IsMouseButtonPressed(int x0)
{
  return IsMouseButtonPressed(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_IsMouseButtonPressed(PyObject *self, PyObject *arg0)
{
  int x0;
  int result;
  PyObject *pyresult;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = IsMouseButtonPressed(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_IsMouseButtonPressed _cffi_d_IsMouseButtonPressed
#endif

static int _cffi_d_IsMouseButtonReleased(int x0)
{
  return IsMouseButtonReleased(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_IsMouseButtonReleased(PyObject *self, PyObject *arg0)
{
  int x0;
  int result;
  PyObject *pyresult;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = IsMouseButtonReleased(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_IsMouseButtonReleased _cffi_d_IsMouseButtonReleased
#endif

static int _cffi_d_IsMouseButtonUp(int x0)
{
  return IsMouseButtonUp(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_IsMouseButtonUp(PyObject *self, PyObject *arg0)
{
  int x0;
  int result;
  PyObject *pyresult;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = IsMouseButtonUp(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_IsMouseButtonUp _cffi_d_IsMouseButtonUp
#endif

static int _cffi_d_IsMusicPlaying(Music x0)
{
  return IsMusicPlaying(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_IsMusicPlaying(PyObject *self, PyObject *arg0)
{
  Music x0;
  int result;
  PyObject *pyresult;

  if (_cffi_to_c((char *)&x0, _cffi_type(371), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = IsMusicPlaying(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
static int _cffi_f_IsMusicPlaying(Music *x0)
{
  int result;
  { result = IsMusicPlaying(*x0); }
  return result;
}
#endif

static int _cffi_d_IsSoundPlaying(Sound x0)
{
  return IsSoundPlaying(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_IsSoundPlaying(PyObject *self, PyObject *arg0)
{
  Sound x0;
  int result;
  PyObject *pyresult;

  if (_cffi_to_c((char *)&x0, _cffi_type(433), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = IsSoundPlaying(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
static int _cffi_f_IsSoundPlaying(Sound *x0)
{
  int result;
  { result = IsSoundPlaying(*x0); }
  return result;
}
#endif

static int _cffi_d_IsVrSimulatorReady(void)
{
  return IsVrSimulatorReady();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_IsVrSimulatorReady(PyObject *self, PyObject *noarg)
{
  int result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = IsVrSimulatorReady(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_IsVrSimulatorReady _cffi_d_IsVrSimulatorReady
#endif

static int _cffi_d_IsWindowHidden(void)
{
  return IsWindowHidden();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_IsWindowHidden(PyObject *self, PyObject *noarg)
{
  int result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = IsWindowHidden(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_IsWindowHidden _cffi_d_IsWindowHidden
#endif

static int _cffi_d_IsWindowMinimized(void)
{
  return IsWindowMinimized();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_IsWindowMinimized(PyObject *self, PyObject *noarg)
{
  int result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = IsWindowMinimized(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_IsWindowMinimized _cffi_d_IsWindowMinimized
#endif

static int _cffi_d_IsWindowReady(void)
{
  return IsWindowReady();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_IsWindowReady(PyObject *self, PyObject *noarg)
{
  int result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = IsWindowReady(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_IsWindowReady _cffi_d_IsWindowReady
#endif

static int _cffi_d_IsWindowResized(void)
{
  return IsWindowResized();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_IsWindowResized(PyObject *self, PyObject *noarg)
{
  int result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = IsWindowResized(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_IsWindowResized _cffi_d_IsWindowResized
#endif

static Font _cffi_d_LoadFont(char const * x0)
{
  return LoadFont(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_LoadFont(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  Font result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = LoadFont(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(66));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
static void _cffi_f_LoadFont(Font *result, char const * x0)
{
  { *result = LoadFont(x0); }
}
#endif

static CharInfo * _cffi_d_LoadFontData(char const * x0, int x1, int * x2, int x3, int x4)
{
  return LoadFontData(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_LoadFontData(PyObject *self, PyObject *args)
{
  char const * x0;
  int x1;
  int * x2;
  int x3;
  int x4;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  CharInfo * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "LoadFontData", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(11), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(11), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  x4 = _cffi_to_c_int(arg4, int);
  if (x4 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = LoadFontData(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(1148));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_LoadFontData _cffi_d_LoadFontData
#endif

static Font _cffi_d_LoadFontEx(char const * x0, int x1, int * x2, int x3)
{
  return LoadFontEx(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_LoadFontEx(PyObject *self, PyObject *args)
{
  char const * x0;
  int x1;
  int * x2;
  int x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  Font result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "LoadFontEx", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(11), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(11), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = LoadFontEx(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(66));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
static void _cffi_f_LoadFontEx(Font *result, char const * x0, int x1, int * x2, int x3)
{
  { *result = LoadFontEx(x0, x1, x2, x3); }
}
#endif

static Font _cffi_d_LoadFontFromImage(Image x0, Color x1, int x2)
{
  return LoadFontFromImage(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_LoadFontFromImage(PyObject *self, PyObject *args)
{
  Image x0;
  Color x1;
  int x2;
  Font result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "LoadFontFromImage", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(16), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(24), arg1) < 0)
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = LoadFontFromImage(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(66));
  return pyresult;
}
#else
static void _cffi_f_LoadFontFromImage(Font *result, Image *x0, Color *x1, int x2)
{
  { *result = LoadFontFromImage(*x0, *x1, x2); }
}
#endif

static Image _cffi_d_LoadImage(char const * x0)
{
  return LoadImage(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_LoadImage(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  Image result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = LoadImage(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(16));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
static void _cffi_f_LoadImage(Image *result, char const * x0)
{
  { *result = LoadImage(x0); }
}
#endif

static Image _cffi_d_LoadImageEx(Color * x0, int x1, int x2)
{
  return LoadImageEx(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_LoadImageEx(PyObject *self, PyObject *args)
{
  Color * x0;
  int x1;
  int x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  Image result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "LoadImageEx", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(61), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Color *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(61), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = LoadImageEx(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(16));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
static void _cffi_f_LoadImageEx(Image *result, Color * x0, int x1, int x2)
{
  { *result = LoadImageEx(x0, x1, x2); }
}
#endif

static Image _cffi_d_LoadImagePro(void * x0, int x1, int x2, int x3)
{
  return LoadImagePro(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_LoadImagePro(PyObject *self, PyObject *args)
{
  void * x0;
  int x1;
  int x2;
  int x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  Image result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "LoadImagePro", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(141), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (void *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(141), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = LoadImagePro(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(16));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
static void _cffi_f_LoadImagePro(Image *result, void * x0, int x1, int x2, int x3)
{
  { *result = LoadImagePro(x0, x1, x2, x3); }
}
#endif

static Image _cffi_d_LoadImageRaw(char const * x0, int x1, int x2, int x3, int x4)
{
  return LoadImageRaw(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_LoadImageRaw(PyObject *self, PyObject *args)
{
  char const * x0;
  int x1;
  int x2;
  int x3;
  int x4;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  Image result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "LoadImageRaw", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  x4 = _cffi_to_c_int(arg4, int);
  if (x4 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = LoadImageRaw(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(16));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
static void _cffi_f_LoadImageRaw(Image *result, char const * x0, int x1, int x2, int x3, int x4)
{
  { *result = LoadImageRaw(x0, x1, x2, x3, x4); }
}
#endif

static Material _cffi_d_LoadMaterialDefault(void)
{
  return LoadMaterialDefault();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_LoadMaterialDefault(PyObject *self, PyObject *noarg)
{
  Material result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = LoadMaterialDefault(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(704));
  return pyresult;
}
#else
static void _cffi_f_LoadMaterialDefault(Material *result)
{
  { *result = LoadMaterialDefault(); }
}
#endif

static Material * _cffi_d_LoadMaterials(char const * x0, int * x1)
{
  return LoadMaterials(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_LoadMaterials(PyObject *self, PyObject *args)
{
  char const * x0;
  int * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  Material * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "LoadMaterials", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(11), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(11), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = LoadMaterials(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(699));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_LoadMaterials _cffi_d_LoadMaterials
#endif

static Mesh * _cffi_d_LoadMeshes(char const * x0, int * x1)
{
  return LoadMeshes(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_LoadMeshes(PyObject *self, PyObject *args)
{
  char const * x0;
  int * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  Mesh * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "LoadMeshes", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(11), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(11), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = LoadMeshes(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(710));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_LoadMeshes _cffi_d_LoadMeshes
#endif

static Model _cffi_d_LoadModel(char const * x0)
{
  return LoadModel(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_LoadModel(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  Model result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = LoadModel(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(214));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
static void _cffi_f_LoadModel(Model *result, char const * x0)
{
  { *result = LoadModel(x0); }
}
#endif

static ModelAnimation * _cffi_d_LoadModelAnimations(char const * x0, int * x1)
{
  return LoadModelAnimations(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_LoadModelAnimations(PyObject *self, PyObject *args)
{
  char const * x0;
  int * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  ModelAnimation * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "LoadModelAnimations", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(11), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(11), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = LoadModelAnimations(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(1161));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_LoadModelAnimations _cffi_d_LoadModelAnimations
#endif

static Model _cffi_d_LoadModelFromMesh(Mesh x0)
{
  return LoadModelFromMesh(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_LoadModelFromMesh(PyObject *self, PyObject *arg0)
{
  Mesh x0;
  Model result;
  PyObject *pyresult;

  if (_cffi_to_c((char *)&x0, _cffi_type(6), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = LoadModelFromMesh(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(214));
  return pyresult;
}
#else
static void _cffi_f_LoadModelFromMesh(Model *result, Mesh *x0)
{
  { *result = LoadModelFromMesh(*x0); }
}
#endif

static Music _cffi_d_LoadMusicStream(char const * x0)
{
  return LoadMusicStream(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_LoadMusicStream(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  Music result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = LoadMusicStream(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(371));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
static void _cffi_f_LoadMusicStream(Music *result, char const * x0)
{
  { *result = LoadMusicStream(x0); }
}
#endif

static RenderTexture2D _cffi_d_LoadRenderTexture(int x0, int x1)
{
  return LoadRenderTexture(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_LoadRenderTexture(PyObject *self, PyObject *args)
{
  int x0;
  int x1;
  RenderTexture2D result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "LoadRenderTexture", 2, 2, &arg0, &arg1))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = LoadRenderTexture(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(800));
  return pyresult;
}
#else
static void _cffi_f_LoadRenderTexture(RenderTexture2D *result, int x0, int x1)
{
  { *result = LoadRenderTexture(x0, x1); }
}
#endif

static Shader _cffi_d_LoadShader(char const * x0, char const * x1)
{
  return LoadShader(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_LoadShader(PyObject *self, PyObject *args)
{
  char const * x0;
  char const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  Shader result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "LoadShader", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = LoadShader(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(258));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
static void _cffi_f_LoadShader(Shader *result, char const * x0, char const * x1)
{
  { *result = LoadShader(x0, x1); }
}
#endif

static Shader _cffi_d_LoadShaderCode(char const * x0, char const * x1)
{
  return LoadShaderCode(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_LoadShaderCode(PyObject *self, PyObject *args)
{
  char const * x0;
  char const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  Shader result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "LoadShaderCode", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = LoadShaderCode(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(258));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
static void _cffi_f_LoadShaderCode(Shader *result, char const * x0, char const * x1)
{
  { *result = LoadShaderCode(x0, x1); }
}
#endif

static Sound _cffi_d_LoadSound(char const * x0)
{
  return LoadSound(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_LoadSound(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  Sound result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = LoadSound(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(433));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
static void _cffi_f_LoadSound(Sound *result, char const * x0)
{
  { *result = LoadSound(x0); }
}
#endif

static Sound _cffi_d_LoadSoundFromWave(Wave x0)
{
  return LoadSoundFromWave(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_LoadSoundFromWave(PyObject *self, PyObject *arg0)
{
  Wave x0;
  Sound result;
  PyObject *pyresult;

  if (_cffi_to_c((char *)&x0, _cffi_type(245), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = LoadSoundFromWave(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(433));
  return pyresult;
}
#else
static void _cffi_f_LoadSoundFromWave(Sound *result, Wave *x0)
{
  { *result = LoadSoundFromWave(*x0); }
}
#endif

static char * _cffi_d_LoadText(char const * x0)
{
  return LoadText(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_LoadText(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = LoadText(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(319));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_LoadText _cffi_d_LoadText
#endif

static Texture2D _cffi_d_LoadTexture(char const * x0)
{
  return LoadTexture(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_LoadTexture(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  Texture2D result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = LoadTexture(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(80));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
static void _cffi_f_LoadTexture(Texture2D *result, char const * x0)
{
  { *result = LoadTexture(x0); }
}
#endif

static Texture2D _cffi_d_LoadTextureCubemap(Image x0, int x1)
{
  return LoadTextureCubemap(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_LoadTextureCubemap(PyObject *self, PyObject *args)
{
  Image x0;
  int x1;
  Texture2D result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "LoadTextureCubemap", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(16), arg0) < 0)
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = LoadTextureCubemap(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(80));
  return pyresult;
}
#else
static void _cffi_f_LoadTextureCubemap(Texture2D *result, Image *x0, int x1)
{
  { *result = LoadTextureCubemap(*x0, x1); }
}
#endif

static Texture2D _cffi_d_LoadTextureFromImage(Image x0)
{
  return LoadTextureFromImage(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_LoadTextureFromImage(PyObject *self, PyObject *arg0)
{
  Image x0;
  Texture2D result;
  PyObject *pyresult;

  if (_cffi_to_c((char *)&x0, _cffi_type(16), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = LoadTextureFromImage(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(80));
  return pyresult;
}
#else
static void _cffi_f_LoadTextureFromImage(Texture2D *result, Image *x0)
{
  { *result = LoadTextureFromImage(*x0); }
}
#endif

static Wave _cffi_d_LoadWave(char const * x0)
{
  return LoadWave(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_LoadWave(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  Wave result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = LoadWave(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(245));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
static void _cffi_f_LoadWave(Wave *result, char const * x0)
{
  { *result = LoadWave(x0); }
}
#endif

static int _cffi_d_MeasureText(char const * x0, int x1)
{
  return MeasureText(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_MeasureText(PyObject *self, PyObject *args)
{
  char const * x0;
  int x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "MeasureText", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = MeasureText(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_MeasureText _cffi_d_MeasureText
#endif

static Vector2 _cffi_d_MeasureTextEx(Font x0, char const * x1, float x2, float x3)
{
  return MeasureTextEx(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_MeasureTextEx(PyObject *self, PyObject *args)
{
  Font x0;
  char const * x1;
  float x2;
  float x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  Vector2 result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "MeasureTextEx", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(66), arg0) < 0)
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = (float)_cffi_to_c_float(arg2);
  if (x2 == (float)-1 && PyErr_Occurred())
    return NULL;

  x3 = (float)_cffi_to_c_float(arg3);
  if (x3 == (float)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = MeasureTextEx(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(209));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
static void _cffi_f_MeasureTextEx(Vector2 *result, Font *x0, char const * x1, float x2, float x3)
{
  { *result = MeasureTextEx(*x0, x1, x2, x3); }
}
#endif

static void _cffi_d_MeshBinormals(Mesh * x0)
{
  MeshBinormals(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_MeshBinormals(PyObject *self, PyObject *arg0)
{
  Mesh * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(710), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Mesh *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(710), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { MeshBinormals(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_MeshBinormals _cffi_d_MeshBinormals
#endif

static BoundingBox _cffi_d_MeshBoundingBox(Mesh x0)
{
  return MeshBoundingBox(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_MeshBoundingBox(PyObject *self, PyObject *arg0)
{
  Mesh x0;
  BoundingBox result;
  PyObject *pyresult;

  if (_cffi_to_c((char *)&x0, _cffi_type(6), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = MeshBoundingBox(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(387));
  return pyresult;
}
#else
static void _cffi_f_MeshBoundingBox(BoundingBox *result, Mesh *x0)
{
  { *result = MeshBoundingBox(*x0); }
}
#endif

static void _cffi_d_MeshTangents(Mesh * x0)
{
  MeshTangents(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_MeshTangents(PyObject *self, PyObject *arg0)
{
  Mesh * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(710), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Mesh *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(710), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { MeshTangents(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_MeshTangents _cffi_d_MeshTangents
#endif

static void _cffi_d_OpenURL(char const * x0)
{
  OpenURL(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_OpenURL(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { OpenURL(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_OpenURL _cffi_d_OpenURL
#endif

static void _cffi_d_PauseAudioStream(AudioStream x0)
{
  PauseAudioStream(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PauseAudioStream(PyObject *self, PyObject *arg0)
{
  AudioStream x0;

  if (_cffi_to_c((char *)&x0, _cffi_type(384), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { PauseAudioStream(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_PauseAudioStream(AudioStream *x0)
{
  { PauseAudioStream(*x0); }
}
#endif

static void _cffi_d_PauseMusicStream(Music x0)
{
  PauseMusicStream(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PauseMusicStream(PyObject *self, PyObject *arg0)
{
  Music x0;

  if (_cffi_to_c((char *)&x0, _cffi_type(371), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { PauseMusicStream(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_PauseMusicStream(Music *x0)
{
  { PauseMusicStream(*x0); }
}
#endif

static void _cffi_d_PauseSound(Sound x0)
{
  PauseSound(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PauseSound(PyObject *self, PyObject *arg0)
{
  Sound x0;

  if (_cffi_to_c((char *)&x0, _cffi_type(433), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { PauseSound(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_PauseSound(Sound *x0)
{
  { PauseSound(*x0); }
}
#endif

static void _cffi_d_PlayAudioStream(AudioStream x0)
{
  PlayAudioStream(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PlayAudioStream(PyObject *self, PyObject *arg0)
{
  AudioStream x0;

  if (_cffi_to_c((char *)&x0, _cffi_type(384), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { PlayAudioStream(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_PlayAudioStream(AudioStream *x0)
{
  { PlayAudioStream(*x0); }
}
#endif

static void _cffi_d_PlayMusicStream(Music x0)
{
  PlayMusicStream(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PlayMusicStream(PyObject *self, PyObject *arg0)
{
  Music x0;

  if (_cffi_to_c((char *)&x0, _cffi_type(371), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { PlayMusicStream(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_PlayMusicStream(Music *x0)
{
  { PlayMusicStream(*x0); }
}
#endif

static void _cffi_d_PlaySound(Sound x0)
{
  PlaySound(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PlaySound(PyObject *self, PyObject *arg0)
{
  Sound x0;

  if (_cffi_to_c((char *)&x0, _cffi_type(433), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { PlaySound(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_PlaySound(Sound *x0)
{
  { PlaySound(*x0); }
}
#endif

static void _cffi_d_PlaySoundMulti(Sound x0)
{
  PlaySoundMulti(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PlaySoundMulti(PyObject *self, PyObject *arg0)
{
  Sound x0;

  if (_cffi_to_c((char *)&x0, _cffi_type(433), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { PlaySoundMulti(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_PlaySoundMulti(Sound *x0)
{
  { PlaySoundMulti(*x0); }
}
#endif

static void _cffi_d_ResumeAudioStream(AudioStream x0)
{
  ResumeAudioStream(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ResumeAudioStream(PyObject *self, PyObject *arg0)
{
  AudioStream x0;

  if (_cffi_to_c((char *)&x0, _cffi_type(384), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ResumeAudioStream(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_ResumeAudioStream(AudioStream *x0)
{
  { ResumeAudioStream(*x0); }
}
#endif

static void _cffi_d_ResumeMusicStream(Music x0)
{
  ResumeMusicStream(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ResumeMusicStream(PyObject *self, PyObject *arg0)
{
  Music x0;

  if (_cffi_to_c((char *)&x0, _cffi_type(371), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ResumeMusicStream(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_ResumeMusicStream(Music *x0)
{
  { ResumeMusicStream(*x0); }
}
#endif

static void _cffi_d_ResumeSound(Sound x0)
{
  ResumeSound(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ResumeSound(PyObject *self, PyObject *arg0)
{
  Sound x0;

  if (_cffi_to_c((char *)&x0, _cffi_type(433), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ResumeSound(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_ResumeSound(Sound *x0)
{
  { ResumeSound(*x0); }
}
#endif

static void _cffi_d_SetAudioStreamPitch(AudioStream x0, float x1)
{
  SetAudioStreamPitch(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_SetAudioStreamPitch(PyObject *self, PyObject *args)
{
  AudioStream x0;
  float x1;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "SetAudioStreamPitch", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(384), arg0) < 0)
    return NULL;

  x1 = (float)_cffi_to_c_float(arg1);
  if (x1 == (float)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { SetAudioStreamPitch(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_SetAudioStreamPitch(AudioStream *x0, float x1)
{
  { SetAudioStreamPitch(*x0, x1); }
}
#endif

static void _cffi_d_SetAudioStreamVolume(AudioStream x0, float x1)
{
  SetAudioStreamVolume(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_SetAudioStreamVolume(PyObject *self, PyObject *args)
{
  AudioStream x0;
  float x1;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "SetAudioStreamVolume", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(384), arg0) < 0)
    return NULL;

  x1 = (float)_cffi_to_c_float(arg1);
  if (x1 == (float)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { SetAudioStreamVolume(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_SetAudioStreamVolume(AudioStream *x0, float x1)
{
  { SetAudioStreamVolume(*x0, x1); }
}
#endif

static void _cffi_d_SetCameraAltControl(int x0)
{
  SetCameraAltControl(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_SetCameraAltControl(PyObject *self, PyObject *arg0)
{
  int x0;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { SetCameraAltControl(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_SetCameraAltControl _cffi_d_SetCameraAltControl
#endif

static void _cffi_d_SetCameraMode(Camera3D x0, int x1)
{
  SetCameraMode(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_SetCameraMode(PyObject *self, PyObject *args)
{
  Camera3D x0;
  int x1;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "SetCameraMode", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(158), arg0) < 0)
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { SetCameraMode(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_SetCameraMode(Camera3D *x0, int x1)
{
  { SetCameraMode(*x0, x1); }
}
#endif

static void _cffi_d_SetCameraMoveControls(int x0, int x1, int x2, int x3, int x4, int x5)
{
  SetCameraMoveControls(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_SetCameraMoveControls(PyObject *self, PyObject *args)
{
  int x0;
  int x1;
  int x2;
  int x3;
  int x4;
  int x5;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "SetCameraMoveControls", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  x4 = _cffi_to_c_int(arg4, int);
  if (x4 == (int)-1 && PyErr_Occurred())
    return NULL;

  x5 = _cffi_to_c_int(arg5, int);
  if (x5 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { SetCameraMoveControls(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_SetCameraMoveControls _cffi_d_SetCameraMoveControls
#endif

static void _cffi_d_SetCameraPanControl(int x0)
{
  SetCameraPanControl(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_SetCameraPanControl(PyObject *self, PyObject *arg0)
{
  int x0;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { SetCameraPanControl(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_SetCameraPanControl _cffi_d_SetCameraPanControl
#endif

static void _cffi_d_SetCameraSmoothZoomControl(int x0)
{
  SetCameraSmoothZoomControl(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_SetCameraSmoothZoomControl(PyObject *self, PyObject *arg0)
{
  int x0;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { SetCameraSmoothZoomControl(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_SetCameraSmoothZoomControl _cffi_d_SetCameraSmoothZoomControl
#endif

static void _cffi_d_SetClipboardText(char const * x0)
{
  SetClipboardText(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_SetClipboardText(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { SetClipboardText(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_SetClipboardText _cffi_d_SetClipboardText
#endif

static void _cffi_d_SetConfigFlags(unsigned int x0)
{
  SetConfigFlags(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_SetConfigFlags(PyObject *self, PyObject *arg0)
{
  unsigned int x0;

  x0 = _cffi_to_c_int(arg0, unsigned int);
  if (x0 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { SetConfigFlags(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_SetConfigFlags _cffi_d_SetConfigFlags
#endif

static void _cffi_d_SetExitKey(int x0)
{
  SetExitKey(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_SetExitKey(PyObject *self, PyObject *arg0)
{
  int x0;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { SetExitKey(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_SetExitKey _cffi_d_SetExitKey
#endif

static void _cffi_d_SetGesturesEnabled(unsigned int x0)
{
  SetGesturesEnabled(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_SetGesturesEnabled(PyObject *self, PyObject *arg0)
{
  unsigned int x0;

  x0 = _cffi_to_c_int(arg0, unsigned int);
  if (x0 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { SetGesturesEnabled(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_SetGesturesEnabled _cffi_d_SetGesturesEnabled
#endif

static void _cffi_d_SetMasterVolume(float x0)
{
  SetMasterVolume(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_SetMasterVolume(PyObject *self, PyObject *arg0)
{
  float x0;

  x0 = (float)_cffi_to_c_float(arg0);
  if (x0 == (float)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { SetMasterVolume(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_SetMasterVolume _cffi_d_SetMasterVolume
#endif

static void _cffi_d_SetMaterialTexture(Material * x0, int x1, Texture2D x2)
{
  SetMaterialTexture(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_SetMaterialTexture(PyObject *self, PyObject *args)
{
  Material * x0;
  int x1;
  Texture2D x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "SetMaterialTexture", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(699), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Material *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(699), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(80), arg2) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { SetMaterialTexture(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_SetMaterialTexture(Material * x0, int x1, Texture2D *x2)
{
  { SetMaterialTexture(x0, x1, *x2); }
}
#endif

static void _cffi_d_SetMatrixModelview(Matrix x0)
{
  SetMatrixModelview(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_SetMatrixModelview(PyObject *self, PyObject *arg0)
{
  Matrix x0;

  if (_cffi_to_c((char *)&x0, _cffi_type(707), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { SetMatrixModelview(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_SetMatrixModelview(Matrix *x0)
{
  { SetMatrixModelview(*x0); }
}
#endif

static void _cffi_d_SetMatrixProjection(Matrix x0)
{
  SetMatrixProjection(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_SetMatrixProjection(PyObject *self, PyObject *arg0)
{
  Matrix x0;

  if (_cffi_to_c((char *)&x0, _cffi_type(707), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { SetMatrixProjection(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_SetMatrixProjection(Matrix *x0)
{
  { SetMatrixProjection(*x0); }
}
#endif

static void _cffi_d_SetModelMeshMaterial(Model * x0, int x1, int x2)
{
  SetModelMeshMaterial(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_SetModelMeshMaterial(PyObject *self, PyObject *args)
{
  Model * x0;
  int x1;
  int x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "SetModelMeshMaterial", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(720), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Model *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(720), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { SetModelMeshMaterial(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_SetModelMeshMaterial _cffi_d_SetModelMeshMaterial
#endif

static void _cffi_d_SetMouseOffset(int x0, int x1)
{
  SetMouseOffset(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_SetMouseOffset(PyObject *self, PyObject *args)
{
  int x0;
  int x1;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "SetMouseOffset", 2, 2, &arg0, &arg1))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { SetMouseOffset(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_SetMouseOffset _cffi_d_SetMouseOffset
#endif

static void _cffi_d_SetMousePosition(int x0, int x1)
{
  SetMousePosition(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_SetMousePosition(PyObject *self, PyObject *args)
{
  int x0;
  int x1;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "SetMousePosition", 2, 2, &arg0, &arg1))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { SetMousePosition(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_SetMousePosition _cffi_d_SetMousePosition
#endif

static void _cffi_d_SetMouseScale(float x0, float x1)
{
  SetMouseScale(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_SetMouseScale(PyObject *self, PyObject *args)
{
  float x0;
  float x1;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "SetMouseScale", 2, 2, &arg0, &arg1))
    return NULL;

  x0 = (float)_cffi_to_c_float(arg0);
  if (x0 == (float)-1 && PyErr_Occurred())
    return NULL;

  x1 = (float)_cffi_to_c_float(arg1);
  if (x1 == (float)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { SetMouseScale(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_SetMouseScale _cffi_d_SetMouseScale
#endif

static void _cffi_d_SetMusicLoopCount(Music x0, int x1)
{
  SetMusicLoopCount(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_SetMusicLoopCount(PyObject *self, PyObject *args)
{
  Music x0;
  int x1;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "SetMusicLoopCount", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(371), arg0) < 0)
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { SetMusicLoopCount(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_SetMusicLoopCount(Music *x0, int x1)
{
  { SetMusicLoopCount(*x0, x1); }
}
#endif

static void _cffi_d_SetMusicPitch(Music x0, float x1)
{
  SetMusicPitch(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_SetMusicPitch(PyObject *self, PyObject *args)
{
  Music x0;
  float x1;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "SetMusicPitch", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(371), arg0) < 0)
    return NULL;

  x1 = (float)_cffi_to_c_float(arg1);
  if (x1 == (float)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { SetMusicPitch(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_SetMusicPitch(Music *x0, float x1)
{
  { SetMusicPitch(*x0, x1); }
}
#endif

static void _cffi_d_SetMusicVolume(Music x0, float x1)
{
  SetMusicVolume(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_SetMusicVolume(PyObject *self, PyObject *args)
{
  Music x0;
  float x1;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "SetMusicVolume", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(371), arg0) < 0)
    return NULL;

  x1 = (float)_cffi_to_c_float(arg1);
  if (x1 == (float)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { SetMusicVolume(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_SetMusicVolume(Music *x0, float x1)
{
  { SetMusicVolume(*x0, x1); }
}
#endif

static void _cffi_d_SetShaderValue(Shader x0, int x1, void const * x2, int x3)
{
  SetShaderValue(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_SetShaderValue(PyObject *self, PyObject *args)
{
  Shader x0;
  int x1;
  void const * x2;
  int x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "SetShaderValue", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(258), arg0) < 0)
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(526), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (void const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(526), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { SetShaderValue(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_SetShaderValue(Shader *x0, int x1, void const * x2, int x3)
{
  { SetShaderValue(*x0, x1, x2, x3); }
}
#endif

static void _cffi_d_SetShaderValueMatrix(Shader x0, int x1, Matrix x2)
{
  SetShaderValueMatrix(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_SetShaderValueMatrix(PyObject *self, PyObject *args)
{
  Shader x0;
  int x1;
  Matrix x2;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "SetShaderValueMatrix", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(258), arg0) < 0)
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(707), arg2) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { SetShaderValueMatrix(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_SetShaderValueMatrix(Shader *x0, int x1, Matrix *x2)
{
  { SetShaderValueMatrix(*x0, x1, *x2); }
}
#endif

static void _cffi_d_SetShaderValueTexture(Shader x0, int x1, Texture2D x2)
{
  SetShaderValueTexture(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_SetShaderValueTexture(PyObject *self, PyObject *args)
{
  Shader x0;
  int x1;
  Texture2D x2;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "SetShaderValueTexture", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(258), arg0) < 0)
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x2, _cffi_type(80), arg2) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { SetShaderValueTexture(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_SetShaderValueTexture(Shader *x0, int x1, Texture2D *x2)
{
  { SetShaderValueTexture(*x0, x1, *x2); }
}
#endif

static void _cffi_d_SetShaderValueV(Shader x0, int x1, void const * x2, int x3, int x4)
{
  SetShaderValueV(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_SetShaderValueV(PyObject *self, PyObject *args)
{
  Shader x0;
  int x1;
  void const * x2;
  int x3;
  int x4;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "SetShaderValueV", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(258), arg0) < 0)
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(526), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (void const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(526), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  x4 = _cffi_to_c_int(arg4, int);
  if (x4 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { SetShaderValueV(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_SetShaderValueV(Shader *x0, int x1, void const * x2, int x3, int x4)
{
  { SetShaderValueV(*x0, x1, x2, x3, x4); }
}
#endif

static void _cffi_d_SetShapesTexture(Texture2D x0, Rectangle x1)
{
  SetShapesTexture(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_SetShapesTexture(PyObject *self, PyObject *args)
{
  Texture2D x0;
  Rectangle x1;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "SetShapesTexture", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(80), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(77), arg1) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { SetShapesTexture(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_SetShapesTexture(Texture2D *x0, Rectangle *x1)
{
  { SetShapesTexture(*x0, *x1); }
}
#endif

static void _cffi_d_SetSoundPitch(Sound x0, float x1)
{
  SetSoundPitch(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_SetSoundPitch(PyObject *self, PyObject *args)
{
  Sound x0;
  float x1;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "SetSoundPitch", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(433), arg0) < 0)
    return NULL;

  x1 = (float)_cffi_to_c_float(arg1);
  if (x1 == (float)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { SetSoundPitch(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_SetSoundPitch(Sound *x0, float x1)
{
  { SetSoundPitch(*x0, x1); }
}
#endif

static void _cffi_d_SetSoundVolume(Sound x0, float x1)
{
  SetSoundVolume(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_SetSoundVolume(PyObject *self, PyObject *args)
{
  Sound x0;
  float x1;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "SetSoundVolume", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(433), arg0) < 0)
    return NULL;

  x1 = (float)_cffi_to_c_float(arg1);
  if (x1 == (float)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { SetSoundVolume(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_SetSoundVolume(Sound *x0, float x1)
{
  { SetSoundVolume(*x0, x1); }
}
#endif

static void _cffi_d_SetTargetFPS(int x0)
{
  SetTargetFPS(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_SetTargetFPS(PyObject *self, PyObject *arg0)
{
  int x0;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { SetTargetFPS(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_SetTargetFPS _cffi_d_SetTargetFPS
#endif

static void _cffi_d_SetTextureFilter(Texture2D x0, int x1)
{
  SetTextureFilter(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_SetTextureFilter(PyObject *self, PyObject *args)
{
  Texture2D x0;
  int x1;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "SetTextureFilter", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(80), arg0) < 0)
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { SetTextureFilter(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_SetTextureFilter(Texture2D *x0, int x1)
{
  { SetTextureFilter(*x0, x1); }
}
#endif

static void _cffi_d_SetTextureWrap(Texture2D x0, int x1)
{
  SetTextureWrap(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_SetTextureWrap(PyObject *self, PyObject *args)
{
  Texture2D x0;
  int x1;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "SetTextureWrap", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(80), arg0) < 0)
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { SetTextureWrap(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_SetTextureWrap(Texture2D *x0, int x1)
{
  { SetTextureWrap(*x0, x1); }
}
#endif

static void _cffi_d_SetTraceLogExit(int x0)
{
  SetTraceLogExit(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_SetTraceLogExit(PyObject *self, PyObject *arg0)
{
  int x0;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { SetTraceLogExit(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_SetTraceLogExit _cffi_d_SetTraceLogExit
#endif

static void _cffi_d_SetTraceLogLevel(int x0)
{
  SetTraceLogLevel(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_SetTraceLogLevel(PyObject *self, PyObject *arg0)
{
  int x0;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { SetTraceLogLevel(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_SetTraceLogLevel _cffi_d_SetTraceLogLevel
#endif

static void _cffi_d_SetVrConfiguration(VrDeviceInfo x0, Shader x1)
{
  SetVrConfiguration(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_SetVrConfiguration(PyObject *self, PyObject *args)
{
  VrDeviceInfo x0;
  Shader x1;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "SetVrConfiguration", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(1020), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(258), arg1) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { SetVrConfiguration(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_SetVrConfiguration(VrDeviceInfo *x0, Shader *x1)
{
  { SetVrConfiguration(*x0, *x1); }
}
#endif

static void _cffi_d_SetWindowIcon(Image x0)
{
  SetWindowIcon(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_SetWindowIcon(PyObject *self, PyObject *arg0)
{
  Image x0;

  if (_cffi_to_c((char *)&x0, _cffi_type(16), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { SetWindowIcon(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_SetWindowIcon(Image *x0)
{
  { SetWindowIcon(*x0); }
}
#endif

static void _cffi_d_SetWindowMinSize(int x0, int x1)
{
  SetWindowMinSize(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_SetWindowMinSize(PyObject *self, PyObject *args)
{
  int x0;
  int x1;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "SetWindowMinSize", 2, 2, &arg0, &arg1))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { SetWindowMinSize(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_SetWindowMinSize _cffi_d_SetWindowMinSize
#endif

static void _cffi_d_SetWindowMonitor(int x0)
{
  SetWindowMonitor(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_SetWindowMonitor(PyObject *self, PyObject *arg0)
{
  int x0;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { SetWindowMonitor(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_SetWindowMonitor _cffi_d_SetWindowMonitor
#endif

static void _cffi_d_SetWindowPosition(int x0, int x1)
{
  SetWindowPosition(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_SetWindowPosition(PyObject *self, PyObject *args)
{
  int x0;
  int x1;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "SetWindowPosition", 2, 2, &arg0, &arg1))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { SetWindowPosition(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_SetWindowPosition _cffi_d_SetWindowPosition
#endif

static void _cffi_d_SetWindowSize(int x0, int x1)
{
  SetWindowSize(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_SetWindowSize(PyObject *self, PyObject *args)
{
  int x0;
  int x1;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "SetWindowSize", 2, 2, &arg0, &arg1))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { SetWindowSize(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_SetWindowSize _cffi_d_SetWindowSize
#endif

static void _cffi_d_SetWindowTitle(char const * x0)
{
  SetWindowTitle(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_SetWindowTitle(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { SetWindowTitle(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_SetWindowTitle _cffi_d_SetWindowTitle
#endif

static void _cffi_d_ShowCursor(void)
{
  ShowCursor();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ShowCursor(PyObject *self, PyObject *noarg)
{

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ShowCursor(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ShowCursor _cffi_d_ShowCursor
#endif

static void _cffi_d_StopAudioStream(AudioStream x0)
{
  StopAudioStream(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_StopAudioStream(PyObject *self, PyObject *arg0)
{
  AudioStream x0;

  if (_cffi_to_c((char *)&x0, _cffi_type(384), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { StopAudioStream(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_StopAudioStream(AudioStream *x0)
{
  { StopAudioStream(*x0); }
}
#endif

static void _cffi_d_StopMusicStream(Music x0)
{
  StopMusicStream(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_StopMusicStream(PyObject *self, PyObject *arg0)
{
  Music x0;

  if (_cffi_to_c((char *)&x0, _cffi_type(371), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { StopMusicStream(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_StopMusicStream(Music *x0)
{
  { StopMusicStream(*x0); }
}
#endif

static void _cffi_d_StopSound(Sound x0)
{
  StopSound(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_StopSound(PyObject *self, PyObject *arg0)
{
  Sound x0;

  if (_cffi_to_c((char *)&x0, _cffi_type(433), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { StopSound(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_StopSound(Sound *x0)
{
  { StopSound(*x0); }
}
#endif

static void _cffi_d_StopSoundMulti(void)
{
  StopSoundMulti();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_StopSoundMulti(PyObject *self, PyObject *noarg)
{

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { StopSoundMulti(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_StopSoundMulti _cffi_d_StopSoundMulti
#endif

static int _cffi_d_StorageLoadValue(int x0)
{
  return StorageLoadValue(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_StorageLoadValue(PyObject *self, PyObject *arg0)
{
  int x0;
  int result;
  PyObject *pyresult;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = StorageLoadValue(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_StorageLoadValue _cffi_d_StorageLoadValue
#endif

static void _cffi_d_StorageSaveValue(int x0, int x1)
{
  StorageSaveValue(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_StorageSaveValue(PyObject *self, PyObject *args)
{
  int x0;
  int x1;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "StorageSaveValue", 2, 2, &arg0, &arg1))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { StorageSaveValue(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_StorageSaveValue _cffi_d_StorageSaveValue
#endif

static void _cffi_d_TakeScreenshot(char const * x0)
{
  TakeScreenshot(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_TakeScreenshot(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { TakeScreenshot(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_TakeScreenshot _cffi_d_TakeScreenshot
#endif

static void _cffi_d_TextAppend(char * x0, char const * x1, int * x2)
{
  TextAppend(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_TextAppend(PyObject *self, PyObject *args)
{
  char * x0;
  char const * x1;
  int * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "TextAppend", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(319), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(319), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(11), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(11), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { TextAppend(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_TextAppend _cffi_d_TextAppend
#endif

static int _cffi_d_TextCopy(char * x0, char const * x1)
{
  return TextCopy(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_TextCopy(PyObject *self, PyObject *args)
{
  char * x0;
  char const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "TextCopy", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(319), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(319), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = TextCopy(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_TextCopy _cffi_d_TextCopy
#endif

static int _cffi_d_TextFindIndex(char const * x0, char const * x1)
{
  return TextFindIndex(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_TextFindIndex(PyObject *self, PyObject *args)
{
  char const * x0;
  char const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "TextFindIndex", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = TextFindIndex(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_TextFindIndex _cffi_d_TextFindIndex
#endif

static void _cffi_const_TextFormat(char *o)
{
  *(char const *(* *)(char const *, ...))o = TextFormat;
}

static char * _cffi_d_TextInsert(char const * x0, char const * x1, int x2)
{
  return TextInsert(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_TextInsert(PyObject *self, PyObject *args)
{
  char const * x0;
  char const * x1;
  int x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "TextInsert", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = TextInsert(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(319));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_TextInsert _cffi_d_TextInsert
#endif

static int _cffi_d_TextIsEqual(char const * x0, char const * x1)
{
  return TextIsEqual(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_TextIsEqual(PyObject *self, PyObject *args)
{
  char const * x0;
  char const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "TextIsEqual", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = TextIsEqual(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_TextIsEqual _cffi_d_TextIsEqual
#endif

static char const * _cffi_d_TextJoin(char const * * x0, int x1, char const * x2)
{
  return TextJoin(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_TextJoin(PyObject *self, PyObject *args)
{
  char const * * x0;
  int x1;
  char const * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char const * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "TextJoin", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(341), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const * *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(341), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = TextJoin(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(9));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_TextJoin _cffi_d_TextJoin
#endif

static unsigned int _cffi_d_TextLength(char const * x0)
{
  return TextLength(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_TextLength(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  unsigned int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = TextLength(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, unsigned int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_TextLength _cffi_d_TextLength
#endif

static char * _cffi_d_TextReplace(char * x0, char const * x1, char const * x2)
{
  return TextReplace(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_TextReplace(PyObject *self, PyObject *args)
{
  char * x0;
  char const * x1;
  char const * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "TextReplace", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(319), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(319), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = TextReplace(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(319));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_TextReplace _cffi_d_TextReplace
#endif

static char const * * _cffi_d_TextSplit(char const * x0, char x1, int * x2)
{
  return TextSplit(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_TextSplit(PyObject *self, PyObject *args)
{
  char const * x0;
  char x1;
  int * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char const * * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "TextSplit", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = (char)_cffi_to_c_char(arg1);
  if (x1 == (char)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(11), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(11), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = TextSplit(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(341));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_TextSplit _cffi_d_TextSplit
#endif

static char const * _cffi_d_TextSubtext(char const * x0, int x1, int x2)
{
  return TextSubtext(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_TextSubtext(PyObject *self, PyObject *args)
{
  char const * x0;
  int x1;
  int x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char const * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "TextSubtext", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = TextSubtext(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(9));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_TextSubtext _cffi_d_TextSubtext
#endif

static int _cffi_d_TextToInteger(char const * x0)
{
  return TextToInteger(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_TextToInteger(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = TextToInteger(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_TextToInteger _cffi_d_TextToInteger
#endif

static char const * _cffi_d_TextToLower(char const * x0)
{
  return TextToLower(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_TextToLower(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char const * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = TextToLower(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(9));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_TextToLower _cffi_d_TextToLower
#endif

static char const * _cffi_d_TextToPascal(char const * x0)
{
  return TextToPascal(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_TextToPascal(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char const * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = TextToPascal(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(9));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_TextToPascal _cffi_d_TextToPascal
#endif

static char const * _cffi_d_TextToUpper(char const * x0)
{
  return TextToUpper(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_TextToUpper(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char const * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(9), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(9), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = TextToUpper(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(9));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_TextToUpper _cffi_d_TextToUpper
#endif

static char * _cffi_d_TextToUtf8(int * x0, int x1)
{
  return TextToUtf8(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_TextToUtf8(PyObject *self, PyObject *args)
{
  int * x0;
  int x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "TextToUtf8", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(11), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(11), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = TextToUtf8(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(319));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_TextToUtf8 _cffi_d_TextToUtf8
#endif

static void _cffi_d_ToggleFullscreen(void)
{
  ToggleFullscreen();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ToggleFullscreen(PyObject *self, PyObject *noarg)
{

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ToggleFullscreen(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ToggleFullscreen _cffi_d_ToggleFullscreen
#endif

static void _cffi_d_ToggleVrMode(void)
{
  ToggleVrMode();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ToggleVrMode(PyObject *self, PyObject *noarg)
{

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { ToggleVrMode(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_ToggleVrMode _cffi_d_ToggleVrMode
#endif

static void _cffi_const_TraceLog(char *o)
{
  *(void(* *)(int, char const *, ...))o = TraceLog;
}

static void _cffi_d_UnhideWindow(void)
{
  UnhideWindow();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_UnhideWindow(PyObject *self, PyObject *noarg)
{

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { UnhideWindow(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_UnhideWindow _cffi_d_UnhideWindow
#endif

static void _cffi_d_UnloadFont(Font x0)
{
  UnloadFont(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_UnloadFont(PyObject *self, PyObject *arg0)
{
  Font x0;

  if (_cffi_to_c((char *)&x0, _cffi_type(66), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { UnloadFont(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_UnloadFont(Font *x0)
{
  { UnloadFont(*x0); }
}
#endif

static void _cffi_d_UnloadImage(Image x0)
{
  UnloadImage(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_UnloadImage(PyObject *self, PyObject *arg0)
{
  Image x0;

  if (_cffi_to_c((char *)&x0, _cffi_type(16), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { UnloadImage(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_UnloadImage(Image *x0)
{
  { UnloadImage(*x0); }
}
#endif

static void _cffi_d_UnloadMaterial(Material x0)
{
  UnloadMaterial(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_UnloadMaterial(PyObject *self, PyObject *arg0)
{
  Material x0;

  if (_cffi_to_c((char *)&x0, _cffi_type(704), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { UnloadMaterial(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_UnloadMaterial(Material *x0)
{
  { UnloadMaterial(*x0); }
}
#endif

static void _cffi_d_UnloadMesh(Mesh x0)
{
  UnloadMesh(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_UnloadMesh(PyObject *self, PyObject *arg0)
{
  Mesh x0;

  if (_cffi_to_c((char *)&x0, _cffi_type(6), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { UnloadMesh(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_UnloadMesh(Mesh *x0)
{
  { UnloadMesh(*x0); }
}
#endif

static void _cffi_d_UnloadModel(Model x0)
{
  UnloadModel(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_UnloadModel(PyObject *self, PyObject *arg0)
{
  Model x0;

  if (_cffi_to_c((char *)&x0, _cffi_type(214), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { UnloadModel(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_UnloadModel(Model *x0)
{
  { UnloadModel(*x0); }
}
#endif

static void _cffi_d_UnloadModelAnimation(ModelAnimation x0)
{
  UnloadModelAnimation(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_UnloadModelAnimation(PyObject *self, PyObject *arg0)
{
  ModelAnimation x0;

  if (_cffi_to_c((char *)&x0, _cffi_type(404), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { UnloadModelAnimation(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_UnloadModelAnimation(ModelAnimation *x0)
{
  { UnloadModelAnimation(*x0); }
}
#endif

static void _cffi_d_UnloadMusicStream(Music x0)
{
  UnloadMusicStream(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_UnloadMusicStream(PyObject *self, PyObject *arg0)
{
  Music x0;

  if (_cffi_to_c((char *)&x0, _cffi_type(371), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { UnloadMusicStream(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_UnloadMusicStream(Music *x0)
{
  { UnloadMusicStream(*x0); }
}
#endif

static void _cffi_d_UnloadRenderTexture(RenderTexture2D x0)
{
  UnloadRenderTexture(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_UnloadRenderTexture(PyObject *self, PyObject *arg0)
{
  RenderTexture2D x0;

  if (_cffi_to_c((char *)&x0, _cffi_type(800), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { UnloadRenderTexture(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_UnloadRenderTexture(RenderTexture2D *x0)
{
  { UnloadRenderTexture(*x0); }
}
#endif

static void _cffi_d_UnloadShader(Shader x0)
{
  UnloadShader(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_UnloadShader(PyObject *self, PyObject *arg0)
{
  Shader x0;

  if (_cffi_to_c((char *)&x0, _cffi_type(258), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { UnloadShader(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_UnloadShader(Shader *x0)
{
  { UnloadShader(*x0); }
}
#endif

static void _cffi_d_UnloadSound(Sound x0)
{
  UnloadSound(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_UnloadSound(PyObject *self, PyObject *arg0)
{
  Sound x0;

  if (_cffi_to_c((char *)&x0, _cffi_type(433), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { UnloadSound(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_UnloadSound(Sound *x0)
{
  { UnloadSound(*x0); }
}
#endif

static void _cffi_d_UnloadTexture(Texture2D x0)
{
  UnloadTexture(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_UnloadTexture(PyObject *self, PyObject *arg0)
{
  Texture2D x0;

  if (_cffi_to_c((char *)&x0, _cffi_type(80), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { UnloadTexture(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_UnloadTexture(Texture2D *x0)
{
  { UnloadTexture(*x0); }
}
#endif

static void _cffi_d_UnloadWave(Wave x0)
{
  UnloadWave(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_UnloadWave(PyObject *self, PyObject *arg0)
{
  Wave x0;

  if (_cffi_to_c((char *)&x0, _cffi_type(245), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { UnloadWave(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_UnloadWave(Wave *x0)
{
  { UnloadWave(*x0); }
}
#endif

static void _cffi_d_UpdateAudioStream(AudioStream x0, void const * x1, int x2)
{
  UpdateAudioStream(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_UpdateAudioStream(PyObject *self, PyObject *args)
{
  AudioStream x0;
  void const * x1;
  int x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "UpdateAudioStream", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(384), arg0) < 0)
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(526), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (void const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(526), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { UpdateAudioStream(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_UpdateAudioStream(AudioStream *x0, void const * x1, int x2)
{
  { UpdateAudioStream(*x0, x1, x2); }
}
#endif

static void _cffi_d_UpdateCamera(Camera3D * x0)
{
  UpdateCamera(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_UpdateCamera(PyObject *self, PyObject *arg0)
{
  Camera3D * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(537), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Camera3D *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(537), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { UpdateCamera(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_UpdateCamera _cffi_d_UpdateCamera
#endif

static void _cffi_d_UpdateModelAnimation(Model x0, ModelAnimation x1, int x2)
{
  UpdateModelAnimation(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_UpdateModelAnimation(PyObject *self, PyObject *args)
{
  Model x0;
  ModelAnimation x1;
  int x2;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "UpdateModelAnimation", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(214), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(404), arg1) < 0)
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { UpdateModelAnimation(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_UpdateModelAnimation(Model *x0, ModelAnimation *x1, int x2)
{
  { UpdateModelAnimation(*x0, *x1, x2); }
}
#endif

static void _cffi_d_UpdateMusicStream(Music x0)
{
  UpdateMusicStream(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_UpdateMusicStream(PyObject *self, PyObject *arg0)
{
  Music x0;

  if (_cffi_to_c((char *)&x0, _cffi_type(371), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { UpdateMusicStream(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_UpdateMusicStream(Music *x0)
{
  { UpdateMusicStream(*x0); }
}
#endif

static void _cffi_d_UpdateSound(Sound x0, void const * x1, int x2)
{
  UpdateSound(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_UpdateSound(PyObject *self, PyObject *args)
{
  Sound x0;
  void const * x1;
  int x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "UpdateSound", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(433), arg0) < 0)
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(526), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (void const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(526), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { UpdateSound(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_UpdateSound(Sound *x0, void const * x1, int x2)
{
  { UpdateSound(*x0, x1, x2); }
}
#endif

static void _cffi_d_UpdateTexture(Texture2D x0, void const * x1)
{
  UpdateTexture(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_UpdateTexture(PyObject *self, PyObject *args)
{
  Texture2D x0;
  void const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "UpdateTexture", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(80), arg0) < 0)
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(526), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (void const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(526), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { UpdateTexture(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
static void _cffi_f_UpdateTexture(Texture2D *x0, void const * x1)
{
  { UpdateTexture(*x0, x1); }
}
#endif

static void _cffi_d_UpdateVrTracking(Camera3D * x0)
{
  UpdateVrTracking(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_UpdateVrTracking(PyObject *self, PyObject *arg0)
{
  Camera3D * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(537), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Camera3D *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(537), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { UpdateVrTracking(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_UpdateVrTracking _cffi_d_UpdateVrTracking
#endif

static Wave _cffi_d_WaveCopy(Wave x0)
{
  return WaveCopy(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_WaveCopy(PyObject *self, PyObject *arg0)
{
  Wave x0;
  Wave result;
  PyObject *pyresult;

  if (_cffi_to_c((char *)&x0, _cffi_type(245), arg0) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = WaveCopy(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_struct((char *)&result, _cffi_type(245));
  return pyresult;
}
#else
static void _cffi_f_WaveCopy(Wave *result, Wave *x0)
{
  { *result = WaveCopy(*x0); }
}
#endif

static void _cffi_d_WaveCrop(Wave * x0, int x1, int x2)
{
  WaveCrop(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_WaveCrop(PyObject *self, PyObject *args)
{
  Wave * x0;
  int x1;
  int x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "WaveCrop", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1024), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Wave *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1024), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { WaveCrop(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_WaveCrop _cffi_d_WaveCrop
#endif

static void _cffi_d_WaveFormat(Wave * x0, int x1, int x2, int x3)
{
  WaveFormat(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_WaveFormat(PyObject *self, PyObject *args)
{
  Wave * x0;
  int x1;
  int x2;
  int x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "WaveFormat", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1024), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (Wave *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1024), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { WaveFormat(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_WaveFormat _cffi_d_WaveFormat
#endif

static int _cffi_d_WindowShouldClose(void)
{
  return WindowShouldClose();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_WindowShouldClose(PyObject *self, PyObject *noarg)
{
  int result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = WindowShouldClose(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_WindowShouldClose _cffi_d_WindowShouldClose
#endif

static int _cffi_const_LOC_MAP_DIFFUSE(unsigned long long *o)
{
  int n = (LOC_MAP_DIFFUSE) <= 0;
  *o = (unsigned long long)((LOC_MAP_DIFFUSE) | 0);  /* check that LOC_MAP_DIFFUSE is an integer */
  if (!_cffi_check_int(*o, n, 14U))
    n |= 2;
  return n;
}

static int _cffi_const_LOC_MAP_SPECULAR(unsigned long long *o)
{
  int n = (LOC_MAP_SPECULAR) <= 0;
  *o = (unsigned long long)((LOC_MAP_SPECULAR) | 0);  /* check that LOC_MAP_SPECULAR is an integer */
  if (!_cffi_check_int(*o, n, 15U))
    n |= 2;
  return n;
}

static int _cffi_const_MAP_DIFFUSE(unsigned long long *o)
{
  int n = (MAP_DIFFUSE) <= 0;
  *o = (unsigned long long)((MAP_DIFFUSE) | 0);  /* check that MAP_DIFFUSE is an integer */
  if (!_cffi_check_int(*o, n, 0))
    n |= 2;
  return n;
}

static int _cffi_const_MAP_SPECULAR(unsigned long long *o)
{
  int n = (MAP_SPECULAR) <= 0;
  *o = (unsigned long long)((MAP_SPECULAR) | 0);  /* check that MAP_SPECULAR is an integer */
  if (!_cffi_check_int(*o, n, 1U))
    n |= 2;
  return n;
}

static int _cffi_const_MAX_TOUCH_POINTS(unsigned long long *o)
{
  int n = (MAX_TOUCH_POINTS) <= 0;
  *o = (unsigned long long)((MAX_TOUCH_POINTS) | 0);  /* check that MAX_TOUCH_POINTS is an integer */
  if (!_cffi_check_int(*o, n, 10U))
    n |= 2;
  return n;
}

_CFFI_UNUSED_FN
static void _cffi_checkfld__AudioStream(AudioStream *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->sampleRate) | 0);  /* check that 'AudioStream.sampleRate' is an integer */
  (void)((p->sampleSize) | 0);  /* check that 'AudioStream.sampleSize' is an integer */
  (void)((p->channels) | 0);  /* check that 'AudioStream.channels' is an integer */
  { rAudioBuffer * *tmp = &p->buffer; (void)tmp; }
}
struct _cffi_align__AudioStream { char x; AudioStream y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__BoneInfo(BoneInfo *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { char(*tmp)[32] = &p->name; (void)tmp; }
  (void)((p->parent) | 0);  /* check that 'BoneInfo.parent' is an integer */
}
struct _cffi_align__BoneInfo { char x; BoneInfo y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__BoundingBox(BoundingBox *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { Vector3 *tmp = &p->min; (void)tmp; }
  { Vector3 *tmp = &p->max; (void)tmp; }
}
struct _cffi_align__BoundingBox { char x; BoundingBox y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__Camera2D(Camera2D *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { Vector2 *tmp = &p->offset; (void)tmp; }
  { Vector2 *tmp = &p->target; (void)tmp; }
  { float *tmp = &p->rotation; (void)tmp; }
  { float *tmp = &p->zoom; (void)tmp; }
}
struct _cffi_align__Camera2D { char x; Camera2D y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__Camera3D(Camera3D *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { Vector3 *tmp = &p->position; (void)tmp; }
  { Vector3 *tmp = &p->target; (void)tmp; }
  { Vector3 *tmp = &p->up; (void)tmp; }
  { float *tmp = &p->fovy; (void)tmp; }
  (void)((p->type) | 0);  /* check that 'Camera3D.type' is an integer */
}
struct _cffi_align__Camera3D { char x; Camera3D y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__CharInfo(CharInfo *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->value) | 0);  /* check that 'CharInfo.value' is an integer */
  (void)((p->offsetX) | 0);  /* check that 'CharInfo.offsetX' is an integer */
  (void)((p->offsetY) | 0);  /* check that 'CharInfo.offsetY' is an integer */
  (void)((p->advanceX) | 0);  /* check that 'CharInfo.advanceX' is an integer */
  { Image *tmp = &p->image; (void)tmp; }
}
struct _cffi_align__CharInfo { char x; CharInfo y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__Color(Color *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->r) | 0);  /* check that 'Color.r' is an integer */
  (void)((p->g) | 0);  /* check that 'Color.g' is an integer */
  (void)((p->b) | 0);  /* check that 'Color.b' is an integer */
  (void)((p->a) | 0);  /* check that 'Color.a' is an integer */
}
struct _cffi_align__Color { char x; Color y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__Font(Font *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->baseSize) | 0);  /* check that 'Font.baseSize' is an integer */
  (void)((p->charsCount) | 0);  /* check that 'Font.charsCount' is an integer */
  { Texture2D *tmp = &p->texture; (void)tmp; }
  { Rectangle * *tmp = &p->recs; (void)tmp; }
  { CharInfo * *tmp = &p->chars; (void)tmp; }
}
struct _cffi_align__Font { char x; Font y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__Image(Image *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { void * *tmp = &p->data; (void)tmp; }
  (void)((p->width) | 0);  /* check that 'Image.width' is an integer */
  (void)((p->height) | 0);  /* check that 'Image.height' is an integer */
  (void)((p->mipmaps) | 0);  /* check that 'Image.mipmaps' is an integer */
  (void)((p->format) | 0);  /* check that 'Image.format' is an integer */
}
struct _cffi_align__Image { char x; Image y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__Material(Material *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { Shader *tmp = &p->shader; (void)tmp; }
  { MaterialMap * *tmp = &p->maps; (void)tmp; }
  { float * *tmp = &p->params; (void)tmp; }
}
struct _cffi_align__Material { char x; Material y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__MaterialMap(MaterialMap *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { Texture2D *tmp = &p->texture; (void)tmp; }
  { Color *tmp = &p->color; (void)tmp; }
  { float *tmp = &p->value; (void)tmp; }
}
struct _cffi_align__MaterialMap { char x; MaterialMap y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__Matrix(Matrix *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { float *tmp = &p->m0; (void)tmp; }
  { float *tmp = &p->m4; (void)tmp; }
  { float *tmp = &p->m8; (void)tmp; }
  { float *tmp = &p->m12; (void)tmp; }
  { float *tmp = &p->m1; (void)tmp; }
  { float *tmp = &p->m5; (void)tmp; }
  { float *tmp = &p->m9; (void)tmp; }
  { float *tmp = &p->m13; (void)tmp; }
  { float *tmp = &p->m2; (void)tmp; }
  { float *tmp = &p->m6; (void)tmp; }
  { float *tmp = &p->m10; (void)tmp; }
  { float *tmp = &p->m14; (void)tmp; }
  { float *tmp = &p->m3; (void)tmp; }
  { float *tmp = &p->m7; (void)tmp; }
  { float *tmp = &p->m11; (void)tmp; }
  { float *tmp = &p->m15; (void)tmp; }
}
struct _cffi_align__Matrix { char x; Matrix y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__Mesh(Mesh *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->vertexCount) | 0);  /* check that 'Mesh.vertexCount' is an integer */
  (void)((p->triangleCount) | 0);  /* check that 'Mesh.triangleCount' is an integer */
  { float * *tmp = &p->vertices; (void)tmp; }
  { float * *tmp = &p->texcoords; (void)tmp; }
  { float * *tmp = &p->texcoords2; (void)tmp; }
  { float * *tmp = &p->normals; (void)tmp; }
  { float * *tmp = &p->tangents; (void)tmp; }
  { unsigned char * *tmp = &p->colors; (void)tmp; }
  { unsigned short * *tmp = &p->indices; (void)tmp; }
  { float * *tmp = &p->animVertices; (void)tmp; }
  { float * *tmp = &p->animNormals; (void)tmp; }
  { int * *tmp = &p->boneIds; (void)tmp; }
  { float * *tmp = &p->boneWeights; (void)tmp; }
  (void)((p->vaoId) | 0);  /* check that 'Mesh.vaoId' is an integer */
  { unsigned int * *tmp = &p->vboId; (void)tmp; }
}
struct _cffi_align__Mesh { char x; Mesh y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__Model(Model *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { Matrix *tmp = &p->transform; (void)tmp; }
  (void)((p->meshCount) | 0);  /* check that 'Model.meshCount' is an integer */
  { Mesh * *tmp = &p->meshes; (void)tmp; }
  (void)((p->materialCount) | 0);  /* check that 'Model.materialCount' is an integer */
  { Material * *tmp = &p->materials; (void)tmp; }
  { int * *tmp = &p->meshMaterial; (void)tmp; }
  (void)((p->boneCount) | 0);  /* check that 'Model.boneCount' is an integer */
  { BoneInfo * *tmp = &p->bones; (void)tmp; }
  { Transform * *tmp = &p->bindPose; (void)tmp; }
}
struct _cffi_align__Model { char x; Model y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__ModelAnimation(ModelAnimation *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->boneCount) | 0);  /* check that 'ModelAnimation.boneCount' is an integer */
  { BoneInfo * *tmp = &p->bones; (void)tmp; }
  (void)((p->frameCount) | 0);  /* check that 'ModelAnimation.frameCount' is an integer */
  { Transform * * *tmp = &p->framePoses; (void)tmp; }
}
struct _cffi_align__ModelAnimation { char x; ModelAnimation y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__Music(Music *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->ctxType) | 0);  /* check that 'Music.ctxType' is an integer */
  { void * *tmp = &p->ctxData; (void)tmp; }
  (void)((p->sampleCount) | 0);  /* check that 'Music.sampleCount' is an integer */
  (void)((p->loopCount) | 0);  /* check that 'Music.loopCount' is an integer */
  { AudioStream *tmp = &p->stream; (void)tmp; }
}
struct _cffi_align__Music { char x; Music y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__NPatchInfo(NPatchInfo *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { Rectangle *tmp = &p->sourceRec; (void)tmp; }
  (void)((p->left) | 0);  /* check that 'NPatchInfo.left' is an integer */
  (void)((p->top) | 0);  /* check that 'NPatchInfo.top' is an integer */
  (void)((p->right) | 0);  /* check that 'NPatchInfo.right' is an integer */
  (void)((p->bottom) | 0);  /* check that 'NPatchInfo.bottom' is an integer */
  (void)((p->type) | 0);  /* check that 'NPatchInfo.type' is an integer */
}
struct _cffi_align__NPatchInfo { char x; NPatchInfo y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__Ray(Ray *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { Vector3 *tmp = &p->position; (void)tmp; }
  { Vector3 *tmp = &p->direction; (void)tmp; }
}
struct _cffi_align__Ray { char x; Ray y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__RayHitInfo(RayHitInfo *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->hit) | 0);  /* check that 'RayHitInfo.hit' is an integer */
  { float *tmp = &p->distance; (void)tmp; }
  { Vector3 *tmp = &p->position; (void)tmp; }
  { Vector3 *tmp = &p->normal; (void)tmp; }
}
struct _cffi_align__RayHitInfo { char x; RayHitInfo y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__Rectangle(Rectangle *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { float *tmp = &p->x; (void)tmp; }
  { float *tmp = &p->y; (void)tmp; }
  { float *tmp = &p->width; (void)tmp; }
  { float *tmp = &p->height; (void)tmp; }
}
struct _cffi_align__Rectangle { char x; Rectangle y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__RenderTexture2D(RenderTexture2D *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->id) | 0);  /* check that 'RenderTexture2D.id' is an integer */
  { Texture2D *tmp = &p->texture; (void)tmp; }
  { Texture2D *tmp = &p->depth; (void)tmp; }
  (void)((p->depthTexture) | 0);  /* check that 'RenderTexture2D.depthTexture' is an integer */
}
struct _cffi_align__RenderTexture2D { char x; RenderTexture2D y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__Shader(Shader *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->id) | 0);  /* check that 'Shader.id' is an integer */
  { int * *tmp = &p->locs; (void)tmp; }
}
struct _cffi_align__Shader { char x; Shader y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__Sound(Sound *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->sampleCount) | 0);  /* check that 'Sound.sampleCount' is an integer */
  { AudioStream *tmp = &p->stream; (void)tmp; }
}
struct _cffi_align__Sound { char x; Sound y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__Texture2D(Texture2D *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->id) | 0);  /* check that 'Texture2D.id' is an integer */
  (void)((p->width) | 0);  /* check that 'Texture2D.width' is an integer */
  (void)((p->height) | 0);  /* check that 'Texture2D.height' is an integer */
  (void)((p->mipmaps) | 0);  /* check that 'Texture2D.mipmaps' is an integer */
  (void)((p->format) | 0);  /* check that 'Texture2D.format' is an integer */
}
struct _cffi_align__Texture2D { char x; Texture2D y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__Transform(Transform *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { Vector3 *tmp = &p->translation; (void)tmp; }
  { Vector4 *tmp = &p->rotation; (void)tmp; }
  { Vector3 *tmp = &p->scale; (void)tmp; }
}
struct _cffi_align__Transform { char x; Transform y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__Vector2(Vector2 *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { float *tmp = &p->x; (void)tmp; }
  { float *tmp = &p->y; (void)tmp; }
}
struct _cffi_align__Vector2 { char x; Vector2 y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__Vector3(Vector3 *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { float *tmp = &p->x; (void)tmp; }
  { float *tmp = &p->y; (void)tmp; }
  { float *tmp = &p->z; (void)tmp; }
}
struct _cffi_align__Vector3 { char x; Vector3 y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__Vector4(Vector4 *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { float *tmp = &p->x; (void)tmp; }
  { float *tmp = &p->y; (void)tmp; }
  { float *tmp = &p->z; (void)tmp; }
  { float *tmp = &p->w; (void)tmp; }
}
struct _cffi_align__Vector4 { char x; Vector4 y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__VrDeviceInfo(VrDeviceInfo *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->hResolution) | 0);  /* check that 'VrDeviceInfo.hResolution' is an integer */
  (void)((p->vResolution) | 0);  /* check that 'VrDeviceInfo.vResolution' is an integer */
  { float *tmp = &p->hScreenSize; (void)tmp; }
  { float *tmp = &p->vScreenSize; (void)tmp; }
  { float *tmp = &p->vScreenCenter; (void)tmp; }
  { float *tmp = &p->eyeToScreenDistance; (void)tmp; }
  { float *tmp = &p->lensSeparationDistance; (void)tmp; }
  { float *tmp = &p->interpupillaryDistance; (void)tmp; }
  { float(*tmp)[4] = &p->lensDistortionValues; (void)tmp; }
  { float(*tmp)[4] = &p->chromaAbCorrection; (void)tmp; }
}
struct _cffi_align__VrDeviceInfo { char x; VrDeviceInfo y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__Wave(Wave *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->sampleCount) | 0);  /* check that 'Wave.sampleCount' is an integer */
  (void)((p->sampleRate) | 0);  /* check that 'Wave.sampleRate' is an integer */
  (void)((p->sampleSize) | 0);  /* check that 'Wave.sampleSize' is an integer */
  (void)((p->channels) | 0);  /* check that 'Wave.channels' is an integer */
  { void * *tmp = &p->data; (void)tmp; }
}
struct _cffi_align__Wave { char x; Wave y; };

static const struct _cffi_global_s _cffi_globals[] = {
  { "BLEND_ADDITIVE", (void *)_cffi_const_BLEND_ADDITIVE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "BLEND_ALPHA", (void *)_cffi_const_BLEND_ALPHA, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "BLEND_MULTIPLIED", (void *)_cffi_const_BLEND_MULTIPLIED, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "BeginBlendMode", (void *)_cffi_f_BeginBlendMode, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 1063), (void *)_cffi_d_BeginBlendMode },
  { "BeginDrawing", (void *)_cffi_f_BeginDrawing, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 1140), (void *)_cffi_d_BeginDrawing },
  { "BeginMode2D", (void *)_cffi_f_BeginMode2D, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 533), (void *)_cffi_d_BeginMode2D },
  { "BeginMode3D", (void *)_cffi_f_BeginMode3D, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 539), (void *)_cffi_d_BeginMode3D },
  { "BeginScissorMode", (void *)_cffi_f_BeginScissorMode, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 1108), (void *)_cffi_d_BeginScissorMode },
  { "BeginShaderMode", (void *)_cffi_f_BeginShaderMode, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 802), (void *)_cffi_d_BeginShaderMode },
  { "BeginTextureMode", (void *)_cffi_f_BeginTextureMode, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 799), (void *)_cffi_d_BeginTextureMode },
  { "BeginVrDrawing", (void *)_cffi_f_BeginVrDrawing, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 1140), (void *)_cffi_d_BeginVrDrawing },
  { "CAMERA_CUSTOM", (void *)_cffi_const_CAMERA_CUSTOM, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "CAMERA_FIRST_PERSON", (void *)_cffi_const_CAMERA_FIRST_PERSON, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "CAMERA_FREE", (void *)_cffi_const_CAMERA_FREE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "CAMERA_ORBITAL", (void *)_cffi_const_CAMERA_ORBITAL, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "CAMERA_ORTHOGRAPHIC", (void *)_cffi_const_CAMERA_ORTHOGRAPHIC, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "CAMERA_PERSPECTIVE", (void *)_cffi_const_CAMERA_PERSPECTIVE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "CAMERA_THIRD_PERSON", (void *)_cffi_const_CAMERA_THIRD_PERSON, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "COMPRESSED_ASTC_4x4_RGBA", (void *)_cffi_const_COMPRESSED_ASTC_4x4_RGBA, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "COMPRESSED_ASTC_8x8_RGBA", (void *)_cffi_const_COMPRESSED_ASTC_8x8_RGBA, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "COMPRESSED_DXT1_RGB", (void *)_cffi_const_COMPRESSED_DXT1_RGB, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "COMPRESSED_DXT1_RGBA", (void *)_cffi_const_COMPRESSED_DXT1_RGBA, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "COMPRESSED_DXT3_RGBA", (void *)_cffi_const_COMPRESSED_DXT3_RGBA, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "COMPRESSED_DXT5_RGBA", (void *)_cffi_const_COMPRESSED_DXT5_RGBA, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "COMPRESSED_ETC1_RGB", (void *)_cffi_const_COMPRESSED_ETC1_RGB, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "COMPRESSED_ETC2_EAC_RGBA", (void *)_cffi_const_COMPRESSED_ETC2_EAC_RGBA, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "COMPRESSED_ETC2_RGB", (void *)_cffi_const_COMPRESSED_ETC2_RGB, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "COMPRESSED_PVRT_RGB", (void *)_cffi_const_COMPRESSED_PVRT_RGB, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "COMPRESSED_PVRT_RGBA", (void *)_cffi_const_COMPRESSED_PVRT_RGBA, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "CUBEMAP_AUTO_DETECT", (void *)_cffi_const_CUBEMAP_AUTO_DETECT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "CUBEMAP_CROSS_FOUR_BY_THREE", (void *)_cffi_const_CUBEMAP_CROSS_FOUR_BY_THREE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "CUBEMAP_CROSS_THREE_BY_FOUR", (void *)_cffi_const_CUBEMAP_CROSS_THREE_BY_FOUR, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "CUBEMAP_LINE_HORIZONTAL", (void *)_cffi_const_CUBEMAP_LINE_HORIZONTAL, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "CUBEMAP_LINE_VERTICAL", (void *)_cffi_const_CUBEMAP_LINE_VERTICAL, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "CUBEMAP_PANORAMA", (void *)_cffi_const_CUBEMAP_PANORAMA, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "ChangeDirectory", (void *)_cffi_f_ChangeDirectory, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 471), (void *)_cffi_d_ChangeDirectory },
  { "CheckCollisionBoxSphere", (void *)_cffi_f_CheckCollisionBoxSphere, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 390), (void *)_cffi_d_CheckCollisionBoxSphere },
  { "CheckCollisionBoxes", (void *)_cffi_f_CheckCollisionBoxes, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 386), (void *)_cffi_d_CheckCollisionBoxes },
  { "CheckCollisionCircleRec", (void *)_cffi_f_CheckCollisionCircleRec, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 450), (void *)_cffi_d_CheckCollisionCircleRec },
  { "CheckCollisionCircles", (void *)_cffi_f_CheckCollisionCircles, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 455), (void *)_cffi_d_CheckCollisionCircles },
  { "CheckCollisionPointCircle", (void *)_cffi_f_CheckCollisionPointCircle, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 445), (void *)_cffi_d_CheckCollisionPointCircle },
  { "CheckCollisionPointRec", (void *)_cffi_f_CheckCollisionPointRec, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 435), (void *)_cffi_d_CheckCollisionPointRec },
  { "CheckCollisionPointTriangle", (void *)_cffi_f_CheckCollisionPointTriangle, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 439), (void *)_cffi_d_CheckCollisionPointTriangle },
  { "CheckCollisionRayBox", (void *)_cffi_f_CheckCollisionRayBox, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 409), (void *)_cffi_d_CheckCollisionRayBox },
  { "CheckCollisionRaySphere", (void *)_cffi_f_CheckCollisionRaySphere, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 413), (void *)_cffi_d_CheckCollisionRaySphere },
  { "CheckCollisionRaySphereEx", (void *)_cffi_f_CheckCollisionRaySphereEx, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 418), (void *)_cffi_d_CheckCollisionRaySphereEx },
  { "CheckCollisionRecs", (void *)_cffi_f_CheckCollisionRecs, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 424), (void *)_cffi_d_CheckCollisionRecs },
  { "CheckCollisionSpheres", (void *)_cffi_f_CheckCollisionSpheres, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 461), (void *)_cffi_d_CheckCollisionSpheres },
  { "ClearBackground", (void *)_cffi_f_ClearBackground, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 561), (void *)_cffi_d_ClearBackground },
  { "ClearDirectoryFiles", (void *)_cffi_f_ClearDirectoryFiles, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 1140), (void *)_cffi_d_ClearDirectoryFiles },
  { "ClearDroppedFiles", (void *)_cffi_f_ClearDroppedFiles, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 1140), (void *)_cffi_d_ClearDroppedFiles },
  { "CloseAudioDevice", (void *)_cffi_f_CloseAudioDevice, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 1140), (void *)_cffi_d_CloseAudioDevice },
  { "CloseAudioStream", (void *)_cffi_f_CloseAudioStream, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 517), (void *)_cffi_d_CloseAudioStream },
  { "CloseVrSimulator", (void *)_cffi_f_CloseVrSimulator, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 1140), (void *)_cffi_d_CloseVrSimulator },
  { "CloseWindow", (void *)_cffi_f_CloseWindow, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 1140), (void *)_cffi_d_CloseWindow },
  { "CodepointToUtf8", (void *)_cffi_f_CodepointToUtf8, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 359), (void *)_cffi_d_CodepointToUtf8 },
  { "ColorFromHSV", (void *)_cffi_f_ColorFromHSV, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 27), (void *)_cffi_d_ColorFromHSV },
  { "ColorFromNormalized", (void *)_cffi_f_ColorFromNormalized, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 30), (void *)_cffi_d_ColorFromNormalized },
  { "ColorNormalize", (void *)_cffi_f_ColorNormalize, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 302), (void *)_cffi_d_ColorNormalize },
  { "ColorToHSV", (void *)_cffi_f_ColorToHSV, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 296), (void *)_cffi_d_ColorToHSV },
  { "ColorToInt", (void *)_cffi_f_ColorToInt, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 395), (void *)_cffi_d_ColorToInt },
  { "CompressData", (void *)_cffi_f_CompressData, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 507), (void *)_cffi_d_CompressData },
  { "DecompressData", (void *)_cffi_f_DecompressData, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 507), (void *)_cffi_d_DecompressData },
  { "DirectoryExists", (void *)_cffi_f_DirectoryExists, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 471), (void *)_cffi_d_DirectoryExists },
  { "DisableCursor", (void *)_cffi_f_DisableCursor, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 1140), (void *)_cffi_d_DisableCursor },
  { "DrawBillboard", (void *)_cffi_f_DrawBillboard, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 550), (void *)_cffi_d_DrawBillboard },
  { "DrawBillboardRec", (void *)_cffi_f_DrawBillboardRec, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 542), (void *)_cffi_d_DrawBillboardRec },
  { "DrawBoundingBox", (void *)_cffi_f_DrawBoundingBox, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 529), (void *)_cffi_d_DrawBoundingBox },
  { "DrawCircle", (void *)_cffi_f_DrawCircle, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 1088), (void *)_cffi_d_DrawCircle },
  { "DrawCircle3D", (void *)_cffi_f_DrawCircle3D, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 990), (void *)_cffi_d_DrawCircle3D },
  { "DrawCircleGradient", (void *)_cffi_f_DrawCircleGradient, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 1094), (void *)_cffi_d_DrawCircleGradient },
  { "DrawCircleLines", (void *)_cffi_f_DrawCircleLines, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 1088), (void *)_cffi_d_DrawCircleLines },
  { "DrawCircleSector", (void *)_cffi_f_DrawCircleSector, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 953), (void *)_cffi_d_DrawCircleSector },
  { "DrawCircleSectorLines", (void *)_cffi_f_DrawCircleSectorLines, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 953), (void *)_cffi_d_DrawCircleSectorLines },
  { "DrawCircleV", (void *)_cffi_f_DrawCircleV, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 939), (void *)_cffi_d_DrawCircleV },
  { "DrawCube", (void *)_cffi_f_DrawCube, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 997), (void *)_cffi_d_DrawCube },
  { "DrawCubeTexture", (void *)_cffi_f_DrawCubeTexture, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 891), (void *)_cffi_d_DrawCubeTexture },
  { "DrawCubeV", (void *)_cffi_f_DrawCubeV, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 980), (void *)_cffi_d_DrawCubeV },
  { "DrawCubeWires", (void *)_cffi_f_DrawCubeWires, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 997), (void *)_cffi_d_DrawCubeWires },
  { "DrawCubeWiresV", (void *)_cffi_f_DrawCubeWiresV, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 980), (void *)_cffi_d_DrawCubeWiresV },
  { "DrawCylinder", (void *)_cffi_f_DrawCylinder, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 1004), (void *)_cffi_d_DrawCylinder },
  { "DrawCylinderWires", (void *)_cffi_f_DrawCylinderWires, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 1004), (void *)_cffi_d_DrawCylinderWires },
  { "DrawEllipse", (void *)_cffi_f_DrawEllipse, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 1101), (void *)_cffi_d_DrawEllipse },
  { "DrawEllipseLines", (void *)_cffi_f_DrawEllipseLines, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 1101), (void *)_cffi_d_DrawEllipseLines },
  { "DrawFPS", (void *)_cffi_f_DrawFPS, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 1074), (void *)_cffi_d_DrawFPS },
  { "DrawGizmo", (void *)_cffi_f_DrawGizmo, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 968), (void *)_cffi_d_DrawGizmo },
  { "DrawGrid", (void *)_cffi_f_DrawGrid, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 1070), (void *)_cffi_d_DrawGrid },
  { "DrawLine", (void *)_cffi_f_DrawLine, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 1114), (void *)_cffi_d_DrawLine },
  { "DrawLine3D", (void *)_cffi_f_DrawLine3D, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 980), (void *)_cffi_d_DrawLine3D },
  { "DrawLineBezier", (void *)_cffi_f_DrawLineBezier, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 933), (void *)_cffi_d_DrawLineBezier },
  { "DrawLineEx", (void *)_cffi_f_DrawLineEx, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 933), (void *)_cffi_d_DrawLineEx },
  { "DrawLineStrip", (void *)_cffi_f_DrawLineStrip, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 913), (void *)_cffi_d_DrawLineStrip },
  { "DrawLineV", (void *)_cffi_f_DrawLineV, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 922), (void *)_cffi_d_DrawLineV },
  { "DrawModel", (void *)_cffi_f_DrawModel, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 740), (void *)_cffi_d_DrawModel },
  { "DrawModelEx", (void *)_cffi_f_DrawModelEx, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 732), (void *)_cffi_d_DrawModelEx },
  { "DrawModelWires", (void *)_cffi_f_DrawModelWires, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 740), (void *)_cffi_d_DrawModelWires },
  { "DrawModelWiresEx", (void *)_cffi_f_DrawModelWiresEx, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 732), (void *)_cffi_d_DrawModelWiresEx },
  { "DrawPixel", (void *)_cffi_f_DrawPixel, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 1078), (void *)_cffi_d_DrawPixel },
  { "DrawPixelV", (void *)_cffi_f_DrawPixelV, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 918), (void *)_cffi_d_DrawPixelV },
  { "DrawPlane", (void *)_cffi_f_DrawPlane, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 975), (void *)_cffi_d_DrawPlane },
  { "DrawPoint3D", (void *)_cffi_f_DrawPoint3D, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 971), (void *)_cffi_d_DrawPoint3D },
  { "DrawPoly", (void *)_cffi_f_DrawPoly, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 961), (void *)_cffi_d_DrawPoly },
  { "DrawPolyLines", (void *)_cffi_f_DrawPolyLines, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 961), (void *)_cffi_d_DrawPolyLines },
  { "DrawRay", (void *)_cffi_f_DrawRay, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 760), (void *)_cffi_d_DrawRay },
  { "DrawRectangle", (void *)_cffi_f_DrawRectangle, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 1114), (void *)_cffi_d_DrawRectangle },
  { "DrawRectangleGradientEx", (void *)_cffi_f_DrawRectangleGradientEx, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 768), (void *)_cffi_d_DrawRectangleGradientEx },
  { "DrawRectangleGradientH", (void *)_cffi_f_DrawRectangleGradientH, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 1121), (void *)_cffi_d_DrawRectangleGradientH },
  { "DrawRectangleGradientV", (void *)_cffi_f_DrawRectangleGradientV, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 1121), (void *)_cffi_d_DrawRectangleGradientV },
  { "DrawRectangleLines", (void *)_cffi_f_DrawRectangleLines, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 1114), (void *)_cffi_d_DrawRectangleLines },
  { "DrawRectangleLinesEx", (void *)_cffi_f_DrawRectangleLinesEx, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 794), (void *)_cffi_d_DrawRectangleLinesEx },
  { "DrawRectanglePro", (void *)_cffi_f_DrawRectanglePro, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 775), (void *)_cffi_d_DrawRectanglePro },
  { "DrawRectangleRec", (void *)_cffi_f_DrawRectangleRec, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 764), (void *)_cffi_d_DrawRectangleRec },
  { "DrawRectangleRounded", (void *)_cffi_f_DrawRectangleRounded, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 781), (void *)_cffi_d_DrawRectangleRounded },
  { "DrawRectangleRoundedLines", (void *)_cffi_f_DrawRectangleRoundedLines, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 787), (void *)_cffi_d_DrawRectangleRoundedLines },
  { "DrawRectangleV", (void *)_cffi_f_DrawRectangleV, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 922), (void *)_cffi_d_DrawRectangleV },
  { "DrawRing", (void *)_cffi_f_DrawRing, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 944), (void *)_cffi_d_DrawRing },
  { "DrawRingLines", (void *)_cffi_f_DrawRingLines, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 944), (void *)_cffi_d_DrawRingLines },
  { "DrawSphere", (void *)_cffi_f_DrawSphere, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 985), (void *)_cffi_d_DrawSphere },
  { "DrawSphereEx", (void *)_cffi_f_DrawSphereEx, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 1012), (void *)_cffi_d_DrawSphereEx },
  { "DrawSphereWires", (void *)_cffi_f_DrawSphereWires, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 1012), (void *)_cffi_d_DrawSphereWires },
  { "DrawText", (void *)_cffi_f_DrawText, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 1049), (void *)_cffi_d_DrawText },
  { "DrawTextCodepoint", (void *)_cffi_f_DrawTextCodepoint, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 597), (void *)_cffi_d_DrawTextCodepoint },
  { "DrawTextEx", (void *)_cffi_f_DrawTextEx, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 589), (void *)_cffi_d_DrawTextEx },
  { "DrawTextRec", (void *)_cffi_f_DrawTextRec, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 567), (void *)_cffi_d_DrawTextRec },
  { "DrawTextRecEx", (void *)_cffi_f_DrawTextRecEx, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 576), (void *)_cffi_d_DrawTextRecEx },
  { "DrawTexture", (void *)_cffi_f_DrawTexture, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 903), (void *)_cffi_d_DrawTexture },
  { "DrawTextureEx", (void *)_cffi_f_DrawTextureEx, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 884), (void *)_cffi_d_DrawTextureEx },
  { "DrawTextureNPatch", (void *)_cffi_f_DrawTextureNPatch, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 846), (void *)_cffi_d_DrawTextureNPatch },
  { "DrawTexturePro", (void *)_cffi_f_DrawTexturePro, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 858), (void *)_cffi_d_DrawTexturePro },
  { "DrawTextureQuad", (void *)_cffi_f_DrawTextureQuad, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 877), (void *)_cffi_d_DrawTextureQuad },
  { "DrawTextureRec", (void *)_cffi_f_DrawTextureRec, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 866), (void *)_cffi_d_DrawTextureRec },
  { "DrawTextureV", (void *)_cffi_f_DrawTextureV, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 872), (void *)_cffi_d_DrawTextureV },
  { "DrawTriangle", (void *)_cffi_f_DrawTriangle, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 927), (void *)_cffi_d_DrawTriangle },
  { "DrawTriangleFan", (void *)_cffi_f_DrawTriangleFan, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 913), (void *)_cffi_d_DrawTriangleFan },
  { "DrawTriangleLines", (void *)_cffi_f_DrawTriangleLines, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 927), (void *)_cffi_d_DrawTriangleLines },
  { "DrawTriangleStrip", (void *)_cffi_f_DrawTriangleStrip, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 913), (void *)_cffi_d_DrawTriangleStrip },
  { "EnableCursor", (void *)_cffi_f_EnableCursor, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 1140), (void *)_cffi_d_EnableCursor },
  { "EndBlendMode", (void *)_cffi_f_EndBlendMode, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 1140), (void *)_cffi_d_EndBlendMode },
  { "EndDrawing", (void *)_cffi_f_EndDrawing, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 1140), (void *)_cffi_d_EndDrawing },
  { "EndMode2D", (void *)_cffi_f_EndMode2D, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 1140), (void *)_cffi_d_EndMode2D },
  { "EndMode3D", (void *)_cffi_f_EndMode3D, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 1140), (void *)_cffi_d_EndMode3D },
  { "EndScissorMode", (void *)_cffi_f_EndScissorMode, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 1140), (void *)_cffi_d_EndScissorMode },
  { "EndShaderMode", (void *)_cffi_f_EndShaderMode, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 1140), (void *)_cffi_d_EndShaderMode },
  { "EndTextureMode", (void *)_cffi_f_EndTextureMode, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 1140), (void *)_cffi_d_EndTextureMode },
  { "EndVrDrawing", (void *)_cffi_f_EndVrDrawing, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 1140), (void *)_cffi_d_EndVrDrawing },
  { "ExportImage", (void *)_cffi_f_ExportImage, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 694), (void *)_cffi_d_ExportImage },
  { "ExportImageAsCode", (void *)_cffi_f_ExportImageAsCode, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 694), (void *)_cffi_d_ExportImageAsCode },
  { "ExportMesh", (void *)_cffi_f_ExportMesh, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 715), (void *)_cffi_d_ExportMesh },
  { "ExportWave", (void *)_cffi_f_ExportWave, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 1037), (void *)_cffi_d_ExportWave },
  { "ExportWaveAsCode", (void *)_cffi_f_ExportWaveAsCode, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 1037), (void *)_cffi_d_ExportWaveAsCode },
  { "FILTER_ANISOTROPIC_16X", (void *)_cffi_const_FILTER_ANISOTROPIC_16X, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "FILTER_ANISOTROPIC_4X", (void *)_cffi_const_FILTER_ANISOTROPIC_4X, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "FILTER_ANISOTROPIC_8X", (void *)_cffi_const_FILTER_ANISOTROPIC_8X, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "FILTER_BILINEAR", (void *)_cffi_const_FILTER_BILINEAR, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "FILTER_POINT", (void *)_cffi_const_FILTER_POINT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "FILTER_TRILINEAR", (void *)_cffi_const_FILTER_TRILINEAR, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "FLAG_FULLSCREEN_MODE", (void *)_cffi_const_FLAG_FULLSCREEN_MODE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "FLAG_MSAA_4X_HINT", (void *)_cffi_const_FLAG_MSAA_4X_HINT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "FLAG_RESERVED", (void *)_cffi_const_FLAG_RESERVED, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "FLAG_VSYNC_HINT", (void *)_cffi_const_FLAG_VSYNC_HINT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "FLAG_WINDOW_ALWAYS_RUN", (void *)_cffi_const_FLAG_WINDOW_ALWAYS_RUN, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "FLAG_WINDOW_HIDDEN", (void *)_cffi_const_FLAG_WINDOW_HIDDEN, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "FLAG_WINDOW_RESIZABLE", (void *)_cffi_const_FLAG_WINDOW_RESIZABLE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "FLAG_WINDOW_TRANSPARENT", (void *)_cffi_const_FLAG_WINDOW_TRANSPARENT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "FLAG_WINDOW_UNDECORATED", (void *)_cffi_const_FLAG_WINDOW_UNDECORATED, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "FONT_BITMAP", (void *)_cffi_const_FONT_BITMAP, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "FONT_DEFAULT", (void *)_cffi_const_FONT_DEFAULT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "FONT_SDF", (void *)_cffi_const_FONT_SDF, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "Fade", (void *)_cffi_f_Fade, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 23), (void *)_cffi_d_Fade },
  { "FileExists", (void *)_cffi_f_FileExists, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 471), (void *)_cffi_d_FileExists },
  { "GAMEPAD_AXIS_LEFT_TRIGGER", (void *)_cffi_const_GAMEPAD_AXIS_LEFT_TRIGGER, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GAMEPAD_AXIS_LEFT_X", (void *)_cffi_const_GAMEPAD_AXIS_LEFT_X, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GAMEPAD_AXIS_LEFT_Y", (void *)_cffi_const_GAMEPAD_AXIS_LEFT_Y, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GAMEPAD_AXIS_RIGHT_TRIGGER", (void *)_cffi_const_GAMEPAD_AXIS_RIGHT_TRIGGER, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GAMEPAD_AXIS_RIGHT_X", (void *)_cffi_const_GAMEPAD_AXIS_RIGHT_X, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GAMEPAD_AXIS_RIGHT_Y", (void *)_cffi_const_GAMEPAD_AXIS_RIGHT_Y, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GAMEPAD_AXIS_UNKNOWN", (void *)_cffi_const_GAMEPAD_AXIS_UNKNOWN, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GAMEPAD_BUTTON_LEFT_FACE_DOWN", (void *)_cffi_const_GAMEPAD_BUTTON_LEFT_FACE_DOWN, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GAMEPAD_BUTTON_LEFT_FACE_LEFT", (void *)_cffi_const_GAMEPAD_BUTTON_LEFT_FACE_LEFT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GAMEPAD_BUTTON_LEFT_FACE_RIGHT", (void *)_cffi_const_GAMEPAD_BUTTON_LEFT_FACE_RIGHT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GAMEPAD_BUTTON_LEFT_FACE_UP", (void *)_cffi_const_GAMEPAD_BUTTON_LEFT_FACE_UP, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GAMEPAD_BUTTON_LEFT_THUMB", (void *)_cffi_const_GAMEPAD_BUTTON_LEFT_THUMB, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GAMEPAD_BUTTON_LEFT_TRIGGER_1", (void *)_cffi_const_GAMEPAD_BUTTON_LEFT_TRIGGER_1, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GAMEPAD_BUTTON_LEFT_TRIGGER_2", (void *)_cffi_const_GAMEPAD_BUTTON_LEFT_TRIGGER_2, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GAMEPAD_BUTTON_MIDDLE", (void *)_cffi_const_GAMEPAD_BUTTON_MIDDLE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GAMEPAD_BUTTON_MIDDLE_LEFT", (void *)_cffi_const_GAMEPAD_BUTTON_MIDDLE_LEFT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GAMEPAD_BUTTON_MIDDLE_RIGHT", (void *)_cffi_const_GAMEPAD_BUTTON_MIDDLE_RIGHT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GAMEPAD_BUTTON_RIGHT_FACE_DOWN", (void *)_cffi_const_GAMEPAD_BUTTON_RIGHT_FACE_DOWN, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GAMEPAD_BUTTON_RIGHT_FACE_LEFT", (void *)_cffi_const_GAMEPAD_BUTTON_RIGHT_FACE_LEFT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GAMEPAD_BUTTON_RIGHT_FACE_RIGHT", (void *)_cffi_const_GAMEPAD_BUTTON_RIGHT_FACE_RIGHT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GAMEPAD_BUTTON_RIGHT_FACE_UP", (void *)_cffi_const_GAMEPAD_BUTTON_RIGHT_FACE_UP, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GAMEPAD_BUTTON_RIGHT_THUMB", (void *)_cffi_const_GAMEPAD_BUTTON_RIGHT_THUMB, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GAMEPAD_BUTTON_RIGHT_TRIGGER_1", (void *)_cffi_const_GAMEPAD_BUTTON_RIGHT_TRIGGER_1, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GAMEPAD_BUTTON_RIGHT_TRIGGER_2", (void *)_cffi_const_GAMEPAD_BUTTON_RIGHT_TRIGGER_2, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GAMEPAD_BUTTON_UNKNOWN", (void *)_cffi_const_GAMEPAD_BUTTON_UNKNOWN, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GAMEPAD_PLAYER1", (void *)_cffi_const_GAMEPAD_PLAYER1, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GAMEPAD_PLAYER2", (void *)_cffi_const_GAMEPAD_PLAYER2, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GAMEPAD_PLAYER3", (void *)_cffi_const_GAMEPAD_PLAYER3, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GAMEPAD_PLAYER4", (void *)_cffi_const_GAMEPAD_PLAYER4, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GESTURE_DOUBLETAP", (void *)_cffi_const_GESTURE_DOUBLETAP, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GESTURE_DRAG", (void *)_cffi_const_GESTURE_DRAG, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GESTURE_HOLD", (void *)_cffi_const_GESTURE_HOLD, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GESTURE_NONE", (void *)_cffi_const_GESTURE_NONE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GESTURE_PINCH_IN", (void *)_cffi_const_GESTURE_PINCH_IN, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GESTURE_PINCH_OUT", (void *)_cffi_const_GESTURE_PINCH_OUT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GESTURE_SWIPE_DOWN", (void *)_cffi_const_GESTURE_SWIPE_DOWN, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GESTURE_SWIPE_LEFT", (void *)_cffi_const_GESTURE_SWIPE_LEFT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GESTURE_SWIPE_RIGHT", (void *)_cffi_const_GESTURE_SWIPE_RIGHT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GESTURE_SWIPE_UP", (void *)_cffi_const_GESTURE_SWIPE_UP, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GESTURE_TAP", (void *)_cffi_const_GESTURE_TAP, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GenImageCellular", (void *)_cffi_f_GenImageCellular, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 120), (void *)_cffi_d_GenImageCellular },
  { "GenImageChecked", (void *)_cffi_f_GenImageChecked, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 125), (void *)_cffi_d_GenImageChecked },
  { "GenImageColor", (void *)_cffi_f_GenImageColor, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 97), (void *)_cffi_d_GenImageColor },
  { "GenImageFontAtlas", (void *)_cffi_f_GenImageFontAtlas, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 52), (void *)_cffi_d_GenImageFontAtlas },
  { "GenImageGradientH", (void *)_cffi_f_GenImageGradientH, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 102), (void *)_cffi_d_GenImageGradientH },
  { "GenImageGradientRadial", (void *)_cffi_f_GenImageGradientRadial, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 113), (void *)_cffi_d_GenImageGradientRadial },
  { "GenImageGradientV", (void *)_cffi_f_GenImageGradientV, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 102), (void *)_cffi_d_GenImageGradientV },
  { "GenImagePerlinNoise", (void *)_cffi_f_GenImagePerlinNoise, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 133), (void *)_cffi_d_GenImagePerlinNoise },
  { "GenImageWhiteNoise", (void *)_cffi_f_GenImageWhiteNoise, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 108), (void *)_cffi_d_GenImageWhiteNoise },
  { "GenMeshCube", (void *)_cffi_f_GenMeshCube, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 170), (void *)_cffi_d_GenMeshCube },
  { "GenMeshCubicmap", (void *)_cffi_f_GenMeshCubicmap, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 166), (void *)_cffi_d_GenMeshCubicmap },
  { "GenMeshCylinder", (void *)_cffi_f_GenMeshCylinder, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 175), (void *)_cffi_d_GenMeshCylinder },
  { "GenMeshHeightmap", (void *)_cffi_f_GenMeshHeightmap, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 166), (void *)_cffi_d_GenMeshHeightmap },
  { "GenMeshHemiSphere", (void *)_cffi_f_GenMeshHemiSphere, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 186), (void *)_cffi_d_GenMeshHemiSphere },
  { "GenMeshKnot", (void *)_cffi_f_GenMeshKnot, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 180), (void *)_cffi_d_GenMeshKnot },
  { "GenMeshPlane", (void *)_cffi_f_GenMeshPlane, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 180), (void *)_cffi_d_GenMeshPlane },
  { "GenMeshPoly", (void *)_cffi_f_GenMeshPoly, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 191), (void *)_cffi_d_GenMeshPoly },
  { "GenMeshSphere", (void *)_cffi_f_GenMeshSphere, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 186), (void *)_cffi_d_GenMeshSphere },
  { "GenMeshTorus", (void *)_cffi_f_GenMeshTorus, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 180), (void *)_cffi_d_GenMeshTorus },
  { "GenTextureBRDF", (void *)_cffi_f_GenTextureBRDF, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 262), (void *)_cffi_d_GenTextureBRDF },
  { "GenTextureCubemap", (void *)_cffi_f_GenTextureCubemap, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 257), (void *)_cffi_d_GenTextureCubemap },
  { "GenTextureIrradiance", (void *)_cffi_f_GenTextureIrradiance, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 257), (void *)_cffi_d_GenTextureIrradiance },
  { "GenTextureMipmaps", (void *)_cffi_f_GenTextureMipmaps, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 840), (void *)_cffi_d_GenTextureMipmaps },
  { "GenTexturePrefilter", (void *)_cffi_f_GenTexturePrefilter, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 257), (void *)_cffi_d_GenTexturePrefilter },
  { "GetCameraMatrix", (void *)_cffi_f_GetCameraMatrix, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 157), (void *)_cffi_d_GetCameraMatrix },
  { "GetCameraMatrix2D", (void *)_cffi_f_GetCameraMatrix2D, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 154), (void *)_cffi_d_GetCameraMatrix2D },
  { "GetClipboardText", (void *)_cffi_f_GetClipboardText, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 363), (void *)_cffi_d_GetClipboardText },
  { "GetCodepoints", (void *)_cffi_f_GetCodepoints, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 379), (void *)_cffi_d_GetCodepoints },
  { "GetCodepointsCount", (void *)_cffi_f_GetCodepointsCount, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 471), (void *)_cffi_d_GetCodepointsCount },
  { "GetCollisionRayGround", (void *)_cffi_f_GetCollisionRayGround, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 222), (void *)_cffi_d_GetCollisionRayGround },
  { "GetCollisionRayModel", (void *)_cffi_f_GetCollisionRayModel, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 212), (void *)_cffi_d_GetCollisionRayModel },
  { "GetCollisionRayTriangle", (void *)_cffi_f_GetCollisionRayTriangle, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 216), (void *)_cffi_d_GetCollisionRayTriangle },
  { "GetCollisionRec", (void *)_cffi_f_GetCollisionRec, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 230), (void *)_cffi_d_GetCollisionRec },
  { "GetColor", (void *)_cffi_f_GetColor, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 33), (void *)_cffi_d_GetColor },
  { "GetDirectoryFiles", (void *)_cffi_f_GetDirectoryFiles, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 311), (void *)_cffi_d_GetDirectoryFiles },
  { "GetDirectoryPath", (void *)_cffi_f_GetDirectoryPath, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 345), (void *)_cffi_d_GetDirectoryPath },
  { "GetDroppedFiles", (void *)_cffi_f_GetDroppedFiles, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 315), (void *)_cffi_d_GetDroppedFiles },
  { "GetExtension", (void *)_cffi_f_GetExtension, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 345), (void *)_cffi_d_GetExtension },
  { "GetFPS", (void *)_cffi_f_GetFPS, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 502), (void *)_cffi_d_GetFPS },
  { "GetFileModTime", (void *)_cffi_f_GetFileModTime, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 504), (void *)_cffi_d_GetFileModTime },
  { "GetFileName", (void *)_cffi_f_GetFileName, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 345), (void *)_cffi_d_GetFileName },
  { "GetFileNameWithoutExt", (void *)_cffi_f_GetFileNameWithoutExt, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 345), (void *)_cffi_d_GetFileNameWithoutExt },
  { "GetFontDefault", (void *)_cffi_f_GetFontDefault, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 50), (void *)_cffi_d_GetFontDefault },
  { "GetFrameTime", (void *)_cffi_f_GetFrameTime, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 377), (void *)_cffi_d_GetFrameTime },
  { "GetGamepadAxisCount", (void *)_cffi_f_GetGamepadAxisCount, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 486), (void *)_cffi_d_GetGamepadAxisCount },
  { "GetGamepadAxisMovement", (void *)_cffi_f_GetGamepadAxisMovement, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 373), (void *)_cffi_d_GetGamepadAxisMovement },
  { "GetGamepadButtonPressed", (void *)_cffi_f_GetGamepadButtonPressed, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 502), (void *)_cffi_d_GetGamepadButtonPressed },
  { "GetGamepadName", (void *)_cffi_f_GetGamepadName, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 356), (void *)_cffi_d_GetGamepadName },
  { "GetGestureDetected", (void *)_cffi_f_GetGestureDetected, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 502), (void *)_cffi_d_GetGestureDetected },
  { "GetGestureDragAngle", (void *)_cffi_f_GetGestureDragAngle, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 377), (void *)_cffi_d_GetGestureDragAngle },
  { "GetGestureDragVector", (void *)_cffi_f_GetGestureDragVector, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 294), (void *)_cffi_d_GetGestureDragVector },
  { "GetGestureHoldDuration", (void *)_cffi_f_GetGestureHoldDuration, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 377), (void *)_cffi_d_GetGestureHoldDuration },
  { "GetGesturePinchAngle", (void *)_cffi_f_GetGesturePinchAngle, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 377), (void *)_cffi_d_GetGesturePinchAngle },
  { "GetGesturePinchVector", (void *)_cffi_f_GetGesturePinchVector, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 294), (void *)_cffi_d_GetGesturePinchVector },
  { "GetGlyphIndex", (void *)_cffi_f_GetGlyphIndex, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 398), (void *)_cffi_d_GetGlyphIndex },
  { "GetImageAlphaBorder", (void *)_cffi_f_GetImageAlphaBorder, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 226), (void *)_cffi_d_GetImageAlphaBorder },
  { "GetImageData", (void *)_cffi_f_GetImageData, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 15), (void *)_cffi_d_GetImageData },
  { "GetImageDataNormalized", (void *)_cffi_f_GetImageDataNormalized, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 299), (void *)_cffi_d_GetImageDataNormalized },
  { "GetKeyPressed", (void *)_cffi_f_GetKeyPressed, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 502), (void *)_cffi_d_GetKeyPressed },
  { "GetMatrixModelview", (void *)_cffi_f_GetMatrixModelview, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 160), (void *)_cffi_d_GetMatrixModelview },
  { "GetMatrixProjection", (void *)_cffi_f_GetMatrixProjection, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 160), (void *)_cffi_d_GetMatrixProjection },
  { "GetMonitorCount", (void *)_cffi_f_GetMonitorCount, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 502), (void *)_cffi_d_GetMonitorCount },
  { "GetMonitorHeight", (void *)_cffi_f_GetMonitorHeight, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 486), (void *)_cffi_d_GetMonitorHeight },
  { "GetMonitorName", (void *)_cffi_f_GetMonitorName, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 356), (void *)_cffi_d_GetMonitorName },
  { "GetMonitorPhysicalHeight", (void *)_cffi_f_GetMonitorPhysicalHeight, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 486), (void *)_cffi_d_GetMonitorPhysicalHeight },
  { "GetMonitorPhysicalWidth", (void *)_cffi_f_GetMonitorPhysicalWidth, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 486), (void *)_cffi_d_GetMonitorPhysicalWidth },
  { "GetMonitorWidth", (void *)_cffi_f_GetMonitorWidth, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 486), (void *)_cffi_d_GetMonitorWidth },
  { "GetMousePosition", (void *)_cffi_f_GetMousePosition, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 294), (void *)_cffi_d_GetMousePosition },
  { "GetMouseRay", (void *)_cffi_f_GetMouseRay, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 208), (void *)_cffi_d_GetMouseRay },
  { "GetMouseWheelMove", (void *)_cffi_f_GetMouseWheelMove, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 502), (void *)_cffi_d_GetMouseWheelMove },
  { "GetMouseX", (void *)_cffi_f_GetMouseX, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 502), (void *)_cffi_d_GetMouseX },
  { "GetMouseY", (void *)_cffi_f_GetMouseY, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 502), (void *)_cffi_d_GetMouseY },
  { "GetMusicTimeLength", (void *)_cffi_f_GetMusicTimeLength, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 370), (void *)_cffi_d_GetMusicTimeLength },
  { "GetMusicTimePlayed", (void *)_cffi_f_GetMusicTimePlayed, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 370), (void *)_cffi_d_GetMusicTimePlayed },
  { "GetNextCodepoint", (void *)_cffi_f_GetNextCodepoint, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 478), (void *)_cffi_d_GetNextCodepoint },
  { "GetPixelDataSize", (void *)_cffi_f_GetPixelDataSize, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 497), (void *)_cffi_d_GetPixelDataSize },
  { "GetPrevDirectoryPath", (void *)_cffi_f_GetPrevDirectoryPath, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 345), (void *)_cffi_d_GetPrevDirectoryPath },
  { "GetRandomValue", (void *)_cffi_f_GetRandomValue, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 493), (void *)_cffi_d_GetRandomValue },
  { "GetScreenData", (void *)_cffi_f_GetScreenData, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 146), (void *)_cffi_d_GetScreenData },
  { "GetScreenHeight", (void *)_cffi_f_GetScreenHeight, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 502), (void *)_cffi_d_GetScreenHeight },
  { "GetScreenToWorld2D", (void *)_cffi_f_GetScreenToWorld2D, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 277), (void *)_cffi_d_GetScreenToWorld2D },
  { "GetScreenWidth", (void *)_cffi_f_GetScreenWidth, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 502), (void *)_cffi_d_GetScreenWidth },
  { "GetShaderDefault", (void *)_cffi_f_GetShaderDefault, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 242), (void *)_cffi_d_GetShaderDefault },
  { "GetShaderLocation", (void *)_cffi_f_GetShaderLocation, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 428), (void *)_cffi_d_GetShaderLocation },
  { "GetSoundsPlaying", (void *)_cffi_f_GetSoundsPlaying, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 502), (void *)_cffi_d_GetSoundsPlaying },
  { "GetTextureData", (void *)_cffi_f_GetTextureData, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 79), (void *)_cffi_d_GetTextureData },
  { "GetTextureDefault", (void *)_cffi_f_GetTextureDefault, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 269), (void *)_cffi_d_GetTextureDefault },
  { "GetTime", (void *)_cffi_f_GetTime, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 365), (void *)_cffi_d_GetTime },
  { "GetTouchPointsCount", (void *)_cffi_f_GetTouchPointsCount, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 502), (void *)_cffi_d_GetTouchPointsCount },
  { "GetTouchPosition", (void *)_cffi_f_GetTouchPosition, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 291), (void *)_cffi_d_GetTouchPosition },
  { "GetTouchX", (void *)_cffi_f_GetTouchX, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 502), (void *)_cffi_d_GetTouchX },
  { "GetTouchY", (void *)_cffi_f_GetTouchY, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 502), (void *)_cffi_d_GetTouchY },
  { "GetWaveData", (void *)_cffi_f_GetWaveData, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 367), (void *)_cffi_d_GetWaveData },
  { "GetWindowHandle", (void *)_cffi_f_GetWindowHandle, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 515), (void *)_cffi_d_GetWindowHandle },
  { "GetWindowPosition", (void *)_cffi_f_GetWindowPosition, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 294), (void *)_cffi_d_GetWindowPosition },
  { "GetWorkingDirectory", (void *)_cffi_f_GetWorkingDirectory, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 363), (void *)_cffi_d_GetWorkingDirectory },
  { "GetWorldToScreen", (void *)_cffi_f_GetWorldToScreen, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 281), (void *)_cffi_d_GetWorldToScreen },
  { "GetWorldToScreen2D", (void *)_cffi_f_GetWorldToScreen2D, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 277), (void *)_cffi_d_GetWorldToScreen2D },
  { "GetWorldToScreenEx", (void *)_cffi_f_GetWorldToScreenEx, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 285), (void *)_cffi_d_GetWorldToScreenEx },
  { "HideCursor", (void *)_cffi_f_HideCursor, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 1140), (void *)_cffi_d_HideCursor },
  { "HideWindow", (void *)_cffi_f_HideWindow, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 1140), (void *)_cffi_d_HideWindow },
  { "ImageAlphaClear", (void *)_cffi_f_ImageAlphaClear, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 616), (void *)_cffi_d_ImageAlphaClear },
  { "ImageAlphaCrop", (void *)_cffi_f_ImageAlphaCrop, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 663), (void *)_cffi_d_ImageAlphaCrop },
  { "ImageAlphaMask", (void *)_cffi_f_ImageAlphaMask, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 621), (void *)_cffi_d_ImageAlphaMask },
  { "ImageAlphaPremultiply", (void *)_cffi_f_ImageAlphaPremultiply, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 604), (void *)_cffi_d_ImageAlphaPremultiply },
  { "ImageColorBrightness", (void *)_cffi_f_ImageColorBrightness, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 667), (void *)_cffi_d_ImageColorBrightness },
  { "ImageColorContrast", (void *)_cffi_f_ImageColorContrast, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 663), (void *)_cffi_d_ImageColorContrast },
  { "ImageColorGrayscale", (void *)_cffi_f_ImageColorGrayscale, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 604), (void *)_cffi_d_ImageColorGrayscale },
  { "ImageColorInvert", (void *)_cffi_f_ImageColorInvert, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 604), (void *)_cffi_d_ImageColorInvert },
  { "ImageColorReplace", (void *)_cffi_f_ImageColorReplace, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 611), (void *)_cffi_d_ImageColorReplace },
  { "ImageColorTint", (void *)_cffi_f_ImageColorTint, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 607), (void *)_cffi_d_ImageColorTint },
  { "ImageCopy", (void *)_cffi_f_ImageCopy, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 72), (void *)_cffi_d_ImageCopy },
  { "ImageCrop", (void *)_cffi_f_ImageCrop, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 632), (void *)_cffi_d_ImageCrop },
  { "ImageDither", (void *)_cffi_f_ImageDither, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 676), (void *)_cffi_d_ImageDither },
  { "ImageDraw", (void *)_cffi_f_ImageDraw, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 625), (void *)_cffi_d_ImageDraw },
  { "ImageDrawRectangle", (void *)_cffi_f_ImageDrawRectangle, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 636), (void *)_cffi_d_ImageDrawRectangle },
  { "ImageDrawRectangleLines", (void *)_cffi_f_ImageDrawRectangleLines, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 641), (void *)_cffi_d_ImageDrawRectangleLines },
  { "ImageDrawText", (void *)_cffi_f_ImageDrawText, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 656), (void *)_cffi_d_ImageDrawText },
  { "ImageDrawTextEx", (void *)_cffi_f_ImageDrawTextEx, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 647), (void *)_cffi_d_ImageDrawTextEx },
  { "ImageExtractPalette", (void *)_cffi_f_ImageExtractPalette, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 18), (void *)_cffi_d_ImageExtractPalette },
  { "ImageFlipHorizontal", (void *)_cffi_f_ImageFlipHorizontal, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 604), (void *)_cffi_d_ImageFlipHorizontal },
  { "ImageFlipVertical", (void *)_cffi_f_ImageFlipVertical, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 604), (void *)_cffi_d_ImageFlipVertical },
  { "ImageFormat", (void *)_cffi_f_ImageFormat, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 667), (void *)_cffi_d_ImageFormat },
  { "ImageFromImage", (void *)_cffi_f_ImageFromImage, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 75), (void *)_cffi_d_ImageFromImage },
  { "ImageMipmaps", (void *)_cffi_f_ImageMipmaps, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 604), (void *)_cffi_d_ImageMipmaps },
  { "ImageResize", (void *)_cffi_f_ImageResize, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 671), (void *)_cffi_d_ImageResize },
  { "ImageResizeCanvas", (void *)_cffi_f_ImageResizeCanvas, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 683), (void *)_cffi_d_ImageResizeCanvas },
  { "ImageResizeNN", (void *)_cffi_f_ImageResizeNN, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 671), (void *)_cffi_d_ImageResizeNN },
  { "ImageRotateCCW", (void *)_cffi_f_ImageRotateCCW, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 604), (void *)_cffi_d_ImageRotateCCW },
  { "ImageRotateCW", (void *)_cffi_f_ImageRotateCW, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 604), (void *)_cffi_d_ImageRotateCW },
  { "ImageText", (void *)_cffi_f_ImageText, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 85), (void *)_cffi_d_ImageText },
  { "ImageTextEx", (void *)_cffi_f_ImageTextEx, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 65), (void *)_cffi_d_ImageTextEx },
  { "ImageToPOT", (void *)_cffi_f_ImageToPOT, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 607), (void *)_cffi_d_ImageToPOT },
  { "InitAudioDevice", (void *)_cffi_f_InitAudioDevice, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 1140), (void *)_cffi_d_InitAudioDevice },
  { "InitAudioStream", (void *)_cffi_f_InitAudioStream, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_InitAudioStream },
  { "InitVrSimulator", (void *)_cffi_f_InitVrSimulator, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 1140), (void *)_cffi_d_InitVrSimulator },
  { "InitWindow", (void *)_cffi_f_InitWindow, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 1083), (void *)_cffi_d_InitWindow },
  { "IsAudioDeviceReady", (void *)_cffi_f_IsAudioDeviceReady, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 502), (void *)_cffi_d_IsAudioDeviceReady },
  { "IsAudioStreamPlaying", (void *)_cffi_f_IsAudioStreamPlaying, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 383), (void *)_cffi_d_IsAudioStreamPlaying },
  { "IsAudioStreamProcessed", (void *)_cffi_f_IsAudioStreamProcessed, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 383), (void *)_cffi_d_IsAudioStreamProcessed },
  { "IsCursorHidden", (void *)_cffi_f_IsCursorHidden, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 502), (void *)_cffi_d_IsCursorHidden },
  { "IsFileDropped", (void *)_cffi_f_IsFileDropped, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 502), (void *)_cffi_d_IsFileDropped },
  { "IsFileExtension", (void *)_cffi_f_IsFileExtension, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 474), (void *)_cffi_d_IsFileExtension },
  { "IsGamepadAvailable", (void *)_cffi_f_IsGamepadAvailable, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 486), (void *)_cffi_d_IsGamepadAvailable },
  { "IsGamepadButtonDown", (void *)_cffi_f_IsGamepadButtonDown, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 493), (void *)_cffi_d_IsGamepadButtonDown },
  { "IsGamepadButtonPressed", (void *)_cffi_f_IsGamepadButtonPressed, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 493), (void *)_cffi_d_IsGamepadButtonPressed },
  { "IsGamepadButtonReleased", (void *)_cffi_f_IsGamepadButtonReleased, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 493), (void *)_cffi_d_IsGamepadButtonReleased },
  { "IsGamepadButtonUp", (void *)_cffi_f_IsGamepadButtonUp, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 493), (void *)_cffi_d_IsGamepadButtonUp },
  { "IsGamepadName", (void *)_cffi_f_IsGamepadName, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 489), (void *)_cffi_d_IsGamepadName },
  { "IsGestureDetected", (void *)_cffi_f_IsGestureDetected, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 486), (void *)_cffi_d_IsGestureDetected },
  { "IsKeyDown", (void *)_cffi_f_IsKeyDown, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 486), (void *)_cffi_d_IsKeyDown },
  { "IsKeyPressed", (void *)_cffi_f_IsKeyPressed, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 486), (void *)_cffi_d_IsKeyPressed },
  { "IsKeyReleased", (void *)_cffi_f_IsKeyReleased, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 486), (void *)_cffi_d_IsKeyReleased },
  { "IsKeyUp", (void *)_cffi_f_IsKeyUp, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 486), (void *)_cffi_d_IsKeyUp },
  { "IsModelAnimationValid", (void *)_cffi_f_IsModelAnimationValid, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 402), (void *)_cffi_d_IsModelAnimationValid },
  { "IsMouseButtonDown", (void *)_cffi_f_IsMouseButtonDown, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 486), (void *)_cffi_d_IsMouseButtonDown },
  { "IsMouseButtonPressed", (void *)_cffi_f_IsMouseButtonPressed, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 486), (void *)_cffi_d_IsMouseButtonPressed },
  { "IsMouseButtonReleased", (void *)_cffi_f_IsMouseButtonReleased, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 486), (void *)_cffi_d_IsMouseButtonReleased },
  { "IsMouseButtonUp", (void *)_cffi_f_IsMouseButtonUp, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 486), (void *)_cffi_d_IsMouseButtonUp },
  { "IsMusicPlaying", (void *)_cffi_f_IsMusicPlaying, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 406), (void *)_cffi_d_IsMusicPlaying },
  { "IsSoundPlaying", (void *)_cffi_f_IsSoundPlaying, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 432), (void *)_cffi_d_IsSoundPlaying },
  { "IsVrSimulatorReady", (void *)_cffi_f_IsVrSimulatorReady, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 502), (void *)_cffi_d_IsVrSimulatorReady },
  { "IsWindowHidden", (void *)_cffi_f_IsWindowHidden, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 502), (void *)_cffi_d_IsWindowHidden },
  { "IsWindowMinimized", (void *)_cffi_f_IsWindowMinimized, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 502), (void *)_cffi_d_IsWindowMinimized },
  { "IsWindowReady", (void *)_cffi_f_IsWindowReady, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 502), (void *)_cffi_d_IsWindowReady },
  { "IsWindowResized", (void *)_cffi_f_IsWindowResized, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 502), (void *)_cffi_d_IsWindowResized },
  { "KEY_A", (void *)_cffi_const_KEY_A, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_APOSTROPHE", (void *)_cffi_const_KEY_APOSTROPHE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_B", (void *)_cffi_const_KEY_B, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_BACK", (void *)_cffi_const_KEY_BACK, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_BACKSLASH", (void *)_cffi_const_KEY_BACKSLASH, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_BACKSPACE", (void *)_cffi_const_KEY_BACKSPACE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_C", (void *)_cffi_const_KEY_C, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_CAPS_LOCK", (void *)_cffi_const_KEY_CAPS_LOCK, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_COMMA", (void *)_cffi_const_KEY_COMMA, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_D", (void *)_cffi_const_KEY_D, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_DELETE", (void *)_cffi_const_KEY_DELETE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_DOWN", (void *)_cffi_const_KEY_DOWN, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_E", (void *)_cffi_const_KEY_E, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_EIGHT", (void *)_cffi_const_KEY_EIGHT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_END", (void *)_cffi_const_KEY_END, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_ENTER", (void *)_cffi_const_KEY_ENTER, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_EQUAL", (void *)_cffi_const_KEY_EQUAL, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_ESCAPE", (void *)_cffi_const_KEY_ESCAPE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_F", (void *)_cffi_const_KEY_F, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_F1", (void *)_cffi_const_KEY_F1, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_F10", (void *)_cffi_const_KEY_F10, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_F11", (void *)_cffi_const_KEY_F11, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_F12", (void *)_cffi_const_KEY_F12, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_F2", (void *)_cffi_const_KEY_F2, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_F3", (void *)_cffi_const_KEY_F3, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_F4", (void *)_cffi_const_KEY_F4, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_F5", (void *)_cffi_const_KEY_F5, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_F6", (void *)_cffi_const_KEY_F6, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_F7", (void *)_cffi_const_KEY_F7, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_F8", (void *)_cffi_const_KEY_F8, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_F9", (void *)_cffi_const_KEY_F9, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_FIVE", (void *)_cffi_const_KEY_FIVE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_FOUR", (void *)_cffi_const_KEY_FOUR, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_G", (void *)_cffi_const_KEY_G, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_GRAVE", (void *)_cffi_const_KEY_GRAVE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_H", (void *)_cffi_const_KEY_H, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_HOME", (void *)_cffi_const_KEY_HOME, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_I", (void *)_cffi_const_KEY_I, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_INSERT", (void *)_cffi_const_KEY_INSERT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_J", (void *)_cffi_const_KEY_J, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_K", (void *)_cffi_const_KEY_K, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_KB_MENU", (void *)_cffi_const_KEY_KB_MENU, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_KP_0", (void *)_cffi_const_KEY_KP_0, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_KP_1", (void *)_cffi_const_KEY_KP_1, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_KP_2", (void *)_cffi_const_KEY_KP_2, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_KP_3", (void *)_cffi_const_KEY_KP_3, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_KP_4", (void *)_cffi_const_KEY_KP_4, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_KP_5", (void *)_cffi_const_KEY_KP_5, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_KP_6", (void *)_cffi_const_KEY_KP_6, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_KP_7", (void *)_cffi_const_KEY_KP_7, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_KP_8", (void *)_cffi_const_KEY_KP_8, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_KP_9", (void *)_cffi_const_KEY_KP_9, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_KP_ADD", (void *)_cffi_const_KEY_KP_ADD, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_KP_DECIMAL", (void *)_cffi_const_KEY_KP_DECIMAL, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_KP_DIVIDE", (void *)_cffi_const_KEY_KP_DIVIDE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_KP_ENTER", (void *)_cffi_const_KEY_KP_ENTER, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_KP_EQUAL", (void *)_cffi_const_KEY_KP_EQUAL, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_KP_MULTIPLY", (void *)_cffi_const_KEY_KP_MULTIPLY, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_KP_SUBTRACT", (void *)_cffi_const_KEY_KP_SUBTRACT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_L", (void *)_cffi_const_KEY_L, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_LEFT", (void *)_cffi_const_KEY_LEFT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_LEFT_ALT", (void *)_cffi_const_KEY_LEFT_ALT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_LEFT_BRACKET", (void *)_cffi_const_KEY_LEFT_BRACKET, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_LEFT_CONTROL", (void *)_cffi_const_KEY_LEFT_CONTROL, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_LEFT_SHIFT", (void *)_cffi_const_KEY_LEFT_SHIFT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_LEFT_SUPER", (void *)_cffi_const_KEY_LEFT_SUPER, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_M", (void *)_cffi_const_KEY_M, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_MENU", (void *)_cffi_const_KEY_MENU, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_MINUS", (void *)_cffi_const_KEY_MINUS, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_N", (void *)_cffi_const_KEY_N, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_NINE", (void *)_cffi_const_KEY_NINE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_NUM_LOCK", (void *)_cffi_const_KEY_NUM_LOCK, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_O", (void *)_cffi_const_KEY_O, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_ONE", (void *)_cffi_const_KEY_ONE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_P", (void *)_cffi_const_KEY_P, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_PAGE_DOWN", (void *)_cffi_const_KEY_PAGE_DOWN, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_PAGE_UP", (void *)_cffi_const_KEY_PAGE_UP, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_PAUSE", (void *)_cffi_const_KEY_PAUSE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_PERIOD", (void *)_cffi_const_KEY_PERIOD, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_PRINT_SCREEN", (void *)_cffi_const_KEY_PRINT_SCREEN, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_Q", (void *)_cffi_const_KEY_Q, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_R", (void *)_cffi_const_KEY_R, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_RIGHT", (void *)_cffi_const_KEY_RIGHT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_RIGHT_ALT", (void *)_cffi_const_KEY_RIGHT_ALT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_RIGHT_BRACKET", (void *)_cffi_const_KEY_RIGHT_BRACKET, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_RIGHT_CONTROL", (void *)_cffi_const_KEY_RIGHT_CONTROL, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_RIGHT_SHIFT", (void *)_cffi_const_KEY_RIGHT_SHIFT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_RIGHT_SUPER", (void *)_cffi_const_KEY_RIGHT_SUPER, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_S", (void *)_cffi_const_KEY_S, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_SCROLL_LOCK", (void *)_cffi_const_KEY_SCROLL_LOCK, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_SEMICOLON", (void *)_cffi_const_KEY_SEMICOLON, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_SEVEN", (void *)_cffi_const_KEY_SEVEN, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_SIX", (void *)_cffi_const_KEY_SIX, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_SLASH", (void *)_cffi_const_KEY_SLASH, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_SPACE", (void *)_cffi_const_KEY_SPACE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_T", (void *)_cffi_const_KEY_T, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_TAB", (void *)_cffi_const_KEY_TAB, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_THREE", (void *)_cffi_const_KEY_THREE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_TWO", (void *)_cffi_const_KEY_TWO, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_U", (void *)_cffi_const_KEY_U, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_UP", (void *)_cffi_const_KEY_UP, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_V", (void *)_cffi_const_KEY_V, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_VOLUME_DOWN", (void *)_cffi_const_KEY_VOLUME_DOWN, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_VOLUME_UP", (void *)_cffi_const_KEY_VOLUME_UP, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_W", (void *)_cffi_const_KEY_W, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_X", (void *)_cffi_const_KEY_X, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_Y", (void *)_cffi_const_KEY_Y, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_Z", (void *)_cffi_const_KEY_Z, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "KEY_ZERO", (void *)_cffi_const_KEY_ZERO, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LOC_COLOR_AMBIENT", (void *)_cffi_const_LOC_COLOR_AMBIENT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LOC_COLOR_DIFFUSE", (void *)_cffi_const_LOC_COLOR_DIFFUSE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LOC_COLOR_SPECULAR", (void *)_cffi_const_LOC_COLOR_SPECULAR, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LOC_MAP_ALBEDO", (void *)_cffi_const_LOC_MAP_ALBEDO, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LOC_MAP_BRDF", (void *)_cffi_const_LOC_MAP_BRDF, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LOC_MAP_CUBEMAP", (void *)_cffi_const_LOC_MAP_CUBEMAP, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LOC_MAP_DIFFUSE", (void *)_cffi_const_LOC_MAP_DIFFUSE, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "LOC_MAP_EMISSION", (void *)_cffi_const_LOC_MAP_EMISSION, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LOC_MAP_HEIGHT", (void *)_cffi_const_LOC_MAP_HEIGHT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LOC_MAP_IRRADIANCE", (void *)_cffi_const_LOC_MAP_IRRADIANCE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LOC_MAP_METALNESS", (void *)_cffi_const_LOC_MAP_METALNESS, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LOC_MAP_NORMAL", (void *)_cffi_const_LOC_MAP_NORMAL, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LOC_MAP_OCCLUSION", (void *)_cffi_const_LOC_MAP_OCCLUSION, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LOC_MAP_PREFILTER", (void *)_cffi_const_LOC_MAP_PREFILTER, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LOC_MAP_ROUGHNESS", (void *)_cffi_const_LOC_MAP_ROUGHNESS, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LOC_MAP_SPECULAR", (void *)_cffi_const_LOC_MAP_SPECULAR, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "LOC_MATRIX_MODEL", (void *)_cffi_const_LOC_MATRIX_MODEL, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LOC_MATRIX_MVP", (void *)_cffi_const_LOC_MATRIX_MVP, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LOC_MATRIX_PROJECTION", (void *)_cffi_const_LOC_MATRIX_PROJECTION, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LOC_MATRIX_VIEW", (void *)_cffi_const_LOC_MATRIX_VIEW, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LOC_VECTOR_VIEW", (void *)_cffi_const_LOC_VECTOR_VIEW, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LOC_VERTEX_COLOR", (void *)_cffi_const_LOC_VERTEX_COLOR, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LOC_VERTEX_NORMAL", (void *)_cffi_const_LOC_VERTEX_NORMAL, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LOC_VERTEX_POSITION", (void *)_cffi_const_LOC_VERTEX_POSITION, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LOC_VERTEX_TANGENT", (void *)_cffi_const_LOC_VERTEX_TANGENT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LOC_VERTEX_TEXCOORD01", (void *)_cffi_const_LOC_VERTEX_TEXCOORD01, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LOC_VERTEX_TEXCOORD02", (void *)_cffi_const_LOC_VERTEX_TEXCOORD02, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LOG_ALL", (void *)_cffi_const_LOG_ALL, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LOG_DEBUG", (void *)_cffi_const_LOG_DEBUG, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LOG_ERROR", (void *)_cffi_const_LOG_ERROR, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LOG_FATAL", (void *)_cffi_const_LOG_FATAL, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LOG_INFO", (void *)_cffi_const_LOG_INFO, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LOG_NONE", (void *)_cffi_const_LOG_NONE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LOG_TRACE", (void *)_cffi_const_LOG_TRACE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LOG_WARNING", (void *)_cffi_const_LOG_WARNING, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LoadFont", (void *)_cffi_f_LoadFont, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 41), (void *)_cffi_d_LoadFont },
  { "LoadFontData", (void *)_cffi_f_LoadFontData, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 8), (void *)_cffi_d_LoadFontData },
  { "LoadFontEx", (void *)_cffi_f_LoadFontEx, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 44), (void *)_cffi_d_LoadFontEx },
  { "LoadFontFromImage", (void *)_cffi_f_LoadFontFromImage, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 36), (void *)_cffi_d_LoadFontFromImage },
  { "LoadImage", (void *)_cffi_f_LoadImage, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 82), (void *)_cffi_d_LoadImage },
  { "LoadImageEx", (void *)_cffi_f_LoadImageEx, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 60), (void *)_cffi_d_LoadImageEx },
  { "LoadImagePro", (void *)_cffi_f_LoadImagePro, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 140), (void *)_cffi_d_LoadImagePro },
  { "LoadImageRaw", (void *)_cffi_f_LoadImageRaw, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 90), (void *)_cffi_d_LoadImageRaw },
  { "LoadMaterialDefault", (void *)_cffi_f_LoadMaterialDefault, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 152), (void *)_cffi_d_LoadMaterialDefault },
  { "LoadMaterials", (void *)_cffi_f_LoadMaterials, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 148), (void *)_cffi_d_LoadMaterials },
  { "LoadMeshes", (void *)_cffi_f_LoadMeshes, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 162), (void *)_cffi_d_LoadMeshes },
  { "LoadModel", (void *)_cffi_f_LoadModel, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 198), (void *)_cffi_d_LoadModel },
  { "LoadModelAnimations", (void *)_cffi_f_LoadModelAnimations, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 201), (void *)_cffi_d_LoadModelAnimations },
  { "LoadModelFromMesh", (void *)_cffi_f_LoadModelFromMesh, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 195), (void *)_cffi_d_LoadModelFromMesh },
  { "LoadMusicStream", (void *)_cffi_f_LoadMusicStream, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 205), (void *)_cffi_d_LoadMusicStream },
  { "LoadRenderTexture", (void *)_cffi_f_LoadRenderTexture, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 234), (void *)_cffi_d_LoadRenderTexture },
  { "LoadShader", (void *)_cffi_f_LoadShader, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 238), (void *)_cffi_d_LoadShader },
  { "LoadShaderCode", (void *)_cffi_f_LoadShaderCode, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 238), (void *)_cffi_d_LoadShaderCode },
  { "LoadSound", (void *)_cffi_f_LoadSound, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 247), (void *)_cffi_d_LoadSound },
  { "LoadSoundFromWave", (void *)_cffi_f_LoadSoundFromWave, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 244), (void *)_cffi_d_LoadSoundFromWave },
  { "LoadText", (void *)_cffi_f_LoadText, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 323), (void *)_cffi_d_LoadText },
  { "LoadTexture", (void *)_cffi_f_LoadTexture, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 266), (void *)_cffi_d_LoadTexture },
  { "LoadTextureCubemap", (void *)_cffi_f_LoadTextureCubemap, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 253), (void *)_cffi_d_LoadTextureCubemap },
  { "LoadTextureFromImage", (void *)_cffi_f_LoadTextureFromImage, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 250), (void *)_cffi_d_LoadTextureFromImage },
  { "LoadWave", (void *)_cffi_f_LoadWave, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 308), (void *)_cffi_d_LoadWave },
  { "MAP_ALBEDO", (void *)_cffi_const_MAP_ALBEDO, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "MAP_BRDF", (void *)_cffi_const_MAP_BRDF, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "MAP_CUBEMAP", (void *)_cffi_const_MAP_CUBEMAP, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "MAP_DIFFUSE", (void *)_cffi_const_MAP_DIFFUSE, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "MAP_EMISSION", (void *)_cffi_const_MAP_EMISSION, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "MAP_HEIGHT", (void *)_cffi_const_MAP_HEIGHT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "MAP_IRRADIANCE", (void *)_cffi_const_MAP_IRRADIANCE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "MAP_METALNESS", (void *)_cffi_const_MAP_METALNESS, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "MAP_NORMAL", (void *)_cffi_const_MAP_NORMAL, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "MAP_OCCLUSION", (void *)_cffi_const_MAP_OCCLUSION, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "MAP_PREFILTER", (void *)_cffi_const_MAP_PREFILTER, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "MAP_ROUGHNESS", (void *)_cffi_const_MAP_ROUGHNESS, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "MAP_SPECULAR", (void *)_cffi_const_MAP_SPECULAR, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "MAX_TOUCH_POINTS", (void *)_cffi_const_MAX_TOUCH_POINTS, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "MOUSE_LEFT_BUTTON", (void *)_cffi_const_MOUSE_LEFT_BUTTON, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "MOUSE_MIDDLE_BUTTON", (void *)_cffi_const_MOUSE_MIDDLE_BUTTON, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "MOUSE_RIGHT_BUTTON", (void *)_cffi_const_MOUSE_RIGHT_BUTTON, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "MeasureText", (void *)_cffi_f_MeasureText, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 482), (void *)_cffi_d_MeasureText },
  { "MeasureTextEx", (void *)_cffi_f_MeasureTextEx, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 271), (void *)_cffi_d_MeasureTextEx },
  { "MeshBinormals", (void *)_cffi_f_MeshBinormals, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 709), (void *)_cffi_d_MeshBinormals },
  { "MeshBoundingBox", (void *)_cffi_f_MeshBoundingBox, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 5), (void *)_cffi_d_MeshBoundingBox },
  { "MeshTangents", (void *)_cffi_f_MeshTangents, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 709), (void *)_cffi_d_MeshTangents },
  { "NPT_3PATCH_HORIZONTAL", (void *)_cffi_const_NPT_3PATCH_HORIZONTAL, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "NPT_3PATCH_VERTICAL", (void *)_cffi_const_NPT_3PATCH_VERTICAL, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "NPT_9PATCH", (void *)_cffi_const_NPT_9PATCH, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "OpenURL", (void *)_cffi_f_OpenURL, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 1046), (void *)_cffi_d_OpenURL },
  { "PauseAudioStream", (void *)_cffi_f_PauseAudioStream, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 517), (void *)_cffi_d_PauseAudioStream },
  { "PauseMusicStream", (void *)_cffi_f_PauseMusicStream, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 749), (void *)_cffi_d_PauseMusicStream },
  { "PauseSound", (void *)_cffi_f_PauseSound, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 828), (void *)_cffi_d_PauseSound },
  { "PlayAudioStream", (void *)_cffi_f_PlayAudioStream, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 517), (void *)_cffi_d_PlayAudioStream },
  { "PlayMusicStream", (void *)_cffi_f_PlayMusicStream, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 749), (void *)_cffi_d_PlayMusicStream },
  { "PlaySound", (void *)_cffi_f_PlaySound, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 828), (void *)_cffi_d_PlaySound },
  { "PlaySoundMulti", (void *)_cffi_f_PlaySoundMulti, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 828), (void *)_cffi_d_PlaySoundMulti },
  { "ResumeAudioStream", (void *)_cffi_f_ResumeAudioStream, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 517), (void *)_cffi_d_ResumeAudioStream },
  { "ResumeMusicStream", (void *)_cffi_f_ResumeMusicStream, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 749), (void *)_cffi_d_ResumeMusicStream },
  { "ResumeSound", (void *)_cffi_f_ResumeSound, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 828), (void *)_cffi_d_ResumeSound },
  { "SetAudioStreamPitch", (void *)_cffi_f_SetAudioStreamPitch, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 520), (void *)_cffi_d_SetAudioStreamPitch },
  { "SetAudioStreamVolume", (void *)_cffi_f_SetAudioStreamVolume, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 520), (void *)_cffi_d_SetAudioStreamVolume },
  { "SetCameraAltControl", (void *)_cffi_f_SetCameraAltControl, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 1063), (void *)_cffi_d_SetCameraAltControl },
  { "SetCameraMode", (void *)_cffi_f_SetCameraMode, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 557), (void *)_cffi_d_SetCameraMode },
  { "SetCameraMoveControls", (void *)_cffi_f_SetCameraMoveControls, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 1129), (void *)_cffi_d_SetCameraMoveControls },
  { "SetCameraPanControl", (void *)_cffi_f_SetCameraPanControl, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 1063), (void *)_cffi_d_SetCameraPanControl },
  { "SetCameraSmoothZoomControl", (void *)_cffi_f_SetCameraSmoothZoomControl, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 1063), (void *)_cffi_d_SetCameraSmoothZoomControl },
  { "SetClipboardText", (void *)_cffi_f_SetClipboardText, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 1046), (void *)_cffi_d_SetClipboardText },
  { "SetConfigFlags", (void *)_cffi_f_SetConfigFlags, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 1137), (void *)_cffi_d_SetConfigFlags },
  { "SetExitKey", (void *)_cffi_f_SetExitKey, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 1063), (void *)_cffi_d_SetExitKey },
  { "SetGesturesEnabled", (void *)_cffi_f_SetGesturesEnabled, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 1137), (void *)_cffi_d_SetGesturesEnabled },
  { "SetMasterVolume", (void *)_cffi_f_SetMasterVolume, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 1056), (void *)_cffi_d_SetMasterVolume },
  { "SetMaterialTexture", (void *)_cffi_f_SetMaterialTexture, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 698), (void *)_cffi_d_SetMaterialTexture },
  { "SetMatrixModelview", (void *)_cffi_f_SetMatrixModelview, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 706), (void *)_cffi_d_SetMatrixModelview },
  { "SetMatrixProjection", (void *)_cffi_f_SetMatrixProjection, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 706), (void *)_cffi_d_SetMatrixProjection },
  { "SetModelMeshMaterial", (void *)_cffi_f_SetModelMeshMaterial, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 719), (void *)_cffi_d_SetModelMeshMaterial },
  { "SetMouseOffset", (void *)_cffi_f_SetMouseOffset, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 1074), (void *)_cffi_d_SetMouseOffset },
  { "SetMousePosition", (void *)_cffi_f_SetMousePosition, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 1074), (void *)_cffi_d_SetMousePosition },
  { "SetMouseScale", (void *)_cffi_f_SetMouseScale, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 1059), (void *)_cffi_d_SetMouseScale },
  { "SetMusicLoopCount", (void *)_cffi_f_SetMusicLoopCount, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 756), (void *)_cffi_d_SetMusicLoopCount },
  { "SetMusicPitch", (void *)_cffi_f_SetMusicPitch, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 752), (void *)_cffi_d_SetMusicPitch },
  { "SetMusicVolume", (void *)_cffi_f_SetMusicVolume, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 752), (void *)_cffi_d_SetMusicVolume },
  { "SetShaderValue", (void *)_cffi_f_SetShaderValue, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 815), (void *)_cffi_d_SetShaderValue },
  { "SetShaderValueMatrix", (void *)_cffi_f_SetShaderValueMatrix, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 805), (void *)_cffi_d_SetShaderValueMatrix },
  { "SetShaderValueTexture", (void *)_cffi_f_SetShaderValueTexture, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 810), (void *)_cffi_d_SetShaderValueTexture },
  { "SetShaderValueV", (void *)_cffi_f_SetShaderValueV, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 821), (void *)_cffi_d_SetShaderValueV },
  { "SetShapesTexture", (void *)_cffi_f_SetShapesTexture, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 854), (void *)_cffi_d_SetShapesTexture },
  { "SetSoundPitch", (void *)_cffi_f_SetSoundPitch, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 831), (void *)_cffi_d_SetSoundPitch },
  { "SetSoundVolume", (void *)_cffi_f_SetSoundVolume, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 831), (void *)_cffi_d_SetSoundVolume },
  { "SetTargetFPS", (void *)_cffi_f_SetTargetFPS, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 1063), (void *)_cffi_d_SetTargetFPS },
  { "SetTextureFilter", (void *)_cffi_f_SetTextureFilter, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 899), (void *)_cffi_d_SetTextureFilter },
  { "SetTextureWrap", (void *)_cffi_f_SetTextureWrap, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 899), (void *)_cffi_d_SetTextureWrap },
  { "SetTraceLogExit", (void *)_cffi_f_SetTraceLogExit, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 1063), (void *)_cffi_d_SetTraceLogExit },
  { "SetTraceLogLevel", (void *)_cffi_f_SetTraceLogLevel, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 1063), (void *)_cffi_d_SetTraceLogLevel },
  { "SetVrConfiguration", (void *)_cffi_f_SetVrConfiguration, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 1019), (void *)_cffi_d_SetVrConfiguration },
  { "SetWindowIcon", (void *)_cffi_f_SetWindowIcon, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 691), (void *)_cffi_d_SetWindowIcon },
  { "SetWindowMinSize", (void *)_cffi_f_SetWindowMinSize, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 1074), (void *)_cffi_d_SetWindowMinSize },
  { "SetWindowMonitor", (void *)_cffi_f_SetWindowMonitor, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 1063), (void *)_cffi_d_SetWindowMonitor },
  { "SetWindowPosition", (void *)_cffi_f_SetWindowPosition, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 1074), (void *)_cffi_d_SetWindowPosition },
  { "SetWindowSize", (void *)_cffi_f_SetWindowSize, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 1074), (void *)_cffi_d_SetWindowSize },
  { "SetWindowTitle", (void *)_cffi_f_SetWindowTitle, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 1046), (void *)_cffi_d_SetWindowTitle },
  { "ShowCursor", (void *)_cffi_f_ShowCursor, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 1140), (void *)_cffi_d_ShowCursor },
  { "StopAudioStream", (void *)_cffi_f_StopAudioStream, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 517), (void *)_cffi_d_StopAudioStream },
  { "StopMusicStream", (void *)_cffi_f_StopMusicStream, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 749), (void *)_cffi_d_StopMusicStream },
  { "StopSound", (void *)_cffi_f_StopSound, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 828), (void *)_cffi_d_StopSound },
  { "StopSoundMulti", (void *)_cffi_f_StopSoundMulti, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 1140), (void *)_cffi_d_StopSoundMulti },
  { "StorageLoadValue", (void *)_cffi_f_StorageLoadValue, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 486), (void *)_cffi_d_StorageLoadValue },
  { "StorageSaveValue", (void *)_cffi_f_StorageSaveValue, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 1074), (void *)_cffi_d_StorageSaveValue },
  { "TakeScreenshot", (void *)_cffi_f_TakeScreenshot, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 1046), (void *)_cffi_d_TakeScreenshot },
  { "TextAppend", (void *)_cffi_f_TextAppend, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 1041), (void *)_cffi_d_TextAppend },
  { "TextCopy", (void *)_cffi_f_TextCopy, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 467), (void *)_cffi_d_TextCopy },
  { "TextFindIndex", (void *)_cffi_f_TextFindIndex, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 474), (void *)_cffi_d_TextFindIndex },
  { "TextFormat", (void *)_cffi_const_TextFormat, _CFFI_OP(_CFFI_OP_CONSTANT, 1177), (void *)0 },
  { "TextInsert", (void *)_cffi_f_TextInsert, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 326), (void *)_cffi_d_TextInsert },
  { "TextIsEqual", (void *)_cffi_f_TextIsEqual, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 474), (void *)_cffi_d_TextIsEqual },
  { "TextJoin", (void *)_cffi_f_TextJoin, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 340), (void *)_cffi_d_TextJoin },
  { "TextLength", (void *)_cffi_f_TextLength, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 512), (void *)_cffi_d_TextLength },
  { "TextReplace", (void *)_cffi_f_TextReplace, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 318), (void *)_cffi_d_TextReplace },
  { "TextSplit", (void *)_cffi_f_TextSplit, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 335), (void *)_cffi_d_TextSplit },
  { "TextSubtext", (void *)_cffi_f_TextSubtext, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 351), (void *)_cffi_d_TextSubtext },
  { "TextToInteger", (void *)_cffi_f_TextToInteger, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 471), (void *)_cffi_d_TextToInteger },
  { "TextToLower", (void *)_cffi_f_TextToLower, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 345), (void *)_cffi_d_TextToLower },
  { "TextToPascal", (void *)_cffi_f_TextToPascal, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 345), (void *)_cffi_d_TextToPascal },
  { "TextToUpper", (void *)_cffi_f_TextToUpper, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 345), (void *)_cffi_d_TextToUpper },
  { "TextToUtf8", (void *)_cffi_f_TextToUtf8, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 331), (void *)_cffi_d_TextToUtf8 },
  { "ToggleFullscreen", (void *)_cffi_f_ToggleFullscreen, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 1140), (void *)_cffi_d_ToggleFullscreen },
  { "ToggleVrMode", (void *)_cffi_f_ToggleVrMode, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 1140), (void *)_cffi_d_ToggleVrMode },
  { "TraceLog", (void *)_cffi_const_TraceLog, _CFFI_OP(_CFFI_OP_CONSTANT, 1191), (void *)0 },
  { "UNCOMPRESSED_GRAYSCALE", (void *)_cffi_const_UNCOMPRESSED_GRAYSCALE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "UNCOMPRESSED_GRAY_ALPHA", (void *)_cffi_const_UNCOMPRESSED_GRAY_ALPHA, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "UNCOMPRESSED_R32", (void *)_cffi_const_UNCOMPRESSED_R32, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "UNCOMPRESSED_R32G32B32", (void *)_cffi_const_UNCOMPRESSED_R32G32B32, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "UNCOMPRESSED_R32G32B32A32", (void *)_cffi_const_UNCOMPRESSED_R32G32B32A32, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "UNCOMPRESSED_R4G4B4A4", (void *)_cffi_const_UNCOMPRESSED_R4G4B4A4, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "UNCOMPRESSED_R5G5B5A1", (void *)_cffi_const_UNCOMPRESSED_R5G5B5A1, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "UNCOMPRESSED_R5G6B5", (void *)_cffi_const_UNCOMPRESSED_R5G6B5, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "UNCOMPRESSED_R8G8B8", (void *)_cffi_const_UNCOMPRESSED_R8G8B8, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "UNCOMPRESSED_R8G8B8A8", (void *)_cffi_const_UNCOMPRESSED_R8G8B8A8, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "UNIFORM_FLOAT", (void *)_cffi_const_UNIFORM_FLOAT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "UNIFORM_INT", (void *)_cffi_const_UNIFORM_INT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "UNIFORM_IVEC2", (void *)_cffi_const_UNIFORM_IVEC2, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "UNIFORM_IVEC3", (void *)_cffi_const_UNIFORM_IVEC3, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "UNIFORM_IVEC4", (void *)_cffi_const_UNIFORM_IVEC4, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "UNIFORM_SAMPLER2D", (void *)_cffi_const_UNIFORM_SAMPLER2D, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "UNIFORM_VEC2", (void *)_cffi_const_UNIFORM_VEC2, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "UNIFORM_VEC3", (void *)_cffi_const_UNIFORM_VEC3, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "UNIFORM_VEC4", (void *)_cffi_const_UNIFORM_VEC4, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "UnhideWindow", (void *)_cffi_f_UnhideWindow, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 1140), (void *)_cffi_d_UnhideWindow },
  { "UnloadFont", (void *)_cffi_f_UnloadFont, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 564), (void *)_cffi_d_UnloadFont },
  { "UnloadImage", (void *)_cffi_f_UnloadImage, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 691), (void *)_cffi_d_UnloadImage },
  { "UnloadMaterial", (void *)_cffi_f_UnloadMaterial, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 703), (void *)_cffi_d_UnloadMaterial },
  { "UnloadMesh", (void *)_cffi_f_UnloadMesh, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 712), (void *)_cffi_d_UnloadMesh },
  { "UnloadModel", (void *)_cffi_f_UnloadModel, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 724), (void *)_cffi_d_UnloadModel },
  { "UnloadModelAnimation", (void *)_cffi_f_UnloadModelAnimation, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 746), (void *)_cffi_d_UnloadModelAnimation },
  { "UnloadMusicStream", (void *)_cffi_f_UnloadMusicStream, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 749), (void *)_cffi_d_UnloadMusicStream },
  { "UnloadRenderTexture", (void *)_cffi_f_UnloadRenderTexture, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 799), (void *)_cffi_d_UnloadRenderTexture },
  { "UnloadShader", (void *)_cffi_f_UnloadShader, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 802), (void *)_cffi_d_UnloadShader },
  { "UnloadSound", (void *)_cffi_f_UnloadSound, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 828), (void *)_cffi_d_UnloadSound },
  { "UnloadTexture", (void *)_cffi_f_UnloadTexture, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 843), (void *)_cffi_d_UnloadTexture },
  { "UnloadWave", (void *)_cffi_f_UnloadWave, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 1034), (void *)_cffi_d_UnloadWave },
  { "UpdateAudioStream", (void *)_cffi_f_UpdateAudioStream, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 524), (void *)_cffi_d_UpdateAudioStream },
  { "UpdateCamera", (void *)_cffi_f_UpdateCamera, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 536), (void *)_cffi_d_UpdateCamera },
  { "UpdateModelAnimation", (void *)_cffi_f_UpdateModelAnimation, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 727), (void *)_cffi_d_UpdateModelAnimation },
  { "UpdateMusicStream", (void *)_cffi_f_UpdateMusicStream, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 749), (void *)_cffi_d_UpdateMusicStream },
  { "UpdateSound", (void *)_cffi_f_UpdateSound, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 835), (void *)_cffi_d_UpdateSound },
  { "UpdateTexture", (void *)_cffi_f_UpdateTexture, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 909), (void *)_cffi_d_UpdateTexture },
  { "UpdateVrTracking", (void *)_cffi_f_UpdateVrTracking, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 536), (void *)_cffi_d_UpdateVrTracking },
  { "WRAP_CLAMP", (void *)_cffi_const_WRAP_CLAMP, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "WRAP_MIRROR_CLAMP", (void *)_cffi_const_WRAP_MIRROR_CLAMP, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "WRAP_MIRROR_REPEAT", (void *)_cffi_const_WRAP_MIRROR_REPEAT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "WRAP_REPEAT", (void *)_cffi_const_WRAP_REPEAT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "WaveCopy", (void *)_cffi_f_WaveCopy, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 305), (void *)_cffi_d_WaveCopy },
  { "WaveCrop", (void *)_cffi_f_WaveCrop, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 1023), (void *)_cffi_d_WaveCrop },
  { "WaveFormat", (void *)_cffi_f_WaveFormat, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 1028), (void *)_cffi_d_WaveFormat },
  { "WindowShouldClose", (void *)_cffi_f_WindowShouldClose, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 502), (void *)_cffi_d_WindowShouldClose },
};

static const struct _cffi_field_s _cffi_fields[] = {
  { "sampleRate", offsetof(AudioStream, sampleRate),
                  sizeof(((AudioStream *)0)->sampleRate),
                  _CFFI_OP(_CFFI_OP_NOOP, 1) },
  { "sampleSize", offsetof(AudioStream, sampleSize),
                  sizeof(((AudioStream *)0)->sampleSize),
                  _CFFI_OP(_CFFI_OP_NOOP, 1) },
  { "channels", offsetof(AudioStream, channels),
                sizeof(((AudioStream *)0)->channels),
                _CFFI_OP(_CFFI_OP_NOOP, 1) },
  { "buffer", offsetof(AudioStream, buffer),
              sizeof(((AudioStream *)0)->buffer),
              _CFFI_OP(_CFFI_OP_NOOP, 1185) },
  { "name", offsetof(BoneInfo, name),
            sizeof(((BoneInfo *)0)->name),
            _CFFI_OP(_CFFI_OP_NOOP, 1178) },
  { "parent", offsetof(BoneInfo, parent),
              sizeof(((BoneInfo *)0)->parent),
              _CFFI_OP(_CFFI_OP_NOOP, 10) },
  { "min", offsetof(BoundingBox, min),
           sizeof(((BoundingBox *)0)->min),
           _CFFI_OP(_CFFI_OP_NOOP, 28) },
  { "max", offsetof(BoundingBox, max),
           sizeof(((BoundingBox *)0)->max),
           _CFFI_OP(_CFFI_OP_NOOP, 28) },
  { "offset", offsetof(Camera2D, offset),
              sizeof(((Camera2D *)0)->offset),
              _CFFI_OP(_CFFI_OP_NOOP, 209) },
  { "target", offsetof(Camera2D, target),
              sizeof(((Camera2D *)0)->target),
              _CFFI_OP(_CFFI_OP_NOOP, 209) },
  { "rotation", offsetof(Camera2D, rotation),
                sizeof(((Camera2D *)0)->rotation),
                _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "zoom", offsetof(Camera2D, zoom),
            sizeof(((Camera2D *)0)->zoom),
            _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "position", offsetof(Camera3D, position),
                sizeof(((Camera3D *)0)->position),
                _CFFI_OP(_CFFI_OP_NOOP, 28) },
  { "target", offsetof(Camera3D, target),
              sizeof(((Camera3D *)0)->target),
              _CFFI_OP(_CFFI_OP_NOOP, 28) },
  { "up", offsetof(Camera3D, up),
          sizeof(((Camera3D *)0)->up),
          _CFFI_OP(_CFFI_OP_NOOP, 28) },
  { "fovy", offsetof(Camera3D, fovy),
            sizeof(((Camera3D *)0)->fovy),
            _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "type", offsetof(Camera3D, type),
            sizeof(((Camera3D *)0)->type),
            _CFFI_OP(_CFFI_OP_NOOP, 10) },
  { "value", offsetof(CharInfo, value),
             sizeof(((CharInfo *)0)->value),
             _CFFI_OP(_CFFI_OP_NOOP, 10) },
  { "offsetX", offsetof(CharInfo, offsetX),
               sizeof(((CharInfo *)0)->offsetX),
               _CFFI_OP(_CFFI_OP_NOOP, 10) },
  { "offsetY", offsetof(CharInfo, offsetY),
               sizeof(((CharInfo *)0)->offsetY),
               _CFFI_OP(_CFFI_OP_NOOP, 10) },
  { "advanceX", offsetof(CharInfo, advanceX),
                sizeof(((CharInfo *)0)->advanceX),
                _CFFI_OP(_CFFI_OP_NOOP, 10) },
  { "image", offsetof(CharInfo, image),
             sizeof(((CharInfo *)0)->image),
             _CFFI_OP(_CFFI_OP_NOOP, 16) },
  { "r", offsetof(Color, r),
         sizeof(((Color *)0)->r),
         _CFFI_OP(_CFFI_OP_NOOP, 1187) },
  { "g", offsetof(Color, g),
         sizeof(((Color *)0)->g),
         _CFFI_OP(_CFFI_OP_NOOP, 1187) },
  { "b", offsetof(Color, b),
         sizeof(((Color *)0)->b),
         _CFFI_OP(_CFFI_OP_NOOP, 1187) },
  { "a", offsetof(Color, a),
         sizeof(((Color *)0)->a),
         _CFFI_OP(_CFFI_OP_NOOP, 1187) },
  { "baseSize", offsetof(Font, baseSize),
                sizeof(((Font *)0)->baseSize),
                _CFFI_OP(_CFFI_OP_NOOP, 10) },
  { "charsCount", offsetof(Font, charsCount),
                  sizeof(((Font *)0)->charsCount),
                  _CFFI_OP(_CFFI_OP_NOOP, 10) },
  { "texture", offsetof(Font, texture),
               sizeof(((Font *)0)->texture),
               _CFFI_OP(_CFFI_OP_NOOP, 80) },
  { "recs", offsetof(Font, recs),
            sizeof(((Font *)0)->recs),
            _CFFI_OP(_CFFI_OP_NOOP, 1166) },
  { "chars", offsetof(Font, chars),
             sizeof(((Font *)0)->chars),
             _CFFI_OP(_CFFI_OP_NOOP, 1148) },
  { "data", offsetof(Image, data),
            sizeof(((Image *)0)->data),
            _CFFI_OP(_CFFI_OP_NOOP, 141) },
  { "width", offsetof(Image, width),
             sizeof(((Image *)0)->width),
             _CFFI_OP(_CFFI_OP_NOOP, 10) },
  { "height", offsetof(Image, height),
              sizeof(((Image *)0)->height),
              _CFFI_OP(_CFFI_OP_NOOP, 10) },
  { "mipmaps", offsetof(Image, mipmaps),
               sizeof(((Image *)0)->mipmaps),
               _CFFI_OP(_CFFI_OP_NOOP, 10) },
  { "format", offsetof(Image, format),
              sizeof(((Image *)0)->format),
              _CFFI_OP(_CFFI_OP_NOOP, 10) },
  { "shader", offsetof(Material, shader),
              sizeof(((Material *)0)->shader),
              _CFFI_OP(_CFFI_OP_NOOP, 258) },
  { "maps", offsetof(Material, maps),
            sizeof(((Material *)0)->maps),
            _CFFI_OP(_CFFI_OP_NOOP, 1158) },
  { "params", offsetof(Material, params),
              sizeof(((Material *)0)->params),
              _CFFI_OP(_CFFI_OP_NOOP, 1181) },
  { "texture", offsetof(MaterialMap, texture),
               sizeof(((MaterialMap *)0)->texture),
               _CFFI_OP(_CFFI_OP_NOOP, 80) },
  { "color", offsetof(MaterialMap, color),
             sizeof(((MaterialMap *)0)->color),
             _CFFI_OP(_CFFI_OP_NOOP, 24) },
  { "value", offsetof(MaterialMap, value),
             sizeof(((MaterialMap *)0)->value),
             _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "m0", offsetof(Matrix, m0),
          sizeof(((Matrix *)0)->m0),
          _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "m4", offsetof(Matrix, m4),
          sizeof(((Matrix *)0)->m4),
          _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "m8", offsetof(Matrix, m8),
          sizeof(((Matrix *)0)->m8),
          _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "m12", offsetof(Matrix, m12),
           sizeof(((Matrix *)0)->m12),
           _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "m1", offsetof(Matrix, m1),
          sizeof(((Matrix *)0)->m1),
          _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "m5", offsetof(Matrix, m5),
          sizeof(((Matrix *)0)->m5),
          _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "m9", offsetof(Matrix, m9),
          sizeof(((Matrix *)0)->m9),
          _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "m13", offsetof(Matrix, m13),
           sizeof(((Matrix *)0)->m13),
           _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "m2", offsetof(Matrix, m2),
          sizeof(((Matrix *)0)->m2),
          _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "m6", offsetof(Matrix, m6),
          sizeof(((Matrix *)0)->m6),
          _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "m10", offsetof(Matrix, m10),
           sizeof(((Matrix *)0)->m10),
           _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "m14", offsetof(Matrix, m14),
           sizeof(((Matrix *)0)->m14),
           _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "m3", offsetof(Matrix, m3),
          sizeof(((Matrix *)0)->m3),
          _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "m7", offsetof(Matrix, m7),
          sizeof(((Matrix *)0)->m7),
          _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "m11", offsetof(Matrix, m11),
           sizeof(((Matrix *)0)->m11),
           _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "m15", offsetof(Matrix, m15),
           sizeof(((Matrix *)0)->m15),
           _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "vertexCount", offsetof(Mesh, vertexCount),
                   sizeof(((Mesh *)0)->vertexCount),
                   _CFFI_OP(_CFFI_OP_NOOP, 10) },
  { "triangleCount", offsetof(Mesh, triangleCount),
                     sizeof(((Mesh *)0)->triangleCount),
                     _CFFI_OP(_CFFI_OP_NOOP, 10) },
  { "vertices", offsetof(Mesh, vertices),
                sizeof(((Mesh *)0)->vertices),
                _CFFI_OP(_CFFI_OP_NOOP, 1181) },
  { "texcoords", offsetof(Mesh, texcoords),
                 sizeof(((Mesh *)0)->texcoords),
                 _CFFI_OP(_CFFI_OP_NOOP, 1181) },
  { "texcoords2", offsetof(Mesh, texcoords2),
                  sizeof(((Mesh *)0)->texcoords2),
                  _CFFI_OP(_CFFI_OP_NOOP, 1181) },
  { "normals", offsetof(Mesh, normals),
               sizeof(((Mesh *)0)->normals),
               _CFFI_OP(_CFFI_OP_NOOP, 1181) },
  { "tangents", offsetof(Mesh, tangents),
                sizeof(((Mesh *)0)->tangents),
                _CFFI_OP(_CFFI_OP_NOOP, 1181) },
  { "colors", offsetof(Mesh, colors),
              sizeof(((Mesh *)0)->colors),
              _CFFI_OP(_CFFI_OP_NOOP, 508) },
  { "indices", offsetof(Mesh, indices),
               sizeof(((Mesh *)0)->indices),
               _CFFI_OP(_CFFI_OP_NOOP, 1189) },
  { "animVertices", offsetof(Mesh, animVertices),
                    sizeof(((Mesh *)0)->animVertices),
                    _CFFI_OP(_CFFI_OP_NOOP, 1181) },
  { "animNormals", offsetof(Mesh, animNormals),
                   sizeof(((Mesh *)0)->animNormals),
                   _CFFI_OP(_CFFI_OP_NOOP, 1181) },
  { "boneIds", offsetof(Mesh, boneIds),
               sizeof(((Mesh *)0)->boneIds),
               _CFFI_OP(_CFFI_OP_NOOP, 11) },
  { "boneWeights", offsetof(Mesh, boneWeights),
                   sizeof(((Mesh *)0)->boneWeights),
                   _CFFI_OP(_CFFI_OP_NOOP, 1181) },
  { "vaoId", offsetof(Mesh, vaoId),
             sizeof(((Mesh *)0)->vaoId),
             _CFFI_OP(_CFFI_OP_NOOP, 1) },
  { "vboId", offsetof(Mesh, vboId),
             sizeof(((Mesh *)0)->vboId),
             _CFFI_OP(_CFFI_OP_NOOP, 1188) },
  { "transform", offsetof(Model, transform),
                 sizeof(((Model *)0)->transform),
                 _CFFI_OP(_CFFI_OP_NOOP, 707) },
  { "meshCount", offsetof(Model, meshCount),
                 sizeof(((Model *)0)->meshCount),
                 _CFFI_OP(_CFFI_OP_NOOP, 10) },
  { "meshes", offsetof(Model, meshes),
              sizeof(((Model *)0)->meshes),
              _CFFI_OP(_CFFI_OP_NOOP, 710) },
  { "materialCount", offsetof(Model, materialCount),
                     sizeof(((Model *)0)->materialCount),
                     _CFFI_OP(_CFFI_OP_NOOP, 10) },
  { "materials", offsetof(Model, materials),
                 sizeof(((Model *)0)->materials),
                 _CFFI_OP(_CFFI_OP_NOOP, 699) },
  { "meshMaterial", offsetof(Model, meshMaterial),
                    sizeof(((Model *)0)->meshMaterial),
                    _CFFI_OP(_CFFI_OP_NOOP, 11) },
  { "boneCount", offsetof(Model, boneCount),
                 sizeof(((Model *)0)->boneCount),
                 _CFFI_OP(_CFFI_OP_NOOP, 10) },
  { "bones", offsetof(Model, bones),
             sizeof(((Model *)0)->bones),
             _CFFI_OP(_CFFI_OP_NOOP, 1144) },
  { "bindPose", offsetof(Model, bindPose),
                sizeof(((Model *)0)->bindPose),
                _CFFI_OP(_CFFI_OP_NOOP, 1173) },
  { "boneCount", offsetof(ModelAnimation, boneCount),
                 sizeof(((ModelAnimation *)0)->boneCount),
                 _CFFI_OP(_CFFI_OP_NOOP, 10) },
  { "bones", offsetof(ModelAnimation, bones),
             sizeof(((ModelAnimation *)0)->bones),
             _CFFI_OP(_CFFI_OP_NOOP, 1144) },
  { "frameCount", offsetof(ModelAnimation, frameCount),
                  sizeof(((ModelAnimation *)0)->frameCount),
                  _CFFI_OP(_CFFI_OP_NOOP, 10) },
  { "framePoses", offsetof(ModelAnimation, framePoses),
                  sizeof(((ModelAnimation *)0)->framePoses),
                  _CFFI_OP(_CFFI_OP_NOOP, 1172) },
  { "ctxType", offsetof(Music, ctxType),
               sizeof(((Music *)0)->ctxType),
               _CFFI_OP(_CFFI_OP_NOOP, 10) },
  { "ctxData", offsetof(Music, ctxData),
               sizeof(((Music *)0)->ctxData),
               _CFFI_OP(_CFFI_OP_NOOP, 141) },
  { "sampleCount", offsetof(Music, sampleCount),
                   sizeof(((Music *)0)->sampleCount),
                   _CFFI_OP(_CFFI_OP_NOOP, 1) },
  { "loopCount", offsetof(Music, loopCount),
                 sizeof(((Music *)0)->loopCount),
                 _CFFI_OP(_CFFI_OP_NOOP, 1) },
  { "stream", offsetof(Music, stream),
              sizeof(((Music *)0)->stream),
              _CFFI_OP(_CFFI_OP_NOOP, 384) },
  { "sourceRec", offsetof(NPatchInfo, sourceRec),
                 sizeof(((NPatchInfo *)0)->sourceRec),
                 _CFFI_OP(_CFFI_OP_NOOP, 77) },
  { "left", offsetof(NPatchInfo, left),
            sizeof(((NPatchInfo *)0)->left),
            _CFFI_OP(_CFFI_OP_NOOP, 10) },
  { "top", offsetof(NPatchInfo, top),
           sizeof(((NPatchInfo *)0)->top),
           _CFFI_OP(_CFFI_OP_NOOP, 10) },
  { "right", offsetof(NPatchInfo, right),
             sizeof(((NPatchInfo *)0)->right),
             _CFFI_OP(_CFFI_OP_NOOP, 10) },
  { "bottom", offsetof(NPatchInfo, bottom),
              sizeof(((NPatchInfo *)0)->bottom),
              _CFFI_OP(_CFFI_OP_NOOP, 10) },
  { "type", offsetof(NPatchInfo, type),
            sizeof(((NPatchInfo *)0)->type),
            _CFFI_OP(_CFFI_OP_NOOP, 10) },
  { "position", offsetof(Ray, position),
                sizeof(((Ray *)0)->position),
                _CFFI_OP(_CFFI_OP_NOOP, 28) },
  { "direction", offsetof(Ray, direction),
                 sizeof(((Ray *)0)->direction),
                 _CFFI_OP(_CFFI_OP_NOOP, 28) },
  { "hit", offsetof(RayHitInfo, hit),
           sizeof(((RayHitInfo *)0)->hit),
           _CFFI_OP(_CFFI_OP_NOOP, 10) },
  { "distance", offsetof(RayHitInfo, distance),
                sizeof(((RayHitInfo *)0)->distance),
                _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "position", offsetof(RayHitInfo, position),
                sizeof(((RayHitInfo *)0)->position),
                _CFFI_OP(_CFFI_OP_NOOP, 28) },
  { "normal", offsetof(RayHitInfo, normal),
              sizeof(((RayHitInfo *)0)->normal),
              _CFFI_OP(_CFFI_OP_NOOP, 28) },
  { "x", offsetof(Rectangle, x),
         sizeof(((Rectangle *)0)->x),
         _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "y", offsetof(Rectangle, y),
         sizeof(((Rectangle *)0)->y),
         _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "width", offsetof(Rectangle, width),
             sizeof(((Rectangle *)0)->width),
             _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "height", offsetof(Rectangle, height),
              sizeof(((Rectangle *)0)->height),
              _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "id", offsetof(RenderTexture2D, id),
          sizeof(((RenderTexture2D *)0)->id),
          _CFFI_OP(_CFFI_OP_NOOP, 1) },
  { "texture", offsetof(RenderTexture2D, texture),
               sizeof(((RenderTexture2D *)0)->texture),
               _CFFI_OP(_CFFI_OP_NOOP, 80) },
  { "depth", offsetof(RenderTexture2D, depth),
             sizeof(((RenderTexture2D *)0)->depth),
             _CFFI_OP(_CFFI_OP_NOOP, 80) },
  { "depthTexture", offsetof(RenderTexture2D, depthTexture),
                    sizeof(((RenderTexture2D *)0)->depthTexture),
                    _CFFI_OP(_CFFI_OP_NOOP, 10) },
  { "id", offsetof(Shader, id),
          sizeof(((Shader *)0)->id),
          _CFFI_OP(_CFFI_OP_NOOP, 1) },
  { "locs", offsetof(Shader, locs),
            sizeof(((Shader *)0)->locs),
            _CFFI_OP(_CFFI_OP_NOOP, 11) },
  { "sampleCount", offsetof(Sound, sampleCount),
                   sizeof(((Sound *)0)->sampleCount),
                   _CFFI_OP(_CFFI_OP_NOOP, 1) },
  { "stream", offsetof(Sound, stream),
              sizeof(((Sound *)0)->stream),
              _CFFI_OP(_CFFI_OP_NOOP, 384) },
  { "id", offsetof(Texture2D, id),
          sizeof(((Texture2D *)0)->id),
          _CFFI_OP(_CFFI_OP_NOOP, 1) },
  { "width", offsetof(Texture2D, width),
             sizeof(((Texture2D *)0)->width),
             _CFFI_OP(_CFFI_OP_NOOP, 10) },
  { "height", offsetof(Texture2D, height),
              sizeof(((Texture2D *)0)->height),
              _CFFI_OP(_CFFI_OP_NOOP, 10) },
  { "mipmaps", offsetof(Texture2D, mipmaps),
               sizeof(((Texture2D *)0)->mipmaps),
               _CFFI_OP(_CFFI_OP_NOOP, 10) },
  { "format", offsetof(Texture2D, format),
              sizeof(((Texture2D *)0)->format),
              _CFFI_OP(_CFFI_OP_NOOP, 10) },
  { "translation", offsetof(Transform, translation),
                   sizeof(((Transform *)0)->translation),
                   _CFFI_OP(_CFFI_OP_NOOP, 28) },
  { "rotation", offsetof(Transform, rotation),
                sizeof(((Transform *)0)->rotation),
                _CFFI_OP(_CFFI_OP_NOOP, 31) },
  { "scale", offsetof(Transform, scale),
             sizeof(((Transform *)0)->scale),
             _CFFI_OP(_CFFI_OP_NOOP, 28) },
  { "x", offsetof(Vector2, x),
         sizeof(((Vector2 *)0)->x),
         _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "y", offsetof(Vector2, y),
         sizeof(((Vector2 *)0)->y),
         _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "x", offsetof(Vector3, x),
         sizeof(((Vector3 *)0)->x),
         _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "y", offsetof(Vector3, y),
         sizeof(((Vector3 *)0)->y),
         _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "z", offsetof(Vector3, z),
         sizeof(((Vector3 *)0)->z),
         _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "x", offsetof(Vector4, x),
         sizeof(((Vector4 *)0)->x),
         _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "y", offsetof(Vector4, y),
         sizeof(((Vector4 *)0)->y),
         _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "z", offsetof(Vector4, z),
         sizeof(((Vector4 *)0)->z),
         _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "w", offsetof(Vector4, w),
         sizeof(((Vector4 *)0)->w),
         _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "hResolution", offsetof(VrDeviceInfo, hResolution),
                   sizeof(((VrDeviceInfo *)0)->hResolution),
                   _CFFI_OP(_CFFI_OP_NOOP, 10) },
  { "vResolution", offsetof(VrDeviceInfo, vResolution),
                   sizeof(((VrDeviceInfo *)0)->vResolution),
                   _CFFI_OP(_CFFI_OP_NOOP, 10) },
  { "hScreenSize", offsetof(VrDeviceInfo, hScreenSize),
                   sizeof(((VrDeviceInfo *)0)->hScreenSize),
                   _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "vScreenSize", offsetof(VrDeviceInfo, vScreenSize),
                   sizeof(((VrDeviceInfo *)0)->vScreenSize),
                   _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "vScreenCenter", offsetof(VrDeviceInfo, vScreenCenter),
                     sizeof(((VrDeviceInfo *)0)->vScreenCenter),
                     _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "eyeToScreenDistance", offsetof(VrDeviceInfo, eyeToScreenDistance),
                           sizeof(((VrDeviceInfo *)0)->eyeToScreenDistance),
                           _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "lensSeparationDistance", offsetof(VrDeviceInfo, lensSeparationDistance),
                              sizeof(((VrDeviceInfo *)0)->lensSeparationDistance),
                              _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "interpupillaryDistance", offsetof(VrDeviceInfo, interpupillaryDistance),
                              sizeof(((VrDeviceInfo *)0)->interpupillaryDistance),
                              _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "lensDistortionValues", offsetof(VrDeviceInfo, lensDistortionValues),
                            sizeof(((VrDeviceInfo *)0)->lensDistortionValues),
                            _CFFI_OP(_CFFI_OP_NOOP, 1182) },
  { "chromaAbCorrection", offsetof(VrDeviceInfo, chromaAbCorrection),
                          sizeof(((VrDeviceInfo *)0)->chromaAbCorrection),
                          _CFFI_OP(_CFFI_OP_NOOP, 1182) },
  { "sampleCount", offsetof(Wave, sampleCount),
                   sizeof(((Wave *)0)->sampleCount),
                   _CFFI_OP(_CFFI_OP_NOOP, 1) },
  { "sampleRate", offsetof(Wave, sampleRate),
                  sizeof(((Wave *)0)->sampleRate),
                  _CFFI_OP(_CFFI_OP_NOOP, 1) },
  { "sampleSize", offsetof(Wave, sampleSize),
                  sizeof(((Wave *)0)->sampleSize),
                  _CFFI_OP(_CFFI_OP_NOOP, 1) },
  { "channels", offsetof(Wave, channels),
                sizeof(((Wave *)0)->channels),
                _CFFI_OP(_CFFI_OP_NOOP, 1) },
  { "data", offsetof(Wave, data),
            sizeof(((Wave *)0)->data),
            _CFFI_OP(_CFFI_OP_NOOP, 141) },
};

static const struct _cffi_struct_union_s _cffi_struct_unions[] = {
  { "AudioStream", 384, _CFFI_F_CHECK_FIELDS,
    sizeof(AudioStream), offsetof(struct _cffi_align__AudioStream, y), 0, 4 },
  { "BoneInfo", 1145, _CFFI_F_CHECK_FIELDS,
    sizeof(BoneInfo), offsetof(struct _cffi_align__BoneInfo, y), 4, 2 },
  { "BoundingBox", 387, _CFFI_F_CHECK_FIELDS,
    sizeof(BoundingBox), offsetof(struct _cffi_align__BoundingBox, y), 6, 2 },
  { "Camera2D", 155, _CFFI_F_CHECK_FIELDS,
    sizeof(Camera2D), offsetof(struct _cffi_align__Camera2D, y), 8, 4 },
  { "Camera3D", 158, _CFFI_F_CHECK_FIELDS,
    sizeof(Camera3D), offsetof(struct _cffi_align__Camera3D, y), 12, 5 },
  { "CharInfo", 1149, _CFFI_F_CHECK_FIELDS,
    sizeof(CharInfo), offsetof(struct _cffi_align__CharInfo, y), 17, 5 },
  { "Color", 24, _CFFI_F_CHECK_FIELDS,
    sizeof(Color), offsetof(struct _cffi_align__Color, y), 22, 4 },
  { "Font", 66, _CFFI_F_CHECK_FIELDS,
    sizeof(Font), offsetof(struct _cffi_align__Font, y), 26, 5 },
  { "Image", 16, _CFFI_F_CHECK_FIELDS,
    sizeof(Image), offsetof(struct _cffi_align__Image, y), 31, 5 },
  { "Material", 704, _CFFI_F_CHECK_FIELDS,
    sizeof(Material), offsetof(struct _cffi_align__Material, y), 36, 3 },
  { "MaterialMap", 1159, _CFFI_F_CHECK_FIELDS,
    sizeof(MaterialMap), offsetof(struct _cffi_align__MaterialMap, y), 39, 3 },
  { "Matrix", 707, _CFFI_F_CHECK_FIELDS,
    sizeof(Matrix), offsetof(struct _cffi_align__Matrix, y), 42, 16 },
  { "Mesh", 6, _CFFI_F_CHECK_FIELDS,
    sizeof(Mesh), offsetof(struct _cffi_align__Mesh, y), 58, 15 },
  { "Model", 214, _CFFI_F_CHECK_FIELDS,
    sizeof(Model), offsetof(struct _cffi_align__Model, y), 73, 9 },
  { "ModelAnimation", 404, _CFFI_F_CHECK_FIELDS,
    sizeof(ModelAnimation), offsetof(struct _cffi_align__ModelAnimation, y), 82, 4 },
  { "Music", 371, _CFFI_F_CHECK_FIELDS,
    sizeof(Music), offsetof(struct _cffi_align__Music, y), 86, 5 },
  { "NPatchInfo", 848, _CFFI_F_CHECK_FIELDS,
    sizeof(NPatchInfo), offsetof(struct _cffi_align__NPatchInfo, y), 91, 6 },
  { "Ray", 213, _CFFI_F_CHECK_FIELDS,
    sizeof(Ray), offsetof(struct _cffi_align__Ray, y), 97, 2 },
  { "RayHitInfo", 1165, _CFFI_F_CHECK_FIELDS,
    sizeof(RayHitInfo), offsetof(struct _cffi_align__RayHitInfo, y), 99, 4 },
  { "Rectangle", 77, _CFFI_F_CHECK_FIELDS,
    sizeof(Rectangle), offsetof(struct _cffi_align__Rectangle, y), 103, 4 },
  { "RenderTexture2D", 800, _CFFI_F_CHECK_FIELDS,
    sizeof(RenderTexture2D), offsetof(struct _cffi_align__RenderTexture2D, y), 107, 4 },
  { "Shader", 258, _CFFI_F_CHECK_FIELDS,
    sizeof(Shader), offsetof(struct _cffi_align__Shader, y), 111, 2 },
  { "Sound", 433, _CFFI_F_CHECK_FIELDS,
    sizeof(Sound), offsetof(struct _cffi_align__Sound, y), 113, 2 },
  { "Texture2D", 80, _CFFI_F_CHECK_FIELDS,
    sizeof(Texture2D), offsetof(struct _cffi_align__Texture2D, y), 115, 5 },
  { "Transform", 1174, _CFFI_F_CHECK_FIELDS,
    sizeof(Transform), offsetof(struct _cffi_align__Transform, y), 120, 3 },
  { "Vector2", 209, _CFFI_F_CHECK_FIELDS,
    sizeof(Vector2), offsetof(struct _cffi_align__Vector2, y), 123, 2 },
  { "Vector3", 28, _CFFI_F_CHECK_FIELDS,
    sizeof(Vector3), offsetof(struct _cffi_align__Vector3, y), 125, 3 },
  { "Vector4", 31, _CFFI_F_CHECK_FIELDS,
    sizeof(Vector4), offsetof(struct _cffi_align__Vector4, y), 128, 4 },
  { "VrDeviceInfo", 1020, _CFFI_F_CHECK_FIELDS,
    sizeof(VrDeviceInfo), offsetof(struct _cffi_align__VrDeviceInfo, y), 132, 10 },
  { "Wave", 245, _CFFI_F_CHECK_FIELDS,
    sizeof(Wave), offsetof(struct _cffi_align__Wave, y), 142, 5 },
  { "rAudioBuffer", 1186, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
};

static const struct _cffi_enum_s _cffi_enums[] = {
  { "$AndroidButton", 1142, _cffi_prim_int(sizeof(AndroidButton), ((AndroidButton)-1) <= 0),
    "KEY_BACK,KEY_MENU,KEY_VOLUME_UP,KEY_VOLUME_DOWN" },
  { "$BlendMode", 1143, _cffi_prim_int(sizeof(BlendMode), ((BlendMode)-1) <= 0),
    "BLEND_ALPHA,BLEND_ADDITIVE,BLEND_MULTIPLIED" },
  { "$CameraMode", 1146, _cffi_prim_int(sizeof(CameraMode), ((CameraMode)-1) <= 0),
    "CAMERA_CUSTOM,CAMERA_FREE,CAMERA_ORBITAL,CAMERA_FIRST_PERSON,CAMERA_THIRD_PERSON" },
  { "$CameraType", 1147, _cffi_prim_int(sizeof(CameraType), ((CameraType)-1) <= 0),
    "CAMERA_PERSPECTIVE,CAMERA_ORTHOGRAPHIC" },
  { "$ConfigFlag", 1150, _cffi_prim_int(sizeof(ConfigFlag), ((ConfigFlag)-1) <= 0),
    "FLAG_RESERVED,FLAG_FULLSCREEN_MODE,FLAG_WINDOW_RESIZABLE,FLAG_WINDOW_UNDECORATED,FLAG_WINDOW_TRANSPARENT,FLAG_WINDOW_HIDDEN,FLAG_WINDOW_ALWAYS_RUN,FLAG_MSAA_4X_HINT,FLAG_VSYNC_HINT" },
  { "$CubemapLayoutType", 1151, _cffi_prim_int(sizeof(CubemapLayoutType), ((CubemapLayoutType)-1) <= 0),
    "CUBEMAP_AUTO_DETECT,CUBEMAP_LINE_VERTICAL,CUBEMAP_LINE_HORIZONTAL,CUBEMAP_CROSS_THREE_BY_FOUR,CUBEMAP_CROSS_FOUR_BY_THREE,CUBEMAP_PANORAMA" },
  { "$FontType", 1152, _cffi_prim_int(sizeof(FontType), ((FontType)-1) <= 0),
    "FONT_DEFAULT,FONT_BITMAP,FONT_SDF" },
  { "$GamepadAxis", 1153, _cffi_prim_int(sizeof(GamepadAxis), ((GamepadAxis)-1) <= 0),
    "GAMEPAD_AXIS_UNKNOWN,GAMEPAD_AXIS_LEFT_X,GAMEPAD_AXIS_LEFT_Y,GAMEPAD_AXIS_RIGHT_X,GAMEPAD_AXIS_RIGHT_Y,GAMEPAD_AXIS_LEFT_TRIGGER,GAMEPAD_AXIS_RIGHT_TRIGGER" },
  { "$GamepadButton", 1154, _cffi_prim_int(sizeof(GamepadButton), ((GamepadButton)-1) <= 0),
    "GAMEPAD_BUTTON_UNKNOWN,GAMEPAD_BUTTON_LEFT_FACE_UP,GAMEPAD_BUTTON_LEFT_FACE_RIGHT,GAMEPAD_BUTTON_LEFT_FACE_DOWN,GAMEPAD_BUTTON_LEFT_FACE_LEFT,GAMEPAD_BUTTON_RIGHT_FACE_UP,GAMEPAD_BUTTON_RIGHT_FACE_RIGHT,GAMEPAD_BUTTON_RIGHT_FACE_DOWN,GAMEPAD_BUTTON_RIGHT_FACE_LEFT,GAMEPAD_BUTTON_LEFT_TRIGGER_1,GAMEPAD_BUTTON_LEFT_TRIGGER_2,GAMEPAD_BUTTON_RIGHT_TRIGGER_1,GAMEPAD_BUTTON_RIGHT_TRIGGER_2,GAMEPAD_BUTTON_MIDDLE_LEFT,GAMEPAD_BUTTON_MIDDLE,GAMEPAD_BUTTON_MIDDLE_RIGHT,GAMEPAD_BUTTON_LEFT_THUMB,GAMEPAD_BUTTON_RIGHT_THUMB" },
  { "$GamepadNumber", 1155, _cffi_prim_int(sizeof(GamepadNumber), ((GamepadNumber)-1) <= 0),
    "GAMEPAD_PLAYER1,GAMEPAD_PLAYER2,GAMEPAD_PLAYER3,GAMEPAD_PLAYER4" },
  { "$GestureType", 1156, _cffi_prim_int(sizeof(GestureType), ((GestureType)-1) <= 0),
    "GESTURE_NONE,GESTURE_TAP,GESTURE_DOUBLETAP,GESTURE_HOLD,GESTURE_DRAG,GESTURE_SWIPE_RIGHT,GESTURE_SWIPE_LEFT,GESTURE_SWIPE_UP,GESTURE_SWIPE_DOWN,GESTURE_PINCH_IN,GESTURE_PINCH_OUT" },
  { "$KeyboardKey", 1157, _cffi_prim_int(sizeof(KeyboardKey), ((KeyboardKey)-1) <= 0),
    "KEY_APOSTROPHE,KEY_COMMA,KEY_MINUS,KEY_PERIOD,KEY_SLASH,KEY_ZERO,KEY_ONE,KEY_TWO,KEY_THREE,KEY_FOUR,KEY_FIVE,KEY_SIX,KEY_SEVEN,KEY_EIGHT,KEY_NINE,KEY_SEMICOLON,KEY_EQUAL,KEY_A,KEY_B,KEY_C,KEY_D,KEY_E,KEY_F,KEY_G,KEY_H,KEY_I,KEY_J,KEY_K,KEY_L,KEY_M,KEY_N,KEY_O,KEY_P,KEY_Q,KEY_R,KEY_S,KEY_T,KEY_U,KEY_V,KEY_W,KEY_X,KEY_Y,KEY_Z,KEY_SPACE,KEY_ESCAPE,KEY_ENTER,KEY_TAB,KEY_BACKSPACE,KEY_INSERT,KEY_DELETE,KEY_RIGHT,KEY_LEFT,KEY_DOWN,KEY_UP,KEY_PAGE_UP,KEY_PAGE_DOWN,KEY_HOME,KEY_END,KEY_CAPS_LOCK,KEY_SCROLL_LOCK,KEY_NUM_LOCK,KEY_PRINT_SCREEN,KEY_PAUSE,KEY_F1,KEY_F2,KEY_F3,KEY_F4,KEY_F5,KEY_F6,KEY_F7,KEY_F8,KEY_F9,KEY_F10,KEY_F11,KEY_F12,KEY_LEFT_SHIFT,KEY_LEFT_CONTROL,KEY_LEFT_ALT,KEY_LEFT_SUPER,KEY_RIGHT_SHIFT,KEY_RIGHT_CONTROL,KEY_RIGHT_ALT,KEY_RIGHT_SUPER,KEY_KB_MENU,KEY_LEFT_BRACKET,KEY_BACKSLASH,KEY_RIGHT_BRACKET,KEY_GRAVE,KEY_KP_0,KEY_KP_1,KEY_KP_2,KEY_KP_3,KEY_KP_4,KEY_KP_5,KEY_KP_6,KEY_KP_7,KEY_KP_8,KEY_KP_9,KEY_KP_DECIMAL,KEY_KP_DIVIDE,KEY_KP_MULTIPLY,KEY_KP_SUBTRACT,KEY_KP_ADD,KEY_KP_ENTER,KEY_KP_EQUAL" },
  { "$MaterialMapType", 1160, _cffi_prim_int(sizeof(MaterialMapType), ((MaterialMapType)-1) <= 0),
    "MAP_ALBEDO,MAP_METALNESS,MAP_NORMAL,MAP_ROUGHNESS,MAP_OCCLUSION,MAP_EMISSION,MAP_HEIGHT,MAP_CUBEMAP,MAP_IRRADIANCE,MAP_PREFILTER,MAP_BRDF" },
  { "$MouseButton", 1162, _cffi_prim_int(sizeof(MouseButton), ((MouseButton)-1) <= 0),
    "MOUSE_LEFT_BUTTON,MOUSE_RIGHT_BUTTON,MOUSE_MIDDLE_BUTTON" },
  { "$NPatchType", 1163, _cffi_prim_int(sizeof(NPatchType), ((NPatchType)-1) <= 0),
    "NPT_9PATCH,NPT_3PATCH_VERTICAL,NPT_3PATCH_HORIZONTAL" },
  { "$PixelFormat", 1164, _cffi_prim_int(sizeof(PixelFormat), ((PixelFormat)-1) <= 0),
    "UNCOMPRESSED_GRAYSCALE,UNCOMPRESSED_GRAY_ALPHA,UNCOMPRESSED_R5G6B5,UNCOMPRESSED_R8G8B8,UNCOMPRESSED_R5G5B5A1,UNCOMPRESSED_R4G4B4A4,UNCOMPRESSED_R8G8B8A8,UNCOMPRESSED_R32,UNCOMPRESSED_R32G32B32,UNCOMPRESSED_R32G32B32A32,COMPRESSED_DXT1_RGB,COMPRESSED_DXT1_RGBA,COMPRESSED_DXT3_RGBA,COMPRESSED_DXT5_RGBA,COMPRESSED_ETC1_RGB,COMPRESSED_ETC2_RGB,COMPRESSED_ETC2_EAC_RGBA,COMPRESSED_PVRT_RGB,COMPRESSED_PVRT_RGBA,COMPRESSED_ASTC_4x4_RGBA,COMPRESSED_ASTC_8x8_RGBA" },
  { "$ShaderLocationIndex", 1167, _cffi_prim_int(sizeof(ShaderLocationIndex), ((ShaderLocationIndex)-1) <= 0),
    "LOC_VERTEX_POSITION,LOC_VERTEX_TEXCOORD01,LOC_VERTEX_TEXCOORD02,LOC_VERTEX_NORMAL,LOC_VERTEX_TANGENT,LOC_VERTEX_COLOR,LOC_MATRIX_MVP,LOC_MATRIX_MODEL,LOC_MATRIX_VIEW,LOC_MATRIX_PROJECTION,LOC_VECTOR_VIEW,LOC_COLOR_DIFFUSE,LOC_COLOR_SPECULAR,LOC_COLOR_AMBIENT,LOC_MAP_ALBEDO,LOC_MAP_METALNESS,LOC_MAP_NORMAL,LOC_MAP_ROUGHNESS,LOC_MAP_OCCLUSION,LOC_MAP_EMISSION,LOC_MAP_HEIGHT,LOC_MAP_CUBEMAP,LOC_MAP_IRRADIANCE,LOC_MAP_PREFILTER,LOC_MAP_BRDF" },
  { "$ShaderUniformDataType", 1168, _cffi_prim_int(sizeof(ShaderUniformDataType), ((ShaderUniformDataType)-1) <= 0),
    "UNIFORM_FLOAT,UNIFORM_VEC2,UNIFORM_VEC3,UNIFORM_VEC4,UNIFORM_INT,UNIFORM_IVEC2,UNIFORM_IVEC3,UNIFORM_IVEC4,UNIFORM_SAMPLER2D" },
  { "$TextureFilterMode", 1169, _cffi_prim_int(sizeof(TextureFilterMode), ((TextureFilterMode)-1) <= 0),
    "FILTER_POINT,FILTER_BILINEAR,FILTER_TRILINEAR,FILTER_ANISOTROPIC_4X,FILTER_ANISOTROPIC_8X,FILTER_ANISOTROPIC_16X" },
  { "$TextureWrapMode", 1170, _cffi_prim_int(sizeof(TextureWrapMode), ((TextureWrapMode)-1) <= 0),
    "WRAP_REPEAT,WRAP_CLAMP,WRAP_MIRROR_REPEAT,WRAP_MIRROR_CLAMP" },
  { "$TraceLogType", 1171, _cffi_prim_int(sizeof(TraceLogType), ((TraceLogType)-1) <= 0),
    "LOG_ALL,LOG_TRACE,LOG_DEBUG,LOG_INFO,LOG_WARNING,LOG_ERROR,LOG_FATAL,LOG_NONE" },
};

static const struct _cffi_typename_s _cffi_typenames[] = {
  { "AndroidButton", 1142 },
  { "AudioStream", 384 },
  { "BlendMode", 1143 },
  { "BoneInfo", 1145 },
  { "BoundingBox", 387 },
  { "Camera", 158 },
  { "Camera2D", 155 },
  { "Camera3D", 158 },
  { "CameraMode", 1146 },
  { "CameraType", 1147 },
  { "CharInfo", 1149 },
  { "Color", 24 },
  { "ConfigFlag", 1150 },
  { "CubemapLayoutType", 1151 },
  { "Font", 66 },
  { "FontType", 1152 },
  { "GamepadAxis", 1153 },
  { "GamepadButton", 1154 },
  { "GamepadNumber", 1155 },
  { "GestureType", 1156 },
  { "Image", 16 },
  { "KeyboardKey", 1157 },
  { "Material", 704 },
  { "MaterialMap", 1159 },
  { "MaterialMapType", 1160 },
  { "Matrix", 707 },
  { "Mesh", 6 },
  { "Model", 214 },
  { "ModelAnimation", 404 },
  { "MouseButton", 1162 },
  { "Music", 371 },
  { "NPatchInfo", 848 },
  { "NPatchType", 1163 },
  { "PixelFormat", 1164 },
  { "Quaternion", 31 },
  { "Ray", 213 },
  { "RayHitInfo", 1165 },
  { "Rectangle", 77 },
  { "RenderTexture", 800 },
  { "RenderTexture2D", 800 },
  { "Shader", 258 },
  { "ShaderLocationIndex", 1167 },
  { "ShaderUniformDataType", 1168 },
  { "Sound", 433 },
  { "Texture", 80 },
  { "Texture2D", 80 },
  { "TextureCubemap", 80 },
  { "TextureFilterMode", 1169 },
  { "TextureWrapMode", 1170 },
  { "TraceLogType", 1171 },
  { "Transform", 1174 },
  { "Vector2", 209 },
  { "Vector3", 28 },
  { "Vector4", 31 },
  { "VrDeviceInfo", 1020 },
  { "Wave", 245 },
  { "rAudioBuffer", 1186 },
};

static const struct _cffi_type_context_s _cffi_type_context = {
  _cffi_types,
  _cffi_globals,
  _cffi_fields,
  _cffi_struct_unions,
  _cffi_enums,
  _cffi_typenames,
  696,  /* num_globals */
  31,  /* num_struct_unions */
  21,  /* num_enums */
  57,  /* num_typenames */
  NULL,  /* no includes */
  1193,  /* num_types */
  0,  /* flags */
};

#ifdef __GNUC__
#  pragma GCC visibility push(default)  /* for -fvisibility= */
#endif

#ifdef PYPY_VERSION
PyMODINIT_FUNC
_cffi_pypyinit__raylib_cffi(const void *p[])
{
    p[0] = (const void *)0x2601;
    p[1] = &_cffi_type_context;
#if PY_MAJOR_VERSION >= 3
    return NULL;
#endif
}
#  ifdef _MSC_VER
     PyMODINIT_FUNC
#  if PY_MAJOR_VERSION >= 3
     PyInit__raylib_cffi(void) { return NULL; }
#  else
     init_raylib_cffi(void) { }
#  endif
#  endif
#elif PY_MAJOR_VERSION >= 3
PyMODINIT_FUNC
PyInit__raylib_cffi(void)
{
  return _cffi_init("_raylib_cffi", 0x2601, &_cffi_type_context);
}
#else
PyMODINIT_FUNC
init_raylib_cffi(void)
{
  _cffi_init("_raylib_cffi", 0x2601, &_cffi_type_context);
}
#endif

#ifdef __GNUC__
#  pragma GCC visibility pop
#endif
